/*
 * CodeElement.java
 *
 * Created on September 30, 2001, 6:43 PM
 */

package IDE.CodeEditor;
import javax.swing.text.*;
/**
 * Represent a code element which is the direct parent of a leaf (lisp atoms,keywords ...)
 * @author  marc
 * @version
 */
public abstract class CodeAtomElement extends CodeElement
{
    /**  The direct leaf where this element is the only parent of */
    protected CodeLeafElement leaf;
    /** Indicates if element is complete */
    private boolean complete=true;
    /** Contructor of CodeElement */
    public CodeAtomElement(CodeLeafElement leaf)
    {
        this.leaf=leaf;
    }
    /** Get the begin of the CodeElement */
    public int begin()
    {
        return leaf.begin();
    }
    /** Get the end of the element */
    public int end()
    {
        return leaf.end();
    }
    /** Return the begin position */
    public Position beginPosition()
    {
    	return leaf.beginPosition();
    }
    /** Return the length */
    public int length()
    {
    	return end()-begin();
    }
    /** Make this element incomplete , the default is complete */
    public void markIncomplete()
    {
        complete=false;
    }
    /** Indicates if a element is complete, normally we must override this method */
    public boolean isComplete()
    {
        return complete;
    }
    /** Get the string defined by begin() <= i < end()  */
    public String getString(CodeDocument document)
    {
        return document.getString(begin(),end()-begin());
    }
    /** Tell the childs of this element that we are their parent,grandparent ... */
    public void acceptChilds()
    {
        leaf.setParent(this);
    }
    /** Indicates if a child is the first child in this element and if this element is first child in its parent */
    protected boolean isFirst(CodeElement child)
    {
        if (parent==null)
        {
            return child==leaf;
        }
        else
        {
            return child==leaf && parent.isFirst(this);
        }
    }
    /** Get the leaf element of this atom */
    public CodeLeafElement getLeaf()
    {
        return leaf;
    }
}
