/*
 * CodeCursor.java
 *
 * Created on May 25, 2002, 9:52 AM
 */

package IDE.CodeEditor;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import IDE.*;
import IDE.Help.*;
import IDE.Configuration.*;
import IDE.Messaging.*;

/**
 * Draws the cursor on a editor
 * @author  marc
 */
public class CodeCursor
{
    /** Static fields used for undrawing the old draw cursor */
    /** The old X position of the cursor */
    private static int oldX=-1;
    /** The old Y position of the cursor */
    private static int oldY=-1;
    /** The old height of the cursor */
    private static int oldHeight=-1;
    /** The old editor where the cursor was drawn */
    private static CodeEditor oldEditor=null;
    /** Static field telling if we must scroll the editor or not */
    private static boolean scroll;
    /** Static Runnable field to undraw the cursor */
    private static Runnable removeCursor=new Runnable()
    {
        public void run()
        {
            if (oldEditor!=null)
            {
                oldEditor.repaint(0,oldX,oldY,2,oldHeight);
                oldX=-1;
                oldY=-1;
                oldHeight=-1;
                oldEditor=null;
            }
        }
    };
    /** Contains the timer object to draw cursor and highlight parentheses and lists */
    private static java.util.Timer timer=new java.util.Timer();
    /** Contains the TimerTask to highlight the parentheses (so we can interrupt this */
    private static CodeCursor.HighlightParentheses highlightParenthesesTask=null;
    /** Contains the TimerTask to display a blinking cursor */
    private static CodeCursor.CursorBlinker cursorBlinkerTask=null;
    /** Contains the TimerTask to display a list */
    private static CodeCursor.HighLightList highlightListTask=null;
    /** Non static fields hold by each cursor */
    /** Editor where this cursor is draw in */
    private CodeEditor editor;
    /** Cursor position in the document of the editor */
    private int position=0;
    /** Magic cursor position, used in scroll up and scroll downs */
    private int magicposition=0;
    /** Creates a new instance of CodeCursor */
    public CodeCursor(CodeEditor editor)
    {
        this.editor=editor;
    }
    /** Move the cursor to the left */
    public void moveLeft()
    {
        setCursor(position-1);
    }
    /** Move the cursor to the right */
    public void moveRight()
    {
        setCursor(position+1);
    }
    /** Move the cursor one line up, taking in account the magicposition */
    public void moveUp()
    {
        CodeDocument document=editor.getDocument();
        int iline=document.getIndexLineAt(position);
        if (iline>0)
        {
            CodeLine line=document.getLines().lineAt(iline-1);
            int lbegin=line.begin();
            int pos=Math.max(lbegin,Math.min(line.end()-1,lbegin+magicposition));
            setCursorImp(pos,true);
        };
    }
    /** Move the cursor one line down, taking in account the magicposition */
    public void moveDown()
    {
        CodeDocument document=editor.getDocument();
        int iline=document.getIndexLineAt(position);
        if (iline+1<document.getLines().size())
        {
            CodeLine line=document.getLines().lineAt(iline+1);
            int lbegin=line.begin();
            int pos=Math.max(lbegin,Math.min(line.end()-1,lbegin+magicposition));
            setCursorImp(pos,true);
        };
    }
    /** Moves the cursor to the end of the document, do no highlighting or make the editor current */
    public void moveToEnd()
    {
        moveCursorTo(editor.getDocument().length()-1);
    }
    /** Modify the cursor offset and the magic position but do not update the editor screen */
    public void moveCursorTo(int position)
    {
        if (position!=this.position)
        {
            CodeDocument document=editor.getDocument();
            CodeLine line=document.getLineAt(position);
            if (line!=null)
                magicposition=position-line.begin();
        };
        Point point=editor.getPoint(position);
        int lnw=editor.getLineNumberingWidth();
        if (point!=null)
            if (point.x<=lnw+20)
                editor.scrollRectToVisible(new Rectangle(0,Math.max(0,point.y-50),20,80));
            else
                editor.scrollRectToVisible(new Rectangle(point.x,Math.max(0,point.y-50),20,80));
        this.position=position;
    }
    /** Go one page down */
    public void pageDown()
    {
        // Get the rectangle of the viewport
        Rectangle rectangle=editor.getCodePane().getVisibleRectangleEditor();
        rectangle.y=rectangle.y+rectangle.height;
        editor.scrollRectToVisible(rectangle);
        Point point=editor.getPoint(position); // Get the point of the cursor
        point.y=point.y+rectangle.height;
        setCursor(editor.getPosition(point.x,point.y));
    }
    /** Go one page up */
    public void pageUp()
    {
        // Get the rectangle of the viewport
        Rectangle rectangle=editor.getCodePane().getVisibleRectangleEditor();
        rectangle.y=Math.max(0,rectangle.y-rectangle.height);
        editor.scrollRectToVisible(rectangle);
        Point point=editor.getPoint(position); // Get the point of the cursor
        point.y=Math.max(0,point.y-rectangle.height);
        setCursor(editor.getPosition(point.x,point.y));
    }
    /** Get the cursor offset */
    public int  cursorOffset()
    {
        return position;
    }
    /** Set the cursor to a specific location and make the editor of the cursor current  and highlight the list and parentheses */
    public void setCursor(int position)
    {
        setCursor(position,true);
    }
    /** Set the cursor to a specific location, makeCurrent indicates if we must make the editor of the cursor current and highlight the list and parentheses */
    public void setCursor(int position,boolean makecurrent)
    {
        if (position!=this.position)
        {
            CodeDocument document=editor.getDocument();
            CodeLine line=document.getLineAt(position);
            if (line!=null)
                magicposition=position-line.begin();
        };
        setCursorImp(position,makecurrent);
    }
    /** private implementation of setCursor */
    private void setCursorImp(int position,boolean makeCurrent)
    {
        CodeDocument document=editor.getDocument();
        if (document!=null)
        {
            if (position<0 || position>=document.length())
                return;
            this.position=position;
            int iline=document.getIndexLineAt(position);
            if (makeCurrent) 
                Main.setCurrent(editor);
            // Update the line info field
            Main.updateCursorInfo(iline+1,(iline==-1) ? 0 : position-document.getLines().lineAt(iline).begin()+1);
            // Cancel all pending cursor tasks and restart cursor
            resetCursor(new CursorBlinker());
            // If needed add a task to display the parentheses and highlight the list
            if (makeCurrent)
            {
                timer.schedule(highlightParenthesesTask=new CodeCursor.HighlightParentheses(),100);
                if (Configuration.highligthList && !editor.hasSelectedText())
                    timer.schedule(highlightListTask=new CodeCursor.HighLightList(),150);
                Hint.activateHint();
            };
        };
    }
    /** Avoid any problems with hangin cursors */
    private static synchronized void resetCursor(CursorBlinker newCursorBlinker)
    {
        // Cancel all pending cursor tasks
        if (cursorBlinkerTask!=null) 
            cursorBlinkerTask.cancel();
        if (highlightParenthesesTask!=null)
        {
            highlightParenthesesTask.interrupt();
            highlightParenthesesTask.cancel();
        };
        if (highlightListTask!=null)
            highlightListTask.cancel();
        // Ensure that we scroll
        scroll=true;
        // Add task to draw a blinking cursor
        cursorBlinkerTask=newCursorBlinker;
        timer.schedule(cursorBlinkerTask,0,Configuration.blinkingRate);
    }
    /** Cancel the drawing of the cursor */
    public static void cancelCursor()
    {
        SwingUtilities.invokeLater(removeCursor);
        if (cursorBlinkerTask!=null)
            cursorBlinkerTask.cancel();
        if (highlightParenthesesTask!=null)
            highlightParenthesesTask.cancel();
        if (highlightListTask!=null)
            highlightListTask.cancel();
    }
    /* Called to stop the highlighting of a list */
    public void stopHighlightingList()
    {
        if (highlightListTask!=null)
            highlightListTask.cancel();
        editor.highlighter.removeHighLight("10list");
    }
    /**
     * Innerclass to do the work of drawing a cursor, can use the non static fields of the cursor draw, must be called in the
     * context of the GUI thread (via SwingUtilites.invokeLater or SwingUtilities.invokeAndWait
     */
    private class CursorDraw implements Runnable
    {
        public void run()
        {
            CodeDocument document=editor.getDocument();
            position=Math.max(0,position);
            position=Math.min(document.length()-1,position);
            Point p=editor.getPoint(position);
            CodeLeafElement leaf=document.getLeafAt(position);
            FontMetrics fm=editor.getFontMetrics((leaf==null) ? editor.getFont() : leaf.getFont());
            Graphics g=editor.getGraphics();
            int lnw=editor.getLineNumberingWidth();
            if (g!=null)
            {
                int X=p.x;
                int Y=p.y;
                int height=fm.getHeight();
                // Draw the new cursor
                // Scroll the editor if needed
                if (scroll)
                {
                    if (X<=lnw+20)
                        editor.scrollRectToVisible(new Rectangle(0,Math.max(0,Y-50),20,80));
                    else
                        editor.scrollRectToVisible(new Rectangle(X,Math.max(0,Y-50),20,80));
                    scroll=false;
                };
                // Remove the old cursor
                removeCursor.run();
                // Draw the cursor
                g.setColor(Configuration.colorCursor);
                g.fillRect(X,Y,2,height);
                // Save the old position
                oldX=X;
                oldY=Y;
                oldHeight=height;
                oldEditor=editor;
            };
        }
    }
    /**
     * Inner class used for blinking a cursor
     */
    private class CursorBlinker extends TimerTask
    {
        boolean display=true;
        Runnable cursordraw=new CursorDraw();
        public void run()
        {
            try
            {
                if (display)
                {
                    SwingUtilities.invokeAndWait(cursordraw);
                    display=false;
                }
                else
                {
                    SwingUtilities.invokeAndWait(removeCursor);
                    display=true;
                };
            }
            catch (Exception e)
            {
                // We don't do anything with the exception
            };
        }
    }
    /** Inner class for highlighting the current element */
    private class HighLightList extends TimerTask
    {
        public void run()
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    editor.highlightList();
                    new CodeCursor.CursorDraw().run();
                }
            });
        }
    }
    /**
     * Inner class used for highlighting the parentheses
     */
    private class HighlightParentheses extends TimerTask
    {
        private boolean interrupted=false;
        private HighlightParentheses()
        {
            interrupted=false;
        }
        public void run()
        {
            CodeDocument document=editor.getDocument();
            Region region=document.getParser().findHooks(position-1);
            if (region!=null)
            {
                final int b=region.begin;
                final int e=region.end;
                int e1=Math.min(e,b+10);
                int b1=Math.max(e1-1,e-10);
                final String str=document.getString(b,e1-b)+" ... "+document.getString(b1,e-b1+1);
                if (!interrupted)
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            editor.highlighter.removeHighLight("20hook");
                            editor.highlighter.addHighLightRegion("20hook",Configuration.colorHook,b,1,false);
                            editor.highlighter.addHighLightRegion("20hook",Configuration.colorHook,e,1,true);
                            if (!InstructionField.inFocus())
                                MessageHandler.say(str.replace('\n',' '));
                        }
                    });
                };
            };
        }
        /** Interrupt the eventual highlighting of the parenteses (if possible) */
        private void interrupt()
        {
            interrupted=true;
        }
    }
}
