/*
 * CodeDialogBox.java
 *
 * Created on November 20, 2001, 8:52 PM
 */

package IDE.CodeEditor;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import IDE.*;
import IDE.CodeEditor.Parser.*;
/**
 * Class showing a dialog box which let the user types in some code which is returned
 * @author  marc
 */
public class CodeDialogBox extends JDialog
{
    /** Title of the dialog box */
    String title;
    /**  represents key pressed  true if ok is pressed false if cancel is pressed */
    private boolean ok=false;
    /**  CodePane used to type in text */
    CodePane codepane;
    /** Creates new CodeDialogBox */
    private CodeDialogBox(String title,Parser parser) 
    {
        super(Main.frame,title,true); // Must be a modal dialogbox because we wait on show
        JPanel panel=new JPanel(new BorderLayout());
        JButton ok=new JButton(new AbstractAction("Ok")
        {
            public void actionPerformed(ActionEvent event)
            {
                CodeDialogBox.this.ok=true;
                hide();
                dispose();
            }
        });
        JButton cancel=new JButton(new AbstractAction("Cancel")
        {
            public void actionPerformed(ActionEvent event)
            {
                CodeDialogBox.this.ok=false;
                hide();
                dispose();
            }
        });
        JPanel buttons=new JPanel();
        buttons.add(ok);
        buttons.add(cancel);
        getContentPane().add(buttons,BorderLayout.SOUTH);
        codepane=new CodePane(parser);
        getContentPane().add(codepane,BorderLayout.CENTER);
        setSize(200,300);
        Main.setCurrent(codepane.getEditor());
    }

    /** 
     * Show the dialog box  let the user types in some code and returns a string containing the typed in code ,
     * returns null if cancel is pressed
     */
    public static String showDialog(String title,Parser parser)
    {
        return showDialog(title,parser,"");
    }
    public static String showDialog(String title,Parser parser,String message)
    {
        CodeEditor save=Main.currentEditor();
        CodeDialogBox box=new CodeDialogBox(title,parser);
        Main.setCurrent(box.codepane.getEditor());
        CodeDocument document=box.codepane.getEditor().getDocument();
        document.insertStringAt(message,0);
        box.show();
        Main.setCurrent(save);
        if (box.ok)
            return document.getString(0,document.length()-1);
        else 
            return null;
    }
}
