/*
 * CodeElement.java
 *
 * Created on September 30, 2001, 6:43 PM
 */

package IDE.CodeEditor;

import IDE.Configuration.*;
import IDE.Project.*;
import java.awt.*;
import java.util.*;
import javax.swing.text.*;

/**
 * Represent a code element such as a token, a keyword a comment ....
 * @author  marc
 * @version
 */
public abstract class CodeElement
{
    /**  Parent of this element */
    protected CodeElement parent;
    /** Contructor of CodeElement */
    public CodeElement()
    {
    }
    /** Get the begin of the CodeElement */
    public abstract int begin();
    /** Get the end of the element =begin()+length*/
    public abstract int end();
    /** Get the begin position of the CodeElement */
    public abstract Position beginPosition();
    /** Get the length of the element */
    public abstract int length();
    /** Tell the childs of this element that we are their parent,grandparent ...,must be overriden by the CodeNonLeafElement */
    public void acceptChilds()
    {
    }
    /** Get the string defined by begin() <= i < end()  */
    public abstract String getString(CodeDocument document);
    /** Get the parent of this element */
    public CodeElement getParent()
    {
        return parent;
    }
    /** Sets the parent element of this element */
    public void setParent(CodeElement parent)
    {
        this.parent=parent;
    }
    /** Indicates if a element is complete, normally we must override this method */
    public boolean isComplete()
    {
        return false;
    }
    /** Indicates if a child is the first child in this element and if this element is first child in its parent */
    protected abstract boolean isFirst(CodeElement child);
    /** Indicates if this element is a list  (what a list is, is up to the parser) */
    public boolean isList()
    {
        return false;
    }
    // Get the font associated with this element
    public Font  getFont()
    {
        return Configuration.font;
    }
    // Get the color associated with this element
    public Color getColor()
    {
        return Configuration.colorForeground;
    }
    // Get the topParent of this element
    public CodeElement getTopParent()
    {
        if (parent==null)
            return this;
        else 
            return parent.getTopParent();
    }
    /** Analyze the element to determine its syntax */
    public boolean analyze()
    {
        return true;
    }
    /** Find the hints for this element */
    public boolean findHints(CodeElement element,Vector hints)
    {
    	return true;
    }
    /** Updates the definitions stored in the Repository, this will be called by the Repository on the toplevel elements
     *  of a document.
 	 * @param repository Repository instance to use to update all the information in the repository, the CodeElement, must do a cast of this instance to a type derived from AbstractRepository 
     * @param filenode The filenode wher the definition must be stored in 
     */
    public void updateDefinition(IRepository repository,CodeDocument doc)
    {
    	// Default is do nothing
    }
}
