/*
 *
 *
 * Created on October 27, 2001, 6:05 PM
 */

package IDE.CodeEditor;
import java.util.*;
/**
 *  Lightwight version of Vector with less memory overhead
 * @author  marc
 */
public final class CodeElementVector
{
    // Arrary to hold the leafs
    private CodeElement[] elements;
    // Size of the array
    private int size;
    // Increment
    private byte increment;
    // Constructors of the LeafVector
    public CodeElementVector()
    {
        this(1,(byte) 1);
    }
    public CodeElementVector(int capacity, byte increment)
    {
        elements=new CodeElement[capacity];
        this.increment=increment;
        size=0;
    }
    /** Get the leaf at position i */
    public CodeElement codeElementAt(int i)
    {
        return elements[i];
    }
    /** Get the first leaf */
    public CodeElement firstCodeElement()
    {
        return elements[0];
    }
    /** Get the last leaf */
    public CodeElement lastCodeElement()
    {
        return elements[size-1];
    }
    /* Add a leaf at the end of the list */
    public void addCodeElement(CodeElement element)
    {
        if (size<elements.length)
        {
            // We don't have to increase the internal array
            elements[size]=element;
            size++;
        }
        else
        {
            // Increase the array with increment
            CodeElement[] newelements=new CodeElement[elements.length+increment];
            System.arraycopy(elements,0,newelements,0,elements.length);
            newelements[size]=element;
            size++;
            elements=newelements;
        };
    }
    /** Add all the leafs in the LeafVector to this LeafVector (at position i) */
    public void insertAll(int i,CodeElementVector newelements)
    {
        if (size+newelements.size<elements.length)
        {
            System.arraycopy(elements,i,elements,i+newelements.size,size-i);
            System.arraycopy(newelements.elements,0,elements,i,newelements.size);
            size=size+newelements.size;
        }
        else
        {
            CodeElement[] tempelements=new CodeElement[elements.length+newelements.size];
            System.arraycopy(elements,0,tempelements,0,i);
            System.arraycopy(newelements.elements,0,tempelements,i,newelements.size);
            System.arraycopy(elements,i,tempelements,i+newelements.size(),size-i);
            size=size+newelements.size;
            elements=tempelements;
        };
    }
    /** Insert a leaf at position i */
    public void insertLeafAt(CodeElement element,int i)
    {
        if (i==size)
        {
            addCodeElement(element);
        }
        else if (i>=0 && i<size)
        {
            if (size<elements.length)
            {
                // Insert without increasing the array
                System.arraycopy(elements,i,elements,i+1,size-i);
                elements[i]=element;
                size++;
            }
            else
            {
                // Insert with increasing the array
                CodeElement[] newelements=new CodeElement[elements.length+increment];
                System.arraycopy(elements,0,newelements,0,i);
                newelements[i]=element;
                System.arraycopy(elements,i,newelements,i+1,size-i);
                elements=newelements;
                size++;
            };
        };
    }
    /** Delete a range of leafs from b (inclusive) to e exclusive */
    public void removeRangeElements(int b,int e)
    {
        if (b>=0 && b<e && e<=size)
        {
            // First copy all entries higher then e to position b
            if (e<size)
                System.arraycopy(elements,e,elements,b,size-e);
            // Next set all the element between size-(e-b) and size to null to avoid leaks
            for (int i=size-(e-b);i<size;i++)
                elements[i]=null;
            // Decrease the size
            size=size-(e-b);
        }
    }
    /** Clear this element (specifying the initial capacity) */
    public void clear(int capacity)
    {
        size=0;
        elements=new CodeElement[capacity];
    }    
    /** Clear the code element  vector */
    public void clear()
    {
        elements=new CodeElement[1];
        size=0;
    }
    /** Give back the size */
    public int size()
    {
        return size;
    }
    /** Make a first level clone of this CodeElementVector */
    public CodeElementVector makeClone()
    {
        CodeElementVector clone=new CodeElementVector(size,increment);
        System.arraycopy(elements,0,clone.elements,0,size);
        clone.size=size;
        return clone;
    }
    /** Search for the first occurrance of the element starting at index  start (returns -1 if not found) */
    public int indexOf(CodeElement element,int start)
    {
        for (int i=start;i<size;i++)
            if (elements[i]==element)
                return i;
        return -1;
    }
    /** Make a iterator over this CodeElementVector */
    public CodeElementIterator makeIterator()
    {
        return new CodeElementIteratorImp();
    }
    /** Convert this to a vector */
    public Vector toVector()
    {
    	Vector result=new Vector(size);
    	for (int i=0;i<size;i++)
    		result.add(elements[i]);
    	return result;
    }
    /**
     * Verifies if the element at position index (if it exist) is equal to element
     * @param index Index of the element to compare to
     * @param element Element to comapre with
     * @return
     */
    public boolean elementAtIs(int index,CodeElement element)
    {
    	return 0<=index && index<size && elements[index]==element;
    }
    /** Private class to implement a iterator over all the elements */
    private class CodeElementIteratorImp implements CodeElementIterator
    {
        /** Index in the array */
        private int index=0;
        /** Get the current position  */
        public int getPosition()
        {
            return index;
        }
        /** Set the current position  */
        public void setPosition(int index)
        {
            this.index=index;
        }
        /** Get the next element  */
        public CodeElement next()
        {
            CodeElement next;
            if (index<size)
            {
                next=elements[index];
                index++;
                return next;
            }
            else
            {
                return null;
            }
        }
        /** Peek for the next element  */
        public CodeElement peek()
        {
            if (index<size)
            {
                return elements[index];
            }
            else
            {
                return null;
            }
        }
		/* (non-Javadoc)
		 * @see IDE.CodeEditor.CodeElementIterator#getNextCount()
		 */
		public int getNextCount() 
		{
			return size-index;
		}
    }
}
