/*
 * CodeHighLighter.java
 *
 * Created on October 7, 2001, 5:13 PM
 */

package IDE.CodeEditor;

import java.awt.*;
import java.util.*;
import javax.swing.text.*;
import IDE.Configuration.*;
import Install.*;

/**
 *
 * @author  marc
 * @version
 */
public class CodeHighLighter
{
    /** Document used as base of highlighter */
    CodeDocument document;
    /** Editor to draw entry in */
    CodeEditor editor;
    /** Set of highlights based on its name */
    Vector highlights=new Vector();
    /** Creates new CodeHighLighter */
    public CodeHighLighter(CodeEditor editor)
    {
        this.document=editor.getDocument();
        this.editor=editor;
    }
    /**  Create a new highlight and his color */
    public void addHighLight(String name,Color color)
    {
        HighLight hl=findHighLight(name);
        if (hl==null)
        {
            hl=new HighLight(name);
            hl.color=color;
            // Work arround for the fact that setXORMode in windows does not work
            // highlights.addElement(hl);
            insertHighLight(hl);
        }
        else
        {
            hl.color=color;
        }
    }
    /** Add a highlight with a certain name to the set of highlights  or extentd the existing highlight */
    public void addHighLightRegion(String name,Color color,int begin,int length,boolean repaint)
    {
        HighLight hl=findHighLight(name);
        if (hl!=null)
        {
            hl.color=color;
            hl.addRegion(begin,begin+length);
        }
        else
        {
            hl=new HighLight(name);
            hl.color=color;
            hl.addRegion(begin,begin+length);
            // Work arround for the fact that setXORMode in windows does not work
            // highlights.addElement(hl);
            insertHighLight(hl);
        };
        if (repaint)
            editor.repaintVisible();
    }
    /* 
     * Add the highlight so that they are sorted, needed to ensure that they are painted 
     * in the correct order even for platforms where there is a bug in setXORMode
     */
    private void insertHighLight(HighLight hl)
    {
        int m=highlights.size();
        int i;
        for (i=0;i<m && ((HighLight) highlights.elementAt(i)).name.compareTo(hl.name)<0;i++);
        highlights.insertElementAt(hl, i);
    }
    /* Remove a highlighted region from a existing highlight */
    public void removeHighLightRegion(String name,int begin,int length,boolean repaint)
    {
        HighLight hl=findHighLight(name);
        if (hl!=null)
        {
            hl.removeRegion(begin,begin+length);
            if (repaint)
                editor.repaintVisible();
        }
    }
    
    /** Remove a highlight */
    public void removeHighLight(String name)
    {
        removeHighLight(name,true);
    }
    public void removeHighLight(String name,boolean repaint)
    {
        HighLight hl=findHighLight(name);
        if (hl!=null)
        {
            highlights.remove(hl);
            if (repaint)
                editor.repaintVisible();
        };
    }
    /** Clear a highlight */
    public void clearHighLight(String name)
    {
        HighLight hl=findHighLight(name);
        if (hl!=null)
        {
            hl.endpoints.clear();
            editor.repaintVisible();
        };
    }
    /** Get the string of highlighted code */
    public String getHighLightedCode(String name)
    {
        HighLight hl=findHighLight(name);
        if (hl==null)
            return null;
        else
            return hl.getString();
    }
    /** Delete the highlighted code for highlight with name name */
    public void deleteHighLightedCode(String name)
    {
        HighLight hl=findHighLight(name);
        if (hl!=null)
        {
            hl.deleteHighLightedCode();
            highlights.remove(hl);
            editor.repaintVisible();
        };
    }
    /** draw the background and all the highlights */
    public void drawBackground(Graphics g,FontMetrics fm,int height,int x,int y,int begin,int end,String str)
    {
        Color color=Configuration.colorBackground;
        // Draw the highlighs
        if (!OsSystem.setXORModeHasBug())
        {
            g.setColor(color);
            g.setXORMode(color);
        }
        for (int i=0;i<highlights.size();i++)
            ((HighLight) highlights.elementAt(i)).drawHighLight(g,fm,height,x,y,begin,end,str);
        if (!OsSystem.setXORModeHasBug())
            g.setPaintMode();
    }
    /** Find highlight return null if not found */
    private HighLight findHighLight(String name)
    {
        int i;
        int m=highlights.size();
        for (i=0;i<m && name.compareTo(((HighLight) highlights.elementAt(i)).name)!=0;i++);
        if (i<m)
            return (HighLight) highlights.elementAt(i);
        else
            return null;
    }
    /** Inner class containing highlight information */
    private class HighLight
    {
        /** Name of the highlight */
        private String name;
        /** Create a HighLight */
        private HighLight(String name)
        {
            this.name=name;
        }
        /** Background Color of highlighter */
        private Color color;
        /** Vector containing pair of begin and endpoints of the segments making up the highlight */
        private Vector endpoints=new Vector();
        /** Add a region to highlight  [begin,end[ to this highlight */
        private void addRegion(int begin,int end)
        {
            // Check the boundaries
            if (begin>=end || begin<0 || end>document.length()-1)
                return;
            // Add the region
            int i=0;
            int b,e;
            while (true)
            {
                if (i+1<endpoints.size())
                {
                    b=((Position) endpoints.elementAt(i)).getOffset();
                    e=((Position) endpoints.elementAt(i+1)).getOffset();
                    if (end<b)
                    {
                        // [begin,end[ lies before [b,e[
                        endpoints.insertElementAt(document.createPosition(end),i);
                        endpoints.insertElementAt(document.createPosition(begin),i);
                        break;
                    }
                    else if (begin<=b && end<e)
                    {
                        // begin<=b<=end<e
                        endpoints.removeElementAt(i);
                        endpoints.removeElementAt(i);
                        end=e;
                    }
                    else if (b<=begin && end<=e)
                    {
                        // b<=begin<end<e
                        break;
                    }
                    else if (b<=begin && begin<=e && e<end)
                    {
                        // b<=begin<e<end
                        endpoints.removeElementAt(i);
                        endpoints.removeElementAt(i);
                        begin=b;
                    }
                    else
                    {
                        // b<e<begin<end
                        i=i+2;
                    }
                    
                }
                else
                {
                    endpoints.addElement(document.createPosition(begin));
                    endpoints.addElement(document.createPosition(end));
                    break;
                };
            }
        }
        /** Remove a region [begin,end[ from the highlight */
        private void removeRegion(int begin,int end)
        {
            int b,e;
            int i=0;
            while  (i+1<endpoints.size())
            {
                b=((Position) endpoints.elementAt(i)).getOffset();
                e=((Position) endpoints.elementAt(i+1)).getOffset();
                if (end<=b)
                {
                    // We reached the end
                    break;
                }
                else if (begin<=b && e<=end)
                {
                    endpoints.remove(i);
                    endpoints.remove(i);
                }
                else if (b<begin && begin<=e && e<=end)
                {
                    endpoints.setElementAt(document.createPosition(begin),i+1);
                    i=i+2;
                }
                else if (begin<=b && begin<e && end<e)
                {
                    endpoints.setElementAt(document.createPosition(end),i);
                    i=i+2;
                }
                else if (b<begin && end<e)
                {
                    endpoints.insertElementAt(document.createPosition(end),i+2);
                    endpoints.insertElementAt(document.createPosition(begin),i+2);
                    i=i+4;
                }
                else
                {
                    i=i+2;
                }
            }
        }
        /** Draw the highlight */
        public void drawHighLight(Graphics g,FontMetrics fm,int height,int x,int y,int begin,int end,String str)
        {
            //Only draws the part between begin and end
            int b,e;
            int m=endpoints.size();
            for (int i=0;i<m;i=i+2)
            {
                b=Math.max(begin,((Position) endpoints.elementAt(i)).getOffset());
                e=Math.min(end,((Position) endpoints.elementAt(i+1)).getOffset());
                if (b<e)
                {
                    // Draw highlight
                    g.setColor(color);
                    g.fillRect(x+fm.stringWidth(str.substring(0,b-begin)),y,fm.stringWidth(str.substring(b-begin,e-begin)),height);
                };
            };
        }
        /**  Get the highlighted string */
        public String getString()
        {
            StringBuffer buffer=new StringBuffer();
            int b,e;
            int m=endpoints.size();
            for (int i=0;i<m;i=i+2)
            {
                b=((Position) endpoints.elementAt(i)).getOffset();
                e=((Position) endpoints.elementAt(i+1)).getOffset();
                buffer.append(document.getString(b,e-b));
            };
            return buffer.toString();
        }
        /** Delete the highlighted code */
        public void deleteHighLightedCode()
        {
            int b,e;
            // Starting backwards to avoid moving to much text
            for (int i=endpoints.size()-1;i>0;i=i-2)
            {
                e=((Position) endpoints.elementAt(i)).getOffset();
                b=((Position) endpoints.elementAt(i-1)).getOffset();
                document.getCodeEditor().remove(b,e-b);
            };
            // Set the cursor position to begin highlighted code
            if (endpoints.size()>0)
                editor.cursor.setCursor(((Position) endpoints.elementAt(0)).getOffset());
        }
    }
}
