/*
 * CodeLeafElement.java
 *
 * Created on September 30, 2001, 7:26 PM
 */

package IDE.CodeEditor;

import java.awt.*;
import javax.swing.text.*;

import IDE.Configuration.*;

/**
 * Represents a leaf
 * @author  marc
 * @version
 */
public  class CodeLeafElement extends CodeElement
{
    // Begin position of the leaf
    private Position begin;
    //  The length of the leaf
    private int length;
    // Create the code leafelement
    public CodeLeafElement(CodeDocument document,int begin)
    {
        
        this.begin=document.createPosition(begin);
    }
    /** Get the begin of this element */
    public int begin()
    {
        return begin.getOffset();
    }
    /** Return the begin position */
    public Position getBeginPosition()
    {
        return begin;
    }
    
    /** Get the end of this element */
    public int end()
    {
        return begin.getOffset()+length;
    }
    // Get the font associated with this element
    public Font  getFont()
    {
        if (parent==null)
            return Configuration.font;
        else
            return parent.getFont();
    }
    // Get the color associated with this element
    public Color getColor()
    {
        if (parent==null)
            return Configuration.colorForeground;
        else
            return parent.getColor();
    }
    // Overwrite the getTopParent() method
    public CodeElement getTopParent()
    {
        if (parent==null)
            return null;
        else
            return parent.getTopParent();
    }
    /** Set the length of a leaf element */
    public void setLength(int length)
    {
        this.length=length;
    }
    /*
     * Get the separator character (is '\0' for a leaf which is not a separator
     * By definining this method also for a general leaf we avoid having to test
     * if a leaf is a separator before we get its separator element
     */
    public char getSeparatorChar()
    {
        return '\0';
    }
    /** Get the string representing this code leaf element */
    public String getString(CodeDocument document)
    {
        return document.getString(begin.getOffset(),length);
    }
    /** Indicates if a child is the first child in this element and if this element is first child in its parent */
    public boolean isFirst(CodeElement child)
    {
        return false; // Leafs do not have childs
    }
    /** Check if this is the first leaf in a toplevel element */
    public boolean isFirstLeaf()
    {
        if (parent==null)
            return false;
        else
            return parent.isFirst(this);
    }
	/* (non-Javadoc)
	 * @see IDE.CodeEditor.CodeElement#beginPosition()
	 */
	public Position beginPosition()
	{
		return begin;
	}
	/* (non-Javadoc)
	 * @see IDE.CodeEditor.CodeElement#endPosition()
	 */
	public int length()
	 {
	 	return length;
	}
}