/*
 * CodePane.java
 *
 * Created on November 8, 2001, 9:10 PM
 */

package IDE.CodeEditor;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;
import IDE.CodeEditor.Parser.*;
import IDE.Configuration.*;
import IDE.Interaction.*;
import IDE.Project.*;
import IDE.Utilities.*;
import IDE.WorkArea.*;

/**
 * Scrollpane wrapper around a CodeEditor
 * @author  marc
 */
public class CodePane extends JScrollPane implements FocusListener
{
    /** Private codeeditor displayed in this scrollpane */
    private CodeEditor editor;
    /** Creates new CodePane */
    private CodePane()
    {
        // This is used only for the cloning of a codepane
        addFocusListener(this);
    }
    public CodePane(Parser parser)
    {
        editor=new CodeEditor(this,new CodeDocument(parser));
        setViewportView(editor);
        getViewport().setBackground(Configuration.colorBackground);
        getVerticalScrollBar().setUnitIncrement(getFontMetrics(Configuration.font).getHeight());
        addFocusListener(this);
    }
    /** Only called via createCodePane */
    private CodePane(Parser parser,File file)
    {
        SplashScreen.showMessage("Opening file: "+file.getName()); // During startup show the opening file message
        CodeDocument document=new CodeDocument(parser);
        editor=new CodeEditor(this,document,file);
        setViewportView(editor);
        getViewport().setBackground(Configuration.colorBackground);
        getVerticalScrollBar().setUnitIncrement(getFontMetrics(Configuration.font).getHeight());
        addFocusListener(this);
    }
    public static CodePane createCodePane(Parser parser,File file)
    {
        CodeEditor existing=WorkArea.getEditorForFile(file);
        if (existing==null)
            return new CodePane(parser,file);
        else
            if (existing.getCodePane()==null)
            {
                JOptionPane.showConfirmDialog(null,"File is already opened","Error creating CodePane",JOptionPane.OK_OPTION);
                return new CodePane(parser);
            }
            else
            {
                return existing.getCodePane().cloneCodePane();
            }
    }
    /** Clone this CodePane */
    public CodePane cloneCodePane()
    {
        CodePane codepane=new CodePane();
        codepane.editor=editor.cloneCodeEditor(codepane);
        codepane.setViewportView(codepane.editor);
        codepane.getViewport().setBackground(Configuration.colorBackground);
        codepane.getVerticalScrollBar().setUnitIncrement(getFontMetrics(Configuration.font).getHeight());
        return codepane;
    }
    /** Close all the resources taken up by the pane */
    public void close()
    {
    }
    /** Refresh the background color */
    public void refreshBackground()
    {
        getViewport().setBackground(Configuration.colorBackground);
    }
    /** Get the editor in this pane */
    public CodeEditor getEditor()
    {
        return editor;
    }
    /** Get the rectangle of the code editor displayed in the scrollpane */
    public Rectangle getVisibleRectangleEditor()
    {
        return getViewport().getViewRect();
    }
    /** Save the document in the editor to a file */
    public void save()
    {
        File file=editor.getDocument().getFile();
        File oldfile=file;
        if (file==null)
        {
            JFileChooser chooser=new JFileChooser(Interactions.currentDirectory);
            javax.swing.filechooser.FileFilter filter=Repositories.getChooserFilter();
            chooser.addChoosableFileFilter(filter);
            chooser.addChoosableFileFilter(chooser.getAcceptAllFileFilter());
            chooser.setFileFilter(filter);
            chooser.setSelectedFile(new File("<filename>."+Repositories.getDefaultExtension(editor.getDocument())));
            // Loop until a correct file is choosen or if a abort is done
            while (true)
            {
                if (chooser.showSaveDialog(null)==JFileChooser.APPROVE_OPTION)
                {
                    file=chooser.getSelectedFile();
                    if (WorkArea.fileOpenByAnotherEditor(file,editor))
                        if (JOptionPane.showConfirmDialog(null,"File is already opened in another buffer, retry?","Error saving",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION)
                            continue;
                        else
                            break;
                    if (file.exists())
                        if (!(JOptionPane.showConfirmDialog(null,"File exist, overwrite the file?","Error saving",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION))
                            break;
                    editor.getDocument().setFile(file);
                    editor.getDocument().saveFile(file);
                    // Tell the repository about the change
                    Repositories.renameDocument(editor.getDocument(),oldfile);
                    // Refresh the name in the workarea
                    WorkArea.refreshTitles();
                    break;
                }
                else
                {
                    break;
                }
            };
        }
        else
        {
            editor.getDocument().saveFile(file);
        };
    }
    /** SaveAs operation */
    public boolean saveAs()
    {
        JFileChooser chooser=new JFileChooser(Interactions.currentDirectory);
        File oldfile=editor.getDocument().getFile();
        File file;
        javax.swing.filechooser.FileFilter filter=Repositories.getChooserFilter();
        chooser.addChoosableFileFilter(filter);
        chooser.addChoosableFileFilter(chooser.getAcceptAllFileFilter());
        chooser.setFileFilter(filter);
        if (oldfile==null)
            file=new File("<filename>."+Repositories.getDefaultExtension(editor.getDocument()));
        else
            file=oldfile;
        chooser.setSelectedFile(file);
        while (true)
        {
            if (chooser.showSaveDialog(null)==JFileChooser.APPROVE_OPTION)
            {
                file=chooser.getSelectedFile();
                if (WorkArea.fileOpenByAnotherEditor(file,editor))
                    if (JOptionPane.showConfirmDialog(null,"File is already opened in another buffer, retry?","Error saving",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION)
                        continue;
                    else
                        return false;
                if (file.exists() && (oldfile==null || Util.getCanonicalFileName(file).compareTo(Util.getCanonicalFileName(oldfile))!=0))
                    if (!(JOptionPane.showConfirmDialog(null,"File exist, overwrite the file?","Error saving",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION))
                        return false;
                editor.getDocument().setFile(file);
                editor.getDocument().saveFile(file);
                // Tell the repository that the document has been renamed
                Repositories.renameDocument(editor.getDocument(),oldfile);
                // Refresh the names in the workarea
                WorkArea.refreshTitles();
                return true;
            }
            else
            {
                return false;
            }
        }
    }
    /** Init and repaint the visible content from the editor */
    public void initAndRepaintVisible()
    {
        editor.initAndRepaintVisible();
    }
    
    public void focusGained(java.awt.event.FocusEvent focusEvent)
    {
        editor.requestFocus();
    }
    
    public void focusLost(java.awt.event.FocusEvent focusEvent)
    {
    }
}