/*
 * CodeElement.java
 *
 * Created on September 30, 2001, 6:43 PM
 */

package IDE.CodeEditor;
import javax.swing.text.*;
/**
 * Represent a code element that contains more then one childs
 * @author  marc
 * @version
 */
public abstract class CodeStructureElement extends CodeElement
{
    /** List of childs of this code element */
    protected CodeElementVector childs=new CodeElementVector();
    /** Indicates if element is complete */
    private boolean complete=true;
    /** Contructor of CodeElement */
    public CodeStructureElement()
    {
    }
    /** Get the begin of the CodeElement */
    public int begin()
    {
        return childs.firstCodeElement().begin();
    }
    /** Get the begin position of the CodeElement */
    public Position beginPosition()
    {
    	return childs.firstCodeElement().beginPosition();
    }
    /** Get the end of the element =begin()+length*/
    public int end()
    {
        return childs.lastCodeElement().end();
    }
    /** Get the length of the element */
    public int length()
    {
    	return end()-begin();
    }
    /** Get the string defined by begin() <= i < end()  */
    public String getString(CodeDocument document)
    {
        return document.getString(begin(),end()-begin());
    }
    /** Add a child to this element */
    public void addChild(CodeElement child)
    {
        childs.addCodeElement(child);
    }
    /**
     * Tell the children,grantchildren ... that are CodeStructure elements that we are there
     * ancestor (needed for hinting)
     */
    public void acceptCodeStructureChilds()
    {
    	int m=childs.size();
    	CodeElement element;
    	for (int i=0;i<m;i++)
    	{
    		element=childs.codeElementAt(i);
    		if (element instanceof CodeStructureElement)
    		{
    			element.setParent(this);
    			((CodeStructureElement) element).acceptCodeStructureChilds();
    		};
    	}
    }
    /** Tell the descendants of this element that we are their ancestor*/
    public void acceptChilds()
    {
        int m=childs.size();
        CodeElement element;
        for (int i=0;i<m;i++)
        {
            element=childs.codeElementAt(i);
            if (element instanceof CodeLeafElement)
            {
                element.setParent(this);
            }
            else if (element instanceof CodeAtomElement)
            {
                element.setParent(this);
                ((CodeAtomElement) element).acceptChilds();
            }
            else if (element instanceof CodeStructureElement)
            {
                element.setParent(this);
                ((CodeStructureElement) element).acceptChilds();
            };
        };
    }
    /** Get the childs of this element */
    public CodeElementVector getChilds()
    {
        return childs;
    }
    /**
     * Insert the given element at the given position, the insertion is done at the lowest level in the hierarchy
     * @param element The element to be inserted
     * @param pos The position to insert the element into
     */
 	public void insertElementAt(CodeElement element,int pos)
 	{
 		int m=childs.size();
 		int i;
 		// Skip all the elements ending before this position
		for (i=0;i<m && childs.codeElementAt(i).end()<=pos;i++);
		// Now either i>=m or childs.codeElementAt(i).end>pos
		if (i>=m)
		{
			// Insert the element at the end of the childs list
			element.setParent(this);
			childs.addCodeElement(element);
		}
		else
		{
			// i<m  and childs.codeElementAt(i).end>pos
			if (childs.codeElementAt(i).begin()<=pos)
			{
				// This child covers the position
				if (childs.codeElementAt(i) instanceof CodeStructureElement)
				{
					// Insert the element in the child element
					((CodeStructureElement) childs.codeElementAt(i)).insertElementAt(element,pos);
				}
				else
				{
					// Insert the element before this element
					element.setParent(this);
					childs.insertLeafAt(element,i);
				};
			}
			else
			{
				// This child does not cover the position
				element.setParent(this);
				childs.insertLeafAt(element,i);
			}
		}
 	}
    /** Make this element incomplete , the default is complete */
    public void markIncomplete()
    {
        complete=false;
    }
    /** Indicates if a element is complete, normally we must override this method */
    public boolean isComplete()
    {
        return complete;
    }
    /** Replace the use of a definition by a newdefinition  (in LISP replace a function/macro call by another call) */
    public void replaceUsedDefinition(CodeDocument document,CodeDocument.UpdateBuffer updatebuffer,String olddefinition,String newdefinition)
    {
    };
    /** Indicates if a child is the first child in this element and if this element is first child in its parent */
    protected boolean isFirst(CodeElement child)
    {
        if (parent==null)
            return child==childs.codeElementAt(0);
        else
            return child==childs.codeElementAt(0) && parent.isFirst(this);
    }
}
