/*
 * LineVector.java
 *
 * Created on October 27, 2001, 6:05 PM
 */

package IDE.CodeEditor;


/**
 *  Lightwight version of Vector with less memory overhead
 * @author  marc
 */
public final class LeafVector
{
    // Arrary to hold the leafs
    private CodeLeafElement[] leafs;
    // Size of the array
    private int size;
    // Increment 
    private byte increment;
    // Constructors of the LeafVector
    public LeafVector()
    {
        this(1,(byte) 1);
    }
    public LeafVector(int capacity,byte increment)
    {
        leafs=new CodeLeafElement[capacity];
        this.increment=increment;
        size=0;
    }
    /** Get the leaf at position i */
    public CodeLeafElement leafAt(int i)
    {
        return leafs[i];
    }
    /** Get the first leaf */
    public CodeLeafElement firstLeaf()
    {
        return leafs[0];
    }
    /** Get the last leaf */
    public CodeLeafElement lastLeaf()
    {
        return leafs[size-1];
    }
    /* Add a leaf at position i */
    public void addLeaf(CodeLeafElement leaf)
    {
        if (size<leafs.length)
        {
            // We don't have to increase the internal array
            leafs[size]=leaf;
            size++;
        }
        else
        {
            // Increase the array with increment
            CodeLeafElement[] newleafs=new CodeLeafElement[leafs.length+increment];
            System.arraycopy(leafs,0,newleafs,0,leafs.length);
            newleafs[size]=leaf;
            size++;
            leafs=newleafs;
        };
    }
    /** Add all the leafs in the LeafVector to this LeafVector (at position i) */
    public void insertAll(int i,LeafVector newleafs)
    {
        if (size+newleafs.size<leafs.length)
        {
            System.arraycopy(leafs,i,leafs,i+newleafs.size,size-i);
            System.arraycopy(newleafs.leafs,0,leafs,i,newleafs.size);
            size=size+newleafs.size;
        }
        else
        {
            CodeLeafElement[] templeafs=new CodeLeafElement[leafs.length+newleafs.size];
            System.arraycopy(leafs,0,templeafs,0,i);
            System.arraycopy(newleafs.leafs,0,templeafs,i,newleafs.size);
            System.arraycopy(leafs,i,templeafs,i+newleafs.size(),size-i);
            size=size+newleafs.size;
            leafs=templeafs;
        };
    }
    /** Insert a leaf at position i */
    public void insertLeafAt(CodeLeafElement leaf,int i)
    {
        if (i==size)
        {
            addLeaf(leaf);
        }
         else if (i>=0 && i<size)
        {
            if (size<leafs.length)
            {
                // Insert without increasing the array
                System.arraycopy(leafs,i,leafs,i+1,size-i);
                leafs[i]=leaf;
                size++;
            }
            else
            {
                // Insert with increasing the array
                CodeLeafElement[] newleafs=new CodeLeafElement[leafs.length+increment];
                System.arraycopy(leafs,0,newleafs,0,i);
                newleafs[i]=leaf;
                System.arraycopy(leafs,i,newleafs,i+1,size-i);
                leafs=newleafs;
                size++;
            };
        };
    }
    /** Delete a range of leafs from b (inclusive) to e exclusive */
    public void removeRangeLeafs(int b,int e)
    {
        if (b>=0 && b<e && e<=size)
        {
            // First copy all entries higher then e to position b
            if (e<size)
                System.arraycopy(leafs,e,leafs,b,size-e);
            // Next set all the element between size-(e-b) and size to null to avoid leaks
            for (int i=size-(e-b);i<size;i++)
                leafs[i]=null;
            // Decrease the size
            size=size-(e-b);
        }
    }
    /** Give back the size */
    public int size()
    {
        return size;
    }
    /** Clear the leaf vector */
    public void clear()
    {
        leafs=new CodeLeafElement[1];
        size=0;
    }
}
