/*
 * LineVector.java
 *
 * Created on October 27, 2001, 6:05 PM
 */

package IDE.CodeEditor;

/**
 *  Lightwight version of Vector with less memory overhead
 * @author  marc
 */
public class LineVector
{
    // Arrary to hold the line
    protected CodeLine[] lines;
    // Size of the array
    protected int size;
    // Increment
    protected byte increment;
    // Constructors of the LineVector
    public LineVector()
    {
        this(1,(byte) 1);
    }
    public LineVector(int capacity, byte increment)
    {
        lines=new CodeLine[capacity];
        this.increment=increment;
        size=0;
    }
    /** Get the line at position i */
    public CodeLine lineAt(int i)
    {
        return lines[i];
    }
    /** Get the first line */
    public CodeLine firstLine()
    {
        return lines[0];
    }
    /** Get the last line */
    public CodeLine lastLine()
    {
        return lines[size-1];
    }
    /* Add a line at position i */
    public void addLine(CodeLine line)
    {
        if (size<lines.length)
        {
            // We don't have to increase the internal array
            lines[size]=line;
            size++;
        }
        else
        {
            // Increase the array with increment
            CodeLine[] newlines=new CodeLine[lines.length+increment];
            System.arraycopy(lines,0,newlines,0,lines.length);
            newlines[size]=line;
            size++;
            lines=newlines;
        };
    }
    /** Add all the lines in the LineVector to this LineVector (at position i) */
    public void insertAll(int i,LineVector newlines)
    {
        if (size+newlines.size<lines.length)
        {
            System.arraycopy(lines,i,lines,i+newlines.size,size-i);
            System.arraycopy(newlines.lines,0,lines,i,newlines.size);
            size=size+newlines.size;
        }
        else
        {
            CodeLine[] templines=new CodeLine[lines.length+newlines.size];
            System.arraycopy(lines,0,templines,0,i);
            System.arraycopy(newlines.lines,0,templines,i,newlines.size);
            System.arraycopy(lines,i,templines,i+newlines.size(),size-i);
            size=size+newlines.size;
            lines=templines;
        };
    }
    /** Insert a line at position i */
    public void insertLineAt(CodeLine line,int i)
    {
        if (i==size)
        {
            addLine(line);
        }
        else if (i>=0 && i<size)
        {
            if (size<lines.length)
            {
                // Insert without increasing the array
                System.arraycopy(lines,i,lines,i+1,size-i);
                lines[i]=line;
                size++;
            }
            else
            {
                // Insert with increasing the array
                CodeLine[] newlines=new CodeLine[lines.length+increment];
                System.arraycopy(lines,0,newlines,0,i);
                newlines[i]=line;
                System.arraycopy(lines,i,newlines,i+1,size-i);
                lines=newlines;
                size++;
            };
        };
    }
    /** Delete a range of lines from b (inclusive) to e exclusive */
    public void removeRangeLines(int b,int e)
    {
        if (b>=0 && b<e && e<=size)
        {
            // First copy all entries higher then e to position b
            if (e<size)
                System.arraycopy(lines,e,lines,b,size-e);
            // Next set all the element between size-(e-b) and size to null to avoid leaks
            for (int i=size-(e-b);i<size;i++)
                lines[i]=null;
            // Decrease the size
            size=size-(e-b);
        }
    }
    /** Give back the size */
    public int size()
    {
        return size;
    }
    /** Clear the line vector */
    public void clear()
    {
        lines=new CodeLine[1];
        size=0;
    }
}
