/*
 * Definition.java
 *
 * Created on December 16, 2001, 1:07 PM
 */

package IDE.CodeEditor.Parser;

import IDE.CodeEditor.*;
import javax.swing.text.*;

/**
 * Represents a definition of a program construct in the code
 * Used to pass on the definitions that must be displayed in the explorer
 * Examples : defun,defmethod,defparameter ...
 * @author  marc
 */
public class Definition implements Comparable 
{
    /** Represent the type of a definition */
    private final String type;
    /** Name of the definition */
    private final String name;
    /** Begin position of the definition */
    private final Position begin;
    /** Creates a new instance of Definition */
    public Definition(String type,String name,CodeLeafElement leaf) 
    {
        this.type=type;
        this.name=name.intern();
        this.begin=leaf.getBeginPosition();
    }
    /** Compare two definitions, defines the order of definitions */
    public int compareTo(Object obj) 
    {
        if (obj instanceof Definition)
        {
            Definition definition=(Definition) obj;
            return (getCompareName()).compareTo(definition.getCompareName());
        }
        else
        {
            return -1;
        }
    } 
    /**
     * Get the name to make the comparisation, ensure that all definitions
     * in a source are unique 
     */
    private String getCompareName()
    {
        return type+name+Integer.toString(begin.getOffset());
    }
    /** Get the type of the definition */
    public String getType()
    {
        return type;
    }
    /** Get the name of the definition */
    public String getName()
    {
        return name;
    }
    /** Get the leaf of the definition */
    public int getBeginPosition()
    {
        return begin.getOffset();
    }
    /** Check if the type of the definition occurs in the list of definition types */
    public boolean isOfType(String[] types)
    {
        for (int i=0;i<types.length;i++)
            if (types[i].compareTo(type)==0)
                return true;
        return false;
    }
    /** Override the equals method, to be consistent with compare  */
    public boolean equals(Object obj)
    {
        if (obj instanceof Definition)
            return ((Definition) obj).name.compareTo(name)==0;
        else
            return false;
    }
    /** Override the hashCode() method because we override equals */
    public int hashCode()
    {
        return name.hashCode();
    }
}
