/*
 * CodeLeafElement.java
 *
 * Created on September 30, 2001, 7:26 PM
 */

package IDE.CodeEditor.Parser;

import java.awt.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;

/**
 * CodeElement representing a lisp atom.
 * @author  marc
 * @version 
 */
public class LispAtomElement extends CodeAtomElement
{
    /** Static final constants enumerating different types of atoms */
    /* Used if the atom is not a variable, keyword or caller */
    public final static int Atom=0;
    /* Represents a lexical bound variable (via a defun,let ...) */
    public final static int Variable=1;
    /* Represents a keyword as it is defined in the lispsyntax file */
    public final static int Keyword=2;
    /* The head symbol in a lisp form (represents the name of a macro or function in the form) */
    public final static int Caller=3;
    /** Static fields depicting the interned strings of different string constants */
    public final static String DEFUN="DEFUN".intern();
    public final static String DEFMACRO="DEFMACRO".intern();
    public final static String DEFGENERIC="DEFGENERIC".intern();
    public final static String DEFMETHOD="DEFMETHOD".intern();
    public final static String DEFSTRUCT="DEFSTRUCT".intern();
    public final static String DEFCLASS="DEFCLASS".intern();
    public final static String DEFPARAMETER="DEFPARAMETER".intern();
    public final static String DEFVAR="DEFVAR".intern();
    public final static String DEFCONSTANT="DEFCONSTANT".intern();
    public final static String DEFTYPE="DEFTYPE".intern();
    public final static String DEFPACKAGE="DEFPACKAGE".intern();
    public final static String MAKEPACKAGE="MAKE-PACKAGE".intern();
	public final static String USEOPTION=":USE".intern();
	public final static String NICKNAMESOPTION=":NICKNAMES".intern();
	public final static String DOCUMENTATIONOPTION=":DOCUMENTATION".intern();
	public final static String SHADOWOPTION=":SHADOW".intern();
	public final static String SHADOWINGIMPORTOPTION=":SHADOWING-IMPORT-FROM".intern();
	public final static String IMPORTFROMOPTION=":IMPORT-FROM".intern();
	public final static String EXPORTOPTION=":EXPORT".intern();
	public final static String INTERNOPTION=":INTERN".intern();
	public final static String SIZEOPTION=":SIZE".intern();
	public final static String EXPORT="EXPORT".intern();
	public final static String IMPORT="IMPORT".intern();
	public final static String INPACKAGE="IN-PACKAGE".intern();
    
    /** Contains type of the element */
    private int type=Atom;
    /**  Document containing this element */
    private CodeDocument document;
    /* Symbol representing the LispAtom */
    private String symbol;
    
    public LispAtomElement(String symbol,CodeLeafElement leaf)
    {
        super(leaf);
        this.symbol=symbol.toUpperCase().intern();
    }
    /** Returns the package and symbol this symbol stands for */
	public String[] getSymbolAndPackage()
	{
		String content=getSymbol();
		String[] result=new String[2];
		int i;
		if ((i=content.indexOf(":"))>=0)
		{
			result[0]=content.substring(0,i);
			result[1]=content.substring(i+1);
		}
		else if ((i=content.indexOf("::"))>=0)
		{
			result[0]=content.substring(0,i);
			result[1]=content.substring(i+2);
		}
		else
		{
			result[0]=null;
			result[1]=content;
		};
		return result;
	}
    /** Check if the atoms associated string is equal to a given string */
    public boolean isEqualTo(String symbol)
    {
        return symbol==this.symbol; // Using the fact that the two symbols are interned
    }
    /** Set the type of the atom */
    public void setType(int type)
    {
        this.type=type;
    }
    /** Get the type of the atom */
    public int getType()
    {
        return type;
    }
    /** Check if the atom represents one of the definition keywords */
    public boolean isDefKeyword()
    {
        return symbol==DEFUN ||
        symbol==DEFMACRO ||
        symbol==DEFGENERIC ||
        symbol==DEFMETHOD ||
        symbol==DEFSTRUCT ||
        symbol==DEFCLASS ||
        symbol==DEFPARAMETER ||
        symbol==DEFVAR ||
        symbol==DEFCONSTANT ||
        symbol==DEFTYPE;
    }
    /** Overwrite the getColor method */
    public Color getColor()
    {
        switch (type)
        {
            case Atom:
                if (parent==null)
                    return Configuration.colorForeground;
                else
                    return parent.getColor();
            case Variable:
                return Configuration.colorVariable;
            case Keyword:
                return Configuration.colorKeyword;
            case Caller:
                return Configuration.colorCaller;
            default:
                if (parent==null)
                    return Configuration.colorForeground;
                else
                    return parent.getColor();
        }
    }
    /** Get the symbol associated with this atom */
    public String getSymbol()
    {
        return symbol;
    }
}
