/*
 * LispElement.java
 *
 * Created on September 30, 2001, 7:59 PM
 */

package IDE.CodeEditor.Parser;

import java.awt.*;
import java.util.*;
import IDE.CodeEditor.*;
import IDE.LispSyntax.*;
import IDE.Messaging.*;
import IDE.Configuration.*;
import IDE.Project.*;
/**
 * Represents a lisp list element
 * @author  marc
 * @version
 */
public class LispListElement extends CodeStructureElement
{
    /** Elements of this list */
    protected CodeElementVector elements=new CodeElementVector();
    /** List of keywords in this list */
    private CodeElementVector keywords=new CodeElementVector();
    /** List of the callers in this list */
    private CodeElementVector callers=new CodeElementVector();
    /** List of the variables in this list */
    private CodeElementVector variables=new CodeElementVector();
    /** Flag set to indicate that we must tell our parent (if we know who it is) that we are a caller */
    private boolean delayedCalled=false;
    /** Boolean indicating this list represents a call */
    boolean isCall=false;
	/** interned LET symbol */
	private static String LET="LET".intern();
	/** interned LET* symbol */
	private static String LETSTAR="LET*".intern();
    /** Creates new LispElement */
    public LispListElement()
    {
        super();
    }
    /** Tell the childs of this element that we are their parent,grandparent ...  */
    public void acceptChilds()
    {
        super.acceptChilds(); // Let the CodeStructureElement code do all the work
        CodeElement first=getFirst();
        if (parent instanceof LispListElement && delayedCalled)
            ((LispListElement) parent).callers.addCodeElement(this);
    }
    /** Add element to list */
    public void addElement(CodeElement element)
    {
        elements.addCodeElement(element);
    }
	/**
	 * Insert the given element at the given position, the insertion is done at the lowest level in the hierarchy,
	 * this method overrides the mode of CodeStructureElement
	 * @param element The element to be inserted
	 * @param pos The position to insert the element into
	 */
	public void insertElementAt(CodeElement element,int pos)
	{
		int m=elements.size();
		int i;
		// Skip all the elements ending before this position
		for (i=0;i<m && elements.codeElementAt(i).end()<=pos;i++);
		// Now either i>=m or childs.codeElementAt(i).end>pos
		if (i>=m)
		{
			// Insert the element at the end of the childs list
			element.setParent(this);
			elements.addCodeElement(element);
		}
		else
		{
			if (elements.codeElementAt(i).begin()==pos)
			{
				// Thist element statrts at the same position as pos
				element.setParent(this);
				elements.insertLeafAt(element,i);
			}
			// i<m  and childs.codeElementAt(i).end>pos
			else if (elements.codeElementAt(i).begin()<=pos)
			{
				// This child covers the position
				if (elements.codeElementAt(i) instanceof CodeStructureElement)
				{
					// Insert the element in the child element
					((CodeStructureElement) elements.codeElementAt(i)).insertElementAt(element,pos);
				}
				else
				{
					// Insert the element before this element
					element.setParent(this);
					elements.insertLeafAt(element,i);
				};
			}
			else
			{
				// This child does not cover the position
				element.setParent(this);
				elements.insertLeafAt(element,i);
			}
		}
	}

    /** Add a keyword to the list */
    public void addKeyWord(CodeElement element)
    {
        keywords.addCodeElement(element);
    }
    /** Add a caller to the list */
    public void addCaller(CodeElement element)
    {
        callers.addCodeElement(element);
    }
    /** Add a variable to the list */
    public void addVariable(CodeElement element)
    {
        variables.addCodeElement(element);
    }
    /** Get the keywords of this list */
    public  CodeElementVector getKeywords()
    {
        return keywords.makeClone();
    }
    /** Set the keywords of this list */
    public void setKeywords(CodeElementVector keywords)
    {
        this.keywords=keywords;
    }
    /** Get the callers of this list */
    public CodeElementVector getCallers()
    {
        return callers.makeClone();
    }
    /** Set the callers of this list */
    public void setCallers(CodeElementVector callers)
    {
        this.callers=callers;
    }
    /** Get the variables of this list */
    public CodeElementVector getVariables()
    {
        return variables.makeClone();
    }
    /** Set the variables of this list */
    public void setVariables(CodeElementVector variables)
    {
        this.variables=variables;
    }
    /** Activate the syntax (the effect of variables and keywords */
    private void activateSyntax()
    {
        int i,m;
        /** Set the color of the keywords,variables and callers and function elements */
        m=variables.size();
        for (i=0;i<m;i++)
            ((LispAtomElement) variables.codeElementAt(i)).setType(LispAtomElement.Variable);
        m=callers.size();
        m=keywords.size();
        for (i=0;i<m;i++)
            ((LispAtomElement) keywords.codeElementAt(i)).setType(LispAtomElement.Keyword);
        m=elements.size();
        for (i=0;i<m;i++)
            if (elements.codeElementAt(i) instanceof LispFunctionElement)
                ((LispFunctionElement) elements.codeElementAt(i)).analyze();
    }
    /** Analyze this element, owner will become the future parent */
    public boolean analyze()
    {
        int i,m;
        CodeElement element;
        /** If analyze is called by the syntax analyzer indicates this is a call to a function or macro */
        isCall=true;
        /** Do a anlyze of the list to find the keywords,variables,callers */
        variables.clear(1);
        callers.clear(1);
        keywords.clear(1);
        /** Do a syntactical analyze of this list */
        if (LispAnalyzerGen.analyze(this))
        {
        	m=callers.size();
        	for (int j=0;j<m;j++)
        		if (callers.codeElementAt(j) instanceof LispListElement)
        		{
        			((LispListElement) callers.codeElementAt(j)).analyze();
        		};
            activateSyntax();
            return true;
        }
        else
        {
            // This is not a special call so we assume it is a call of a normal function
            // Analyze its arguments
            if (elements.size()>0 && elements.codeElementAt(0) instanceof LispAtomElement)
            {
                ((LispAtomElement) elements.codeElementAt(0)).setType(LispAtomElement.Caller);
                delayedCalled=true; //Ensure that parent (not yet know) will know about us as a caller
                m=elements.size();
                for (i=1;i<m;i++)
                {
                    element=elements.codeElementAt(i);
                    if (element instanceof LispListElement)
                        ((LispListElement) element).analyze();
                    else if (element instanceof LispFunctionElement)
                        ((LispFunctionElement) element).analyze();
                };
            };
            return false;
        }
    }
    /** Find the possible hints for this list (where we assume that this is a call) */
    public boolean findHints(CodeElement element,Vector hints)
    {
		CodeElement first=getFirst();
		if (first==element)
		{
			hints.addAll(LispAnalyzer.getFunctionsAndMacros());
			return true;
		}
		else  if (LispAnalyzerGen.findHints(element,hints,this))
		{
			return true;	
		}
		else
		{
			int m=elements.size();
			for (int i=1;i<m;i++)
			{
				if (elements.codeElementAt(i) ==element)
				{
					hints.addAll(LispAnalyzer.getValidForms(element));
					hints.addAll(LispAnalyzer.getTypes(element));
					return true;
				}
				else if (elements.codeElementAt(i) instanceof LispListElement)
				{
					((LispListElement) elements.codeElementAt(i)).findHints(element,hints);
				}
				else if (elements.codeElementAt(i) instanceof LispFunctionElement)
				{
					((LispFunctionElement) elements.codeElementAt(i)).findHints(element,hints);
				};
			};
			return true;
		}
    }
    /** Replace the use of a definition by a newdefinition  (in LISP replace a function/macro call by another call) */
    public void replaceUsedDefinition(CodeDocument document,CodeDocument.UpdateBuffer updatebuffer,String olddefinition,String newdefinition)
    {
        olddefinition=olddefinition.toUpperCase().intern();
        int m;
        boolean insidedef=false;
        CodeElement element;
        // Toplevel call
        CodeElementVector children=getElements();
        m=children.size();
        for (int i=0;i<m;i++)
        {
            element=children.codeElementAt(i);
            if (i==0 && element instanceof LispAtomElement &&
            ((LispAtomElement) element).getType()==LispAtomElement.Caller &&
            ((LispAtomElement) element).isEqualTo(olddefinition))
                updatebuffer.addUpdate(element.begin(),element.end(),newdefinition);
            else if (element instanceof LispListElement)
                ((LispListElement) element).replaceUsedDefinition(document,updatebuffer, olddefinition, newdefinition);
            else if (element instanceof LispFunctionElement)
                ((LispFunctionElement) element).replaceUsedDefinition(document,updatebuffer,olddefinition,newdefinition);
        };
    };
    /** Get the first element in this list, return null if there are no elements */
    public CodeElement  getFirst()
    {
        int m=elements.size();
        int i;
        for (i=0;i<m && elements.codeElementAt(i) instanceof LispCommentElement;i++);
        if (i<m)
            return elements.codeElementAt(i);
        else
            return null;
    }
    /**
     * Search the list of elements of this list for the first element covering
     * pos or which starts after pos
     * the index of this element (take in account the comments)
     */
    
    public int getPosition(int pos)
    {
        CodeElementVector elements=getElements();
        int m=elements.size();
        CodeElement element;
        for (int i=0;i<m;i++)
        {
            element=elements.codeElementAt(i);
            if ((element.begin()<= pos && pos < element.end()) || pos<element.begin())
                return i;
        }
        return m;
    }
    /** Get the variables defined in this list and in its parents */
    public Vector getAllVariables()
    {
        int m;
        Vector result=new Vector();
        CodeElement parent=getParent();
        if (parent instanceof LispListElement)
        {
            Vector parentvariables=((LispListElement) parent).getAllVariables();
            m=parentvariables.size();
            for (int i=0;i<m;i++)
                result.addElement(parentvariables.elementAt(i));
        };
        m=variables.size();
        for (int i=0;i<m;i++)
            result.addElement(variables.codeElementAt(i));
        return result;
    }
    /** Get the variables for the context of a given element (taking in account LET and LET*) */
    public static Vector getAllVariablesFor(CodeElement element)
    {
    	CodeElement top=element.getTopParent();
    	if (top instanceof LispListElement)
    	{
    		Vector vars=((LispListElement) top).getAllVariablesForElement(element,new Vector());
    		if (vars==null)
    			return new Vector();
    		else
    			return vars;
    	}
    	else
    		return new Vector();
    }
    /** Get the variables for the context of a given element (taking in account LET and LET*) */
    private Vector getAllVariablesForElement(CodeElement element,Vector vars)
    {
    	Vector newvars=variables.toVector();
    	newvars.addAll(vars);
    	Vector temp;
    	CodeElement el;
    	int m=elements.size();
    	boolean isLet=isLet();
    	for (int i=0;i<m;i++)
 		{
 			el=elements.codeElementAt(i);
 			if (el==element)
 			{
 				return newvars;		
 			}
 			else if (el instanceof LispListElement)
 			{
 				if (isLet && i==1)
 					temp=((LispListElement) el).getAllVariablesForElementLet(element,newvars);
 				else
 					temp=((LispListElement) el).getAllVariablesForElement(element,newvars);
 				if (temp!=null)
 					return temp;
 			}
 		}  	
    	return null; // Indicate that element is not in the context of a variable 
    }
    
    /** Get the variables for a variable declaration in a LET or LET* */
    private Vector getAllVariablesForElementLet(CodeElement element,Vector vars)
    {
    	// This is the first list after a LET and LET*  
    	// So it is of the form ((var ...) ....)
    	int m=elements.size();
    	Vector newvars;
    	CodeElement el;
    	for (int i=0;i<m;i++)
    	{
    		el=elements.codeElementAt(i);
    		if (el ==element)
    			return null; // this should not happen
    		else if (el instanceof LispListElement)
    		{
    			CodeElementVector els=((LispListElement) el).elements;
 				if (els.size()!=2)
 				{
 					return null; // Should be 2
 				}
 				else
 				{
 					newvars=(Vector) vars.clone();
 					newvars.remove(els.codeElementAt(0));
 					newvars=((LispListElement) el).getAllVariablesForElement(element,newvars);
 					if (newvars!=null)
 						return newvars;
 				}
    		}
    		else 
    		{
    			return null; // This should not happen
    		}
    	}
    	return null;
    }
    /** Check if this is a LET or LET* list */
    private boolean isLet()
    {
    	CodeElement first=getFirst();
    	return (first instanceof LispAtomElement && (((LispAtomElement) first).isEqualTo(LET) || ((LispAtomElement) first).isEqualTo(LETSTAR)));
    }
    /**
     * Will update a treeset with the strings representing all the callers in this list or its sublists
     * @param document Document used to retrieve the string of the callers
     * @param callers TreeSet to store the result in
     */
    // TODO: Remove this after testing 
    /*
    public void updateCallerList(CodeDocument document,TreeSet callersAccumulator)
    {
    	CodeElement element;
    	int m=callers.size();
    	CodeElement first;
    	for (int i=0;i<m;i++)
    		if (callers.codeElementAt(i) instanceof LispListElement)
    		{
    			first=((LispListElement) callers.codeElementAt(i)).getFirst();
    			if (first!=null)
    				callersAccumulator.add(first.getString(document).toLowerCase());
    		}
		m=elements.size();
    	for (int i=0;i<m;i++)
    		if (elements.codeElementAt(i) instanceof LispListElement)
    			((LispListElement) elements.codeElementAt(i)).updateCallerList(document,callersAccumulator);
    }
    */
	/** Updates the definitions stored in the Repository, this will be called by the Repository on the toplevel elements
	 * of a document.
	 */
	public void updateDefinition(IRepository repository,CodeDocument doc)
	{
		LispRepository lisprepository=(LispRepository) repository;
		// Only do updates if this is a call
		if (isCall)
		{
			// Process the callers
			processCallers(doc,lisprepository);
			// Process the different subelements
			CodeElementVector els=getElements();
			int m=els.size();
			// Check if we have at least two arguments
			if (m>1)
			{
				if (els.codeElementAt(0) instanceof LispAtomElement)
				{
					String symbol=((LispAtomElement) els.codeElementAt(0)).getSymbol();
					String name;
					if (symbol==LispAtomElement.DEFPACKAGE)
					{
						// Process a DEFPACKAGE statement
						processDefpackage(els,doc,lisprepository);
					}
					else if (symbol==LispAtomElement.EXPORT)
					{
						processExport(els,doc,lisprepository);
					}
					else if (symbol==LispAtomElement.IMPORT)
					{
						processImport(els,doc,lisprepository);
					}
					else if (symbol==LispAtomElement.MAKEPACKAGE)
					{
						name=nameKeywordOrStringOrQuoted(els.codeElementAt(1),doc);
						if (name==null)
						{
							MessageHandler.codeError("Jabberwocky works only with a string, a keyword or a quoted symbol as argument for makepackage.",doc,beginPosition());
						}
						else
						{
							// Definition of package
							lisprepository.createPackage(name,beginPosition(),length());
							return; // No further processing of the subelements may be done
						};
					}
					else if (symbol==LispAtomElement.INPACKAGE)
					{
						processInPackage(els,doc,lisprepository);		
					}
					else if (els.codeElementAt(1) instanceof LispAtomElement &&
						(symbol==LispAtomElement.DEFCLASS ||
						 symbol==LispAtomElement.DEFCONSTANT ||
						 symbol==LispAtomElement.DEFGENERIC ||
						 symbol==LispAtomElement.DEFMACRO ||
						 symbol==LispAtomElement.DEFPARAMETER ||
						 symbol==LispAtomElement.DEFSTRUCT ||
						 symbol==LispAtomElement.DEFTYPE ||
						 symbol==LispAtomElement.DEFUN ||
						 symbol==LispAtomElement.DEFVAR))
					{
						name=((LispAtomElement) els.codeElementAt(1)).getSymbol();
						lisprepository.addDefinition(symbol,name.toLowerCase(),beginPosition(),length());
					};
					// Apply to possible subcalls
					for (int i=2;i<m;i++)
						els.codeElementAt(i).updateDefinition(lisprepository,doc);
				}
				else
				{
					// Try possible subcalls
					for (int i=1;i<m;i++)
						els.codeElementAt(i).updateDefinition(lisprepository,doc);
				};
			}
		}
	}
	/**
	 * Find out where there is a call to a macro or function and register it in the repository
	 * @param doc  The document analyzed
	 * @param rep The LispRepository asking for the analyze
	 */
	private void processCallers(CodeDocument doc,LispRepository rep)
	{
		CodeElement element;
		int m=callers.size();
		CodeElement first;
		// If this element is itself a caller and has a first element add its first element
		if (isCall)
		{
			first=getFirst();
			if (first!=null)
				rep.registerCaller(first.getString(doc).toLowerCase());
		}
		// Get the list of callers in this element
		for (int i=0;i<m;i++)
			if (callers.codeElementAt(i) instanceof LispListElement)
			{
				first=((LispListElement) callers.codeElementAt(i)).getFirst();
				if (first!=null)
					rep.registerCaller(first.getString(doc).toLowerCase());
			}
		m=elements.size();
		for (int i=0;i<m;i++)
			if (elements.codeElementAt(i) instanceof LispListElement)
				((LispListElement) elements.codeElementAt(i)).processCallers(doc,rep);
	}
	/** 
	 * Process a (in-package "package") statement
	 * @param els The elements of the list
	 * @param doc The document analyzed
	 * @param rep The repository
	 */
	private void processInPackage(CodeElementVector els,CodeDocument doc,LispRepository rep)
	{
		if (els.size()==2)
		{
			if (els.codeElementAt(1) instanceof LispStringElement)
			{
				rep.inPackage(nameString(els.codeElementAt(1),doc),beginPosition(),length());
			};
		}
		else
		{
			MessageHandler.codeWarn("(in-package \"package\") has not the correct nr of arguments",doc,beginPosition());		
		}	
	}
	/** Process the import of a symbol */
	private void processImport(CodeElementVector els,CodeDocument doc,LispRepository rep)
	{
		int m=els.size();
		String to=null; // Package containing symbol, package symbol is exported to
		String[][] packageAndSymbols;
		if (m>1 && els.codeElementAt(1) instanceof LispQuoteElement)
		{
			LispQuoteElement element=(LispQuoteElement) els.codeElementAt(1);
			if (element.isQuotedList())
			{
				int nrsymbols=0;
				CodeElementVector symbols=((LispListElement) element.getQuoted()).getElements();
				int nrelements=symbols.size();
				for (int i=0;i<nrelements;i++)
					if (symbols.codeElementAt(i) instanceof LispAtomElement)
					{
						nrsymbols++;
					}
					else 
					{
						MessageHandler.codeError("(import '(...) ...) symbol list contains not a symbol",doc,beginPosition());
					};
				packageAndSymbols=new String[nrsymbols][2];
				for (int i=0,j=0;i<nrelements;i++)
				{
					if (symbols.codeElementAt(i) instanceof LispAtomElement)
					{
						packageAndSymbols[j]=((LispAtomElement) symbols.codeElementAt(i)).getSymbolAndPackage();
						j++;
					}
				};
			}
			else if (element.isQuotedSymbol())
			{
				packageAndSymbols=new String[1][2];
				packageAndSymbols[0]=element.getSymbolAndPackage();
			}
			else
			{
				MessageHandler.codeWarn("Jabberwocky can only recognize a quoted symbols in the import form.",doc,beginPosition());
				return;
			}
		}
		else
		{
			MessageHandler.codeWarn("Jabberwocky can only recognized quoted symbols in the import form.",doc,beginPosition());
			return;
		}
		if (m==3)
		{
			to=nameString(els.codeElementAt(2),doc);
			if (to==null)
			{
				MessageHandler.codeWarn("Jabberwocky can only import symbols via  (import <symbols> <package> if <package> is a string",doc,beginPosition());
				return;		
			}
		};
		rep.importSymbols(packageAndSymbols,to,beginPosition(),length());
	}
	
	/** Process the export of a symbol */
	private void processExport(CodeElementVector els,CodeDocument doc,LispRepository rep)
	{
		int m=els.size();
		if (m>1)
		{
			if (els.codeElementAt(1) instanceof LispQuoteElement)
			{
				String[] exportedsymbols;
				LispQuoteElement quoteelement=(LispQuoteElement) els.codeElementAt(1);
				if (quoteelement.getQuoted() instanceof LispListElement)
				{
					CodeElementVector symbols=((LispListElement) quoteelement.getQuoted()).getElements();
					int max=symbols.size();
					ArrayList exported=new ArrayList();
					String symbol;
					for (int i=0;i<max;i++)
					{
						if (symbols.codeElementAt(i) instanceof LispAtomElement)
							exported.add(((LispAtomElement) symbols.codeElementAt(i)).getString(doc));
						else
							MessageHandler.codeError("Exported symbol is not a real symbol",doc,beginPosition());
					};
					exportedsymbols=new String[exported.size()];
					for (int i=0;i<exportedsymbols.length;i++)
						exportedsymbols[i]=(String) exported.get(i);
				}
				else if (quoteelement.isQuotedSymbol())
				{
					exportedsymbols=new String[1];
					exportedsymbols[0]=quoteelement.getQuoted().getString(doc);
				}
				else
				{
					exportedsymbols=new String[0];
					MessageHandler.codeWarn("Jabberwocky was not able to understand the export form",doc,beginPosition());
				}
				String from=null;
				if (m>2)
				{
					from=nameKeywordOrStringOrQuoted(els.codeElementAt(2),doc);
					if (from==null)
						MessageHandler.codeError("Namespace in (export ...) must be a string, a keyword or a quoted symbol",doc,beginPosition());
				};
				rep.exportSymbols(exportedsymbols,from,beginPosition(),length());
			}
			else
			{
				MessageHandler.codeError("Second argument of (export ..) is not a list",doc,beginPosition());
			}
		}
		else
		{
			MessageHandler.codeError("(export ...) does not referes to symbols to export",doc,beginPosition());
		}
	}
	/** Processes a defpackage statement */
	private void processDefpackage(CodeElementVector els,CodeDocument doc,LispRepository rep)
	{
		int m=els.size();
		String name=nameKeywordOrString(els.codeElementAt(1),doc);
		ArrayList useOptions=new ArrayList();
		ArrayList shadowOptions=new ArrayList();
		ArrayList shadowImportOptions=new ArrayList();
		ArrayList importOptions=new ArrayList();
		ArrayList internOptions=new ArrayList();
		ArrayList exportOptions=new ArrayList();
		if (name==null)
		{
			MessageHandler.codeError("Package name is neither a string or a keyword",doc,beginPosition());
		}
		else
		{
			rep.createPackage(name,beginPosition(),length());
			// Check out for the different options of defpackage
			for (int i=2;i<m;i++)
				if (els.codeElementAt(i) instanceof LispListElement)
				{
					CodeElementVector option=((LispListElement) els.codeElementAt(i)).getElements();
					if (option.size()<2 || !(option.codeElementAt(0) instanceof LispAtomElement))
					{
						MessageHandler.codeError("Error in defpackage options",doc,beginPosition());
					}
					else
					{
						String op=((LispAtomElement) option.codeElementAt(0)).getSymbol();
						if (op==LispAtomElement.NICKNAMESOPTION)
							processDefpackageNicknames(name,option,doc,rep);
						else if (op==LispAtomElement.DOCUMENTATIONOPTION)
							processDefpackageDocumentation(name,option,doc,rep);
						else if (op==LispAtomElement.USEOPTION)
							useOptions.add(option);
						else if (op==LispAtomElement.SHADOWOPTION)
							shadowOptions.add(option);
						else if (op==LispAtomElement.SHADOWINGIMPORTOPTION)
							shadowImportOptions.add(option);
						else if (op==LispAtomElement.IMPORTFROMOPTION)
							importOptions.add(option);
						else if (op==LispAtomElement.EXPORTOPTION)
							exportOptions.add(option);
						else if (op==LispAtomElement.INTERNOPTION)
							internOptions.add(option);
						else if (op==LispAtomElement.SIZEOPTION)
							processDefpackageSize(name,option,doc,rep);
						else
							MessageHandler.codeError("Error in options for defpackage",doc,beginPosition());	
					}
				}
				else
				{
					MessageHandler.codeError("Option for defpackage is not a list",doc,beginPosition());
				};
			// Process shadow and shadowing import
			m=shadowOptions.size();
			for (int i=0;i<m;i++)
				processDefpackageShadow(name,(CodeElementVector) shadowOptions.get(i),doc,rep);
			m=shadowImportOptions.size();
			for (int i=0;i<m;i++)
				processDefpackageShadowingImportFrom(name,(CodeElementVector) shadowImportOptions.get(i),doc,rep);
			// Process use
			m=useOptions.size();
			for (int i=0;i<m;i++)
				processDefpackageUse(name,(CodeElementVector) useOptions.get(i),doc,rep);
			// Process import-from and intern
			m=importOptions.size();
			for (int i=0;i<m;i++)
				processDefpackageImportFrom(name,(CodeElementVector) importOptions.get(i),doc,rep);
			m=internOptions.size();
			for (int i=0;i<m;i++)
				processDefpackageIntern(name,(CodeElementVector) internOptions.get(i),doc,rep);
			// Process export
			m=exportOptions.size();
			for (int i=0;i<m;i++)
				processDefpackageExport(name,(CodeElementVector) exportOptions.get(i),doc,rep);
		}
	}
	private void processDefpackageNicknames(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		String[] nicknames=new String[option.size()-1];
		String nickname;
		for (int i=0;i<nicknames.length;i++)
		{
			nickname=nameKeywordOrString(option.codeElementAt(i+1),doc);
			if (nickname==null)
			{
				MessageHandler.codeError("Nickname is not a string or a keyword",doc,beginPosition());
				return;
			}
			else
			{
				nicknames[i]=nickname;
			};
		};
		rep.setNickNamesForPackage(namePackage,nicknames,beginPosition(),length());
	}
	private void processDefpackageDocumentation(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		if (option.codeElementAt(1) instanceof LispStringElement)
			rep.setDocumentationForPackage(namePackage,nameKeywordOrString(option.codeElementAt(1),doc),beginPosition(),length());
		else
			MessageHandler.codeError("Documentation is not a string",doc,beginPosition());
	}
	private void processDefpackageUse(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		String[] packages=new String[option.size()-1];
		String packagename;
		for (int i=0;i<packages.length;i++)
		{
			packagename=nameKeywordOrString(option.codeElementAt(i+1),doc);
			if (packagename==null)
			{
				MessageHandler.codeError("Package name is not a string",doc,beginPosition());
			}
			else
			{
				packages[i]=packagename;	
			};
		};
		for (int i=0;i<packages.length;i++)
			rep.useNameSpace(namePackage,packages[i],beginPosition(),length());
	}
	private void processDefpackageShadow(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		
	}
	private void processDefpackageShadowingImportFrom(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		if (option.size()>2)
		{
			String from=nameKeywordOrString(option.codeElementAt(1),doc);
			if (from==null)
			{
				MessageHandler.codeError("Package name is not a string",doc,beginPosition());				
			}
			else
			{
				String[] symbols=new String[option.size()-2];
				for (int i=0;i<symbols.length;i++)
				{
					symbols[i]=nameQuoted(option.codeElementAt(i+2),doc);
					if (symbols[i]==null)
					{
						MessageHandler.codeError("Imported is not a symbol",doc,beginPosition());
						return;
					};
				};
				rep.shadowImportSymbols(namePackage,from,symbols,beginPosition(),length());
			}
		}
		else
		{
			MessageHandler.codeError("Error in (:import-from <package> {<symbol>}*) clause",doc,beginPosition());
		}		
	}
	private void processDefpackageImportFrom(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		if (option.size()>2)
		{
			String from=nameKeywordOrString(option.codeElementAt(1),doc);
			if (from==null)
			{
				MessageHandler.codeError("Package name is not a string",doc,beginPosition());				
			}
			else
			{
				String[] symbols=new String[option.size()-2];
				for (int i=0;i<symbols.length;i++)
				{
					symbols[i]=nameQuoted(option.codeElementAt(i+2),doc);
					if (symbols[i]==null)
					{
						MessageHandler.codeError("Imported is not a symbol",doc,beginPosition());
						return;
					};
				}; 
				rep.importSymbols(namePackage,from,symbols,beginPosition(),length());
			}
		}
		else
		{
			MessageHandler.codeError("Error in (:import-from <package> {<symbol>}*) clause",doc,beginPosition());
		}
	}
	private void processDefpackageExport(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		
	}
	private void processDefpackageIntern(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		
	}
	private void processDefpackageSize(String namePackage,CodeElementVector option,CodeDocument doc,LispRepository rep)
	{
		
	}
	/** Check if a element is either a LispAtomElement and return then the symbol 
	 * string or if element is a LispStringElement and returns then the string */
	private String nameKeywordOrString(CodeElement element,CodeDocument document)
	{
		if (element instanceof LispAtomElement && ((LispAtomElement) element).getSymbol().startsWith(":"))
			return ((LispAtomElement) element).getString(document).substring(1);
		else if (element instanceof LispStringElement)
			return element.getString(document).substring(1,element.getString(document).length()-1);
		else
			return null;
	}
	/** Check if a element is either a LispAtomElement and return then the symbol 
	 * string or if element is a LispStringElement and returns then the string or if 
	 * the element is a LispQutedElement of a LispAtomElement and returns then the quoted symbol string */
	private String nameKeywordOrStringOrQuoted(CodeElement element,CodeDocument document)
	{
		if (element instanceof LispAtomElement && ((LispAtomElement) element).getSymbol().startsWith(":"))
			return ((LispAtomElement) element).getString(document).substring(1);
		else if (element instanceof LispStringElement)
			return element.getString(document).substring(1,element.getString(document).length()-1);
		else if (element instanceof LispQuoteElement && ((LispQuoteElement) element).getQuoted() instanceof LispAtomElement)
			return ((LispAtomElement) ((LispQuoteElement) element).getQuoted()).getString(document);
		else
			return null;
	}
	/** Check if a element is a LispAtomElement and return then the symbol 
	 * string (returns null otherwise)
	 */
	private String nameQuoted(CodeElement element,CodeDocument document)
	{
		if (element instanceof LispQuoteElement && ((LispQuoteElement) element).getQuoted() instanceof LispAtomElement)
			return ((LispAtomElement) ((LispQuoteElement) element).getQuoted()).getString(document);
		else
			return null;
	}
	private String nameString(CodeElement element,CodeDocument document)
	{
		if (element instanceof LispStringElement)
			return element.getString(document).substring(1,element.getString(document).length()-1);
		else
			return null;
	}
    /** Indicates if this element is a call */
    public boolean isCall()
    {
        return isCall;
    }
    /** Check if this list is empty */
    public boolean isEmpty()
    {
        return elements.size()==0;
    }
    /** Check if itst first element of this list starts with a atom with string representation equal to string */
    public boolean startsWith(String symbol)
    {
        return elements.size()>0 &&
        (elements.codeElementAt(0) instanceof LispAtomElement) &&
        ((LispAtomElement) elements.codeElementAt(0)).isEqualTo(symbol);
    }
    /** Get the non comment elements of this list, skipping the dot if it exist */
    public CodeElementVector getElements()
    {
        CodeElementVector result=new CodeElementVector();
        int m=elements.size();
        for (int i=0;i<m;i++)
            if (isLispElement(elements.codeElementAt(i)))
                result.addCodeElement(elements.codeElementAt(i));
        return result;
    }
    /** Check is a real lisp element (comments and . are not real elements) */
    private static boolean isLispElement(CodeElement element)
    {
        if (element instanceof LispCommentElement)
            return false;
        else if (element instanceof LispAtomElement)
        {
            String symbol=((LispAtomElement) element).getSymbol();
            if (symbol.length()==1 && symbol.charAt(0)=='.')
                return false;
            else
                return true;
        }
        else
            return true;
    }
    /** Get the elements inclusive the comments */
    protected CodeElementVector getElementsAndComments()
    {
        return elements;
    }
    /** Indicate that this element is a list */
    public boolean isList()
    {
        return true;
    }
    /** Get the color of this list */
    public Color getColor()
    {
        if (parent==null)
            return Configuration.colorForeground;
        else
            return parent.getColor();
    }
    /* Let the sublists know that we are there parents */
    public void registerParent()
    {
        int m=elements.size();
        for (int i=0;i<m;i++)
            if (elements.codeElementAt(i) instanceof LispListElement)
            {
                elements.codeElementAt(i).setParent(this);
                ((LispListElement) elements.codeElementAt(i)).registerParent();
            };
    }
    /// TODO Comment this after debugging
    public void debugPrint()
    {
    	System.out.print("(");
    	CodeElementVector elements=getElements();
    	for (int i=0;i<elements.size();i++)
    	{
    		if (elements.codeElementAt(i) instanceof LispAtomElement)
    			{
    				System.out.print(((LispAtomElement) elements.codeElementAt(i)).getSymbol());
    				System.out.print(" ");
    			}
    		else if (elements.codeElementAt(i) instanceof LispListElement)
    			{
    				((LispListElement) elements.codeElementAt(i)).debugPrint();
    				System.out.print(" ");
    			}
    	};
    	System.out.print(")");
    }
}
