/*
 * LispParser.java
 *
 * Created on October 1, 2001, 7:46 PM
 */

package IDE.CodeEditor.Parser;

import java.util.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;
import IDE.LispSyntax.*;
import IDE.Project.*;
import IDE.Interaction.*;
import java.awt.event.*;

/**
 * Parser for LISP code
 * @author  marc
 * @version
 */

public class LispParser extends Parser
{
    /**
     * Constructor for the parser, async=true means do the pars in a asynchronous way otherwise
     * do it in synchronous way.
     */
    public LispParser(boolean async)
    {
        super(async);
    }
    /** Get the repository associated with this parser */
    public IRepository getRepository()
    {
    	return LispRepository.lisprepository;
    }
    /** Check if a position is covered by a element that must be a list and that the position is not the first parenthese */
    private boolean isCoveredByList(int position,CodeElement element)
    {
        return element!=null && element instanceof LispListElement && element.begin()<position && position<element.end();
    }
    /** Find the index of openings hook of biggest list enclosing this position */
    private int findIndexEarlistOpeningHook(int pos)
    {
        Region region=findHooks(pos);
        int iopen=region==null ? -1 : region.begin;
        int iclose=region==null ? -1 : region.end;
        while ((region=findHooks(iopen-1))!=null && region!=null && region.begin!=0 && region.begin<=pos && pos<=region.end)
        {
            iopen=region.begin;
            iclose=region.end;
        };
        if (region!=null && region.begin==0 && 0<=pos && pos<=region.end)
            return document.getIndexLeafAt(0);
        if (iopen <= pos && pos<=iclose)
            return document.getIndexLeafAt(iopen);
        else
            return -1;
    }
    /** Get a sorted string array of all the known function and macros */
    private String[] getFunctionsAndMacros()
    {
    	/// TODO After testing removes this
    	/*
        TreeSet result=new TreeSet();
        String[] buildins=LispAnalyzerGen.getLispKeywords();
        File file=document.getFile();
        TODO Remove this DefinitionInfo[] definitions=Explorer.getDefinitionInfos(file,new String[]
        {"defgeneric","defmethod","defun","defmacro"});
        for (int i=0;i<buildins.length;i++)
            result.add(buildins[i].toLowerCase());
        for (int i=0;i<definitions.length;i++)
            result.add(definitions[i].name.toLowerCase());
        Iterator iter=result.iterator();
        String[] functionsandmacros=new String[result.size()];
        for (int i=0;iter.hasNext();i++)
            functionsandmacros[i]=(String) iter.next();
        return functionsandmacros;
    	*/
    	String[] definitions=LispRepository.lisprepository.getDefinitionsOfType(new String[]{"defgeneric","defmethod","defun","defmacro"});
    	String[] buildins=LispAnalyzerGen.getLispKeywords();
    	String[] result=new String[definitions.length+buildins.length];
    	System.arraycopy(definitions,0,result,0,definitions.length);
    	System.arraycopy(buildins,0,result,definitions.length,buildins.length);
    	Arrays.sort(result,DefinitionNameComparator.comparator);
    	return result;
    }
    /** Get a sorted string array of all the known functions */
    private String[] getFunctions()
    {
    	/// TOD after testing removes this
    	/*
        TreeSet result=new TreeSet();
        String[] buildins=LispAnalyzerGen.getLispKeywords();
        File file=document.getFile();
        DefinitionInfo[] definitions=Explorer.getDefinitionInfos(file,new String[]
        {"defgeneric","defmethod","defun"});
        for (int i=0;i<buildins.length;i++)
            result.add(buildins[i].toLowerCase());
        for (int i=0;i<definitions.length;i++)
            result.add(definitions[i].name.toLowerCase());
        Iterator iter=result.iterator();
        String[] functions=new String[result.size()];
        for (int i=0;iter.hasNext();i++)
            functions[i]=(String) iter.next();
        return functions;
        */
		String[] definitions=LispRepository.lisprepository.getDefinitionsOfType(new String[]{"defgeneric","defmethod","defun"});
		String[] buildins=LispAnalyzerGen.getLispKeywords();
		String[] result=new String[definitions.length+buildins.length];
		System.arraycopy(definitions,0,result,0,definitions.length);
		System.arraycopy(buildins,0,result,definitions.length,buildins.length);
		Arrays.sort(result,DefinitionNameComparator.comparator);
		return result;
    }
    
    /** Get a sorted string array of all the known variables */
    private String[] getVariables(LispListElement list)
    {
    	/// TODO After testing remove this
    	/*
        return getDefinitionsAndVariables(list,new String[]
        {"defparameter","defconstant","defvar"});
        */
        return LispRepository.lisprepository.getDefinitionsOfType(new String[]{"defparameter","defconstant","defvar"});
    }
    
    /** Get a sorted list of types */
    private String[] getVariablesAndTypes(LispListElement list)
    {
    	/// TODO After testing remove this
    	/*
        return getDefinitionsAndVariables(list,new String[]
        {"defparameter","defconstant","defvar","deftype","defclass"})
        */
        return LispRepository.lisprepository.getDefinitionsOfType(new String[] {"defparameter","defconstant","defvar","deftype","defclass"});
    }
    /** Get a sorted string array of the classes different from classname */
    private String[] getClasses(String classname)
    {
    	/// TODO After testing remove this
    	/*
        DefinitionInfo[] defs=Explorer.getDefinitionInfos(document.getFile(),new String[]
        {"defclass"});
        Vector temp=new Vector();
        for (int i=0;i<defs.length;i++)
            if (defs[i].name.compareTo(classname)!=0)
                temp.add(defs[i].name);
        int m=temp.size();
        String[] hints=new String[m];
        for (int i=0;i<m;i++)
            hints[i]=(String) temp.elementAt(i);
        return hints;
        */
        String[] defs=LispRepository.lisprepository.getDefinitionsOfType(new String[]{"defclass"});
        ArrayList definitions=new ArrayList();
        for (int i=0;i<defs.length;i++)
        	if (classname.compareTo(defs[i])!=0 && !defs[i].endsWith(":"+classname))
        		definitions.add(defs[i]);
       	String[] result=new String[definitions.size()];
       	for (int i=0;i<result.length;i++)
       		result[i]=(String) definitions.get(i);
       	return result;
    }
    /** Get a sorted string array of the classes in a quoted form defined here */
    private String[] getQuotedClasses()
    {
    	/// TODO Remove this after testing
    	/*
        DefinitionInfo[] defs=Explorer.getDefinitionInfos(document.getFile(),new String[]
        {"defclass"});
        String[] hints=new String[defs.length];
        for (int i=0;i<hints.length;i++)
            hints[i]='\''+defs[i].name;
        return hints;
        */
        String[] defs=LispRepository.lisprepository.getDefinitionsOfType(new String[]{"defclass"});
        for (int i=0;i<defs.length;i++)
        	defs[i]="\'"+defs[i];
        return defs;
    }
    /** Get the type definitions and class definitions and scoped variables defined */
    private String[] getDefinitionsAndVariables(LispListElement list,String[] types)
    {
    	/// TODO removes this after testing
    	/**
        TreeSet hintset=new TreeSet();
        Vector variables=list.getAllVariables();
        String[] hints;
        int m=variables.size();
        for (int i=0;i<m;i++)
            hintset.add(((CodeElement) variables.elementAt(i)).getString(document));
        DefinitionInfo[] definitions=Explorer.getDefinitionInfos(document.getFile(),types);
        for (int i=0;i<definitions.length;i++)
            if (definitions[i].type.compareTo("deftype")==0 || definitions[i].type.compareTo("defclass")==0)
                hintset.add('\''+definitions[i].name);
            else
                hintset.add(definitions[i].name);
        hints=new String[hintset.size()];
        Iterator iter=hintset.iterator();
        for (int i=0;iter.hasNext();i++)
            hints[i]=(String) iter.next();
        return hints;
        */
		TreeSet hintset=new TreeSet(DefinitionNameComparator.comparator);
		 Vector variables=list.getAllVariables();
		 String[] hints;
		 int m=variables.size();
		 for (int i=0;i<m;i++)
			 hintset.add(((CodeElement) variables.elementAt(i)).getString(document));
		 boolean deftype=false,defclass=false;
		 m=types.length;
		 for (int i=0;i<types.length;i++)
		 	if (types[i].compareTo("deftype")==0)
		 	{
		 		deftype=true;
		 		m--;
		 	}
		 	else if (types[i].compareTo("defclass")==0)
		 	{
		 		deftype=true;
		 		m--;
		 	};
		 // Get all the types that are different from deftype and defclass
		 String[] newtypes=new String[m];
		 for (int i=0,j=0;i<types.length;i++)
		 	if (types[i].compareTo("deftype")!=0 && types[i].compareTo("defclass")==0)
		 	{
		 		newtypes[j]=types[i];
		 		j++;
		 	};
		 // Add the types different from deftype and defclass
		 String[] definitions=LispRepository.lisprepository.getDefinitionsOfType(newtypes);
		 for (int i=0;i<definitions.length;i++)
		 	hintset.add(definitions[i]);
		 // Add the deftype 
		 if (deftype)
		 {
			definitions=LispRepository.lisprepository.getDefinitionsOfType(new String[]{"deftype"});
			for (int i=0;i<definitions.length;i++)
			   hintset.add("\'"+definitions[i]);		 	
		 };
		// Add the defclass 
		if (deftype)
		{
		   definitions=LispRepository.lisprepository.getDefinitionsOfType(new String[]{"defclass"});
		   for (int i=0;i<definitions.length;i++)
			  hintset.add("\'"+definitions[i]);		 	
		};
		// Calculate the result
		 hints=new String[hintset.size()];
		 Iterator iter=hintset.iterator();
		 for (int i=0;iter.hasNext();i++)
			 hints[i]=(String) iter.next();
		 return hints;
    }
    /** Search for the nearest match of a string in a sorted array of strings */
    private int search(String[] strings,String key)
    {
        key=key.toLowerCase();
        int pos=Arrays.binarySearch(strings,key,DefinitionNameComparator.comparator);
        return (pos>0) ? pos : Math.abs(pos)-1;
    }
 	/** Update a list of all the callers for the given document */
 	/*
 	public void updateCallers(CodeDocument document,TreeSet callers)
 	{
 		CodeElementVector elements=document.getElements();
 		int m=elements.size();
 		for (int i=0;i<m;i++)
 			if (elements.codeElementAt(i) instanceof LispListElement)
 				((LispListElement) elements.codeElementAt(i)).updateCallerList(document,callers);
 	}
 	*/
 	/** Get the syntax description based on a element somewhere in a call */
 	public String getSyntaxDescriptionCall(CodeElement element,CodeDocument document)
 	{
 		CodeElement parent;
 		for (parent=element.getParent();parent!=null && !(parent instanceof LispListElement && ((LispListElement) parent).isCall());parent=parent.getParent());
 		if (parent instanceof LispListElement)
 		{
 			CodeElementVector elements=((LispListElement) parent).getElements();
 			if (elements.size()==0)
 				return "";
 			else if (elements.codeElementAt(0) instanceof HintMarkerElement && elements.size()>1)
 				return getSyntaxDescription(elements.codeElementAt(1).getString(document));
 			else
 				return getSyntaxDescription(elements.codeElementAt(0).getString(document));
 		}
 		else
 		{
 			return "";
 		}
 	}
    /* Get the syntax hint for a certain function(macro) name */
    public String getSyntaxDescription(String name)
    {
    	if (name==null)
    		return "";
        String result=LispInteraction.getSyntaxDescription(name);
        if (result.length()==0)
        	result=PredefinedSyntaxDescriptions.getSyntaxDescription(name.toUpperCase());
        if (result==null)
        	result="";
        return result;
    }
    /** Replace the leaf by the hint */
    
    public void replaceLeafByHint(CodeLeafElement leaf,String hint)
    {
        int b=leaf.begin();
        int e=leaf.end();
        if (leaf.getSeparatorChar()=='(')
        {
            document.getCodeEditor().insertStringAt(hint,b+1);
            document.getCodeEditor().cursor.setCursor(b+hint.length()+1);
        }
        else if (leaf.getSeparatorChar()==')')
        {
            document.getCodeEditor().insertStringAt(hint,b);
            document.getCodeEditor().cursor.setCursor(b+hint.length());
        }
        else
        {
            document.getCodeEditor().replace(b,e-b,hint);
            document.getCodeEditor().cursor.setCursor(b+hint.length());
        };
    }
    /** Make a clone of this parser */
    public Parser cloneParser(boolean async)
    {
        return LispRepository.lisprepository.createParser(async);
    }
    /**  Find out the correct number of spaces that must be placed at the begin of this line to do correct indentation */
    public int spacesForIndentation(int pos)
    {
        CodeLeafElement leaf;
        String leafstring;
        int i;
        // Find the line covering this position
        CodeLine line=document.getLineAt(pos);
        if (line==null) return 0; // Could not calculate the indentation spaces
        int beginline=line.begin();
        if (beginline==0) return 0; // No indentation needed for the first line
        // Find the index of the leaf containing the begin of this line
        int m=leafs.size();
        int ileaf=document.getIndexLeafAt(line.begin());
        if (ileaf==-1 || ileaf==0) return 0; // No indentation needed because no leaf found or we are at the first leaf
        // Find the position of the opening hook before this element
        int ihook=findIndexOpeningHook(ileaf);
        if (ihook==ileaf)
            ihook=findIndexOpeningHook(ileaf-1);
        
        if (ihook<0) return 0; // No indentation needed beacuse no opening hook found encloising this line
        // Find the next non white space leaf after this opening hook
        int ileafafterhook;
        for (ileafafterhook=ihook+1;ileafafterhook<m && isWhiteSpace(leafs.leafAt(ileafafterhook).getSeparatorChar());ileafafterhook++);
        if (ileafafterhook>=ileaf)
            return charsBefore(ihook)+1; // It looks we are at the begin of the list
        leaf=leafs.leafAt(ileafafterhook);
        leafstring=leaf.getString(document).toUpperCase();
        if (leaf.getSeparatorChar()=='(')
            return charsBefore(ileafafterhook);
        else if ("DEFUN".compareTo(leafstring)==0 || "DEFMACRO".compareTo(leafstring)==0 || "DEFGENERIC".compareTo(leafstring)==0 || "DEFMETHOD".compareTo(leafstring)==0 || "DEFCLASS".compareTo(leafstring)==0)
            return charsBefore(ileafafterhook)+1;
        else if ("COND".compareTo(leafstring)==0)
            return charsBefore(ileafafterhook)+leafstring.length()+1;
        else if ("IF".compareTo(leafstring)==0
        || "DO".compareTo(leafstring)==0
        || "DO*".compareTo(leafstring)==0)
        {
            i=skipElement(ileafafterhook); // Skip the IF,DO or DO*
            if (i>=m) return charsBefore(ileafafterhook)+leafstring.length()+1;
            i=skipElement(i); // Skip the test
            if (i>=ileaf)
                return charsBefore(ileafafterhook)+leafstring.length()+1;
            else
                return charsBefore(ileafafterhook)+1;
        }
        else if ("WHEN".compareTo(leafstring)==0
        || "UNLESS".compareTo(leafstring)==0
        || "LET".compareTo(leafstring)==0
        || "LET*".compareTo(leafstring)==0)
        {
            i=skipElement(ileafafterhook); // Skip the keyword
            if (i>=m) return charsBefore(ileafafterhook)+leafstring.length()+1;
            if (i>=ileaf)
                return charsBefore(ileafafterhook)+leafstring.length()+1;
            else
                return charsBefore(ileafafterhook)+1;
        }
        else if ((i=skipElement(ileafafterhook)) < ileaf)
            return charsBefore(i);
        else
            return charsBefore(ihook)+1;
    }
    /** Skip the element */
    private int skipElement(int ileaf)
    {
        int nextileaf;
        int m=leafs.size();
        if (ileaf>=m)
            return m;
        char c=leafs.leafAt(ileaf).getSeparatorChar();
        if (c=='(')
            nextileaf=skipList(ileaf+1);
        else if (c=='#')
            nextileaf=skipHash(ileaf+1);
        else if (c=='"')
            nextileaf=skipString(ileaf+1);
        else if (c==';')
            nextileaf=skipComment(ileaf+1);
        else
            nextileaf=ileaf+1;
        // Skip the white spaces after this element
        for (;nextileaf<m && isWhiteSpace(leafs.leafAt(nextileaf).getSeparatorChar());nextileaf++);
        if (nextileaf<m && leafs.leafAt(nextileaf).getSeparatorChar()==';')
            return skipElement(nextileaf);
        return nextileaf;
    }
    /** Skip a list */
    private int skipList(int ileaf)
    {
        return findIndexClosingHook(ileaf)+1;
    }
    /** Skip a string */
    private int skipString(int ileaf)
    {
        int m=leafs.size();
        for (;ileaf<m && leafs.leafAt(ileaf).getSeparatorChar()!='"';ileaf++)
            if (leafs.leafAt(ileaf).getSeparatorChar()=='\\')
                ileaf++;
        return ileaf+1;
    }
    /** Skip comment and then the element after the comment */
    private int skipComment(int ileaf)
    {
        int m=leafs.size();
        for (;ileaf<m && leafs.leafAt(ileaf).getSeparatorChar()!='\n';ileaf++);
        for (;ileaf<m && isWhiteSpace(leafs.leafAt(ileaf).getSeparatorChar());ileaf++);
        return ileaf;
    }
    /** Skip elements starting with a hash */
    private int skipHash(int ileaf)
    {
        if (ileaf<leafs.size())
        {
            char c=leafs.leafAt(ileaf).getSeparatorChar();
            if (c=='\\')
            {
                return ileaf+2; // character definition #\rrrr
            }
            else if (c=='\'')
            {
                // function definition
                if (ileaf+1<leafs.size())
                    return skipElement(ileaf+1); // Function definition
                else
                    return ileaf+1;
            }
            else if (c=='(')
            {
                return findClosingHook(ileaf+1)+1;
            }
            else if (c=='|')
            {
                for (ileaf=ileaf+1;ileaf+1<leafs.size() && !(leafs.leafAt(ileaf).getSeparatorChar()=='|' && leafs.leafAt(ileaf+1).getSeparatorChar()=='#');ileaf++);
                if (ileaf+2<leafs.size())
                    return skipElement(ileaf);
                else
                    return ileaf+2;
            }
            else
            {
                return ileaf;
            }
        }
        else
        {
            return ileaf;
        }
    }
    /** Find the number of character between the begin of a line and the begin of a leaf  with index ileaf  (which is located on this line*/
    private int charsBefore(int ileaf)
    {
        CodeLeafElement leaf=leafs.leafAt(ileaf);
        CodeLine line=document.getLineAt(leaf.begin());
        if (line==null)
        {
            return 0;
        }
        else
        {
            return leaf.begin()-line.begin();
        }
    }
    /** Find the enclosing hooks around the pos */
    public Region findHooks(int pos)
    {
        if (document==null)
            return null;
        int ileaf=document.getIndexLeafAt(pos);
        int ibegin;
        int iend;
        if (ileaf==-1)
        {
            return null;
        }
        else
        {
            // Ensure parenthese matching at end of listr
            ibegin=findIndexOpeningHook(ileaf);
            if (ibegin==-1)
                return null;
            iend=findIndexClosingHook(ibegin);
            if (iend==-1)
                return null;
            return new Region(leafs.leafAt(ibegin).begin(),leafs.leafAt(iend).begin());
        }
    }
    
    /** Find the position of the opening hook */
    private int findOpeningHook(int ileaf)
    {
        int ihook=findIndexOpeningHook(ileaf);
        if (ihook<0)
            return -1;
        else
            return leafs.leafAt(ihook).begin();
    }
    /** Find the index of the opening hook, return -1 if we could not find the opening hook */
    private int findIndexOpeningHook(int ileaf)
    {
        CodeLeafElement leaf=leafs.leafAt(ileaf);
        CodeElement parent=leaf.getParent();
        if (parent!=null)
        {
            // We can use the parsed information to find begin hook
            for(;!(parent instanceof LispListElement || parent==null);parent=parent.getParent());
            if (parent==null)
                return -1;
            else
                return document.getIndexLeafAt(parent.begin());
        }
        else
        {
            // Not everything is yet parsed so we find the first leaf with no parent and search for a opening hook in the leaf after this
            int index;
            int ielement;
            int iclose;
            int iopen=-1;
            if (leafs.size()==0)
                return -1;
            for (index=ileaf-1;index>=0 && leafs.leafAt(index).getParent()==null;index--);
            index=Math.max(0,index);
            parent=leafs.leafAt(index).getTopParent();
            if (parent==null)
                index=0;
            else
                index=document.getIndexLeafAt(parent.begin());
            // Find now the openings hook of a list covering ileaf
            while (index<ileaf && index<leafs.size())
            {
                for (;index<leafs.size() && leafs.leafAt(index).getSeparatorChar()!='(';index=goNext(index));
                iclose=findIndexClosingHook(index);
                if (iclose==-1)
                    return iopen<=ileaf ? iopen : -1;
                else if (iclose>=ileaf)
                    if (index<leafs.size() && index<=ileaf)
                        iopen=index;
                index=goNext(index);
            };
            return iopen<=ileaf ? iopen : -1;
        }
    }
    
    /** Go to the next leaf taking in account strings, comments and character definitions */
    private int goNext(int ileaf)
    {
        ileaf++;
        if (ileaf<leafs.size())
        {
            switch (leafs.leafAt(ileaf).getSeparatorChar())
            {
                case '#' :
                    return goNextHash(ileaf);
                case ';':
                    return goNextComment(ileaf);
                case '"':
                    return goNextString(ileaf);
                default :
                    return ileaf;
            }
        }
        else
        {
            return leafs.size();
        }
    }
    /** Go to the leaf after the character definition or the multiline comment */
    private int goNextHash(int ileaf)
    {
        ileaf++; // Skip the #
        if (ileaf<leafs.size())
        {
            switch (leafs.leafAt(ileaf).getSeparatorChar())
            {
                case '|':
                    // Multiline comment find the closing }#
                    for (ileaf=ileaf+1;ileaf+1<leafs.size() && !(leafs.leafAt(ileaf).getSeparatorChar()=='|' && leafs.leafAt(ileaf).getSeparatorChar()=='#');ileaf++);
                    if (ileaf+1>=leafs.size())
                        return leafs.size(); // the rest of the source is comment
                    else
                        return ileaf+2;
                case '\\':
                    return ileaf+1;
                default:
                    return ileaf-1; // Read from #
            }
        }
        else
        {
            return leafs.size();
        }
    }
    /** Go to the leaf after the single line comment */
    private int goNextComment(int ileaf)
    {
        for (ileaf=ileaf+1;ileaf<leafs.size() && leafs.leafAt(ileaf).getSeparatorChar()!='\n';ileaf++);
        return ileaf;
    }
    /** Go to the leaf after the string */
    private int goNextString(int ileaf)
    {
        ileaf++;
        while (ileaf<leafs.size())
        {
            switch (leafs.leafAt(ileaf).getSeparatorChar())
            {
                case '\\':
                    ileaf=ileaf+2;
                    break;
                case '"':
                    return ileaf+1;
                default:
                    ileaf++;
                    break;
            };
        };
        return leafs.size();
    }
    
    /** Find the closing hook */
    private int findClosingHook(int ileaf)
    {
        int i=findIndexClosingHook(ileaf);
        if (i!=-1)
            return leafs.leafAt(i).begin();
        else
            return -1;
    }
    
    /** Find the index of the closing hook (ileaf points to the leaf containing the opening hook), return -1 if we could not find opening hook */
    private int findIndexClosingHook(int ileaf)
    {
        CodeLeafElement leaf=leafs.leafAt(ileaf);
        if (leaf==null)
            return -1;
        CodeElement parent=leaf.getParent();
        if (parent!=null)
        {
            // We can use the parsed information to find begin hook
            for(;!(parent instanceof LispListElement || parent==null);parent=parent.getParent());
            if (parent==null)
                return -1;
            else
                return document.getIndexLeafAt(parent.end()-1);
        }
        else
        {
            int nrhooks=1;
            ileaf=goNext(ileaf);
            while (ileaf<leafs.size())
            {
                switch (leafs.leafAt(ileaf).getSeparatorChar())
                {
                    case ')':
                        if (nrhooks==1)
                            return ileaf;
                        else
                            nrhooks--;
                        break;
                    case '(':
                        nrhooks++;
                        break;
                };
                ileaf=goNext(ileaf);
            };
            return -1;
        }
    }
    
    // Return true for a separator character */
    public final boolean isSeparator(char c)
    {
        return (c==' ' || c=='\n' || c=='\t' || c=='\r' || c=='(' || c==';' || c=='"' || c=='\\' || c=='#' || c==')' || c=='|' || c=='\'' || c=='`');
    }
    // Return true for a white space
    public boolean isWhiteSpace(char c)
    {
        return (c==' ' || c=='\n' || c=='\t' || c=='\r');
    }
    
    /** Parse a element and add this element to elements, returns the index of the leaf after the last leafs of this element  */
    public int parseElement(int ileaf, CodeElementVector elements)
    {
        switch (leafs.leafAt(ileaf).getSeparatorChar())
        {
            case '\'':
                ileaf=parseQuote(ileaf,elements);
                break;
            case '`':
                ileaf=parseBackQuote(ileaf,elements);
                break;
            case '#':
                ileaf=parseHash(ileaf,elements);
                break;
            case ' ':
                ileaf=parseWhite(ileaf,elements);
                break;
            case '\n':
                ileaf=parseWhite(ileaf,elements);
                break;
            case '\t':
                ileaf=parseWhite(ileaf,elements);
                break;
            case '\r':
                ileaf=parseWhite(ileaf,elements);
                break;
            case '(':
                ileaf=parseList(ileaf,elements);
                break;
            case ';':
                ileaf=parseComment(ileaf,elements);
                break;
            case '"':
                ileaf=parseString(ileaf,elements);
                break;
            default:
                ileaf=parseAtom(ileaf,elements);
                break;
        };
        return ileaf;
    }
    /** Parse the hash code */
    private int parseHash(int ileaf,CodeElementVector elements)
    {
        if (ileaf+1<leafs.size())
        {
            switch (leafs.leafAt(ileaf+1).getSeparatorChar())
            {
                case '|':
                    ileaf=parseHashComment(ileaf,elements);
                    break;
                case '\'':
                    ileaf=parseHashFunction(ileaf,elements);
                    break;
                case '\\':
                    ileaf=parseHashCharacter(ileaf,elements);
                    break;
                case '(':
                    ileaf=parseHashArray(ileaf,elements);
                    break;
                case '+':
                case '-':
                    // Case of #+ and #- elements
                    ileaf=parseHashOptional(ileaf,elements);
                    break;
                default:
                    ileaf=parseHashPossibleOptionalElement(ileaf,elements);
                    break;
            };
        }
        else
        {
            // Incomplete hash atom
            LispAtomElement atom=new LispAtomElement(leafs.leafAt(ileaf).getString(document),leafs.leafAt(ileaf));
            atom.markIncomplete();
            elements.addCodeElement(atom);
            ileaf++;
        };
        return ileaf;
    }
    /** Parse a possible hash optional element */
    private int parseHashPossibleOptionalElement(int ileaf,CodeElementVector elements)
    {
        String str=leafs.leafAt(ileaf+1).getString(document);
        if (str.length()>0 && (str.charAt(0)=='+' || str.charAt(0)=='-'))
        {
            // We have a optional element
            LispOptionalElement optional=new LispOptionalElement();
            CodeElementVector childs=optional.getChilds();
            optional.addChild(leafs.leafAt(ileaf));
            ileaf++;
            optional.addChild(leafs.leafAt(ileaf));
            optional.setTest(leafs.leafAt(ileaf)); // Warning this is not the best thing to do because the
            // test contains the + or the -
            ileaf++;
            if (ileaf<leafs.size())
            {
                ileaf=parseElement(ileaf,childs);
                optional.setCode(childs.codeElementAt(childs.size()-1));
            }
            else
            {
                optional.markIncomplete();
            };
            optional.setAdd(str.charAt(0)=='+');
            elements.addCodeElement(optional);
            return ileaf;
        }
        else
        {
            // This is no optional element
            LispAtomElement atom=new LispAtomElement(leafs.leafAt(ileaf).getString(document),leafs.leafAt(ileaf));
            atom.markIncomplete();
            elements.addCodeElement(atom);
            return ileaf+1;
        }
    }
    /** Parse a hash multiline comment (everything until #| */
    private int parseHashComment(int ileaf,CodeElementVector elements)
    {
        LispCommentElement comment=new LispCommentElement();
        comment.addChild(leafs.leafAt(ileaf)); // Add #
        ileaf++;
        comment.addChild(leafs.leafAt(ileaf)); // Add |
        ileaf++;
        int m=leafs.size();
        for (;ileaf+1<m && !(leafs.leafAt(ileaf).getSeparatorChar()=='|' && leafs.leafAt(ileaf+1).getSeparatorChar()=='#');ileaf++)
            comment.addChild(leafs.leafAt(ileaf));
        if (ileaf+1<m)
        {
            comment.addChild(leafs.leafAt(ileaf)); // Add |
            ileaf++;
            comment.addChild(leafs.leafAt(ileaf)); // Add #
            ileaf++;
        }
        else
        {
            comment.markIncomplete();
        };
        elements.addCodeElement(comment);
        return ileaf;
    }
    /** Parse a hash function which is of the form #'element */
    private int parseHashFunction(int ileaf,CodeElementVector elements)
    {
        LispFunctionElement function=new LispFunctionElement();
        CodeElementVector childs=function.getChilds();
        function.addChild(leafs.leafAt(ileaf)); // Add the #
        ileaf++;
        function.addChild(leafs.leafAt(ileaf)); // Add the '
        ileaf++;
        if (ileaf<leafs.size())
        {
            ileaf=parseElement(ileaf,childs);
            function.setFunction(childs.codeElementAt(childs.size()-1));
        }
        else
        {
            function.markIncomplete();
        };
        elements.addCodeElement(function);
        return ileaf;
    }
    /** Parse a #+ or a #- element */
    private int parseHashOptional(int ileaf,CodeElementVector elements)
    {
        LispOptionalElement optional=new LispOptionalElement();
        CodeElementVector childs=optional.getChilds();
        optional.addChild(leafs.leafAt(ileaf)); // Add the #
        ileaf++;
        optional.addChild(leafs.leafAt(ileaf)); // Add the + or the -
        if (leafs.leafAt(ileaf).getSeparatorChar()=='+')
            optional.setAdd(true);
        else if (leafs.leafAt(ileaf).getSeparatorChar()=='-')
            optional.setAdd(false);
        ileaf++;
        if (ileaf<leafs.size())
        {
            ileaf=parseElement(ileaf,childs);
            optional.setTest(childs.codeElementAt(childs.size()-1));
        }
        else
        {
            optional.markIncomplete();
        };
        ileaf++;
        if (ileaf<leafs.size())
        {
            ileaf=parseElement(ileaf,childs);
            optional.setCode(childs.codeElementAt(childs.size()-1));
        }
        else
        {
            optional.markIncomplete();
        };
        elements.addCodeElement(optional);
        return ileaf;
    }
    
    /** Parse a quoted element */
    public int parseQuote(int ileaf,CodeElementVector elements)
    {
        LispQuoteElement quote=new LispQuoteElement();
        CodeElementVector childs=quote.getChilds();
        quote.addChild(leafs.leafAt(ileaf)); // Add the '
        ileaf++;
        if (ileaf<leafs.size())
        {
            ileaf=parseElement(ileaf,childs);
            quote.setQuoted(childs.codeElementAt(childs.size()-1));
        }
        else
        {
            quote.markIncomplete();
        };
        elements.addCodeElement(quote);
        return ileaf;
    }
    /** Parse a back quoted element */
    public int parseBackQuote(int ileaf,CodeElementVector elements)
    {
        LispBackQuoteElement quote=new LispBackQuoteElement();
        CodeElementVector childs=quote.getChilds();
        quote.addChild(leafs.leafAt(ileaf)); // Add the '
        ileaf++;
        if (ileaf<leafs.size())
        {
            ileaf=parseElement(ileaf,childs);
            quote.setQuoted(childs.codeElementAt(childs.size()-1));
        }
        else
        {
            quote.markIncomplete();
        };
        elements.addCodeElement(quote);
        return ileaf;
    }
    /** Parse a hash character of the form #\atom */
    private int parseHashCharacter(int ileaf,CodeElementVector elements)
    {
        LispCharacterElement character=new LispCharacterElement();
        character.addChild(leafs.leafAt(ileaf)); // Add the # leaf element
        ileaf++;
        character.addChild(leafs.leafAt(ileaf)); // Add the \ leaf element
        ileaf++;
        if (ileaf<leafs.size())
        {
            character.addChild(leafs.leafAt(ileaf));
            ileaf++;
        }
        else
        {
            character.markIncomplete();
        };
        elements.addCodeElement(character);
        return ileaf;
    }
    /** Parse a hash array which is of the form  #(....) */
    private int parseHashArray(int ileaf,CodeElementVector elements)
    {
        LispArrayElement array=new LispArrayElement();
        CodeElementVector childs=array.getChilds();
        array.addChild(leafs.leafAt(ileaf)); // Add the # leaf element
        ileaf++;
        array.addChild(leafs.leafAt(ileaf)); // Add the ( leaf element
        ileaf++;
        int m=leafs.size();
        while (ileaf<m && leafs.leafAt(ileaf).getSeparatorChar()!=')')
            ileaf=parseElement(ileaf,childs);
        if (ileaf<m)
            childs.addCodeElement(leafs.leafAt(ileaf)); // Add the )
        else
            array.markIncomplete();
        elements.addCodeElement(array);
        return ileaf+1;
    }
    /** Parse a white space */
    private int parseWhite(int ileaf,CodeElementVector elements)
    {
        LispWhiteElement white=new LispWhiteElement();
        int m=leafs.size();
        for (;ileaf<m && isWhiteSpace(leafs.leafAt(ileaf).getSeparatorChar());ileaf++)
            white.addChild(leafs.leafAt(ileaf));
        elements.addCodeElement(white);
        return ileaf;
    }
    /** Parse a list */
    private int parseList(int ileaf,CodeElementVector elements)
    {
        int m=leafs.size();
        LispListElement list=new LispListElement();
        CodeElementVector childs=list.getChilds();
        //CodeElementVector listelements;
        String caller;
        list.addChild(leafs.leafAt(ileaf));
        ileaf++;
        while (ileaf<m && leafs.leafAt(ileaf).getSeparatorChar()!=')')
            ileaf=parseElement(ileaf,childs);
        // Add the non white space elements (excluding the begin ( to the elements of the list
        int childcount=childs.size();
        for (int i=1;i<childcount;i++)
            if (!(childs.codeElementAt(i) instanceof LispWhiteElement))
                list.addElement(childs.codeElementAt(i));
        // Add final ) and check if list is complete
        if (ileaf<m)
        {
            childs.addCodeElement(leafs.leafAt(ileaf));
        }
        else
        {
            list.markIncomplete();
        };
        elements.addCodeElement(list);
        return ileaf+1;
    }
    /** Parse a single line comment */
    private int parseComment(int ileaf,CodeElementVector elements)
    {
        LispCommentElement comment=new LispCommentElement();
        int m=leafs.size();
        for (;ileaf<m && !(leafs.leafAt(ileaf) instanceof CodeLeafNewLine);ileaf++)
            comment.addChild(leafs.leafAt(ileaf));
        if (ileaf<m)
            comment.addChild(leafs.leafAt(ileaf));
        elements.addCodeElement(comment);
        return ileaf+1;
    }
    /** Parse a string */
    private int parseString(int ileaf,CodeElementVector elements)
    {
        LispStringElement string=new LispStringElement();
        int m=leafs.size();
        CodeLeafElement leaf;
        string.addChild(leafs.leafAt(ileaf));
        ileaf++;
        for (;ileaf<m && (leaf=leafs.leafAt(ileaf)).getSeparatorChar()!='"';ileaf++)
        {
            string.addChild(leaf);
            if (leaf.getSeparatorChar()=='\\')
            {
                ileaf++;
                if (ileaf<m)
                    string.addChild(leafs.leafAt(ileaf));
            };
        };
        if (ileaf<m)
            string.addChild(leafs.leafAt(ileaf));
        else
            string.markIncomplete(); // Didn't found end of string
        elements.addCodeElement(string);
        return ileaf+1;
    }
    /** Parse a atom */
    private int parseAtom(int ileaf,CodeElementVector elements)
    {
        LispAtomElement atom=new LispAtomElement(leafs.leafAt(ileaf).getString(document),leafs.leafAt(ileaf));
        if (isSeparator(leafs.leafAt(ileaf).getSeparatorChar()))
            atom.markIncomplete(); // If we have the begin of a unexpected separator element mark it as incomplete
        elements.addCodeElement(atom);
        return ileaf+1;
    }
    
    /** Complete the insertion of a character, if character is completed return true else return false, if completed the parser must do the insert in the document and the position of the cursor  */
    public boolean completeCharacter(char c, int pos,int modifiers)
    {
        //
        // If character completion is on, ( gives () and ctrl-( gives ( (return false)
        // If character completion is off ( gives ( (return false) and ctrl-( gives ()
        // If character completion is on, " gives "" and ctrl-" gives " (return false)
        // If character completion is off " gives "" (return false) and ctrl-( gives ()
        //
        if (Configuration.characterCompletion)
        {
            if (KeyEvent.CTRL_MASK==(modifiers & KeyEvent.CTRL_MASK))
            {
                return false;
            }
            else if (c=='"')
            {
                document.insertStringAt("\"\"",pos);
                document.getCodeEditor().cursor.setCursor(pos+1);
                return true;
            }
            else if (c=='(')
            {
                document.insertStringAt("()",pos);
                document.getCodeEditor().cursor.setCursor(pos+1);
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            if (!(KeyEvent.CTRL_MASK==(modifiers & KeyEvent.CTRL_MASK)))
            {
                return false;
            }
            else if (c=='"')
            {
                document.insertStringAt("\"\"",pos);
                document.getCodeEditor().cursor.setCursor(pos+1);
                return true;
            }
            else if (c=='(')
            {
                document.insertStringAt("()",pos);
                document.getCodeEditor().cursor.setCursor(pos+1);
                return true;
            }
            else
            {
                return false;
            }
        }
    }
        /** Indentate a LISP form, return a string containing the new form of the LISP form  */
    public String indentateElement(CodeElement element)
    {
        IndentationBuffer buffer=new IndentationBuffer();
        if (element instanceof LispListElement)
        {
            indentateList((LispListElement) element,0,buffer);
            return buffer.toString();
        }
        else if (element instanceof LispCommentElement)
        {
            indentateComment((LispCommentElement) element,0,buffer);
            return buffer.toString();
        }
        else
        {
            return null;
        }
    }
    /** Indentates a atom or a list */
    private void indentateAtomOrList(CodeElement element,int offset,IndentationBuffer buffer)
    {
        if (element instanceof LispListElement)
            indentateList((LispListElement) element,offset,buffer);
        else if (element instanceof LispCommentElement)
            indentateComment((LispCommentElement) element,offset,buffer);
        else if (element instanceof LispBackQuoteElement)
            indentateBackQuote((LispBackQuoteElement) element,offset,buffer);
        else if (element instanceof LispQuoteElement)
            indentateQuote((LispQuoteElement) element,offset,buffer);
        else if (element instanceof LispFunctionElement)
            indentateFunction((LispFunctionElement) element,offset,buffer);
        else
        {
            buffer.append(element.getString(document));
        }
    }
    /** Tries to indentate a list element */
    private void indentateList(LispListElement list,int offset,IndentationBuffer buffer)
    {
        String first;
        CodeElementVector elements=list.getElementsAndComments();
        if (elements.size()==0)
        {
            buffer.append("()");
        }
        else
        {
            first=elements.codeElementAt(0).getString(document).toUpperCase();
            if ("DEFUN".compareTo(first)==0 ||
            "DEFMACRO".compareTo(first)==0 ||
            "DEFGENERIC".compareTo(first)==0 ||
            "DEFMETHOD".compareTo(first)==0 ||
            "DEFCLASS".compareTo(first)==0)
                indentateDefinition(elements,offset,buffer);
            else if ("IF".compareTo(first)==0)
                indentateIf(elements,offset,buffer);
            else if ("COND".compareTo(first)==0)
                indentateCond(elements,offset,buffer);
            else if ("DO".compareTo(first)==0 || "DO*".compareTo(first)==0)
                indentateDo(elements,offset,buffer);
            else if ("LET".compareTo(first)==0 || "LET*".compareTo(first)==0 || "FLET".compareTo(first)==0 || "LABELS".compareTo(first)==0)
                indentateLet(elements,offset,buffer);
            else if ("WHEN".compareTo(first)==0 || "UNLESS".compareTo(first)==0)
                indentateWhen(elements,offset,buffer);
            else if ("CASE".compareTo(first)==0 ||
            "CCASE".compareTo(first)==0 ||
            "ECASE".compareTo(first)==0 ||
            "ETYPECASE".compareTo(first)==0 ||
            "CTYPECASE".compareTo(first)==0)
                indentateCase(elements,offset,buffer);
            else
                indentateNormalList(elements,offset,buffer);
        };
    }
    /**
     * Indentate a definition (writing the indentated code to the buffer
     * We write a definition like
     * (defun name (arg-list)
     *     f1
     *     ....
     *     fn)
     */
    private void indentateDefinition(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        String text;
        String first;
        int offsetarglist;
        int m=list.size();
        int i;
        // Write (defun
        first="("+list.codeElementAt(0).getString(document).toLowerCase()+" ";
        buffer.append(first);
        offsetarglist=offset+first.length();
        // Write comments if they exist (it is a bad idea to have comments after a defun ....
        i=indentateComments(list,1,offset+first.length(),buffer);
        // Write the name if it exist
        if (i<m)
        {
            text=list.codeElementAt(i).getString(document)+" ";
            buffer.append(text);
            offsetarglist=offsetarglist+text.length();
        };
        // Write the comments if they exist (although it is no goode idea to have comments after a name)
        i=indentateComments(list,i+1,offsetarglist,buffer);
        // Write the argilist
        if (i<m)
            indentateVarDefinition(list.codeElementAt(i),offsetarglist,buffer);
        // Write the body of the list
        for (i=i+1;i<m;i++)
            i=indentateElementNextLine(list,i,offset+2,buffer);
        buffer.append(")");
    }
    /**
     * Indentate a if statement
     * this will be of the form :
     * (if condition
     *     first
     *  second)
     */
    private void indentateIf(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int i;
        boolean ok;
        int newoffset;
        
        buffer.append("(if ");
        // Write eventual comment
        i=indentateComments(list,1,offset+4,buffer);
        // First to try to put everything on line
        buffer.setSavePoint();
        ok=true;
        newoffset=offset+4;
        for (int j=i;j<m && ok;j++)
        {
            indentateAtomOrList(list.codeElementAt(j),newoffset,buffer);
            if (j<m-1)
                buffer.append(" ");
            newoffset=buffer.getMaxLineLengthAfterSavePoint();
            if (newoffset>Configuration.maxLineLength || (j<m-1 && buffer.getNumberLinesAfterSavePoint()>0))
                ok=false;
        };
        if (ok)
        {
            // We could put everything on one line
            buffer.commitSavePoint();
        }
        else
        {
            // We could not put everything on one line
            buffer.rollback();
            // The condition
            if (list.size()>1)
                indentateAtomOrList(list.codeElementAt(i),offset+4,buffer);
            // Write eventual comments
            i=indentateComments(list,i+1,offset+4,buffer);
            // The first branch
            if (i<m)
                i=indentateElementNextLine(list,i,offset+4,buffer);
            // The other brances (take in account ther could be syntax errors
            for (i=i+1;i<m;i++)
                i=indentateElementNextLine(list,i,offset+2,buffer);
        }
        buffer.append(")");
    }
    private void indentateCond(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int i;
        buffer.append("(cond ");
        // Eventual comment
        i=indentateComments(list,1,offset+6,buffer);
        // First condition
        if (i<m)
        {
            if (list.codeElementAt(i) instanceof LispListElement)
                indentateVarDefinition(list.codeElementAt(i),offset+6,buffer);
            else
                indentateAtomOrList(list.codeElementAt(i),offset+6,buffer);
        };
        for (i=i+1;i<m;i++)
        {
            if (list.codeElementAt(i) instanceof LispCommentElement)
                i=indentateComments(list,i,offset+6,buffer);
            if (i<m)
            {
                buffer.addNewLine();
                buffer.addSpaces(offset+6);
                indentateVarDefinition(list.codeElementAt(i),offset+6,buffer);
            };
        };
        buffer.append(")");
    }
    /* Indentate do or do* forms, the form used is
     * (do ((...)
     *       (...))
     *      ((...)
     *       ..)
     *   f1
     *   ..
     *   fn)
     */
    private void indentateDo(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int i;
        String first="("+list.codeElementAt(0).getString(document).toLowerCase()+" ";
        buffer.append(first);
        // Eventual comments
        i=indentateComments(list,1,offset+first.length(),buffer);
        // Var definition list
        if (i<m)
            indentateVarDefinition(list.codeElementAt(i),offset+first.length(),buffer);
        // Eventual comments
        i=indentateComments(list,i+1,offset+first.length(),buffer);
        // End conditions
        if (i<m)
            i=indentateElementNextLine(list,i,offset+first.length(),buffer);
        // Body
        for (i=i+1;i<m;i++)
            i=indentateElementNextLine(list,i,offset+2,buffer);
        buffer.append(")");
    }
    /*
     * Indentate a let or let* statement
     * in the form
     */
    private void indentateLet(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        String first="("+list.codeElementAt(0).getString(document).toLowerCase()+" ";
        int m=list.size();
        int i;
        
        buffer.append(first);
        // Eventual comments
        i=indentateComments(list,1,offset+first.length(),buffer);
        // Var list of let
        if (i<m)
        {
            indentateVarDefinition(list.codeElementAt(i),offset+first.length(),buffer);
        };
        // Body of the let or let* form
        for (i=i+1;i<m;i++)
            i=indentateElementNextLine(list,i,offset+2,buffer);
        buffer.append(")");
    }
    private void indentateWhen(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int i;
        int newoffset;
        buffer.append("(when ");
        // Eventual comments
        i=indentateComments(list,1,offset+6,buffer);
        // Try to put everything on the same line
        buffer.setSavePoint();
        indentateAtomOrList(list.codeElementAt(i),offset+6,buffer);
        if (i<m-1)
            buffer.append(" ");
        newoffset=buffer.getMaxLineLengthAfterSavePoint();
        if (newoffset>Configuration.maxLineLength || (m>2 && buffer.getNumberLinesAfterSavePoint()>0))
        {
            // We can not even get the condition on the same line
            buffer.rollback();
            for (int j=i;j<m;j++)
                j=indentateElementNextLine(list,j,offset+2,buffer);
        }
        else
        {
            buffer.commitSavePoint();
            // Now add the rest of the list
            boolean ok=true;
            buffer.setSavePoint();
            for (int j=i+1;j<m && ok;j++)
            {
                indentateAtomOrList(list.codeElementAt(j),newoffset,buffer);
                if (j<m-1)
                    buffer.append(" ");
                newoffset=buffer.getMaxLineLengthAfterSavePoint();
                if (newoffset>Configuration.maxLineLength || (j<m-1 && buffer.getNumberLinesAfterSavePoint()>0))
                    ok=false;
            };
            if (ok)
            {
                // We could put everything on one line (Commit all savepoints)
                buffer.commitSavePoint();
            }
            else
            {
                // We had a problem putting everyhting on one line
                buffer.rollback(); // rollback last savepoint
                // Put the remaining entries on a new line
                for (int j=i+1;j<m;j++)
                    j=indentateElementNextLine(list,j,offset+2,buffer);
            };
        };
        buffer.append(")");
    }
    
    /**
     * Indentate a case form, this will be in the form
     * (case (form)
     *   c1
     *   ...
     *   cn)
     */
    
    private void indentateCase(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        // Add the case header
        String first="("+list.codeElementAt(0).getString(document)+" ";
        buffer.append(first);
        // Add the guide form
        if (m>1)
            indentateAtomOrList(list.codeElementAt(1),offset+first.length(),buffer);
        // Add the remaining args
        for (int i=2;i<m;i++)
            i=indentateElementNextLine(list,i,offset+2,buffer);
        buffer.append(")");
    }
    
    /*
     * Try to indentate a list
     * First we try to put everything on the same line
     * If this fails we try to put the first two elements on the same line and the next elements beneath the second element
     * If this fails we put the first element on the same list and then the other elements beneath the first
     */
    
    private void indentateNormalList(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int newoffset;
        int firstoffset=0;
        boolean ok=true;
        
        if (m==0)
        {
            buffer.append("()");
            return;
        };
        buffer.append("(");
        // Put the first element on the same line
        indentateAtomOrList(list.codeElementAt(0),offset+1,buffer);
        if (m>1)
        {
            // If there are more elements in the list try to indentate them
            buffer.setSavePoint();
            firstoffset=buffer.getMaxLineLengthAfterSavePoint()+1;
            buffer.commitSavePoint();
            newoffset=firstoffset;
            // First try to put remaining elements on the same line
            buffer.setSavePoint();
            buffer.append(" ");
            for (int i=1;i<m && ok;i++)
            {
                indentateAtomOrList(list.codeElementAt(i),newoffset,buffer);
                if (i<m-1)
                    buffer.append(" ");
                newoffset=buffer.getMaxLineLengthAfterSavePoint();
                if (newoffset>Configuration.maxLineLength || (i<m-1 && buffer.getNumberLinesAfterSavePoint()>0))
                    ok=false;
            };
            if (ok)
            {
                // We could put everyhting on the same line
                buffer.commitSavePoint();
            }
            else
            {
                // We could not put everything on the same line
                buffer.rollback();
                // Tries to put the first two elements on the same line
                buffer.setSavePoint();
                buffer.append(" ");
                indentateAtomOrList(list.codeElementAt(1),firstoffset,buffer);
                if (buffer.getMaxLineLengthAfterSavePoint()>Configuration.maxLineLength)
                {
                    // Could not put two elements on same lines so put only one element on one line
                    buffer.rollback();
                    for (int i=1;i<m;i++)
                        i=indentateElementNextLine(list,i,offset+1,buffer);
                }
                else
                {
                    // Could put two elements on same line
                    // If second element is a comment use offset +1 as first offset
                    if (list.codeElementAt(1) instanceof LispCommentElement)
                        firstoffset=offset+1;
                    buffer.commitSavePoint();
                    for (int i=2;i<m;i++)
                        i=indentateElementNextLine(list,i,firstoffset,buffer);
                };
            };
        };
        buffer.append(")");
    }
    
    private void indentateVarDefinition(CodeElement definition,int offset,IndentationBuffer buffer)
    {
        if (definition instanceof LispListElement)
            indentateVarList(((LispListElement) definition).getElementsAndComments(),offset,buffer);
        else
            indentateAtomOrList(definition,offset,buffer);
    }
    
   /*
    * Try to indentate a list
    * First we try to put everything on the same line
    * If this fails we put the first element on the same line and then the other elements beneath the first
    */
    
    private void indentateVarList(CodeElementVector list,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int newoffset;
        int firstoffset=0;
        boolean ok=true;
        
        if (m==0)
        {
            buffer.append("()");
            return;
        };
        buffer.append("(");
        // Put the first element on the same line
        indentateAtomOrList(list.codeElementAt(0),offset+1,buffer);
        if (m>1)
        {
            // If there are more elements in the list try to indentate them
            buffer.append(" ");
            buffer.setSavePoint();
            firstoffset=buffer.getMaxLineLengthAfterSavePoint();
            buffer.commitSavePoint();
            newoffset=firstoffset;
            // First try to put remaining elements on the same line
            buffer.setSavePoint();
            for (int i=1;i<m && ok;i++)
            {
                indentateAtomOrList(list.codeElementAt(i),newoffset,buffer);
                if (i<m-1)
                    buffer.append(" ");
                newoffset=buffer.getMaxLineLengthAfterSavePoint();
                if (newoffset>Configuration.maxLineLength || (i<m-1 && buffer.getNumberLinesAfterSavePoint()>0))
                    ok=false;
            };
            if (ok)
            {
                // We could put everyhting on the same line
                buffer.commitSavePoint();
            }
            else
            {
                // Put only one element on one line
                buffer.rollback();
                for (int i=1;i<m;i++)
                    i=indentateElementNextLine(list,i,offset+1,buffer);
            };
        };
        buffer.append(")");
    }
    
    /**
     * Add element at position i in the list to the buffer at offset in the following way:
     * 1. If element is comment use indentateComments first
     * 2. In other cases, add new line an position element at pos
     */
    private int indentateElementNextLine(CodeElementVector list,int index,int offset,IndentationBuffer buffer)
    {
        CodeElement element=list.codeElementAt(index);
        if (element instanceof LispCommentElement)
        {
            index=indentateComments(list,index,offset,buffer);
        };
        if (index<list.size())
        {
            buffer.addNewLine();
            buffer.addSpaces(offset);
            indentateAtomOrList(list.codeElementAt(index),offset,buffer);
        };
        return index;
    }
    /** Indentate a backquote */
    private void indentateBackQuote(LispBackQuoteElement quote,int offset,IndentationBuffer buffer)
    {
        buffer.append("`");
        indentateAtomOrList(quote.getQuoted(),offset+1,buffer);
    }
    /** Indentate a quote */
    private void indentateQuote(LispQuoteElement quote,int offset,IndentationBuffer buffer)
    {
        buffer.append("'");
        indentateAtomOrList(quote.getQuoted(),offset+1,buffer);
    }
    /** Indentate a backquote */
    private void indentateFunction(LispFunctionElement function,int offset,IndentationBuffer buffer)
    {
        buffer.append("#'");
        indentateAtomOrList(function.getFunction(),offset+1,buffer);
    }
    
    /** Indentate a column in the following way
     * A multiline comment always start on a new line is ended by a new line and starts at posiition 0
     * A comment starting with ;;; starts on a new line and is ended by a new line and starts at positioin 0
     * A comment starting with ;; starts at offset or on a new line at offset if the line is longer then offset
     * A comment starting with ; starts at end of line
     */
    
    private int indentateComment(LispCommentElement comment,int offset,IndentationBuffer buffer)
    {
        String commentstring=comment.getString(document);
        int length=buffer.getCurrentLineLength();
        // Indentate comment
        if (commentstring.startsWith("#|") || commentstring.startsWith(";;;"))
        {
            buffer.addNewLine();
            buffer.append(commentstring);
            buffer.addNewLine();
            return 0;
        }
        else if (commentstring.startsWith(";;"))
        {
            if (length>offset)
            {
                buffer.addNewLine();
                buffer.addSpaces(offset);
                buffer.append(commentstring);
                buffer.addNewLine();
                return offset;
            }
            else
            {
                buffer.addSpaces(offset-length);
                buffer.append(commentstring);
                buffer.addNewLine();
                return offset;
            }
        }
        else
        {
            if (length==0)
            {
                buffer.addSpaces(offset);
                buffer.append(commentstring);
                buffer.addNewLine();
                return offset;
            }
            else
            {
                buffer.append(" ");
                buffer.append(commentstring);
                buffer.addNewLine();
                return length+1;
            }
        }
    }
    /*
     * Find the element at index index-1 and index in list, and indentate and add all the comments between these elements in
     * childs
     */
    private int indentateComments(CodeElementVector list,int index,int offset,IndentationBuffer buffer)
    {
        int m=list.size();
        int newoffset=offset;
        for (;index<m && list.codeElementAt(index) instanceof LispCommentElement;index++)
            newoffset=indentateComment((LispCommentElement) list.codeElementAt(index),newoffset,buffer);
        return index;
    }
}
