/*
 * ASetColorAction.java
 *
 * Created on December 25, 2001, 10:34 AM
 */

package IDE.Configuration;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import IDE.*;
/**
 * Abstract action to set a font
 * @author  marc
 */
public abstract class AbstractFontAction extends AbstractIDEAction
{
    /** List containing the fonts */
    JList list;
    /** Combobox containing the styles */
    JComboBox styles;
    /** TextField containing the size */
    JComboBox size;
    /** TextArea containing the example text */
    JTextArea text;
    /** Font to choose */
    Font font,newfont;
    /** Name of this action */
    private String name;
    /** Creates a new instance of AbstractIntegerAction */
    public AbstractFontAction(String name)
    {
        super(name);
        this.name=name;
    }
    /** Define the action to do */
    public void doWork()
    {
        Font font=chooseFont(getFontValue());
        if (font!=null)
        {
            setFontValue(font);
            Main.repaintEverything();
        };
    }
    /** Get the font value */
    protected abstract Font getFontValue();
    /** Set the font value */
    protected abstract void setFontValue(Font font);
    /** Choose a font out of the available fonts */
    private Font chooseFont(Font defaultfont)
    {
        final JDialog dialog=new JDialog(); // Create a dialog box
        int index=0;
        String[] fontfamilies=GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames(); // Get a list of all the fontfamilies
        Vector allfonts=new Vector();
        for (int i=0;i<fontfamilies.length;i++)
        {
            if (fontfamilies[i].length()!=0)
            {
                allfonts.add(new Font(fontfamilies[i],Font.PLAIN,12));
                if (fontfamilies[i].compareTo(defaultfont.getFamily())==0)
                    index=i;
            };
        };
        font=defaultfont;
        text=new JTextArea("abcdefghijklmnopqrstuvwxyz\nABCDEFGHIJKLMNOPQRSTUVWXYZ\n0123456789");
        
        dialog.setTitle(name);
        dialog.setModal(true);
        dialog.setSize(500,700);
        // Create list
        list=new JList(allfonts);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setCellRenderer(new AbstractFontAction.fontListCellRenderer());
        list.setSelectedIndex(index);
        // Create buttons
        JButton okbutton=new JButton(new AbstractAction("Ok")
        {
            public void actionPerformed(ActionEvent ev)
            {
                dialog.setVisible(false);
                dialog.dispose();
                newfont=font;
            };
        });
        JButton cancelbutton=new JButton(new AbstractAction("Cancel")
        {
            public void actionPerformed(ActionEvent ev)
            {
                dialog.setVisible(false);
                dialog.dispose();
                newfont=null;
            };
        });
        JPanel buttons=new JPanel();
        buttons.add(okbutton);
        buttons.add(cancelbutton);
        // Create the pane to display the example text and the style and size button
        styles=new JComboBox(new String[]
        {"Plain","Bold","Italic","Bold and Italic"});
        switch (font.getStyle())
        {
            case Font.PLAIN:
                styles.setSelectedIndex(0);
                break;
            case Font.BOLD:
                styles.setSelectedIndex(1);
                break;
            case Font.ITALIC:
                styles.setSelectedIndex(2);
                break;
            case Font.BOLD|Font.ITALIC:
                styles.setSelectedIndex(3);
                break;
            default:
                styles.setSelectedIndex(1);
                break;
        };
        String[] sizes=new String[95];
        for (int i=0;i<sizes.length;i++)
            sizes[i]=String.valueOf(i+5);
        size=new JComboBox(sizes);
        size.setSelectedIndex(Math.min(99,font.getSize()-5));
        JPanel stylesize=new JPanel(new FlowLayout(FlowLayout.LEFT));
        stylesize.add(new JLabel("Style: "));
        stylesize.add(styles);
        stylesize.add(new JLabel("Size:"));
        stylesize.add(size);
        JPanel stylesizetext=new JPanel(new BorderLayout());
        stylesizetext.add(stylesize,BorderLayout.CENTER);
        JScrollPane textscroll=new JScrollPane(text);
        stylesizetext.add(textscroll,BorderLayout.SOUTH);
        // Create the central panel
        JPanel central=new JPanel(new BorderLayout());
        JScrollPane scrollpane=new JScrollPane(list);
        central.add(scrollpane,BorderLayout.CENTER);
        central.add(stylesizetext,BorderLayout.SOUTH);
        // Add the listeners to the list,size and style components
        size.addActionListener(new styleSizeAction());
        styles.addActionListener(new styleSizeAction());
        list.addListSelectionListener(new selectionListener());
        setFontExample();
        // Now layout the dialog box
        JPanel content=new JPanel(new BorderLayout());
        content.add(central,BorderLayout.CENTER);
        content.add(buttons,BorderLayout.SOUTH);
        list.ensureIndexIsVisible(index);
        dialog.getContentPane().add(content);
        dialog.setVisible(true);
        
        return newfont;
    }
    /** Set the font in the example text */
    private void setFontExample()
    {
        
        int intsize;
        String family;
        intsize=size.getSelectedIndex()+5;
        family=((Font) list.getSelectedValue()).getFamily();
        switch (styles.getSelectedIndex())
        {
            case 0:
                font=new Font(family,Font.PLAIN,intsize);
                break;
            case 1:
                font=new Font(family,Font.BOLD,intsize);
                break;
            case 2:
                font=new Font(family,Font.ITALIC,intsize);
                break;
            case 3:
                font=new Font(family,Font.BOLD|Font.ITALIC,intsize);
                break;
        };
        text.setFont(font);
        text.revalidate();
        text.repaint();
    }
    // Private class to draw a element of the list */
    private class fontListCellRenderer extends JLabel implements ListCellRenderer
    {
        public fontListCellRenderer()
        {
            setOpaque(true);
            setFont(new Font("Courier",Font.PLAIN,12));
        }
        public Component getListCellRendererComponent(JList list, Object value, int i, boolean isSelected, boolean hasFocus)
        {
            Font font=(Font) value;
            // Set fore and background color
            setForeground(Color.black);
            setBackground((isSelected) ? new Color(153,153,204) : Color.white);
            // Set the text
            setText(font.getName());
            return this;
        }
    }
    // Private class to listen to changes in the styles
    private class styleSizeAction extends AbstractAction
    {
        public void actionPerformed(ActionEvent actionEvent)
        {
            setFontExample();
        }
    }
    // Private class to listen to selection changed events
    private class selectionListener implements ListSelectionListener
    {
        public void valueChanged(ListSelectionEvent listSelectionEvent)
        {
            setFontExample();
        }
    }
}
