/*
 * IDEAction.java
 *
 * Created on December 26, 2001, 11:24 AM
 */

package IDE.Configuration;

import java.awt.event.*;
import javax.swing.*;
import IDE.*;
import IDE.Messaging.*;
import IDE.Recording.*;

/**
 * Also registers the actions executed when recording via action performed this is done via actionPerformed
 * which register the action during macro recording and calls then do, to do the real action
 * @author  marc
 */
public abstract class AbstractIDEAction extends AbstractManagedAction implements IRecordable
{    
    /** INdicates if this action must give the focus back to the current editor buffer */
    protected boolean releaseFocus=true;
    /** Constructor of the action */
    public AbstractIDEAction(String name)
    {
        super(name);
    }
    public AbstractIDEAction(String name, ImageIcon icon)
    {
        super(name,icon);
    }
    /** Action Performed, if recording is on record the execution of the action and then calls doWork */
    public void actionPerformed(ActionEvent event)
    {
        Recording.recordActionEvent(this); // Tell the recorder what the action and the event is
        // Clear any previous error
        MessageHandler.clear();
        // Do the real work
        doWork(); 
        if (Main.currentEditor()!=null && releaseFocus)
            Main.currentEditor().requestFocus();
    }
    /** Restore the stqte of the qction during replaying can be overloaded if a action wants to restore its state */
    public boolean restoreState()
    {
        return true;
    }
    /** Must be implemented by every class which extends the AbstractIDEAction */
    public abstract void doWork();
}
