/*
 * SaveConfiguration.java
 *
 * Created on September 22, 2001, 10:19 AM
 */

package IDE.Configuration;


import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import javax.swing.*;
import IDE.Messaging.*;
import IDE.*;
import IDE.Utilities.*;
import IDE.LispScripting.*;
import IDE.CodeEditor.*;
import IDE.Help.*;
import IDE.Interaction.*;

/**
 * Class to save and load the configuration settings from the IDE
 * @author  marc
 * @version
 */
public class Configuration
{
    /** Contains all the configuration settings of the LispIDE, because of speed we don't use a hashtable */
    /** Name of the IDE */
    public static String IDEName="Jabberwocky";
    /** Version of the IDE */
    public static String IDEVersion="2.0.10";
    /** Name of the browser used for help */
    public static String helpbrowser="konqueror";
    /* installation directory of the LispIDE */
    public static  String installDirectory=InstallParameters.getInstallDir();
    /** Color for selection */
    public static Color colorSelection=new Color(198,198,198);
    /** Color for list */
    public static Color colorList=new Color(240,240,255);
    /** Color for comment */
    public static Color colorComment=new Color(102,102,255);
    /** Color for string */
    public static Color colorString=new Color(255,102,204);
    /** Color for keyword */
    public static Color colorKeyword=new Color(153,50,0);
    /** Color for variable */
    public static Color colorVariable=new Color(0,153,51);
    /** Color for caller */
    public static Color colorCaller=new Color(153,0,153);
    /** Color for character token */
    public static Color colorCharacter=new Color(153,0,153);
    /** Color for Cursor */
    public static Color colorCursor=Color.red;
    /** Color for Foreground of document */
    public static Color colorForeground=Color.black;
    /** Color for Background of document */
    public static Color colorBackground=new Color(255,255,210);
    /** Color for End of line indicator */
    public static Color colorEndline=Color.red;
    /** Color of a array */
    public static Color colorArray=new Color(255,0,0);
    /** Foreground color of the line numbering */
    public static Color colorForegroundLine=Color.black;
    /** Background color of the line numbering */
    public static Color colorBackgroundLine=new Color(230,230,230);
    /** Background color of breakpoint */
    public static Color colorBreakpoint=new Color(255,204,204);
    /** Background color of the executed code */
    public static Color colorExecutedCode=new Color(255,175,175);
    /** Background color of code just executed */
    public static Color colorExecutingCode=new Color(153,153,255);
    /** Foreground color backquote */
    public static Color colorBackQuote=new Color(0,0,255);
    /** Foreground color of quote */
    public static Color colorQuote=new Color(255,153,0);
    /** Background color of the next stop */
    public static Color colorNextStop=Color.yellow;
    /** Color of found strings during search */
    public static Color colorSearch=Color.yellow;
    /** Color used for profiling */
    public static Color colorProfiling=Color.yellow;
    /** Color used as background for the hint windows */
    public static Color colorHint=Color.yellow;
    /** Color of highlighting of hooks */
    public static Color colorHook=new Color(120,180,255);
    /** Background color for selected items in the explorer */
    public static Color colorSelected=new Color(160,180,230);
    /** Indicates if we have linenumbering or not */
    public static boolean lineNumbering=true;
    /** Default font used for the editor buffers */
    public static Font font=new Font("Lucida Sans Typewriter",Font.PLAIN,12);
    /** Font for the comments */
    public static Font commentFont=new Font("Lucida Sans Typewriter",Font.PLAIN,12);
    /** Font for the watch window */
    public static Font watchesFont=new Font("Dialog",Font.BOLD,10);
    /** Font for the macros window */
    public static Font macrosFont=new Font("Dialog",Font.BOLD,10);
    /** Font for the explorer windows */
    public static Font explorerFont=new Font("Dialog",Font.BOLD,10);
    /** tabwidth , used to convert tabs in spaces */
    public static int tabwidth=8;
    /** Indentation to use */
    public static boolean indentateOnEnter=true;
    /** Indicates if clicking on a project will set the current directory to this form the project */
    public static boolean clickProjectSetCurrentDirectory=true;
    /** Solution of a CLISP bug */
    public static boolean clispBug=true;
    /**  Blinking rate of cursor */
    public static int blinkingRate=500;
    /** Timeout for displaying things when hovering over a piece in the editor */
    public static int hoverTimeout=50;
    /** Maximum timeout to wait for the stop of a LISP process when Jabberwocky is finished */
    public static int timeoutStopProcess=10000;
    /** Max length of the lines for indentation */
    public static int maxLineLength=80;
    /** Indicates if we must try to complete special characters (example ( becomes () */
    public static boolean characterCompletion=true;
    /** Indicate if the enclosing list must be highlighted */
    public static boolean highligthList=true;
    /** Indicate if the step after code must be instrumented */
    public static boolean instrumentDebugAfter=true;
    /** Indicate if the error-checking code must be instrumented */
    public static boolean instrumentCheckError=true;
    /**  Command history length */
    public static int commandHistoryLength=100;
    /** Lisp startup script */
    public static String lispStartupFile="";
    /** The look and feel of the interface */
    public static String lookAndFeel="";
    /** Hashtable to store filename to icon mappings (to avoid taking up to much memory for all the icons) */
    public static HashMap nameToIcon=new HashMap();
    /** Creates the configuration Configuration */
    public Configuration()
    {
        /** Define the actions to change the options */
        createOptionActions();
        /** Create the root mode , the mother of all other modes */
        createRootMode();
        /** Create the basic mode */
        createBasicMode();
        /** Create the debug mode */
        createDebugMode();
        /** Create the interaction mode */
        createInteractionMode();
        /** Create the debug interaction mode */
        createDebugInteractionMode();
        /** Load the saved configuration */
        loadConfiguration();
    }
    /** Create the actions to change the options */
    private void createOptionActions()
    {
        Mode.addAction("defaultfont","Default Font","Set the default font used by the buffers.",new AbstractFontAction("Default Font")
        {
            protected Font getFontValue()
            {return Configuration.font;}
            protected void setFontValue(Font font)
            {Configuration.font=font;}
        });
        Mode.addAction("commentfont","Font Comment","Set the font used for comment elements.",new AbstractFontAction("Font Comment")
        {
            protected Font getFontValue()
            {return Configuration.commentFont;}
            protected void setFontValue(Font font)
            {Configuration.commentFont=font;}
        });
        Mode.addAction("explorerfont","Font Project Explorer","Set the font used for the Project Explorer",new AbstractFontAction("Font Explorer")
        {
            protected Font getFontValue()
            {return Configuration.explorerFont;}
            protected void setFontValue(Font font)
            {Configuration.explorerFont=font;}
        });
        Mode.addAction("watchesfont","Font Watches","Set the font used for the Watches pane",new AbstractFontAction("Font Watches")
        {
            protected Font getFontValue()
            {return Configuration.watchesFont;}
            protected void setFontValue(Font font)
            {Configuration.watchesFont=font;}
        });
        Mode.addAction("macrosfont","Font Macros","Set the font used for the Macro pane",new AbstractFontAction("Font Macros")
        {
            protected Font getFontValue()
            {return Configuration.macrosFont;}
            protected void setFontValue(Font font)
            {Configuration.macrosFont=font;}
        });
        Mode.addAction("linenumbers","Line Numbers","Turns on/off the display of line numbers in the buffers.",
        new AbstractBooleanAction("Line Numbers")
        {
            protected void setBoolValue(boolean value)
            {
                lineNumbering=value;
                Main.repaintEverything();
            }
            protected boolean getBoolValue()
            {
                return lineNumbering;
            }
        });
        Mode.addAction("indentateonenter","Indentate on enter","Turns on/of the automatically indentation when pressing enter.",
        new AbstractBooleanAction("Indentate on enter")
        {
            protected void setBoolValue(boolean value)
            {indentateOnEnter=value;}
            protected boolean getBoolValue()
            {return indentateOnEnter;}
        });
        Mode.addAction("clispbug","Workaround CLISP bug","Work around for a bug in CLISP and sending a single return",
        new AbstractBooleanAction("Workaround CLISP bug")
        {
            protected void setBoolValue(boolean value)
            {clispBug=value;}
            protected boolean getBoolValue()
            {return clispBug;}
        });
        Mode.addAction("charactercompletion","Character completion","Turns on/off character completion. With character completion typing a ( you get automatically the ) and typing \" gives the closing \".",
        new AbstractBooleanAction("Character completion")
        {
            protected void setBoolValue(boolean value)
            {characterCompletion=value;}
            protected boolean getBoolValue()
            {return characterCompletion;}
        });
        Mode.addAction("clickProjectSetCurrentDirectory","Automatic setting of current directory to project directory",
        "Turns on/off the setting of the current directory to the project directory when clicking on a project.",
        new AbstractBooleanAction("Automatic setting current directory to project directory")
        {
            protected void setBoolValue(boolean value)
            {clickProjectSetCurrentDirectory=value;}
            protected boolean getBoolValue()
            {return clickProjectSetCurrentDirectory;}
        });
        Mode.addAction("highlightlist","Highlight List","Turns on/off the automatically highlighting of the element covering the cursor positions.",
        new AbstractBooleanAction("Highlight List")
        {
            protected void setBoolValue(boolean value)
            {
                highligthList=value;
                CodeEditor editor=Main.currentEditor();
                if (editor!=null)
                    editor.cursor.stopHighlightingList();
            }
            protected boolean getBoolValue()
            {return highligthList;}
        });
        Mode.addAction("instrumentdebugafter","Instrument the debug after code","Turns on/off the generating of extra debugging code, to allow stopping after evaluating a form",
        new AbstractBooleanAction("Instrument the debug after code")
        {
            protected void setBoolValue(boolean value)
            {
                instrumentDebugAfter=value;
                if (!instrumentDebugAfter && !instrumentCheckError)
                {
                    Mode.getAction("stopafter").setEnabled(false);
                    Mode.getAction("setnewreturnvalue").setEnabled(false);
                }
                else
                {
                    Mode.getAction("stopafter").setEnabled(true);
                    Mode.getAction("setnewreturnvalue").setEnabled(true);
                };
            }
            protected boolean getBoolValue()
            {return instrumentDebugAfter;}
        });
        Mode.addAction("instrumentcheckerror","Instrument the check after code","Turns on/off the generating of extra debugging code, that causes entering the debugger when a error occurs during execution.",
        new AbstractBooleanAction("Instrument the check after code")
        {
            protected void setBoolValue(boolean value)
            {
                instrumentCheckError=value;
                if (!instrumentDebugAfter && !instrumentCheckError)
                {
                    Mode.getAction("stopafter").setEnabled(false);
                    Mode.getAction("setnewreturnvalue").setEnabled(false);
                }
                else
                {
                    Mode.getAction("stopafter").setEnabled(true);
                    Mode.getAction("setnewreturnvalue").setEnabled(true);
                };
            }
            protected boolean getBoolValue()
            {return instrumentCheckError;}
        });
        Mode.addAction("tabwidth","Width of tab for load","Sets the number of spaces to be used as a replacement for tab positions.",
        new AbstractIntegerAction("Width of tab for load",0,100)
        {
            protected int getInt()
            {return tabwidth;}
            protected void setInt(int i)
            {tabwidth=i;}
        });
        Mode.addAction("blinkingrate","Blinking rate cursor","Sets the blinking interval of the cursor.",new AbstractIntegerAction("Blinking Rate Cursor",100,1000)
        {
            protected int getInt()
            {return blinkingRate;}
            protected void setInt(int i)
            {blinkingRate=i;}
        });
        Mode.addAction("maxlinelength","Max Length Line","Set the max line length used by the indentation engine.",new AbstractIntegerAction("Max Length Line",50,500)
        {
            protected int getInt()
            {return maxLineLength;}
            protected void setInt(int i)
            {maxLineLength=i;}
        });
        Mode.addAction("hovertimeout","Wait time before displaying value","Set the wait time before displaying the value of the variable containing the cursor (usefull during debugging).",
        new AbstractIntegerAction("Wait time before displaying value",0,100)
        {
            protected int getInt()
            {return hoverTimeout;}
            protected void setInt(int i)
            {hoverTimeout=i;}
        });
        Mode.addAction("timeoutstopprocess","Timeout for stopping a interaction processs","Set's the time Jabberwocky waits for a interaction process to stop when Jabberwocky stops.",
        new AbstractIntegerAction("Wait time before displaying value",0,100)
        {
            protected int getInt()
            {return timeoutStopProcess;}
            protected void setInt(int i)
            {timeoutStopProcess=i;}
        });
        Mode.addAction("commandhistorylength","Length of the command history","Set the lengh of the command history",new AbstractIntegerAction("Length of the command history",10,10000)
        {
            protected int getInt()
            {return commandHistoryLength;}
            protected void setInt(int i)
            {commandHistoryLength=i;}
        });
        Mode.addAction("colorarray","Color Array","Set the foreground color of arrays.",new AbstractColorAction("Color Array")
        {
            protected Color getColor()
            {return colorArray;}
            protected void setColor(Color color)
            {colorArray=color;}
        });
        Mode.addAction("colorbackground","Color Background","Set the background color of a buffer.",new AbstractColorAction("Color Background")
        {
            protected Color getColor()
            {return colorBackground;}
            protected void setColor(Color color)
            {colorBackground=color;}
        });
        Mode.addAction("colorbackgroundline","Color Background Line","Set the background color of the linenumbers.",new AbstractColorAction("Color Background Line")
        {
            protected Color getColor()
            {return colorBackgroundLine;}
            protected void setColor(Color color)
            {colorBackgroundLine=color;}
        });
        Mode.addAction("colorbreakpoint","Color Breakpoint","Set the background color of breakpoints.",new AbstractColorAction("Color Breakpoint")
        {
            protected Color getColor()
            {return colorBreakpoint;}
            protected void setColor(Color color)
            {colorBreakpoint=color;}
        });
        Mode.addAction("colorcaller","Color Caller","Set the foreground color of the calling function/macro of a lispform.",new AbstractColorAction("Color Caller")
        {
            protected Color getColor()
            {return colorCaller;}
            protected void setColor(Color color)
            {colorCaller=color;}
        });
        Mode.addAction("colorcharacter","Color Character","Set the foreground color of a LISP character literal.",new AbstractColorAction("Color Character")
        {
            protected Color getColor()
            {return colorCharacter;}
            protected void setColor(Color color)
            {colorCharacter=color;}
        });
        Mode.addAction("colorcomment","Color Comment","Set the foreground color of the comments.",new AbstractColorAction("Color Comment")
        {
            protected Color getColor()
            {return colorComment;}
            protected void setColor(Color color)
            {colorComment=color;}
        });
        Mode.addAction("colorcursor","Color Cursor","Set the foreground color of the cursor.",new AbstractColorAction("Color Cursor")
        {
            protected Color getColor()
            {return colorCursor;}
            protected void setColor(Color color)
            {colorCursor=color;}
        });
        Mode.addAction("colorendline","Color Endline","Set the foreground color of the point representing a end of line.",new AbstractColorAction("Color Endline")
        {
            protected Color getColor()
            {return colorEndline;}
            protected void setColor(Color color)
            {colorEndline=color;}
        });
        Mode.addAction("colorexecutedcode","Color Executed Code","During debugging, background color of the code executed before entering the debugger.",
        new AbstractColorAction("Color Executed Code")
        {
            protected Color getColor()
            {return colorExecutedCode;}
            protected void setColor(Color color)
            {colorExecutedCode=color;}
        });
        Mode.addAction("colorexecutingcode","Color Executing Code","During debugging, background color of the code to be executed when leaving the debugger.",
        new AbstractColorAction("Color Executing Code")
        {
            protected Color getColor()
            {return colorExecutingCode;}
            protected void setColor(Color color)
            {colorExecutingCode=color;}
        });
        Mode.addAction("colorforeground","Color Foreground","Foreground color of the buffers.",new AbstractColorAction("Color Foreground")
        {
            protected Color getColor()
            {return colorForeground;}
            protected void setColor(Color color)
            {colorForeground=color;}
        });
        Mode.addAction("colorforegroundline","Color ForegroundLine","Foreground color of the linenumbers.",
        new AbstractColorAction("Color ForegroundLine")
        {
            protected Color getColor()
            {return colorForegroundLine;}
            protected void setColor(Color color)
            {colorForegroundLine=color;}
        });
        Mode.addAction("colorhook","Color Hook","Background color of the hooks (used in parenthese matching).",
        new AbstractColorAction("Color Hook")
        {
            protected Color getColor()
            {return colorHook;}
            protected void setColor(Color color)
            {colorHook=color;}
        });
        Mode.addAction("colorkeyword","Color Keyword","Foreground color keywords.",new AbstractColorAction("Color Keyword")
        {
            protected Color getColor()
            {return colorKeyword;}
            protected void setColor(Color color)
            {colorKeyword=color;}
        });
        Mode.addAction("colorlist","Color List","Background color of the list containing the cursor (if list highlighting is enabled).",
        new AbstractColorAction("Color List")
        {
            protected Color getColor()
            {return colorList;}
            protected void setColor(Color color)
            {colorList=color;}
        });
        Mode.addAction("colornextstop","Color Next Stop","Background color of the indicated next stop during  debugging.",
        new AbstractColorAction("Color Next Stop")
        {
            protected Color getColor()
            {return colorNextStop;}
            protected void setColor(Color color)
            {colorNextStop=color;}
        });
        Mode.addAction("colorsearch","Color Search","Background color to show matches during search operation.",new AbstractColorAction("Color Search")
        {
            protected Color getColor()
            {return colorSearch;}
            protected void setColor(Color color)
            {colorSearch=color;}
        });
        Mode.addAction("colorprofiling","Color Profiling","Background color used to highlight the lisp forms executed at least as many times as indicated by the profile slider.",
        new AbstractColorAction("Color Profiling")
        {
            protected Color getColor()
            {return colorProfiling;}
            protected void setColor(Color color)
            {colorProfiling=color;}
        });
        Mode.addAction("colorselection","Color Selection","Background color used to highlight the selected text.",new AbstractColorAction("Color Selection")
        {
            protected Color getColor()
            {return colorSelection;}
            protected void setColor(Color color)
            {colorSelection=color;}
        });
        Mode.addAction("colorstring","Color String","Foreground color of a string literal.",new AbstractColorAction("Color String")
        {
            protected Color getColor()
            {return colorString;}
            protected void setColor(Color color)
            {colorString=color;}
        });
        Mode.addAction("colorvariable","Color Variable","Sets the color of a lexical bound variable.",new AbstractColorAction("Color Variable")
        {
            protected Color getColor()
            {return colorVariable;}
            protected void setColor(Color color)
            {colorVariable=color;}
        });
        Mode.addAction("colorbackquote","Color Backquote","Sets the foreground color of backquoted forms.",new AbstractColorAction("Color Backquote")
        {
            protected Color getColor()
            {return colorBackQuote;}
            protected void setColor(Color color)
            {colorBackQuote=color;}
        });
        Mode.addAction("colorquote","Color Quote","Sets the foreground color of the quoted forms.",new AbstractColorAction("Color Quote")
        {
            protected Color getColor()
            {return colorQuote;}
            protected void setColor(Color color)
            {colorQuote=color;}
        });
        Mode.addAction("colorhint","Color Hint","Sets the background color of the hint/values",new AbstractColorAction("Color Hint")
        {
            protected Color getColor()
            {return colorHint;}
            protected void setColor(Color color)
            {colorHint=color;}
        });
        Mode.addAction("colorselected","Color Selected","Sets the background color of the selected nodes in a Project Explorer/Watches and Macros",new AbstractColorAction("Color Selected")
        {
            protected Color getColor()
            {return colorSelected;}
            protected void setColor(Color color)
            {colorSelected=color;}
        });
        Mode.addAction("lispstartupscript","Lisp Startup Script","Starts a file dialog to select the file to be loaded at the start of a interaction buffer.",
        new AbstractIDEAction("Lisp Startup Script")
        {
            public void doWork()
            {
                File file;
                JFileChooser chooser=new JFileChooser(System.getProperty("user.dir"));
                chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                chooser.setSelectedFile(new File(lispStartupFile));
                if (chooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION)
                {
                    lispStartupFile=Util.getCanonicalFileName(chooser.getSelectedFile());
                };
            }
        });
        Mode.addAction("hyperspecdir","Directory of the HyperSpec files","Starts a directory selecton dialog to specify the directory containing the HyperSpec documentation",
        new AbstractIDEAction("Directory of the HyperSpec files")
        {
            public void doWork()
            {
                File file;
                JFileChooser chooser=new JFileChooser();
                chooser.setDialogTitle("Choose the directory containing the HyperSpec files");
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                chooser.setDialogType(JFileChooser.OPEN_DIALOG);
                chooser.setSelectedFile(new File(HyperSpec.getHyperSpecDirectory()==null ? "" : HyperSpec.getHyperSpecDirectory()));
                if (chooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION)
                {
                    HyperSpec.setHyperSpecDirectory(Util.getCanonicalFileName(chooser.getSelectedFile()));
                };
            }
        });
        Mode.addAction("saveconfiguration","Save Configuration","Save the configuration settings in the $HOME/.Jabberwocky./LispIDE file.",
        new AbstractIDEAction("Save Configuration")
        {
            public void doWork()
            {
                saveConfiguration();
            }
        });
        Mode.addAction("generatelispcode","Generate Lisp Binding","Generates a file jabberwocky.lisp in the installation directory and the source directory (if it still exists) containing the lisp function definitions to interact with the IDE from within LISP. These functions are created in the package \"JABBERWOCKY\". For every action and question there will be a equivalent function defined.",
        new AbstractIDEAction("Generate Lisp Binding")
        {
            public void doWork()
            {
                GenerateLispCode.generate();
            }
        });
        Mode.addAction("displayversion","About","Display the cersion info of Jabberwocky.t",new AbstractIDEAction("About")
        {
            public void doWork()
            {
                JOptionPane.showConfirmDialog(null,IDEName+" "+IDEVersion,"About Jabberwocky",JOptionPane.PLAIN_MESSAGE);
            };
        });
        Mode.addAction("generatehtmlcode","Generate HTML documentation of actions","Generate a file actions.html in the installation directory and the source directory (if it exists) containing the actions, their keybinding , a short description and their associated LISP function in the JABBERWOCKY package.",
        new AbstractIDEAction("Generate HTML documentation of actions")
        {
            public void doWork()
            {
                Mode.generateDocumentation();
            }
        });
        Mode.addAction("setbrowser","Set the browser for help","Set the browser for help",
        new AbstractChoiceAction("Set the browser",new String[]
        {"native","konqueror","mozilla","netscape","explorer"})
        {
            public void setValue(String value)
            {helpbrowser=value;}
            public String getValue()
            {return helpbrowser;}
        });
        Mode.addAction("setlookandfeel", "Set the java look and feel","Sets the general look and feel of Jabberwocky",
        new AbstractChoiceAction("Set the look and feel", getLookAndFeelNames())
        {
            public void setValue(String value)
            {
                try
                {
                    UIManager.setLookAndFeel(value);
                    SwingUtilities.updateComponentTreeUI(Main.frame);
                    Main.frame.pack();
                    lookAndFeel=value;
                }
                catch (Exception ex)
                {
                    MessageHandler.warn("Failed to set look and feel, error="+ex.getMessage());
                };
            }
            public String getValue()
            {
                return lookAndFeel;
            }
        });
        Mode.addAction("deadkeymanagment","Dead key managment","Allows you to associatie a character with a key combination, used to solve some problems with java an international keyboards.",
        new AbstractIDEAction("Keysequence to character management")
        {
            public void doWork()
            {
                new DeadKeySequence().manageDeadKeySequences();
            }
        });
        Mode.addAction("managecommands","Manage the commands to start a interaction","Manage the command used by a interaction buffer to start the interaction to communicate with",
        new AbstractIDEAction("Manage the commands to start a interaction")
        {
            public void doWork()
            {
                InteractionCommands.editCommands();
            }
        });
        Mode.addAction( "loadconfiguration", "Load Configuration", "Load the configuration settings of a another configuration file",
        new AbstractIDEAction( "loadconfiguration" )
        {
            public void doWork()
            {
                //should we clear existing keybindings before loading new stuff?
                JFileChooser chooser = new JFileChooser();
                chooser.setCurrentDirectory(new File(InstallParameters.getInstallDir()+File.separator+"configurations"));
                chooser.setFileFilter( new javax.swing.filechooser.FileFilter()
                {
                    public boolean accept( File f )
                    {
                        return f.isDirectory()
                        || f.getName().endsWith( ".LispIDE" );
                    }
                    public String getDescription()
                    {
                        return "*.LispIDE (Jabberwocky configuration files)";
                    }
                } );
                
                int returnVal = chooser.showOpenDialog(Main.getWorkArea());
                if( returnVal == JFileChooser.APPROVE_OPTION )
                {
                    loadConfigFile(chooser.getSelectedFile());
                    Main.repaintEverything();
                }
            }
        });
    }
    /** Get the look and feels available */
    private static String[] getLookAndFeelNames()
    {
        UIManager.LookAndFeelInfo[] lfs=UIManager.getInstalledLookAndFeels();
        String result[]=new String[lfs.length];
        for (int i=0;i<lfs.length;i++)
            result[i]=lfs[i].getClassName();
        return result;
    }
    /** Creates the basic mode */
    private static void createRootMode()
    {
        Mode mode=new Mode("root");
        // Define the menu and menu items
        mode.addActionToMenu("'File'","new");
        mode.addActionToMenu("'File'","open");
        mode.addActionToMenu("'File'","save");
        mode.addActionToMenu("'File'","saveas");
        mode.addActionToMenu("'File'","close");
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","setcurrentdirectory");
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","addproject");
        mode.getMenu("'File'").setMnemonic(KeyEvent.VK_F);
        
        mode.addActionToMenu("'Edit'","copy");
        mode.addActionToMenu("'Edit'","cut");
        mode.addActionToMenu("'Edit'","delete");
        mode.addActionToMenu("'Edit'","paste");
        mode.addSeparatorToMenu("'Edit'");
        mode.addActionToMenu("'Edit'","undo");
        mode.addActionToMenu("'Edit'","redo");
        mode.addSeparatorToMenu("'Edit'");
        mode.addActionToMenu("'Edit'","indentate");
        mode.addActionToMenu("'Edit'","indentateelement");
        mode.addActionToMenu("'Edit'","indentatedocument");
        mode.addSeparatorToMenu("'Edit'");
        mode.addActionToMenu("'Edit'","searchbackward");
        mode.addActionToMenu("'Edit'","searchforward");
        mode.addActionToMenu("'Edit'","searchandreplace");
        mode.addActionToMenu("'Edit'","findandreplace");
        mode.addSeparatorToMenu("'Edit'");
        mode.addActionToMenu("'Edit'","gotoline");
        mode.addSeparatorToMenu("'Edit'");
        mode.addActionToMenu("'Edit'","startrecording");
        mode.addActionToMenu("'Edit'","stoprecording");
        mode.addActionToMenu("'Edit'","replayrecording");
        mode.getMenu("'Edit'").setMnemonic(KeyEvent.VK_E);
        
        mode.addActionToMenu("'Workarea'","splitvertical");
        mode.addActionToMenu("'Workarea'","splithorizontal");
        mode.addActionToMenu("'Workarea'","replaceeditor");
        mode.getMenu("'Workarea'").setMnemonic(KeyEvent.VK_W);
        if (InteractionCommands.getInteractionCommand(LispAclInteraction.class.getName())!=null)
            mode.addActionToMenu("'Interaction'","newinteractionacl");
		if (InteractionCommands.getInteractionCommand(LispAclWindowsInteraction.class.getName())!=null)
			mode.addActionToMenu("'Interaction'","newinteractionaclwindows");
        if (InteractionCommands.getInteractionCommand(LispCmuInteraction.class.getName())!=null)
            mode.addActionToMenu("'Interaction'","newinteractioncmucl");
        if (InteractionCommands.getInteractionCommand(LispClispInteraction.class.getName())!=null)
            mode.addActionToMenu("'Interaction'","newinteractionclisp");
        if (InteractionCommands.getInteractionCommand(LispSbclInteraction.class.getName())!=null)
            mode.addActionToMenu("'Interaction'","newinteractionsbcl");
        mode.addActionToMenu("'Interaction'","newwatch");
        mode.addActionToMenu("'Interaction'","deleteinteraction");
        mode.addActionToMenu("'Interaction'","deletewatch");
        mode.addActionToMenu("'Interaction'","execute");
        mode.addActionToMenu("'Interaction'","debugform");
        mode.addActionToMenu("'Interaction'","loadeditor");
        mode.addActionToMenu("'Interaction'","compileeditor");
        mode.addActionToMenu("'Interaction'/'Signal'","sigint");
        mode.addActionToMenu("'Interaction'/'Signal'","sigstop");
        mode.addActionToMenu("'Interaction'/'Signal'","sigcont");
        mode.addActionToMenu("'Interaction'/'Signal'","sigquit");
        mode.addActionToMenu("'Interaction'/'Signal'","sigkill");
        mode.addActionToMenu("'Interaction'","selectcommand");
        mode.getMenu("'Interaction'").setMnemonic(KeyEvent.VK_I);
        
        mode.addActionToMenu("'Options'/'Font'","defaultfont");
        mode.addActionToMenu("'Options'/'Font'","commentfont");
        mode.addActionToMenu("'Options'/'Font'", "explorerfont");
        mode.addActionToMenu("'Options'/'Font'", "watchesfont");
        mode.addActionToMenu("'Options'/'Font'", "macrosfont");
        mode.addActionToMenu("'Options'/'Color'","colorarray");
        mode.addActionToMenu("'Options'/'Color'","colorbackground");
        mode.addActionToMenu("'Options'/'Color'","colorbackgroundline");
        mode.addActionToMenu("'Options'/'Color'","colorbackquote");
        mode.addActionToMenu("'Options'/'Color'","colorbreakpoint");
        mode.addActionToMenu("'Options'/'Color'","colorcaller");
        mode.addActionToMenu("'Options'/'Color'","colorcharacter");
        mode.addActionToMenu("'Options'/'Color'","colorcomment");
        mode.addActionToMenu("'Options'/'Color'","colorcursor");
        mode.addActionToMenu("'Options'/'Color'","colorendline");
        mode.addActionToMenu("'Options'/'Color'","colorelement");
        mode.addActionToMenu("'Options'/'Color'","colorexecutedcode");
        mode.addActionToMenu("'Options'/'Color'","colorexecutingcode");
        mode.addActionToMenu("'Options'/'Color'","colorforeground");
        mode.addActionToMenu("'Options'/'Color'","colorforegroundline");
        mode.addActionToMenu("'Options'/'Color'","colorhint");
        mode.addActionToMenu("'Options'/'Color'","colorhook");
        mode.addActionToMenu("'Options'/'Color'","colorkeyword");
        mode.addActionToMenu("'Options'/'Color'","colorlist");
        mode.addActionToMenu("'Options'/'Color'","colornextstop");
        mode.addActionToMenu("'Options'/'Color'","colorprofiling");
        mode.addActionToMenu("'Options'/'Color'","colorquote");
        mode.addActionToMenu("'Options'/'Color'","colorsearch");
        mode.addActionToMenu("'Options'/'Color'","colorselected");
        mode.addActionToMenu("'Options'/'Color'","colorselection");
        mode.addActionToMenu("'Options'/'Color'","colorstring");
        mode.addActionToMenu("'Options'/'Color'","colorvariable");
        mode.addSeparatorToMenu("'Options'");
        mode.addActionToMenu("'Options'/'General'", "setlookandfeel");
        mode.addCheckboxToMenu("'Options'/'General'", "clickProjectSetCurrentDirectory", clickProjectSetCurrentDirectory);
        mode.addActionToMenu("'Options'/'General'","keybinding");
        mode.addActionToMenu("'Options'/'General'","managecommands");
        mode.addActionToMenu("'Options'/'General'","setbrowser");
        mode.addActionToMenu("'Options'/'General'","hyperspecdir");
        mode.addActionToMenu("'Options'/'General'","generatelispcode");
        mode.addActionToMenu("'Options'/'General'","generatehtmlcode");
        mode.addSeparatorToMenu("'Options'/'General'");
        mode.addActionToMenu("'Options'/'General'","lispstartupscript");
        mode.addCheckboxToMenu("'Options'/'Editor'","linenumbers",lineNumbering);
        mode.addCheckboxToMenu("'Options'/'Editor'","charactercompletion",indentateOnEnter);
        mode.addCheckboxToMenu("'Options'/'Editor'","indentateonenter",characterCompletion);
        mode.addCheckboxToMenu("'Options'/'Editor'","highlightlist",highligthList);
        mode.addSeparatorToMenu("'Options'/'Editor'");
        mode.addActionToMenu("'Options'/'Editor'","maxlinelength");
        mode.addActionToMenu("'Options'/'Editor'","tabwidth");
        mode.addActionToMenu("'Options'/'Editor'","blinkingrate");
        mode.addActionToMenu("'Options'/'Interaction'","hovertimeout");
        mode.addActionToMenu("'Options'/'Interaction'","commandhistorylength");
        mode.addActionToMenu("'Options'/'Interaction'","timeoutstopprocess");
        mode.addCheckboxToMenu("'Options'/'Interaction'","instrumentdebugafter",instrumentDebugAfter);
        mode.addCheckboxToMenu("'Options'/'Interaction'","instrumentcheckerror",instrumentCheckError);
        mode.addCheckboxToMenu("'Options'/'Workarounds'","clispbug",clispBug);
        mode.addActionToMenu("'Options'/'Workarounds'", "deadkeymanagment");
        mode.addSeparatorToMenu("'Options'");
        mode.addActionToMenu("'Options'","loadconfiguration");
        mode.addActionToMenu("'Options'","saveconfiguration");
        mode.getMenu("'Options'").setMnemonic(KeyEvent.VK_O);
        
        
        mode.addActionToMenu("'Help'","help");
        mode.addActionToMenu("'Help'","hyperspec");
        mode.addActionToMenu("'Help'","hyperspeccontext");
        mode.addActionToMenu("'Help'","displayversion");
        mode.addActionToMenu("'Help'/'Hint'","togglehint");
        mode.addActionToMenu("'Help'/'Hint'","replacebyhint");
        mode.getMenu("'Help'").setMnemonic(KeyEvent.VK_H);
        
        // Add actions to a popup menu
        mode.addActionToPopup("'File'","new");
        mode.addActionToPopup("'File'","open");
        mode.addActionToPopup("'File'","save");
        mode.addActionToPopup("'File'","saveas");
        mode.addActionToPopup("'File'","close");
        mode.addActionToPopup("'File'","setcurrentdirectory");
        
        // Add the buttons to the button bar in this mode
        
        // Define the keysequences
        mode.bindActionToKeySequence("up",KeyEvent.VK_UP,0);
        mode.bindActionToKeySequence("down",KeyEvent.VK_DOWN,0);
        mode.bindActionToKeySequence("pageup",KeyEvent.VK_PAGE_UP,0);
        mode.bindActionToKeySequence("pagedown",KeyEvent.VK_PAGE_DOWN,0);
        mode.bindActionToKeySequence("left",KeyEvent.VK_LEFT,0);
        mode.bindActionToKeySequence("right",KeyEvent.VK_RIGHT,0);
        mode.bindActionToKeySequence("remove",KeyEvent.VK_DELETE,0);
        mode.bindActionToKeySequence("backspace",KeyEvent.VK_BACK_SPACE,0);
        mode.bindActionToKeySequence("undo",KeyEvent.VK_U,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("redo",KeyEvent.VK_R,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("help",KeyEvent.VK_F1,0);
        mode.bindActionToKeySequence("hyperspeccontext",KeyEvent.VK_F1,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("hyperspec",KeyEvent.VK_F2,0);
        mode.bindActionToKeySequence("hintup",KeyEvent.VK_UP,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("hintdown",KeyEvent.VK_DOWN,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("replacebyhint",KeyEvent.VK_ENTER,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("togglehint",KeyEvent.VK_SPACE,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("indentate",KeyEvent.VK_TAB,0);
        mode.bindActionToKeySequence("indentateelement",KeyEvent.VK_TAB,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("debugform",KeyEvent.VK_D,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("execute",KeyEvent.VK_ENTER,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("sigint",KeyEvent.VK_C,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("loadeditor",KeyEvent.VK_L,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("open",KeyEvent.VK_X,KeyEvent.CTRL_MASK,KeyEvent.VK_F,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("save",KeyEvent.VK_X,KeyEvent.CTRL_MASK,KeyEvent.VK_S,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("exit",KeyEvent.VK_X,KeyEvent.CTRL_MASK,KeyEvent.VK_C,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("startrecording",KeyEvent.VK_M,KeyEvent.CTRL_MASK,KeyEvent.VK_B,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("stoprecording",KeyEvent.VK_M,KeyEvent.CTRL_MASK,KeyEvent.VK_E,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("replayrecording",KeyEvent.VK_M,KeyEvent.CTRL_MASK,KeyEvent.VK_R,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("copy",KeyEvent.VK_INSERT,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("cut",KeyEvent.VK_DELETE,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("delete",KeyEvent.VK_DELETE,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("paste",KeyEvent.VK_INSERT,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("searchforward",KeyEvent.VK_N,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("searchbackward",KeyEvent.VK_P,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("searchandreplace",KeyEvent.VK_R,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("findandreplace",KeyEvent.VK_F,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("selectcommand",KeyEvent.VK_H,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("movetobeginline",KeyEvent.VK_A,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("movetoendline",KeyEvent.VK_E,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("movetonextleaf",KeyEvent.VK_RIGHT,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("movetoendlist",KeyEvent.VK_RIGHT,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("movetoprevleaf",KeyEvent.VK_LEFT,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("movetobeginlist",KeyEvent.VK_LEFT,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("gotoline",KeyEvent.VK_G,KeyEvent.ALT_MASK);
        mode.bindActionToKeySequence("switch",KeyEvent.VK_I,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("next",KeyEvent.VK_T,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("selectcurrentleaf",KeyEvent.VK_S,KeyEvent.CTRL_MASK,KeyEvent.VK_W,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("selectcurrentelement",KeyEvent.VK_S,KeyEvent.CTRL_MASK,KeyEvent.VK_E,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("selecttoplevelelement",KeyEvent.VK_S,KeyEvent.CTRL_MASK,KeyEvent.VK_T,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("selectline",KeyEvent.VK_S,KeyEvent.CTRL_MASK,KeyEvent.VK_L,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("widenselection",KeyEvent.VK_S,KeyEvent.CTRL_MASK,KeyEvent.VK_U,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("shortenselection",KeyEvent.VK_S,KeyEvent.CTRL_MASK,KeyEvent.VK_D,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("deletecurrentleaf",KeyEvent.VK_X,KeyEvent.CTRL_MASK,KeyEvent.VK_W,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("deletecurrentelement",KeyEvent.VK_X,KeyEvent.CTRL_MASK,KeyEvent.VK_E,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("deletetoplevelelement",KeyEvent.VK_X,KeyEvent.CTRL_MASK,KeyEvent.VK_T,KeyEvent.CTRL_MASK);
        mode.bindActionToKeySequence("deleteline",KeyEvent.VK_K,KeyEvent.CTRL_MASK);
        
    }
    /** Create the basic mode */
    public static void createBasicMode()
    {
        Mode mode=Mode.getMode("root").createCopy("basic");
        
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","debug");
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","exit");
        
        mode.addSeparatorToPopup("'File'");
        mode.addActionToPopup("'File'","debug");
        mode.addActionToPopup("copy");
        mode.addActionToPopup("cut");
        mode.addActionToPopup("delete");
        mode.addActionToPopup("paste");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("undo");
        mode.addActionToPopup("redo");
        
        mode.addToolbar("debug");
        
        mode.addButton("debug","debug");
        mode.addButton("debug","new");
        mode.addButton("debug","open");
        mode.addButton("debug","save");
        mode.addButton("debug","close");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","cut");
        mode.addButton("debug","copy");
        mode.addButton("debug","paste");
        mode.addButton("debug","undo");
        mode.addButton("debug","redo");
        mode.addButton("debug","searchbackward");
        mode.addButton("debug","searchforward");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","startrecording");
        mode.addButton("debug","stoprecording");
        mode.addButton("debug","replayrecording");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","splitvertical");
        mode.addButton("debug","splithorizontal");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","compileeditor");
        mode.addButton("debug","execute");
        mode.addSeparatorToButtonBar("debug");
        if (InteractionCommands.getInteractionCommand(LispAclInteraction.class.getName())!=null)
            mode.addButton("debug","newinteractionacl","ACL");
		if (InteractionCommands.getInteractionCommand(LispAclWindowsInteraction.class.getName())!=null)
			mode.addButton("debug","newinteractionaclwindows","ACL");
        if (InteractionCommands.getInteractionCommand(LispCmuInteraction.class.getName())!=null)
            mode.addButton("debug","newinteractioncmucl","CMUCL");
        if (InteractionCommands.getInteractionCommand(LispClispInteraction.class.getName())!=null)
            mode.addButton("debug","newinteractionclisp","CLISP");
        if (InteractionCommands.getInteractionCommand(LispSbclInteraction.class.getName())!=null)
            mode.addButton("debug","newinteractionsbcl","SBCL");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","deleteinteraction");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","togglehint");
        mode.addButton("debug","help");
        
        mode.bindActionToKeySequence("debug",KeyEvent.VK_F6,0);
        
    }
    /** Create the mode for the interaction pane */
    public static void createInteractionMode()
    {
        Mode mode=Mode.getMode("root").createCopy("interaction");
        
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","exit");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("sigint");
        mode.addActionToPopup("sigstop");
        mode.addActionToPopup("sigcont");
        mode.addActionToPopup("sigquit");
        mode.addActionToPopup("sigkill");
        
        mode.addToolbar("interaction");
        
        mode.addButton("interaction","debug");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","cut");
        mode.addButton("interaction","copy");
        mode.addButton("interaction","paste");
        mode.addButton("interaction","undo");
        mode.addButton("interaction","redo");
        mode.addButton("interaction","searchbackward");
        mode.addButton("interaction","searchforward");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","startrecording");
        mode.addButton("interaction","stoprecording");
        mode.addButton("interaction","replayrecording");
        mode.addSeparatorToButtonBar("interaction");
        if (InteractionCommands.getInteractionCommand(LispAclInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractionacl","ACL");
		if (InteractionCommands.getInteractionCommand(LispAclWindowsInteraction.class.getName())!=null)
			mode.addButton("interaction","newinteractionaclwindows","ACL");
        if (InteractionCommands.getInteractionCommand(LispCmuInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractioncmucl","CMUCL");
        if (InteractionCommands.getInteractionCommand(LispClispInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractionclisp","CLISP");
        if (InteractionCommands.getInteractionCommand(LispSbclInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractionsbcl","SBCL");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","deleteinteraction");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","togglehint");
        mode.addButton("interaction","help");
        
        mode.bindActionToKeySequence("return",KeyEvent.VK_ENTER,KeyEvent.SHIFT_MASK);
    }
    /** Create the debug mode */
    public static void createDebugMode()
    {
        Mode mode=Mode.getMode("root").createCopy("debug");
        
        mode.addActionToMenu("'File'","undebug");
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","exit");
        
        mode.addActionToMenu("'Debug'","togglebreakpoint");
        mode.addActionToMenu("'Debug'","togglecondbreakpoint");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","stepinto");
        mode.addActionToMenu("'Debug'","stepover");
        mode.addActionToMenu("'Debug'","stopat");
        mode.addActionToMenu("'Debug'","continue");
        mode.addActionToMenu("'Debug'","stopafter");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","addwatch");
        mode.addActionToMenu("'Debug'","setnewreturnvalue");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","past");
        mode.addActionToMenu("'Debug'","future");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","startprofiling");
        mode.addActionToMenu("'Debug'","stopprofiling");
        mode.addSeparatorToPopup("'File'");
        mode.addActionToPopup("'File'","undebug");
        // mode.addActionToPopup("Paste");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("togglebreakpoint");
        mode.addActionToPopup("togglecondbreakpoint");
        mode.addActionToPopup("stepinto");
        mode.addActionToPopup("stepover");
        mode.addActionToPopup("stopat");
        mode.addActionToPopup("continue");
        mode.addActionToPopup("stopafter");
        mode.addActionToPopup("addwatch");
        mode.addActionToPopup("setnewreturnvalue");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("past");
        mode.addActionToPopup("future");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("startprofiling");
        mode.addActionToPopup("stopprofiling");
        
        mode.addToolbar("debug");
        
        mode.addButton("debug","undebug");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","togglebreakpoint");
        mode.addButton("debug","togglecondbreakpoint");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","stepinto");
        mode.addButton("debug","stepover");
        mode.addButton("debug","stopat");
        mode.addButton("debug","continue");
        mode.addButton("debug","stopafter");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","addwatch");
        mode.addButton("debug","setnewreturnvalue");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","past");
        mode.addButton("debug","future");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","startprofiling");
        mode.addButton("debug","stopprofiling");
        
        mode.bindActionToKeySequence("undebug",KeyEvent.VK_F6,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("togglebreakpoint",KeyEvent.VK_F9,0);
        mode.bindActionToKeySequence("togglecondbreakpoint",KeyEvent.VK_F9,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("stepinto",KeyEvent.VK_F7,0);
        mode.bindActionToKeySequence("stepover",KeyEvent.VK_F8,0);
        mode.bindActionToKeySequence("stopat",KeyEvent.VK_F4,0);
        mode.bindActionToKeySequence("continue",KeyEvent.VK_F5,0);
        mode.bindActionToKeySequence("stopafter",KeyEvent.VK_F10,0);
        mode.bindActionToKeySequence("addwatch",KeyEvent.VK_F11,0);
        mode.bindActionToKeySequence("setnewreturnvalue",KeyEvent.VK_F12,0);
    }
    /** Create debug interaction mode */
    public static void createDebugInteractionMode()
    {
        Mode mode=Mode.getMode("root").createCopy("debuginteraction");
        
        mode.addSeparatorToMenu("'File'");
        mode.addActionToMenu("'File'","exit");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("sigint");
        mode.addActionToPopup("sigstop");
        mode.addActionToPopup("sigcont");
        mode.addActionToPopup("sigquit");
        mode.addActionToPopup("sigkill");
        
        mode.addToolbar("interaction");
        
        mode.addButton("interaction","cut");
        mode.addButton("interaction","copy");
        mode.addButton("interaction","paste");
        mode.addButton("interaction","undo");
        mode.addButton("interaction","redo");
        mode.addButton("interaction","searchbackward");
        mode.addButton("interaction","searchforward");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","startrecording");
        mode.addButton("interaction","stoprecording");
        mode.addButton("interaction","replayrecording");
        mode.addSeparatorToButtonBar("interaction");
        if (InteractionCommands.getInteractionCommand(LispAclInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractionacl","ACL");
		if (InteractionCommands.getInteractionCommand(LispAclWindowsInteraction.class.getName())!=null)
			mode.addButton("interaction","newinteractionaclwindows","ACL");
        if (InteractionCommands.getInteractionCommand(LispCmuInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractioncmucl","CMUCL");
        if (InteractionCommands.getInteractionCommand(LispClispInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractionclisp","CLISP");
        if (InteractionCommands.getInteractionCommand(LispSbclInteraction.class.getName())!=null)
            mode.addButton("interaction","newinteractionsbcl","SBCL");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","deleteinteraction");
        mode.addSeparatorToButtonBar("interaction");
        mode.addButton("interaction","togglehint");
        mode.addButton("interaction","help");
        // Add debug commands
        mode.addActionToMenu("'Debug'","togglebreakpoint");
        mode.addActionToMenu("'Debug'","togglecondbreakpoint");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","stepinto");
        mode.addActionToMenu("'Debug'","stepover");
        mode.addActionToMenu("'Debug'","stopat");
        mode.addActionToMenu("'Debug'","continue");
        mode.addActionToMenu("'Debug'","stopafter");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","addwatch");
        mode.addActionToMenu("'Debug'","setnewreturnvalue");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","past");
        mode.addActionToMenu("'Debug'","future");
        mode.addSeparatorToMenu("'Debug'");
        mode.addActionToMenu("'Debug'","startprofiling");
        mode.addActionToMenu("'Debug'","stopprofiling");
        
        mode.addSeparatorToPopup("'File'");
        mode.addActionToPopup("'File'","undebug");
        // mode.addActionToPopup("Paste");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("togglebreakpoint");
        mode.addActionToPopup("togglecondbreakpoint");
        mode.addActionToPopup("stepinto");
        mode.addActionToPopup("stepover");
        mode.addActionToPopup("stopat");
        mode.addActionToPopup("continue");
        mode.addActionToPopup("stopafter");
        mode.addActionToPopup("addwatch");
        mode.addActionToPopup("setnewreturnvalue");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("past");
        mode.addActionToPopup("future");
        mode.addSeparatorToPopup("");
        mode.addActionToPopup("startprofiling");
        mode.addActionToPopup("stopprofiling");
        
        mode.addToolbar("debug");
        
        mode.addButton("debug","undebug");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","togglebreakpoint");
        mode.addButton("debug","togglecondbreakpoint");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","stepinto");
        mode.addButton("debug","stepover");
        mode.addButton("debug","stopat");
        mode.addButton("debug","continue");
        mode.addButton("debug","stopafter");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","addwatch");
        mode.addButton("debug","setnewreturnvalue");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","past");
        mode.addButton("debug","future");
        mode.addSeparatorToButtonBar("debug");
        mode.addButton("debug","startprofiling");
        mode.addButton("debug","stopprofiling");
        
        mode.bindActionToKeySequence("undebug",KeyEvent.VK_F6,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("togglebreakpoint",KeyEvent.VK_F9,0);
        mode.bindActionToKeySequence("togglecondbreakpoint",KeyEvent.VK_F9,KeyEvent.SHIFT_MASK);
        mode.bindActionToKeySequence("stepinto",KeyEvent.VK_F7,0);
        mode.bindActionToKeySequence("stepover",KeyEvent.VK_F8,0);
        mode.bindActionToKeySequence("stopat",KeyEvent.VK_F4,0);
        mode.bindActionToKeySequence("continue",KeyEvent.VK_F5,0);
        mode.bindActionToKeySequence("stopafter",KeyEvent.VK_F10,0);
        mode.bindActionToKeySequence("addwatch",KeyEvent.VK_F11,0);
        mode.bindActionToKeySequence("setnewreturnvalue",KeyEvent.VK_F12,0);
        mode.bindActionToKeySequence("return",KeyEvent.VK_ENTER,KeyEvent.SHIFT_MASK);
    }
    /** Get the image file name */
    public static ImageIcon getIcon(String filename)
    {
        String separator=System.getProperty("file.separator");
        String result= Configuration.installDirectory+separator+"image"+separator+filename;
        ImageIcon icon=(ImageIcon) nameToIcon.get(result);
        if (icon==null)
        {
            icon=new ImageIcon(result);
            nameToIcon.put(result,icon);
        };
        return icon;
    }
    /** Save configuration settings */
    public void saveConfiguration()
    {
        File savefile=new File(System.getProperty("user.home")+File.separator+".Jabberwocky"+File.separator+".LispIDE");
        try
        {
            BufferedWriter writer=new BufferedWriter(new FileWriter(savefile));
            // Save the fields of Configuration
            Field[] fields=this.getClass().getFields();
            XMLConfigClass xmlclass=new XMLConfigClass(this.getClass().getName());
            XMLConfigField xmlfield;
            XMLConfigObject xmlobject;
            Class cls;
            Color color;
            Font font;
            for (int i=0;i<fields.length;i++)
            {
                // these fields must not be saved
                if (fields[i].getName().compareTo("IDEName")==0 ||
                fields[i].getName().compareTo("IDEVersion")==0 ||
                fields[i].getName().compareTo("installDirectory")==0)
                    continue;
                // these fields must be saved
                xmlfield=new XMLConfigField(fields[i].getName());
                cls=fields[i].getType();
                if (cls==Integer.TYPE)
                {
                    xmlfield.setValue(fields[i].getInt(null));
                    xmlclass.addField(xmlfield);
                }
                else if (cls==Boolean.TYPE)
                {
                    xmlfield.setValue(fields[i].getBoolean(null));
                    xmlclass.addField(xmlfield);
                }
                else if  (cls==Class.forName("java.lang.String"))
                {
                    xmlfield.setValue((String) fields[i].get(null));
                    xmlclass.addField(xmlfield);
                }
                else if (cls==Class.forName("java.awt.Color"))
                {
                    xmlobject=new XMLConfigObject(cls.getName());
                    color=(Color) fields[i].get(null);
                    xmlobject.addArgument(color.getRed());
                    xmlobject.addArgument(color.getGreen());
                    xmlobject.addArgument(color.getBlue());
                    xmlfield.setValue(xmlobject);
                    xmlclass.addField(xmlfield);
                }
                else if (cls==Class.forName("java.awt.Font"))
                {
                    xmlobject=new XMLConfigObject(cls.getName());
                    font=(Font) fields[i].get(null);
                    xmlobject.addArgument(font.getFontName());
                    xmlobject.addArgument(font.getStyle());
                    xmlobject.addArgument(font.getSize());
                    xmlfield.setValue(xmlobject);
                    xmlclass.addField(xmlfield);
                };
            };
            xmlclass.serialize(writer);
            // Save the mode info (keymappings)
            Mode.saveConfiguration(writer);
            // Save the HyperSpec information
            HyperSpec.saveConfiguration(writer);
            // Save the DeadKey Configuration
            DeadKeySequence.saveConfiguration(writer);
            // Close the stream
            writer.close();
            // Save the global configuration info
            saveGlobalConfiguration();
        }
        catch (IOException e)
        {
            MessageHandler.report("Could not write configuration",e);
        }
        catch (IllegalAccessException e)
        {
            MessageHandler.report("Illegal access error during read Configuration fields",e);
        }
        catch (ClassNotFoundException e)
        {
            MessageHandler.report("Class not found exception ",e);
        };
    }
    /** Save the global configuration state */
    private static void saveGlobalConfiguration()
    {
        try
        {
        // Save the generic configuration file
        BufferedWriter writer=new BufferedWriter(new FileWriter(new File(InstallParameters.getInstallDir(),"jabberwocky.config")));
        InteractionCommands.saveConfiguration(writer);
        writer.close();
        }
        catch (Exception e)
        {
            MessageHandler.say("Failed to save the global configuration");
        }
    }
    /*
     * Load the global configuration
     */
    public static void loadGlobalConfiguration()
    {
        // Load the global configuration file
        File loadfile=new File(InstallParameters.getInstallDir(),"jabberwocky.config");
        if (!loadfile.exists())
        {
            if (JOptionPane.showConfirmDialog(null,
            "Non existing Global Configuration File (this could be a corrupt installation), do you want to create a new file. Be aware that you have to create new startup commands for the lisp interactions because they will be empty.",
            "Error configuring Jabberwocky",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                saveGlobalConfiguration();
        }
        loadConfigFile(loadfile);
        if (AbstractXMLConfig.errorDuringConfig)
        {
            System.out.println("Error Loading config");
            // A error occured during the load of the configuration, ask if it is ok to create a new
            // configuration
            if (JOptionPane.showConfirmDialog(null,
            "Incorrect Global Configuration File (this could be a corrupt installation), do you want to create a new file. Be aware that you have to create new startup commands for the lisp interactions because they will be empty.",
            "Error configuring Jabberwocky",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                saveGlobalConfiguration();
        };
        
    }
    /**
     *  Copy another configuration file over the .LispIDE file in the home directory of the user,
     *  load this file and do a redisplay of the editor bindings
     **/
    
    /** Load the configuration info */
    private void loadConfiguration()
    {
        File loadfile;
        // Check the existance of the .Jabberwocky directory, if i does not exist create a new
        File directory=new File(System.getProperty("user.home")+File.separator+".Jabberwocky");
        if (!directory.exists())
        {
            MessageHandler.warn("Configuration directory .Jabberwocky does not exist, creating new one");
            directory.mkdir();
            MessageHandler.warn("Copy old configuration files to new location");
            loadfile=new File(System.getProperty("user.home")+File.separator+".LispIDE");
            if (loadfile.exists())
                loadfile.renameTo(new File(directory,loadfile.getName()));
            loadfile=new File(System.getProperty("user.home")+File.separator+".LispIDEState");
            if (loadfile.exists())
                loadfile.renameTo(new File(directory,loadfile.getName()));
        };
        // Create the log file for error reporting
        File  logfile=new File(System.getProperty("user.home")+File.separator+".Jabberwocky"+File.separator+"error.log");
        if (logfile.exists())
            logfile.delete();
        try
        {
            logfile.createNewFile();
        }
        catch (Exception e)
        {
            // No way to report this exception
        };
        // Load the configuration settings
        loadfile=new File(directory,".LispIDE");
        if (!loadfile.exists())
        {
            MessageHandler.warn("Configuration file does not exist, creating new one");
            saveConfiguration();
        }
        loadConfigFile(loadfile);
        if (AbstractXMLConfig.errorDuringConfig)
        {
            // A error occured during the load of the configuration, ask if it is ok to create a new
            // configuration
            if (JOptionPane.showConfirmDialog(null,
            "Incorrect Configuration File (this could happen with a new release or because the file is corrupt), do you want to create a new file",
            "Error configuring Jabberwocky",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                saveConfiguration();
        };
    }
    
    private static void loadConfigFile(File configFile)
    {
        try
        {
            XMLConfigReader reader=new XMLConfigReader(new BufferedReader(new FileReader(configFile)));
            XMLConfigClass cls;
            while ((cls=reader.readXMLConfigClass())!=null)
                cls.configure();
        }
        catch (FileNotFoundException e)
        {
            MessageHandler.warn("Configuration file does not exist");
        };
    }
    public static void loadState()
    {
        // Load the state
        File loadfile=new File(System.getProperty("user.home")+File.separator+".Jabberwocky"+File.separator+".LispIDEState");
        try
        {
            XMLConfigReader reader=new XMLConfigReader(new BufferedReader(new FileReader(loadfile)));
            XMLConfigClass cls;
            while ((cls=reader.readXMLConfigClass())!=null)
                cls.configure();
        }
        catch (FileNotFoundException e)
        {
            MessageHandler.warn("File does not exists "+e.getMessage());
        };
    }
}
