/*
 * DeadKeySequence.java
 *
 * Created on April 12, 2003, 8:39 AM
 */

package IDE.Configuration;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import IDE.Messaging.*;

/**
 * Class representing dead key sequences associated with a Unicode character
 * this to solve the really very annoying behavior of Sun of not supporting international
 * keyboards
 * @author  marc
 */
public class DeadKeySequence
{
    /** Static ArrayList holding all the DeadKeySequences know by Jabberwocky */
    private static ArrayList deadKeySequences=new ArrayList();
    /** Array to hold the KeyEvents keycode,modifiers and character */
    private KeyCodeModifiersChar[] keycodemodifierschar=new KeyCodeModifiersChar[20];
    /** Character associated with the dead keysequence */
    private char unicodechar;
    /** Next index to use in the array keymodifierschar */
    private int next=0;
    /** Compiled version of the keysequences */
    private static SortedMap compiledKeySequence=null;
    /** Maintains the state of the checkig of a DeadKeySequence */
    private static SortedMap kmcToNext=null;
    /** Table used to edit the deadkeysequences */
    private JTable table;
    /** Model used in the JTable showing the DeadKeySequences */
    DeadKeySequencesModel model;
    /**
     * Creates a new instance of DeadKeySequence (private constructor, the instances
     * will be created via the static method manageDeadKeySequences
     */
    public DeadKeySequence()
    {
    }
    /** Test */
    /*
    public static void main(String[] args)
    {
        new DeadKeySequence().manageDeadKeySequences();
        // Test of the DeadKey Sequences
        JFrame frame=new JFrame();
        JTextField field=new JTextField("TypeText");
        field.addKeyListener(new DeadKeySequence());
        frame.getContentPane().add(field);
        frame.setSize(100,100);
        frame.setVisible(true);
    }
     **/
    /*
     * Restarts the checking for dead keys
     */
    public static void Restart()
    {
        kmcToNext=null;
    }
    /*
     * Check if a certain KeyEvent is part of a deadKeySequence
     */
    public static char checkForDeadKeys(KeyEvent e)
    {
        if (compiledKeySequence==null)
            return '\0'; // Do not even try
        Object found;
        if (kmcToNext==null)
        {
            // We had a restart or this is the first time
            found=compiledKeySequence.get(new KeyCodeModifiersChar(e.getKeyCode(), e.getModifiers(), e.getKeyChar(),e.isAltGraphDown()));
            if (found==null)
            {
                // Nothing use previous restart
                return '\0';
            }
            else if (found instanceof Character)
            {
                // We have a full match , return the character and restart
                kmcToNext=null;
                return ((Character) found).charValue();
            }
            else
            {
                // We have a partial match
                kmcToNext=(SortedMap) found;
                return '\0';
            }
        }
        else
        {
            // We have a previous match
            found=kmcToNext.get(new KeyCodeModifiersChar(e.getKeyCode(), e.getModifiers(),e.getKeyChar(),e.isAltGraphDown()));
            if (found==null)
            {
                // We have not a match, do a restart
                kmcToNext=null;
                return '\0';
            }
            else if (found instanceof Character)
            {
                // We have a full match, return the character and do a restart
                kmcToNext=null;
                return ((Character) found).charValue();
            }
            else
            {
                // We have a partial match
                kmcToNext=(SortedMap) found;
                return '\0';
            }
        }
    }
    /* Returns a string representing the key stroke */
    public String toString()
    {
        StringBuffer buffer=new StringBuffer();
        for (int i=0;i<next;i++)
        {
            buffer.append("(");
            buffer.append(keycodemodifierschar[i].keycode);
            buffer.append(",");
            buffer.append(keycodemodifierschar[i].modifiers);
            buffer.append(",");
            buffer.append(keycodemodifierschar[i].keychar);
            buffer.append(",");
            buffer.append(keycodemodifierschar[i].isAltGraph);
            buffer.append(")"); 
        };
        return buffer.toString();
    }
    /** Compile the KeySequences in a structure that is easy to check */
    public static boolean compileKeySequences()
    {
        DeadKeySequence seq;
        SortedMap kmcToKmc;
        Object found;
        if (deadKeySequences.size()==0)
        {
            compiledKeySequence=null;
        }
        else
        {
            compiledKeySequence=new TreeMap();
            int m=deadKeySequences.size();
            for (int i=0;i<m;i++)
            {
                seq=(DeadKeySequence) deadKeySequences.get(i);
                if (seq.next>0)
                {
                    kmcToKmc=compiledKeySequence;
                    for (int j=0;j+1<seq.next;j++)
                    {
                        found=kmcToKmc.get(seq.keycodemodifierschar[j]);
                        if (found==null)
                        {
                            found=new TreeMap();
                            kmcToKmc.put(seq.keycodemodifierschar[j], found);
                            kmcToKmc=(SortedMap) found;
                        }
                        else if (found instanceof SortedMap)
                        {
                            kmcToKmc=(SortedMap) found;
                        }
                        else
                        {
                            MessageHandler.warn("Ambiguous key sequence");
                            return false;
                        }
                    };
                    kmcToKmc.put(seq.keycodemodifierschar[seq.next-1],new Character(seq.unicodechar));
                };
            };
        };
        return true;
    }
    /** Close a DeadKeySequence */
    public DeadKeySequence copy()
    {
        DeadKeySequence seq=new DeadKeySequence();
        for (int i=0;i<next;i++)
        {
            KeyCodeModifiersChar kmc=keycodemodifierschar[i];
            seq.keycodemodifierschar[i]=new KeyCodeModifiersChar(kmc.keycode,kmc.modifiers,kmc.keychar,kmc.isAltGraph);
        };
        seq.unicodechar=unicodechar;
        seq.next=next;
        return seq;
    }
    /** Add a new DeadKeySequence, used in the save of configuration information */
    public static void addDeadKeySequence(char unicodechar,
    int next,
    int c0,int m0,int ch0,boolean ag0,
    int c1,int m1,int ch1,boolean ag1,
    int c2,int m2,int ch2,boolean ag2,
    int c3,int m3,int ch3,boolean ag3,
    int c4,int m4,int ch4,boolean ag4,
    int c5,int m5,int ch5,boolean ag5,
    int c6,int m6,int ch6,boolean ag6,
    int c7,int m7,int ch7,boolean ag7,
    int c8,int m8,int ch8,boolean ag8,
    int c9,int m9,int ch9,boolean ag9,
    int c10,int m10,int ch10,boolean ag10,
    int c11,int m11,int ch11,boolean ag11,
    int c12,int m12,int ch12,boolean ag12,
    int c13,int m13,int ch13,boolean ag13,
    int c14,int m14,int ch14,boolean ag14,
    int c15,int m15,int ch15,boolean ag15,
    int c16,int m16,int ch16,boolean ag16,
    int c17,int m17,int ch17,boolean ag17,
    int c18,int m18,int ch18,boolean ag18,
    int c19,int m19,int ch19,boolean ag19)
    {
        DeadKeySequence seq=new DeadKeySequence();
        seq.unicodechar=unicodechar;
        seq.next=next;
        seq.keycodemodifierschar[0]=new KeyCodeModifiersChar(c0,m0,ch0,ag0);
        seq.keycodemodifierschar[1]=new KeyCodeModifiersChar(c1,m1,ch1,ag1);
        seq.keycodemodifierschar[2]=new KeyCodeModifiersChar(c2,m2,ch2,ag2);
        seq.keycodemodifierschar[3]=new KeyCodeModifiersChar(c3,m3,ch3,ag3);
        seq.keycodemodifierschar[4]=new KeyCodeModifiersChar(c4,m4,ch4,ag4);
        seq.keycodemodifierschar[5]=new KeyCodeModifiersChar(c5,m5,ch5,ag5);
        seq.keycodemodifierschar[6]=new KeyCodeModifiersChar(c6,m6,ch6,ag6);
        seq.keycodemodifierschar[7]=new KeyCodeModifiersChar(c7,m7,ch7,ag7);
        seq.keycodemodifierschar[8]=new KeyCodeModifiersChar(c8,m8,ch8,ag8);
        seq.keycodemodifierschar[9]=new KeyCodeModifiersChar(c9,m9,ch9,ag9);
        seq.keycodemodifierschar[10]=new KeyCodeModifiersChar(c10,m10,ch10,ag10);
        seq.keycodemodifierschar[11]=new KeyCodeModifiersChar(c11,m11,ch11,ag11);
        seq.keycodemodifierschar[12]=new KeyCodeModifiersChar(c12,m12,ch12,ag12);
        seq.keycodemodifierschar[13]=new KeyCodeModifiersChar(c13,m13,ch13,ag13);
        seq.keycodemodifierschar[14]=new KeyCodeModifiersChar(c14,m14,ch14,ag14);
        seq.keycodemodifierschar[15]=new KeyCodeModifiersChar(c15,m15,ch15,ag15);
        seq.keycodemodifierschar[16]=new KeyCodeModifiersChar(c16,m16,ch16,ag16);
        seq.keycodemodifierschar[17]=new KeyCodeModifiersChar(c17,m17,ch17,ag17);
        seq.keycodemodifierschar[18]=new KeyCodeModifiersChar(c18,m18,ch18,ag18);
        seq.keycodemodifierschar[19]=new KeyCodeModifiersChar(c19,m19,ch19,ag19);
        deadKeySequences.add(seq);
    }
    /** Static method to save the configuration of this DeadKeySequence */
    public static void saveConfiguration(Writer writer)
    {
        XMLConfigClass cls=new XMLConfigClass(DeadKeySequence.class.getName());
        XMLConfigMethod method;
        DeadKeySequence seq;
        KeyCodeModifiersChar kmc;
        int m=deadKeySequences.size();
        for (int i=0;i<m;i++)
        {
            seq=(DeadKeySequence) deadKeySequences.get(i);
            method=new XMLConfigMethod("addDeadKeySequence");
            method.addArgument(seq.unicodechar);
            method.addArgument(seq.next);
            for (int j=0;j<seq.keycodemodifierschar.length;j++)
            {
                kmc=seq.keycodemodifierschar[j];
                if (kmc==null)
                {
                    method.addArgument(0);
                    method.addArgument(0);
                    method.addArgument(0);
                    method.addArgument(true);
                }
                else
                {
                    method.addArgument(kmc.keycode);
                    method.addArgument(kmc.modifiers);
                    method.addArgument(kmc.keychar);
                    method.addArgument(kmc.isAltGraph);
                };
            };
            cls.addMethod(method);
        };
        method=new XMLConfigMethod("compileKeySequences");
        cls.addMethod(method);
        cls.serialize(writer);
    }
    /** Edit the list of DeadKeySequences */
    public void manageDeadKeySequences()
    {
        final JDialog dialog=new JDialog();
        dialog.setTitle("Manage special characters");
        dialog.setModal(true);
        model=new DeadKeySequencesModel();
        table=new JTable(model);
        table.setDefaultEditor(DeadKeySequence.class,new DeadKeySequenceCellEditor());
        table.setDefaultEditor(Character.class,new CharCellEditor());
        table.setDefaultRenderer(DeadKeySequence.class,new DeadKeySequenceCellRenderer());
        table.setDefaultRenderer(Character.class,new DeadKeySequenceCellRenderer());
      
        JScrollPane scrollpane=new JScrollPane(table);
        JButton add=new JButton(new AbstractAction("Add")
        {
            public void actionPerformed(ActionEvent ev)
            {
                model.addNewDeadKeySequence();
                table.changeSelection(deadKeySequences.size()-1,0, false,false);
            };
        });
        JButton remove=new JButton(new AbstractAction("Remove")
        {
            public void actionPerformed(ActionEvent ev)
            {
                int index=table.getSelectedRow();
                if (index<0)
                    index=table.getEditingRow();
                if (index>=0)
                {
                    model.removeDeadKeySequence(index);
                    table.getCellEditor().cancelCellEditing();
                }
            }
        });
        JButton ok=new JButton(new AbstractAction("Ok")
        {
            public void actionPerformed(ActionEvent ev)
            {
                model.updateDeadkeysequences();
                dialog.setVisible(false);
                dialog.dispose();
            }
        });
        JButton cancel=new JButton(new AbstractAction("Cancel")
        {
            public void actionPerformed(ActionEvent ev)
            {
                dialog.setVisible(false);
                dialog.dispose();
            }
        });
        JPanel content=new JPanel(new BorderLayout());
        JPanel buttons=new JPanel();
        buttons.add(add);
        buttons.add(remove);
        buttons.add(ok);
        buttons.add(cancel);
        content.add(scrollpane,BorderLayout.CENTER);
        content.add(buttons,BorderLayout.SOUTH);
        dialog.setContentPane(content);
        dialog.setSize(400,500);
        dialog.setVisible(true);
    }
        
    /*
     * Class to implemnent the table model for DeadKeySequences map
     */
    private class DeadKeySequencesModel extends AbstractTableModel
    {
        ArrayList sequences=new ArrayList();
        
        public DeadKeySequencesModel()
        {
            int m=deadKeySequences.size();
            for (int i=0;i<m;i++)
            {
                sequences.add(((DeadKeySequence) deadKeySequences.get(i)).copy());
            };
        }
        public int getRowCount()
        {
            return sequences.size();
        }
        public int getColumnCount()
        {
            return 2;
        }
        public Object getValueAt(int r,int c)
        {
            if (c==0)
                return sequences.get(r);
            else
                return new Character(((DeadKeySequence) sequences.get(r)).unicodechar);
        }
        public void setValueAt(Object value,int r,int c)
        {
            switch (c)
            {
                case 0:
                    sequences.set(r, value);
                    break;
                case 1:
                    DeadKeySequence dks=(DeadKeySequence) sequences.get(r);
                    dks.unicodechar=((Character) value).charValue();
                    break;
            }
        }
        public boolean isCellEditable(int r,int c)
        {
            return true;
        }
        public String getColumnName(int c)
        {
            switch (c)
            {
                case 0:
                    return "Key Sequence";
                case 1:
                    return "Unicode Character";
                default:
                    return "Unknown column";
            }
        }
        public Class getColumnClass(int c)
        {
            switch (c)
            {
                case 0:
                    return DeadKeySequence.class;
                case 1:
                    return Character.class;
                default:
                    return Character.class;
            }
        }
        /** Called to add a new empty DeadKeySequence */
        private void addNewDeadKeySequence()
        {
            DeadKeySequence seq=new DeadKeySequence();
            int index=sequences.size();
            sequences.add(seq);
            fireTableRowsInserted(index,index);
        }
        /** Called to remove a empty DeadKeySequence */
        private void removeDeadKeySequence(int index)
        {
            if (index<sequences.size())
            {
                sequences.remove(index);
                fireTableRowsDeleted(index,index);
            };
        }
        /** Update the DeadKeySequences */
        private void updateDeadkeysequences()
        {
            ArrayList oldsequences=(ArrayList) deadKeySequences.clone();
            deadKeySequences.clear();
            int m=sequences.size();
            for (int i=0;i<m;i++)
            {
                if (((DeadKeySequence) sequences.get(i)).next>0)
                    deadKeySequences.add(sequences.get(i));
            };
            if (!compileKeySequences())
            {
                // We have a incorrect key sequence rollback
                deadKeySequences=oldsequences;
                compileKeySequences();
            };
        }
    }
    /**
     * Renderer for the DeadKeySequence
     */
    private class DeadKeySequenceCellRenderer extends JLabel implements TableCellRenderer
    {
        /**  Returns the component used for drawing the cell.  This method is
         *  used to configure the renderer appropriately before drawing.
         *
         * @param	table		the <code>JTable</code> that is asking the
         * 				renderer to draw; can be <code>null</code>
         * @param	value		the value of the cell to be rendered.  It is
         * 				up to the specific renderer to interpret
         * 				and draw the value.  For example, if
         * 				<code>value</code>
         * 				is the string "true", it could be rendered as a
         * 				string or it could be rendered as a check
         * 				box that is checked.  <code>null</code> is a
         * 				valid value
         * @param	isSelected	true if the cell is to be rendered with the
         * 				selection highlighted; otherwise false
         * @param	hasFocus	if true, render cell appropriately.  For
         * 				example, put a special border on the cell, if
         * 				the cell can be edited, render in the color used
         * 				to indicate editing
         * @param	row	        the row index of the cell being drawn.  When
         * 				drawing the header, the value of
         * 				<code>row</code> is -1
         * @param	column	        the column index of the cell being drawn
         *
         */
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
        {
            if (column==0)
                setText(((DeadKeySequence) value).toString());
            else
                setText(String.valueOf(value));
            return this;
        }
    }
    /*
     * Editor to pick up a character
     */
    private class CharCellEditor extends JComboBox implements TableCellEditor,ItemListener
    {
        ArrayList listeners=new ArrayList();
        char current,old;
        int column,row;
        public CharCellEditor()
        {
            for (int i=0;i<256;i++)
                if (!Character.isISOControl((char) i))
                    addItem(String.valueOf((char) i));
            addItemListener(this);
        }
        /** Adds a listener to the list that's notified when the editor
         * stops, or cancels editing.
         *
         * @param	l		the CellEditorListener
         *
         */
        public void addCellEditorListener(CellEditorListener l)
        {
            listeners.add(l);
        }
        
        /** Tells the editor to cancel editing and not accept any partially
         * edited value.
         *
         */
        public void cancelCellEditing()
        {
            current=old;
        }
        
        /** Returns the value contained in the editor.
         * @return the value contained in the editor
         *
         */
        public Object getCellEditorValue()
        {
            return new Character(current);
        }
        
        /**  Sets an initial <code>value</code> for the editor.  This will cause
         *  the editor to <code>stopEditing</code> and lose any partially
         *  edited value if the editor is editing when this method is called. <p>
         *
         *  Returns the component that should be added to the client's
         *  <code>Component</code> hierarchy.  Once installed in the client's
         *  hierarchy this component will then be able to draw and receive
         *  user input.
         *
         * @param	table		the <code>JTable</code> that is asking the
         * 				editor to edit; can be <code>null</code>
         * @param	value		the value of the cell to be edited; it is
         * 				up to the specific editor to interpret
         * 				and draw the value.  For example, if value is
         * 				the string "true", it could be rendered as a
         * 				string or it could be rendered as a check
         * 				box that is checked.  <code>null</code>
         * 				is a valid value
         * @param	isSelected	true if the cell is to be rendered with
         * 				highlighting
         * @param	row     	the row of the cell being edited
         * @param	column  	the column of the cell being edited
         * @return	the component for editing
         *
         */
        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
        {
            Character character=(Character) value;
            current=character.charValue();
            this.column=column;
            this.row=row;
            ListModel mdl=getModel();
            int m=mdl.getSize();
            for (int i=0;i<m;i++)
            {
                if (((String) mdl.getElementAt(i)).charAt(0)==character.charValue())
                {
                    setSelectedIndex(i);
                    break;
                };
            };
            old=current;
            return this;
        }
        
        /** Asks the editor if it can start editing using <code>anEvent</code>.
         * <code>anEvent</code> is in the invoking component coordinate system.
         * The editor can not assume the Component returned by
         * <code>getCellEditorComponent</code> is installed.  This method
         * is intended for the use of client to avoid the cost of setting up
         * and installing the editor component if editing is not possible.
         * If editing can be started this method returns true.
         *
         * @param	anEvent		the event the editor should use to consider
         * 				whether to begin editing or not
         * @return	true if editing can be started
         * @see #shouldSelectCell
         *
         */
        public boolean isCellEditable(EventObject anEvent)
        {
            return true;
        }
        
        /** Removes a listener from the list that's notified
         *
         * @param	l		the CellEditorListener
         *
         */
        public void removeCellEditorListener(CellEditorListener l)
        {
            listeners.remove(l);
        }
        
        /** Returns true if the editing cell should be selected, false otherwise.
         * Typically, the return value is true, because is most cases the editing
         * cell should be selected.  However, it is useful to return false to
         * keep the selection from changing for some types of edits.
         * eg. A table that contains a column of check boxes, the user might
         * want to be able to change those checkboxes without altering the
         * selection.  (See Netscape Communicator for just such an example)
         * Of course, it is up to the client of the editor to use the return
         * value, but it doesn't need to if it doesn't want to.
         *
         * @param	anEvent		the event the editor should use to start
         * 				editing
         * @return	true if the editor would like the editing cell to be selected;
         *    otherwise returns false
         * @see #isCellEditable
         *
         */
        public boolean shouldSelectCell(EventObject anEvent)
        {
            return true;
        }
        
        /** Tells the editor to stop editing and accept any partially edited
         * value as the value of the editor.  The editor returns false if
         * editing was not stopped; this is useful for editors that validate
         * and can not accept invalid entries.
         *
         * @return	true if editing was stopped; false otherwise
         *
         */
        public boolean stopCellEditing()
        {
            return true;
        }
        
        /** Invoked when an item has been selected or deselected by the user.
         * The code written for this method performs the operations
         * that need to occur when an item is selected (or deselected).
         *
         */
        public void itemStateChanged(ItemEvent e)
        {
            String charstring=(String) e.getItem();
            current=charstring.charAt(0);
            model.setValueAt(new Character(current), row,column);
            model.fireTableDataChanged();
        }
        
    }
    /*
     * Editor for the DeadKeySequence
     */
    
    private class DeadKeySequenceCellEditor extends JTextField implements TableCellEditor,KeyListener
    {
        ArrayList listeners=new ArrayList();
        DeadKeySequence current=null;
        DeadKeySequence old=null;
        int column;
        int row;
        public DeadKeySequenceCellEditor()
        {
            setEditable(false);
            addKeyListener(this);
        }
        /** Adds a listener to the list that's notified when the editor
         * stops, or cancels editing.
         *
         * @param	l		the CellEditorListener
         *
         */
        public void addCellEditorListener(CellEditorListener l)
        {
            listeners.add(l);
        }
        
        /** Tells the editor to cancel editing and not accept any partially
         * edited value.
         *
         */
        public void cancelCellEditing()
        {
            current=old;
        }
        
        /** Returns the value contained in the editor.
         * @return the value contained in the editor
         *
         */
        public Object getCellEditorValue()
        {
            return current;
        }
        
        /**  Sets an initial <code>value</code> for the editor.  This will cause
         *  the editor to <code>stopEditing</code> and lose any partially
         *  edited value if the editor is editing when this method is called. <p>
         *
         *  Returns the component that should be added to the client's
         *  <code>Component</code> hierarchy.  Once installed in the client's
         *  hierarchy this component will then be able to draw and receive
         *  user input.
         *
         * @param	table		the <code>JTable</code> that is asking the
         * 				editor to edit; can be <code>null</code>
         * @param	value		the value of the cell to be edited; it is
         * 				up to the specific editor to interpret
         * 				and draw the value.  For example, if value is
         * 				the string "true", it could be rendered as a
         * 				string or it could be rendered as a check
         * 				box that is checked.  <code>null</code>
         * 				is a valid value
         * @param	isSelected	true if the cell is to be rendered with
         * 				highlighting
         * @param	row     	the row of the cell being edited
         * @param	column  	the column of the cell being edited
         * @return	the component for editing
         *
         */
        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
        {
            this.column=column;
            this.row=row;
            current=(DeadKeySequence) value;
            old=current.copy();
            table.setRowSelectionInterval(row,row);
            table.setColumnSelectionInterval(column,column);
            if (column==0)
                setText(current.toString());
            else
                setText(String.valueOf(current.unicodechar));
            return this;
        }
        
        /** Asks the editor if it can start editing using <code>anEvent</code>.
         * <code>anEvent</code> is in the invoking component coordinate system.
         * The editor can not assume the Component returned by
         * <code>getCellEditorComponent</code> is installed.  This method
         * is intended for the use of client to avoid the cost of setting up
         * and installing the editor component if editing is not possible.
         * If editing can be started this method returns true.
         *
         * @param	anEvent		the event the editor should use to consider
         * 				whether to begin editing or not
         * @return	true if editing can be started
         * @see #shouldSelectCell
         *
         */
        public boolean isCellEditable(EventObject anEvent)
        {
            return true;
        }
        
        /** Invoked when a key has been pressed.
         * See the class description for {@link KeyEvent} for a definition of
         * a key pressed event.
         *
         */
        public void keyPressed(KeyEvent e)
        {
        }
        
        /** Invoked when a key has been released.
         * See the class description for {@link KeyEvent} for a definition of
         * a key released event.
         *
         */
        public void keyReleased(KeyEvent e)
        {
            if (column==0)
            {
                if (e.getKeyCode()==KeyEvent.VK_F1)
                {
                    current.next=0;
                    setText("");
                }
                else if (current.next<current.keycodemodifierschar.length)
                {
                    current.keycodemodifierschar[current.next]=new KeyCodeModifiersChar(e.getKeyCode(), e.getModifiers(), e.getKeyChar(),e.isAltGraphDown());
                    current.next=current.next+1;
                    current.copy();
                    setText(current.toString());
                    model.setValueAt(current, row,column);
                    model.fireTableDataChanged();
                };
            };
        }
        
        /** Invoked when a key has been typed.
         * See the class description for {@link KeyEvent} for a definition of
         * a key typed event.
         *
         */
        public void keyTyped(KeyEvent e)
        {
        }
        
        /** Removes a listener from the list that's notified
         *
         * @param	l		the CellEditorListener
         *
         */
        public void removeCellEditorListener(CellEditorListener l)
        {
            listeners.remove(l);
        }
        
        /** Returns true if the editing cell should be selected, false otherwise.
         * Typically, the return value is true, because is most cases the editing
         * cell should be selected.  However, it is useful to return false to
         * keep the selection from changing for some types of edits.
         * eg. A table that contains a column of check boxes, the user might
         * want to be able to change those checkboxes without altering the
         * selection.  (See Netscape Communicator for just such an example)
         * Of course, it is up to the client of the editor to use the return
         * value, but it doesn't need to if it doesn't want to.
         *
         * @param	anEvent		the event the editor should use to start
         * 				editing
         * @return	true if the editor would like the editing cell to be selected;
         *    otherwise returns false
         * @see #isCellEditable
         *
         */
        public boolean shouldSelectCell(EventObject anEvent)
        {
            return true;
        }
        
        /** Tells the editor to stop editing and accept any partially edited
         * value as the value of the editor.  The editor returns false if
         * editing was not stopped; this is useful for editors that validate
         * and can not accept invalid entries.
         *
         * @return	true if editing was stopped; false otherwise
         *
         */
        public boolean stopCellEditing()
        {
            return true;
        }
        
    }
}
    /*
     * Private class to hold the KeyCode, KeyModifiers and KeyChar
     */
class KeyCodeModifiersChar implements Comparable
{
    protected int keycode;
    protected int modifiers;
    protected int keychar;
    protected boolean isAltGraph;
    
    protected KeyCodeModifiersChar(int keycode,int modifiers,int keychar,boolean isAltGraph)
    {
        this.keycode=keycode;
        this.modifiers=modifiers;
        this.keychar=keychar;
        this.isAltGraph=isAltGraph;
    }
    
    /** Compares this object with the specified object for order.  Returns a
     * negative integer, zero, or a positive integer as this object is less
     * than, equal to, or greater than the specified object.<p>
     *
     * In the foregoing description, the notation
     * <tt>sgn(</tt><i>expression</i><tt>)</tt> designates the mathematical
     * <i>signum</i> function, which is defined to return one of <tt>-1</tt>,
     * <tt>0</tt>, or <tt>1</tt> according to whether the value of <i>expression</i>
     * is negative, zero or positive.
     *
     * The implementor must ensure <tt>sgn(x.compareTo(y)) ==
     * -sgn(y.compareTo(x))</tt> for all <tt>x</tt> and <tt>y</tt>.  (This
     * implies that <tt>x.compareTo(y)</tt> must throw an exception iff
     * <tt>y.compareTo(x)</tt> throws an exception.)<p>
     *
     * The implementor must also ensure that the relation is transitive:
     * <tt>(x.compareTo(y)&gt;0 &amp;&amp; y.compareTo(z)&gt;0)</tt> implies
     * <tt>x.compareTo(z)&gt;0</tt>.<p>
     *
     * Finally, the implementer must ensure that <tt>x.compareTo(y)==0</tt>
     * implies that <tt>sgn(x.compareTo(z)) == sgn(y.compareTo(z))</tt>, for
     * all <tt>z</tt>.<p>
     *
     * It is strongly recommended, but <i>not</i> strictly required that
     * <tt>(x.compareTo(y)==0) == (x.equals(y))</tt>.  Generally speaking, any
     * class that implements the <tt>Comparable</tt> interface and violates
     * this condition should clearly indicate this fact.  The recommended
     * language is "Note: this class has a natural ordering that is
     * inconsistent with equals."
     *
     * @param   o the Object to be compared.
     * @return  a negative integer, zero, or a positive integer as this object
     * 		is less than, equal to, or greater than the specified object.
     *
     * @throws ClassCastException if the specified object's type prevents it
     *         from being compared to this Object.
     *
     */
    public int compareTo(Object o)
    {
        if (o instanceof KeyCodeModifiersChar)
        {
            KeyCodeModifiersChar kmc=(KeyCodeModifiersChar) o;
            if (kmc.keycode==keycode)
                if (kmc.keychar==keychar)
                    if (kmc.modifiers==modifiers)
                        if (kmc.isAltGraph==isAltGraph)
                            return 0;
                        else if (isAltGraph)
                            return -1;
                        else
                            return 1;
                    else
                        return kmc.modifiers-modifiers;
                else
                    return kmc.keychar-keychar;
            else
                return kmc.keycode-keycode;
        }
        else
        {
            throw new ClassCastException();
        }
    }
    
}
