/*
 * KeySequence.java
 *
 * Created on May 27, 2001, 10:30 AM
 */

package IDE.Configuration;

import java.awt.event.*;

/**
 *
 * @author  marc
 * @version
 */
/** Class used by keymap to retrieve action based on KeySequences, we have a maximum of 10 KeySequences */
public class KeySequence implements Comparable
{
    private int[] keycode;
    private int[] keymodifiers;
    private int next;
    
    public KeySequence()
    {
        next=0;
        keycode=new int[10];
        keymodifiers=new int[10];
    }
    public KeySequence(int keycode, int modifiers)
   {
       next=1;
       this.keycode=new int[10];
       this.keymodifiers=new int[10];
       this.keycode[0]=keycode;
       this.keymodifiers[0]=modifiers;
   }
   public KeySequence(int[][] keymod)
   {
       next=0;
       this.keycode=new int[10];
       this.keymodifiers=new int[10];
       for (int i=0;i<10 && i<keymod.length;i++)
       {
           keycode[i]=keymod[i][0];
           keymodifiers[i]=keymod[i][1];
           if (keycode[i]!=0 || keymodifiers[i]!=0)
               next=i;
       };
       next++;
   }
   public KeySequence copy()
   {
       return new KeySequence(getKeys());
   }
   protected int[][] getKeys()
   {
       int[][] keys=new int[10][2];
       for (int i=0;i<10;i++)
       {
           keys[i][0]=keycode[i];
           keys[i][1]=keymodifiers[i];
       };
       return keys;
   }
    /* Defines the equal method used for hashtable */
    public boolean equals(Object obj)
    {
        KeySequence KeySequence;
        if (obj instanceof KeySequence)
        {
            KeySequence=(KeySequence) obj;
            for (int i=0;i<10;i++)
                if (KeySequence.keycode[i]!=keycode[i] || KeySequence.keymodifiers[i]!=keymodifiers[i])
                    return false;
            return true;
        }
        else
        {
            return false;
        }
    }
        /* Add a key to a key stroke */
    public void addKey(int keycode,int keymodifiers)
    {
        if (next<10)
        {
            this.keycode[next]=keycode;
            this.keymodifiers[next]=keymodifiers;
            next++;
        };
    }
    /** Size of the key sequence */
    public int size()
    {
        /*
        int size;
        for (size=0;size<10 && keymodifiers[size]!=0 || keycode[size]!=0;size++);
        return size;
         **/
        return next;
    }
    /* Returns a string representing the key stroke */
    public String toString()
    {
        StringBuffer str=new StringBuffer();
        for (int i=0;i<10;i++)
        {
            if (keycode[i]!=0 || keymodifiers[i]!=0)
            {
                str.append("<");
                if (keymodifiers[i]!=0)
                    str.append(KeyEvent.getKeyModifiersText(keymodifiers[i])+"-");
                str.append(KeyEvent.getKeyText(keycode[i]));
                str.append(">");
            };
        };
        return str.toString();
    }
    /** Returns a short version of the keystroke minus the n-first entries */
    public String toStringShort(int n)
    {
        StringBuffer str=new StringBuffer();
        for (int i=n;i<10;i++)
        {
            if (keymodifiers[i]!=0)
                str.append(KeyEvent.getKeyModifiersText(keymodifiers[i])+"-");
            if (keycode[i]!=0)
                str.append(KeyEvent.getKeyText(keycode[i]));
        };
        return str.toString();           
    }
    /** Check if this key stroke is a sub key stroke of another */
    public boolean isPartOf(KeySequence seq)
    {
        if (seq==null)
            return false;
        for (int i=0;i<next;i++)
            if (keymodifiers[i]!=seq.keymodifiers[i] || keycode[i]!=seq.keycode[i])
               return false;
        return true;
    }
    
    /** Used to sort key strokes */
    public int compareTo(Object o)
    {
        KeySequence KeySequence;
        if (o instanceof KeySequence)
        {
            KeySequence=(KeySequence) o;
            for (int i=0;i<10;i++)
            {
                if (keymodifiers[i]<KeySequence.keymodifiers[i])
                {
                    return -1;
                }
                else if (keymodifiers[i]==KeySequence.keymodifiers[i])
                {
                    if (keycode[i]<KeySequence.keycode[i])
                    {
                        return -1;
                    }
                    else if (keycode[i]>KeySequence.keycode[i])
                    {
                        return 1;
                    }
                }
                else if (keymodifiers[i]>KeySequence.keymodifiers[i])
                {
                    return 1;
                };
            }
            return 0;
        }
        else
        {
            return -1;
        }
    }
    
}