/*
 * Mode.java
 *
 * Created on June 23, 2001, 1:00 PM
 */

package IDE.Configuration;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import IDE.*;
import IDE.Messaging.*;
import IDE.LispScripting.*;

/**
 *  This class represents the different modes from the IDE
 * A mode defines the following :
 *  - The menus in the IDE
 *  - The butonbar
 *  - The keybindings
 *  - The general popup menu
 * @author  marc
 * @version
 */
public class Mode
{
    /** Field to indicate the name of a action */
    public static final String ACTIONNAME="ACTION";
    /** Contains the different defined modes */
    private static Hashtable modes=new Hashtable();
    /** Contains all the name to action mapping  */
    private static TreeMap nameToAction=new TreeMap();
    /** Contains a action to name mapping (used for serialization) */
    private static HashMap actionToName=new HashMap();
    /** Name of the mode */
    private String name;
    /** Contains the name of all the menus */
    protected TreeMap menus=new TreeMap();
    /** Menu bar associated with the mode */
    protected JMenuBar menubar;
    /** Panel containing the toolbars */
    protected JPanel toolbars;
    /** Keymap associated with this mode */
    protected ActionKeyMap keymap;
    /** Popupmenu associated with this mode */
    protected JPopupMenu popupmenu;
    /** Contains the names of all the menus in the popupmenu */
    protected HashMap popupmenus=new HashMap();
    /** Hash table mapping toolbar name on toolbar */
    protected HashMap nameToToolbar=new HashMap();
    /** Creates new Mode  with name name */
    public Mode(String name)
    {
        // Give the mode a name and add it to the modes
        this.name=name;
        modes.put(name,this);
        // Initialize the menu,buttonbar,keymap and popup menu
        menubar=new JMenuBar();
        toolbars=new JPanel();
        toolbars.setLayout(new BoxLayout(toolbars, BoxLayout.Y_AXIS));
        keymap=new ActionKeyMap();
        popupmenu=new JPopupMenu();
    }
    /** Make a clone of this mode , and giv it the name name */
    public Mode createCopy(String name)
    {
        Mode mode=new Mode(name);
        
        // Clone the menus
        copyMenus(mode);
        // Clone the toolbars
        copyToolbars(mode);
        //Clone the binding
        copyBindings(mode);
        // Clone the popupmenu
        copyPopupMenu(mode);
        // return the cloned menu
        return mode;
    }
    /** Copy all the menus to the new mode */
    private void copyMenus(Mode mode)
    {
        int l=menubar.getMenuCount();
        JMenu menu;
        for (int i=0;i<l;i++)
        {
            menu=menubar.getMenu(i);
            mode.menubar.add(copyMenu(mode,menu,null));
        };
    }
    /** Copy the menu to the new mode */
    private JMenu copyMenu(Mode mode,JMenu menu,String path)
    {
        int l=menu.getMenuComponentCount();
        Component item;
        JCheckBoxMenuItem newcb;
        AbstractManagedAction action;
        JMenu newmenu=new JMenu(menu.getText());
        newmenu.setMnemonic(menu.getMnemonic());
        if (path==null)
        {
            path="'"+menu.getText()+"'";
            mode.menus.put(path,newmenu);
        }
        else
        {
            path=path+"/'"+menu.getText()+"'";
            mode.menus.put(path,newmenu);
        };
        for (int i=0;i<l;i++)
        {
            item=menu.getMenuComponent(i);
            if (item instanceof JMenu)
            {
                newmenu.add(copyMenu(mode,(JMenu)  item,path));
            }
            else if (item instanceof JSeparator)
            {
                newmenu.addSeparator();
            }
            else if (item instanceof JCheckBoxMenuItem)
            {
                newcb=new JCheckBoxMenuItem(((JCheckBoxMenuItem) item).getText());
                action=(AbstractManagedAction) ((JCheckBoxMenuItem) item).getAction();
                if (action!=null)
                    action.addUser(newcb);
                newcb.setAction(action);
                newcb.setState(((JCheckBoxMenuItem) item).getState());
                newcb.setIcon(null);
                newmenu.add(newcb);
            }
            else if (item instanceof AbstractButton)
            {
                action=(AbstractManagedAction) ((AbstractButton) item).getAction();
                JMenuItem newitem=new JMenuItem(action);
                newitem.setIcon(null);
                if (action!=null)
                    action.addUser(newitem);
                newmenu.add(newitem);
            };
        }
        return newmenu;
    }
    /** Copy the popup menu */
    private void copyPopupMenu(Mode mode)
    {
        Component item;
        AbstractManagedAction action;
        JMenuItem menuitem;
        int m=popupmenu.getComponentCount();
        for (int i=0;i<m;i++)
        {
            item=popupmenu.getComponent(i);
            if (item instanceof JMenu)
            {
                mode.popupmenu.add(copyPopupMenuMenu(mode,(JMenu) item,null));
            }
            else if (item instanceof JSeparator)
            {
                mode.popupmenu.addSeparator();
            }
            else if (item instanceof AbstractButton)
            {
                action=(AbstractManagedAction) ((AbstractButton) item).getAction();
                if (action!=null)
                {
                    menuitem=mode.popupmenu.add(action);
                    action.addUser(menuitem);
                }
            };
        };
    }
    /**  Make a copy of the menu for use in a popupmenu in the specified mode */
    private JMenu copyPopupMenuMenu(Mode mode,JMenu menu,String path)
    {
        JMenu copymenu=new JMenu();
        Component item;
        JMenuItem menuitem;
        AbstractManagedAction action;
        int m=menu.getMenuComponentCount();
        copymenu=new JMenu(menu.getText());
        if (path==null)
        {
            path="'"+menu.getText()+"'";
            mode.popupmenus.put(path,copymenu);
        }
        else
        {
            path=path+"/'"+menu.getText()+"'";
            mode.popupmenus.put(path,copymenu);
        };
        for (int i=0;i<m;i++)
        {
            item=menu.getMenuComponent(i);
            if (item instanceof JMenu)
            {
                copymenu.add(copyPopupMenuMenu(mode,(JMenu) item,path));
            }
            else if (item instanceof JSeparator)
            {
                copymenu.addSeparator();
            }
            else if (item instanceof AbstractButton)
            {
                action=(AbstractManagedAction) ((AbstractButton) item).getAction();
                if (action!=null)
                {
                    menuitem=copymenu.add(action);
                    action.addUser(menuitem);
                }
            };
        };
        return copymenu;
    }
    /** Copy the toolbars to the given mode, we do not copy the miscbar, it is used for tempory access  */
    private void copyToolbars(Mode mode)
    {
        int m=toolbars.getComponentCount();
        JToolBar toolbar,newtoolbar;
        JPanel buttonbar;
        JPanel miscbar;
        for (int i=0;i<m;i++)
        {
            toolbar=(JToolBar) toolbars.getComponent(i);
            newtoolbar=new JToolBar();
            newtoolbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
            newtoolbar.setName(toolbar.getName());
            buttonbar=new JPanel();
            buttonbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
            copyButtonBar((JPanel) toolbar.getComponent(0),buttonbar);
            miscbar=new JPanel();
            miscbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
            newtoolbar.add(buttonbar);
            newtoolbar.add(miscbar);
            mode.toolbars.add(newtoolbar);
            mode.nameToToolbar.put(toolbar.getName(), newtoolbar);
        };
    }
    /** Copy the button bar */
    private void copyButtonBar(JPanel oldbuttonbar,JPanel newbuttonbar)
    {
        int l=oldbuttonbar.getComponentCount();
        JButton button,newbutton;
        AbstractManagedAction action;
        for (int i=0;i<l;i++)
        {
            if (oldbuttonbar.getComponent(i) instanceof JButton)
            {
                button=(JButton) oldbuttonbar.getComponent(i);
                action=(AbstractManagedAction) button.getAction();
                newbutton=new JButton(action);
                if (action!=null)
                    action.addUser(newbutton);
                newbutton.setBorder(BorderFactory.createEtchedBorder());
                newbutton.setText(button.getText());
                newbuttonbar.add(newbutton);
            }
            else if (oldbuttonbar.getComponent(i) instanceof JToolBar.Separator)
            {
                newbuttonbar.add(new JToolBar.Separator());
            };
        }
    }
    /** Copy the bindings */
    private void copyBindings(Mode mode)
    {
        KeySequence[] sequences=keymap.getKeySequences();
        for (int i=0;i<sequences.length;i++)
            mode.bindActionToKeySequence((String) keymap.getAction(sequences[i]).getValue(ACTIONNAME),sequences[i]);
    }
    /** Add a new action to  a menu , creating the menus if needed */
    public void addActionToMenu(String path,String actionname)
    {
        JMenu menu=createMenu(path);
        AbstractManagedAction action=getAction(actionname);
        if (menu!=null && action!=null)
        {
            JMenuItem item=new JMenuItem(action);
            menu.add(item);
            action.addUser(item);
            item.setIcon(null);
        };
    }
    /** Add a new toolbar to the mode */
    public void addToolbar(String name)
    {
        if (nameToToolbar.get(name)!=null)
        {
            MessageHandler.warn("Try to create a existing toolbar");
            return;
        };
        JToolBar toolbar=new JToolBar();
        toolbar.setName(name);
        JPanel miscbar=new JPanel();
        JPanel buttonbar=new JPanel();
        toolbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
        buttonbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
        miscbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
        toolbar.add(buttonbar);
        toolbar.add(miscbar);
        // Register the toolbar
        nameToToolbar.put(name,toolbar);
        toolbars.add(toolbar);
    }
    /** Get the buttonbar of a named toolbar */
    private JPanel getButtonbarIn(String name)
    {
        JToolBar toolbar=(JToolBar) nameToToolbar.get(name);
        if (toolbar==null)
            return null;
        else
            return (JPanel) toolbar.getComponent(0);
    }
    /** Get the miscbar of a named toolbar */
    private JPanel getMiscBarIn(String name)
    {
        JToolBar toolbar=(JToolBar) nameToToolbar.get(name);
        if (toolbar==null)
            return null;
        else
            return (JPanel) toolbar.getComponent(1);
    }
    /** Add a new checkbox to a menu , creating the menus if needed */
    public void addCheckboxToMenu(String path,String actionname,boolean enable)
    {
        JMenu menu=createMenu(path);
        JCheckBoxMenuItem cb;
        AbstractManagedAction action=getAction(actionname);
        if (menu!=null && action!=null)
        {
            cb=new JCheckBoxMenuItem(action);
            action.addUser(cb);
            cb.setState(enable);
            menu.add(cb);
        };
    }
    /** Add a separator to the menu */
    public void addSeparatorToMenu(String path)
    {
        JMenu menu=createMenu(path);
        if (menu!=null)
            menu.addSeparator();
    }
    /*
     * Create a menu based on its menu path string , this string is of the form
     * m1/.../mn , where mi = 'menu text'
     * and its default state for the variable
     * Examples :  "'Edit'/'Copy'" , "'Edit'/'Syntax''/'ShowSyntax'
     * returns null if the syntax is incorrect
     */
    
    private JMenu createMenu(String path)
    {
        int i=0;
        int l=path.length();
        int b;
        StringBuffer name=new StringBuffer();
        boolean enabled=false;
        JMenu  menu=null,previous=null;
        
        while (i<l)
        {
            b=i;
            name.delete(0,name.length());
            i=parseName(path,i,name);
            if (i==-1)
                return null;
            /** Check if the menu already exist and create it if needed */
            menu=(JMenu) menus.get(path.substring(0,i-1));
            if (menu==null)
            {
                menu=new JMenu(name.toString());
                menus.put(path.substring(0,i-1),menu);
                if (previous==null)
                    menubar.add(menu);
                else
                    previous.add(menu);
            };
            previous=menu;
        };
        return menu;
    }
    /** Parse a name and store it in the stringbuffer */
    private int parseName(String path,int i,StringBuffer name)
    {
        int l=path.length();
        if (path.charAt(i)!='\'')
            return -1;
        i++;
        while (i<l && path.charAt(i)!='\'')
        {
            name.append(path.charAt(i));
            i++;
        };
        if (i<l)
            return i+2;
        else
            return -1;
    }
    
    /** Add a action to a popupmenu */
    public void addActionToPopup(String actionname)
    {
        AbstractManagedAction action=getAction(actionname);
        if (action!=null)
        {
            JMenuItem item=popupmenu.add(action);
            item.setIcon(null);
            action.addUser(item);
        };
    }
    /** Add a action to a menu in a popupmenu */
    public void addActionToPopup(String path,String actionname)
    {
        if (getAction(actionname)!=null)
        {
            JMenu menu=createPopupMenu(path);
            AbstractManagedAction action=getAction(actionname);
            JMenuItem item=menu.add(action);
            item.setIcon(null);
            action.addUser(item);
        };
    }
    /** Add a separator to the popup menu */
    public void addSeparatorToPopup(String path)
    {
        JMenu menu=createPopupMenu(path);
        if (menu==null)
            popupmenu.addSeparator();
    }
    /** Add  a menu to a popup menu */
    private JMenu createPopupMenu(String path)
    {
        int i=0;
        int l=path.length();
        int b;
        StringBuffer name=new StringBuffer();
        boolean enabled=false;
        JMenu  menu=null,previous=null;
        
        while (i<l)
        {
            b=i;
            name.delete(0,name.length());
            i=parseName(path,i,name);
            if (i==-1)
                return null;
            /** Check if the menu already exist and create it if needed */
            menu=(JMenu) popupmenus.get(path.substring(0,i-1));
            if (menu==null)
            {
                menu=new JMenu(name.toString());
                popupmenus.put(path.substring(0,i-1),menu);
                if (previous==null)
                    popupmenu.add(menu);
                else
                    popupmenu.add(menu);
            };
            previous=menu;
        };
        return menu;
    }
    /** Add a separator to the button bar */
    public void addSeparatorToButtonBar(String toolbar)
    {
        JPanel buttonbar=getButtonbarIn(toolbar);
        if (buttonbar==null)
        {
            MessageHandler.warn("Add seperator to buttonbar of unknown toolbar "+toolbar);
            return;
        };
        buttonbar.add(new JToolBar.Separator());
    }
    /** Add a button to the buttonbar for toolbar, actionname represents the action associated with the button */
    public void addButton(String toolbar,String actionname)
    {
        addButton(toolbar,actionname,"");
    }
    /** Add a button to the buttonbar for toolbar, actionanem is the action id, text is a string representing the text of the button */
    public void addButton(String toolbar,String actionname,String text)
    {
        AbstractManagedAction action=getAction(actionname);
        if (action==null)
        {
            MessageHandler.warn("Add button for unknown action");
            return;
        };
        JPanel buttonbar=(JPanel) getButtonbarIn(toolbar);
        if (buttonbar==null)
        {
            MessageHandler.warn("Add button to buttonbar of unknown toolbar "+toolbar);
            return;
        };
        JButton button=new JButton(action);
        button.setBorder(BorderFactory.createEtchedBorder());
        button.setText(text);
        action.addUser(button);
        buttonbar.add(button);
    }
    /** Add a slider to the miscbar */
    public void addMiscToMode(String toolbar,JComponent slider)
    {
        JPanel miscbar=getMiscBarIn(toolbar);
        if (miscbar==null)
        {
            MessageHandler.warn("Add component to miscbar of unknown toolbar "+toolbar);
            return;
        };
        miscbar.add(slider);
        miscbar.revalidate();
        miscbar.repaint();
        activate();
    }
    /** Remove the action from the miscbar */
    public void removeComponentFromMode(String toolbar,Component component)
    {
        JPanel miscbar=getMiscBarIn(toolbar);
        if (miscbar==null)
        {
            MessageHandler.warn("Add component to miscbar of unknown toolbar "+toolbar);
            return;
        };
        miscbar.remove(component);
        miscbar.revalidate();
        miscbar.repaint();
        activate();
    }
    /** Bind a action to a keystroke */
    public void bindActionToKeySequence(String action,KeySequence keystroke)
    {
        keymap.addAction(action,keystroke);
    }
    /** Bind a action to a keysequence */
    public void bindActionToKeySequence(String action,int key,int modifier)
    {
        keymap.addAction(action,new KeySequence(key,modifier));
    }
    /** Bind a action to a keysequence */
    public void bindActionToKeySequence(String action,int key1,int modifier1,int key2,int modifier2)
    {
        keymap.addAction(action,new KeySequence(new int[][]
        {
            {key1,modifier1},
            {key2,modifier2}
        }));
    }
    /**  Bind a action to a key sequence */
    public void bindActionToKeySequence(String action,int[][] keystroke)
    {
        keymap.addAction(action,new KeySequence(keystroke));
    }
    /** Add a action to list of actions */
    public static void addAction(String name,String shortDescription,String longDescription,AbstractManagedAction action)
    {
        action.putValue(ACTIONNAME,name);
        action.putValue(AbstractAction.NAME,shortDescription);
        action.putValue(AbstractAction.SHORT_DESCRIPTION,shortDescription);
        action.putValue(AbstractAction.LONG_DESCRIPTION,longDescription);
        nameToAction.put(name,action);
        actionToName.put(action,name);
    }
    /**  Get the action associated with a certain name */
    public static  AbstractManagedAction getAction(String name)
    {
        return (AbstractManagedAction) nameToAction.get(name);
    }
    /** Return the menu with a given name */
    public JMenu getMenu(String name)
    {
        return (JMenu) menus.get(name);
    }
    /** Return the name of this mode */
    public String getName()
    {
        return name;
    }
    /** Get the name of the action */
    public static String getNameAction(AbstractManagedAction action)
    {
        return (String) actionToName.get(action);
    }
    /** Get the number of nameToAction defined */
    public static int getActionCount()
    {
        return nameToAction.size();
    }
    /** Get the mode with a certain name */
    public static Mode getMode(String name)
    {
        Mode mode=(Mode) modes.get(name);
        if (mode==null)
        {
            MessageHandler.warn("Unknow mode="+name+", reverting to base mode");
            return (Mode) modes.get("basic");
        }
        return mode;
    }
    /** Get a array of strings containing the names of the different nodes */
    public static String[] getModes()
    {
        String[] modenames=new String[modes.size()];
        Iterator iter=modes.keySet().iterator();
        for (int i=0;i<modenames.length && iter.hasNext();i++)
            modenames[i]=(String) iter.next();
        Arrays.sort(modenames);
        return modenames;
    }
    /** Returns the keymap for this mode */
    public ActionKeyMap getKeyMap()
    {
        return keymap;
    }
    /** Get the array of all the keymaps for the different modes */
    public static ActionKeyMap[] getKeyMaps()
    {
        Iterator iter=modes.values().iterator();
        ActionKeyMap[] keymaps=new ActionKeyMap[modes.size()];
        int i=0;
        while (iter.hasNext())
        {
            keymaps[i]=((Mode) iter.next()).keymap;
            i++;
        };
        return keymaps;
    }
    /** Get the popupmenu from this mode */
    public JPopupMenu getPopupMenu()
    {
        return popupmenu;
    };
    /** Get a iterator over the actions */
    public static Iterator getActionIterator()
    {
        return nameToAction.values().iterator();
    }
    
    /** Activate the mode */
    
    public void activate()
    {
        if (toolbars.getComponentCount()==0)
            toolbars.setBorder(BorderFactory.createEmptyBorder());
        else
            toolbars.setBorder(BorderFactory.createEtchedBorder());
        Main.setMode(this,menubar,toolbars,keymap);
    }
    
    /** Generate documentation, generate documentation about all the actions , their keybinding  and their menu binding and their equivalent lisp functions */
    public static void generateDocumentation()
    {
        try
        {
            File installdoc=new File(InstallParameters.getInstallDir()+File.separator+"doc"+File.separator+"actions.html");
            File sourcedoc=new File(InstallParameters.getSourceDir()+File.separator+"doc"+File.separatorChar+"actions.html");
            if (sourcedoc.exists())
                sourcedoc.delete();
            sourcedoc.createNewFile();
            BufferedWriter writer=new BufferedWriter(new FileWriter(installdoc));
            Iterator iter;
            AbstractManagedAction action;
            String actionname;
            String actionkey;
            String actiondescription;
            
            TreeMap keyToAction=new TreeMap();
            TreeMap descriptionToAction=new TreeMap();
            writer.write("<html>\n");
            writer.write("<head>\n");
            writer.write("<meta http-equiv=\"Content-Type\" content=\"text/html\">\n");
            writer.write("<title></title>\n");
            writer.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"style.css\">\n");
            writer.write("</head>\n");
            // Write the information sorted by action
            writer.write("<html>\n");
            writer.write("<h3><a name=\"actionssortedbyname\">Actions sorted by action name</a></h3>");
            writer.write("<table align=\"center\" frame=\"box\" border=\"1\">\n");
            writer.write("  <tbody>\n");
            writer.write("  <thead>\n");
            writer.write("     <tr>\n");
            writer.write("         <td>action</td><td>key</td><td>Lisp Function</td><td>Description</td>\n");
            writer.write("     </tr>\n");
            writer.write("  </thead>\n");
            iter=nameToAction.values().iterator();
            while (iter.hasNext())
            {
                action=(AbstractManagedAction) iter.next();
                // Get the action properties
                actionname=(String) action.getValue(ACTIONNAME);
                actiondescription=(String) action.getValue(AbstractAction.LONG_DESCRIPTION);
                actionkey=action.getValue(ActionKeyMap.KEYSEQUENCE) instanceof KeySequence  ? replaceSpecial(((KeySequence) action.getValue(ActionKeyMap.KEYSEQUENCE)).toString()) : "";
                // Fill up the treemaps to sort in other directions
                if (actionkey.length()!=0)
                    keyToAction.put(actionkey,action);
                descriptionToAction.put(actiondescription,action);
                // Write the HTML code
                writer.write("    <tr>\n");
                writer.write("         <td>"+actionname+"</td>\n");
                writer.write("         <td>"+actionkey+"</td>\n");
                writer.write("         <td>IDE-"+GenerateLispCode.createFunctionName(action)+"</td>\n");
                writer.write("         <td>"+actiondescription+"</td>\n");
                writer.write("    </tr>\n");
            };
            // Write the action sorted by description
            writer.write("  </tbody>\n");
            writer.write("</table>\n");
            writer.write("<h3><a name=\"actionssortedbydescription\">Actions sorted by description</a></h3>");
            writer.write("<table align=\"center\" frame=\"box\" border=\"1\">\n");
            writer.write("  <tbody>\n");
            writer.write("  <thead>\n");
            writer.write("     <tr>\n");
            writer.write("         <td>description</td><td>action</td><td>key</td><td>Lisp Function</td>\n");
            writer.write("     </tr>\n");
            writer.write("  </thead>\n");
            iter=descriptionToAction.values().iterator();
            while (iter.hasNext())
            {
                action=(AbstractManagedAction) iter.next();
                // Get the action properties
                actionname=(String) action.getValue(ACTIONNAME);
                actiondescription=(String) action.getValue(AbstractAction.SHORT_DESCRIPTION);
                actionkey=action.getValue(ActionKeyMap.KEYSEQUENCE) instanceof KeySequence  ? replaceSpecial(((KeySequence) action.getValue(ActionKeyMap.KEYSEQUENCE)).toString()) : "";
                // Write the HTML code
                writer.write("    <tr>\n");
                writer.write("         <td>"+actiondescription+"</td>\n");
                writer.write("         <td>"+actionname+"</td>\n");
                writer.write("         <td>"+actionkey+"</td>\n");
                writer.write("         <td>IDE-"+GenerateLispCode.createFunctionName(action)+"</td>\n");
                writer.write("    </tr>\n");
            };
            // Write the actions sorted by key sequence
            writer.write("  </tbody>\n");
            writer.write("</table>\n");
            writer.write("<h3><a name=\"actionssortedbykeysequence\">Actions sorted by key sequence</a></h3>");
            writer.write("<table align=\"center\" frame=\"box\" border=\"1\">\n");
            writer.write("  <tbody>\n");
            writer.write("  <thead>\n");
            writer.write("     <tr>\n");
            writer.write("         <td>key</td><td>action</td><td>Lisp Function</td><td>Description</td>\n");
            writer.write("     </tr>\n");
            writer.write("  </thead>\n");
            iter=keyToAction.values().iterator();
            while (iter.hasNext())
            {
                action=(AbstractManagedAction) iter.next();
                // Get the action properties
                actionname=(String) action.getValue(ACTIONNAME);
                actiondescription=(String) action.getValue(AbstractAction.SHORT_DESCRIPTION);
                actionkey=action.getValue(ActionKeyMap.KEYSEQUENCE) instanceof KeySequence  ? replaceSpecial(((KeySequence) action.getValue(ActionKeyMap.KEYSEQUENCE)).toString()) : "";
                // Fill up the treemaps to sort in other directions
                if (actionkey.length()!=0)
                    keyToAction.put(actionkey,action);
                descriptionToAction.put(actiondescription,action);
                // Write the HTML code
                writer.write("    <tr>\n");
                writer.write("         <td>"+actionkey+"</td>\n");
                writer.write("         <td>"+actionname+"</td>\n");
                writer.write("         <td>IDE-"+GenerateLispCode.createFunctionName(action)+"</td>\n");
                writer.write("         <td>"+actiondescription+"</td>\n");
                writer.write("    </tr>\n");
            };
            writer.write("  </tbody>\n");
            writer.write("</table>\n");
            // Write the menu structure of the different modes
            Iterator modeIter=modes.keySet().iterator();
            Mode mode;
            String modename;
            String menustring;
            JMenuItem menuitem;
            JMenu menu;
            int m;
            
            writer.write("<h3><a name=\"menubinding\">Menu bindings</a></h3>\n");
            writer.write("<ul>\n");
            while (modeIter.hasNext())
            {
                modename=(String) modeIter.next();
                mode=(Mode) modes.get(modename);
                iter=mode.menus.keySet().iterator();
                while (iter.hasNext())
                {
                    menustring=(String) iter.next();
                    menuitem=(JMenuItem) mode.menus.get(menustring);
                    writer.write("<li>\n");
                    writer.write("<p><b>"+menuToHTML(menustring)+"</b></p>\n");
                    writer.write("<ul>\n");
                    if (menuitem instanceof JMenu)
                    {
                        menu=(JMenu) menuitem;
                        m=menu.getMenuComponentCount();
                        for (int i=0;i<m;i++)
                        {
                            menuitem=menu.getItem(i);
                            if (menuitem!=null)
                            {
                                action=(AbstractManagedAction) menuitem.getAction();
                                if (action!=null)
                                {
                                    writer.write("<dl>\n");
                                    writer.write("<dt>"+action.getValue(Mode.ACTIONNAME)+"</dt>\n");
                                    writer.write("<dd>"+action.getValue(AbstractAction.LONG_DESCRIPTION)+"</dd>\n");
                                    writer.write("</dl>\n");
                                };
                            }
                        };
                    };
                    writer.write("</li>\n");
                    writer.write("</ul>\n");
                }
            };
            writer.write("</ul>\n");
            // Close the generated file
            writer.write("</html>\n");
            writer.close();
            // Make a copy in the source directory if possible
            if (new File(InstallParameters.getSourceDir()).exists())
            {
                BufferedReader reader=new BufferedReader(new FileReader(installdoc));
                writer=new BufferedWriter(new FileWriter(sourcedoc));
                int c;
                while ((c=reader.read())!=-1)
                    writer.write(c);
                writer.close();
            };
        }
        catch (Exception e)
        {
            MessageHandler.report("Error creating documentation, "+e.getMessage(),e);
        }
    }
    /** Convert a menu of the form 'ddd'/'ddd'... in the form &lt;ddd&gt;/&lt;ddd&gt; */
    private static String menuToHTML(String menu)
    {
        StringBuffer buffer=new StringBuffer();
        int count=0;
        int m=menu.length();
        for (int i=0;i<m;i++)
        {
            if (menu.charAt(i)=='\'')
            {
                if (count % 2 == 0)
                    buffer.append("&lt;");
                else
                    buffer.append("&gt;");
                count++;
            }
            else
                buffer.append(menu.charAt(i));
        };
        return buffer.toString();
    }
    /** Ensure that the < and > are replaced by &gt; and &lt; */
    private static String replaceSpecial(String keysequence)
    {
        StringBuffer result=new StringBuffer();
        char c;
        int m=keysequence.length();
        for (int i=0;i<m;i++)
        {
            c=keysequence.charAt(i);
            if (c=='<')
                result.append("&lt;");
            else if (c=='>')
                result.append("&gt;");
            else
                result.append(c);
        };
        return result.toString();
    }
    /** Static method to save the keybinding for a certain mode */
    public static void addKeyBinding(String mode,String action,
    int k0,int k1,int k2,int k3,int k4,int k5,int k6,int k7,int k8,int k9,
    int m0,int m1,int m2,int m3,int m4,int m5,int m6,int m7,int m8,int m9)
    {
        int[][] keys=new int[][]
        {
            {k0,m0},
            {k1,m1},
            {k2,m2},
            {k3,m3},
            {k4,m4},
            {k5,m5},
            {k6,m6},
            {k7,m7},
            {k8,m8},
            {k9,m9}};
            KeySequence ks=new KeySequence(keys);
            getMode(mode).keymap.addAction(action,ks);
    }
    /** Save the keybindings stored in all the modes */
    public static void saveConfiguration(Writer writer)
    {
        XMLConfigClass cls=new XMLConfigClass("IDE.Configuration.Mode");
        XMLConfigMethod method;
        XMLConfigObject object;
        Iterator itermode=modes.values().iterator();
        //Iterator iterkeys;
        Mode mode;
        KeySequence ks;
        int[][] keys;
        
        while (itermode.hasNext())
        {
            mode=(Mode) itermode.next();
            KeySequence[] keysequences=mode.keymap.getKeySequences();
            for (int k=0;k<keysequences.length;k++)
            {
                ks=keysequences[k];
                if (mode.keymap.getAction(ks)!=null)
                {
                    method=new XMLConfigMethod("addKeyBinding");
                    method.addArgument(mode.name);
                    method.addArgument((String) mode.keymap.getAction(ks).getValue(ACTIONNAME));
                    keys=ks.getKeys();
                    for (int i=0;i<10;i++)
                        method.addArgument(keys[i][0]);
                    for (int i=0;i<10;i++)
                        method.addArgument(keys[i][1]);
                    cls.addMethod(method);
                };
            }
        };
        cls.serialize(writer);
    }
}
