/*
 * XMLConfig.java
 *
 * Created on December 26, 2001, 5:33 PM
 */

package IDE.Configuration;

import java.util.*;
import java.io.*;
import IDE.Messaging.*;
import IDE.Utilities.*;
/**
 * Class to save/read configuration information stored in a class based on XML code
 * @author  marc
 */
public class XMLConfigClass extends AbstractXMLConfig
{
    /** Name of the class */
    private String name;
    /** List of fields for this class */
    private Vector fields=new Vector();
    /** List of static methods to be called */
    private Vector methods=new Vector();
    /** Creates a new instance of XMLConfig */
    public XMLConfigClass(String name)
    {
        this.name=name;
    }
    /** Add a field to the class */
    public void addField(XMLConfigField field)
    {
        fields.add(field);
    }
    /** Add a method to the class */
    public void addMethod(XMLConfigMethod method)
    {
        methods.add(method);
    }
    /** Serialize the XMLConfigClass in XML format to a output stream */
    public void serialize(Writer writer)
    {
        int m;
        try
        {
            writer.write("<class :name=\""+Util.normalize(name)+"\">");
            m=fields.size();
            for (int i=0;i<m;i++)
            {
                writer.write("\n");
                ((XMLConfigField) fields.elementAt(i)).serialize(writer);
            };
            m=methods.size();
            for (int i=0;i<m;i++)
            {
                writer.write("\n");
                ((XMLConfigMethod) methods.elementAt(i)).serialize(writer);
            }
            writer.write("</class>\n");
        }
        catch (IOException e)
        {
            MessageHandler.report("Error writing XMLConfigClass",e);
        };
    }
    /** Do the configuration */
    public void configure()
    {
        try
        {
            Class cls=Class.forName(name);
            int m;
            m=fields.size();
            for (int i=0;i<m;i++)
                ((XMLConfigField) fields.elementAt(i)).configure(cls);
            m=methods.size();
            for (int i=0;i<m;i++)
                ((XMLConfigMethod) methods.elementAt(i)).configure(cls);
        }
        catch (ClassNotFoundException e)
        {
            errorDuringConfig=true;
            MessageHandler.report("Invalid class in configuration file",e);
        }
    }
}
