/*
 * XMLConfigField.java
 *
 * Created on December 26, 2001, 5:44 PM
 */

package IDE.Configuration;

import java.io.*;
import IDE.Messaging.*;
import IDE.Utilities.*;
import java.lang.reflect.*;
/**
 *
 * @author  marc
 */
public class XMLConfigField extends AbstractXMLConfig
{
    /** Name of the field */
    String name;
    /** Value of the field */
    Object value=null;
    /** Creates a new instance of XMLConfigField */
    public XMLConfigField(String name)
    {
        this.name=name;
    }
    public XMLConfigField(String name,int value)
    {
        this.name=name;
        this.value=new Integer(value);
    }
    public XMLConfigField(String name,long value)
    {
        this.name=name;
        this.value=new Long(value);
    }
    public XMLConfigField(String name,char value)
    {
        this.name=name;
        this.value=new Character(value);
    }
    public XMLConfigField(String name,double value)
    {
        this.name=name;
        this.value=new Double(value);
    }    
    public XMLConfigField(String name,boolean value)
    {
        this.name=name;
        this.value=new Boolean(value);
    }        
    public XMLConfigField(String name,String value)
    {
        this.name=name;
        this.value=value;
    }
    public XMLConfigField(String name,XMLConfigObject value)
    {
        this.name=name;
        this.value=value;
    }
    /** Set the value of the field */
    public void setValue(int value)
    {
        this.value=new Integer(value);
    }
    public void setValue(long value)
    {
        this.value=new Long(value);
    }   
    public void setValue(char value)
    {
        this.value=new Character(value);
    }
    public void setValue(double value)
    {
        this.value=new Double(value);
    }
    public void setValue(boolean value)
    {
        this.value=new Boolean(value);
    }
    public void setValue(String value)
    {
        this.value=value;
    }
    public void setValue(XMLConfigObject value)
    {
        this.value=value;
    }
    /** Serialize this ConfigField to the writer */
    public void serialize(Writer writer)
    {
        try
        {
            writer.write("<field :name=\""+Util.normalize(name)+"\">");
            if (value instanceof Integer)
                writer.write(((Integer) value).toString());
            else if (value instanceof Long)
                writer.write(((Long) value).toString());
            else if (value instanceof Double)
                writer.write(((Double) value).toString());
            else if (value instanceof Boolean)
                writer.write(((Boolean) value).toString());
            else if (value instanceof String)
                writer.write("\""+Util.normalize(((String) value))+"\"");
            else if (value instanceof Character)
                writer.write("'"+((int) ((Character) value).charValue())+"'");
            else if (value instanceof XMLConfigObject)
            {
                writer.write("\n");
                ((XMLConfigObject) value).serialize(writer);
                writer.write("\n");
            };
            writer.write("</field>");
        }
        catch (IOException e)
        {
            MessageHandler.report("Error writing XMLConfigField",e);
        };
    }
    /** Configure the field of the class */
    public void configure(Class cls)
    {
        try
        {
            Field field=cls.getField(name);
            if (value instanceof Integer)
                field.setInt(null,((Integer) value).intValue());
            else if (value instanceof Long)
                field.setLong(null,((Long) value).longValue());
            else if (value instanceof Double)
                field.setDouble(null,((Double) value).doubleValue());
            else if (value instanceof String)
                field.set(null,(String) value);
            else if (value instanceof Character)
                field.setChar(null, ((Character) value).charValue());
            else if (value instanceof Boolean)
                field.setBoolean(null, ((Boolean) value).booleanValue());
            else if (value instanceof XMLConfigObject)
                field.set(null,((XMLConfigObject) value).createObject());
        }
        catch (NoSuchFieldException e)
        {
            errorDuringConfig=true;
            MessageHandler.report("Undefined field value during load configuration ",e);
        }
        catch (SecurityException e)
        {
        }
        catch (IllegalArgumentException e)
        {
        }
        catch (IllegalAccessException e)
        {
        };
        
    }
}
