/*
 * XMLConfig.java
 *
 * Created on December 26, 2001, 5:33 PM
 */

package IDE.Configuration;

import java.io.*;
import IDE.Messaging.*;
import IDE.Utilities.*;
import java.util.*;
import java.lang.reflect.*;

/**
 *
 * @author  marc
 */
public class XMLConfigMethod extends AbstractXMLConfig
{
    /** Name of the method */
    private String name;
    /** Arguments of the method */
    private Vector arguments=new Vector();
    /** Creates a new instance of XMLConfig */
    public XMLConfigMethod(String name)
    {
        this.name=name;
    }
    /** Add a argument to the method call */
    public void addArgument(int arg)
    {
        arguments.addElement(new Integer(arg));
    }
    public void addArgument(long arg)
    {
        arguments.addElement(new Long(arg));
    }
    public void addArgument(double arg)
    {
        arguments.addElement(new Double(arg));
    }
    public void addArgument(boolean arg)
    {
        arguments.addElement(new Boolean(arg));
    }
    public void addArgument(char arg)
    {
        arguments.addElement(new Character(arg));
    }
    public void addArgument(String arg)
    {
        arguments.addElement(arg);
    }
    public void addArgument(XMLConfigObject arg)
    {
        arguments.addElement(arg);
    }
    /** Serialize this XMLConfig to a XML writer stream */
    public void serialize(Writer writer)
    {
        try 
        {
            writer.write("<method :name=\""+Util.normalize(name)+"\">");
            int m=arguments.size();
            for (int i=0;i<m;i++)
            {
                writer.write("\n");
                writer.write("<argument>");
                if (arguments.elementAt(i) instanceof Integer)
                    writer.write(((Integer) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof Long)
                    writer.write(((Long) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof Double)
                    writer.write(((Double) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof Boolean)
                    writer.write(((Boolean) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof String)
                    writer.write("\""+Util.normalize((String) arguments.elementAt(i))+"\"");
                else if (arguments.elementAt(i) instanceof Character)
                    writer.write("'"+((int) ((Character) arguments.elementAt(i)).charValue())+"'");
                else if (arguments.elementAt(i) instanceof XMLConfigObject)
                    ((XMLConfigObject) arguments.elementAt(i)).serialize(writer);
                writer.write("</argument>");
            };
            writer.write("</method>");
        }
        catch (IOException e)
        {
            MessageHandler.report("Error writing XMLConfigMethod",e);
        };
    }
    /** Exceute the method to do the configuration*/
    public void configure(Class cls)
    {
        int m=arguments.size();
        Object args[]=new Object[m]; // Get the arguments for the constructor
        Object arg;
        for (int i=0;i<m;i++)
        {
            arg=arguments.elementAt(i);
            if (arg instanceof Integer)
                args[i]=arg;
            else if (arg instanceof Long)
                args[i]=arg;
            else if (arg instanceof Double)
                args[i]=arg;
            else if (arg instanceof Boolean)
                args[i]=arg;
            else if (arg instanceof String)
                args[i]=arg;
            else if (arg instanceof Character)
                args[i]=arg;
            else if (arg instanceof XMLConfigObject)
                args[i]=((XMLConfigObject) arg).createObject();
        };
        callMethod(cls,name,args);
    }
    /** Do a call to a public method of a class */
    private Object callMethod(Class cls,String name,Object[] args)
    {
        Object result=null;
        Method[] methods=cls.getDeclaredMethods();
        Exception ex=null;
        for (int i=0;i<methods.length;i++)
        {
            try
            {
                if (methods[i].getName().compareTo(name)==0 && methods[i].getParameterTypes().length==args.length)
                {
                    result=methods[i].invoke(null,args);
                    return result;
                }
            }
            catch (Exception e)
            {
                ex=e;
            };
        };
        errorDuringConfig=true;
        MessageHandler.report("Fail to call static method="+name+" of class="+cls.getName()+" during load configuration",ex);
        return result;
    }
}
