/*
 * XMLConfig.java
 *
 * Created on December 26, 2001, 5:33 PM
 */

package IDE.Configuration;

import java.io.*;
import java.util.*;
import IDE.Messaging.*;
import IDE.Utilities.*;
import java.lang.reflect.*;

/**
 *
 * @author  marc
 */
public class XMLConfigObject extends AbstractXMLConfig
{
    /** Name of the class of the object */
    private String name;
    /** Arguments of the constructor for this object */
    private Vector arguments=new Vector();
    /** Creates a new instance of XMLConfig */
    public XMLConfigObject(String name) 
    {
        this.name=name;
    }
    /** Add a argument for the constructor */
    public void addArgument(int arg)
    {
        arguments.addElement(new Integer(arg));
    }
    public void addArgument(long arg)
    {
        arguments.addElement(new Long(arg));
    }
    public void addArgument(double arg)
    {
        arguments.addElement(new Double(arg));
    }
    public void addArgument(boolean arg)
    {
        arguments.addElement(new Boolean(arg));
    }
    public void addArgument(String arg)
    {
        arguments.addElement(arg);
    }
    public void addArgument(XMLConfigObject arg)
    {
        arguments.addElement(arg);
    }
    /** Serialize this XMLConfig to a XML writer stream */
    public void serialize(Writer writer)
    {
        try
        {
            writer.write("<object :name=\""+Util.normalize(name)+"\">");
            int m=arguments.size();
            for (int i=0;i<m;i++)
            {
                writer.write("\n");
                writer.write("<argument>");
                if (arguments.elementAt(i) instanceof Integer)
                    writer.write(((Integer) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof Long)
                    writer.write(((Long) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof Double)
                    writer.write(((Double) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof Boolean)
                    writer.write(((Boolean) arguments.elementAt(i)).toString());
                else if (arguments.elementAt(i) instanceof String)
                    writer.write("\""+Util.normalize((String) arguments.elementAt(i))+"\"");
                else if (arguments.elementAt(i) instanceof Character)
                    writer.write("'"+((int) ((Character) arguments.elementAt(i)).charValue())+"'");
                else if (arguments.elementAt(i) instanceof XMLConfigObject)
                    ((XMLConfigObject) arguments.elementAt(i)).serialize(writer);
                writer.write("</argument>");
            };
            writer.write("</object>");
        }
        catch (IOException e)
        {
            MessageHandler.report("Error writing XMLConfigObject",e);
        };
    }
    /** Create a object of the corrent type */
    public Object createObject()
    {
        Object result=null;
        try
        {
            int m=arguments.size();
            Class cls=Class.forName(name); // Get the class to instantiate object from
            Object args[]=new Object[m]; // Get the arguments for the constructor
            Object arg;
            for (int i=0;i<m;i++)
            {
                arg=arguments.elementAt(i);
                if (arg instanceof Integer)
                    args[i]=arg;
                else if (arg instanceof Long)
                    args[i]=arg;
                else if (arg instanceof Character)
                    args[i]=arg;
                else if (arg instanceof Double)
                    args[i]=arg;
                else if (arg instanceof Boolean)
                    args[i]=arg;
                else if (arg instanceof String)
                    args[i]=arg;
                else if (arg instanceof XMLConfigObject)
                    args[i]=((XMLConfigObject) arg).createObject();
            };
            result=callConstructor(cls,args);
        }
        catch (ClassNotFoundException e)
        {
            errorDuringConfig=true;
            MessageHandler.report("Class not found for object creation during load configuration "+name,e);
        }
        return result;
    }
    /** Private class to call the constructor of a class */
    private Object callConstructor(Class cls,Object[] args)
    {
        Object result=null;
        Constructor[] constructors=cls.getDeclaredConstructors();
        for (int i=0;i<constructors.length;i++)
        {
            try
            {   if (constructors[i].getParameterTypes().length==args.length)
                {
                    result=constructors[i].newInstance(args);
                    return result;
                };
            }
            catch (Exception e)
            {
            };
        }
        errorDuringConfig=true;
        MessageHandler.warn("Failed to create object of class "+cls.getName()+" during load configuration");
        return null;
    }
}
