/*
 * XMLConfigReader.java
 *
 * Created on December 27, 2001, 10:24 AM
 */

package IDE.Configuration;

import java.io.*;
import IDE.Messaging.*;

/**
 * Class to read XMLConfig information from a reader
 * @author  marc
 */
public class XMLConfigReader
{
    /** Peek value */
    private int peekvalue;
    /** Reader to read XMLConfig data from */
    private Reader reader;
    /** Creates a new instance of XMLConfigReader */
    public XMLConfigReader(Reader reader)
    {
        this.reader=reader;
        this.peekvalue=-2;
    }
    /** Read a XMLConfigClass from the reader */
    public XMLConfigClass readXMLConfigClass()
    {
        Object object=readXMLConfigObject();
        if (object instanceof XMLConfigClass)
            return (XMLConfigClass) object;
        else
            return null;
    }
    /** Read a XMLConfigElement */
    private Object readXMLConfigObject()
    {
        Tag tag=readTag();
        if (tag==null)
            return null;
        else if (tag.endtag)
            return tag;
        else
            if ("class".compareTo(tag.type)==0)
                return readClass(tag);
            else if ("method".compareTo(tag.type)==0)
                return readMethod(tag);
            else if ("field".compareTo(tag.type)==0)
                return readField(tag);
            else if ("object".compareTo(tag.type)==0)
                return readObject(tag);
            else if ("argument".compareTo(tag.type)==0)
                return readArgument(tag);
            else
                return null;
    }
    /** Read a XMLConfigClass element */
    public AbstractXMLConfig readClass(Tag tag)
    {
        XMLConfigClass cls=new XMLConfigClass(tag.name);
        Object object=readXMLConfigObject();
        while (object instanceof XMLConfigMethod || object instanceof XMLConfigField)
        {
            if (object instanceof XMLConfigMethod)
                cls.addMethod((XMLConfigMethod) object);
            else if (object instanceof XMLConfigField)
                cls.addField((XMLConfigField) object);
            object=readXMLConfigObject();
        };
        if (object==null)
            return null;
        else if (object instanceof Tag && ((Tag) object).endtag &&  ((Tag) object).type.compareTo("class")==0)
            return cls;
        else
            return null;
    }
    /** Read a XMLConfigMethod element */
    public AbstractXMLConfig readMethod(Tag tag)
    {
        XMLConfigMethod method=new XMLConfigMethod(tag.name);
        Object object=readXMLConfigObject();
        while (object instanceof XMLConfigObject || object instanceof Integer || object instanceof Long || object instanceof Double || object instanceof Boolean || object instanceof String || object instanceof Character)
        {
            if (object instanceof XMLConfigObject)
                method.addArgument((XMLConfigObject) object);
            else if (object instanceof Integer)
                method.addArgument(((Integer) object).intValue());
            else if (object instanceof Long)
                method.addArgument(((Long) object).longValue());
            else if (object instanceof Double)
                method.addArgument(((Double) object).doubleValue());
            else if (object instanceof Boolean)
                method.addArgument(((Boolean) object).booleanValue());
            else if (object instanceof Character)
                method.addArgument(((Character) object).charValue());
            else if (object instanceof String)
                method.addArgument((String) object);
            object=readXMLConfigObject();
        };
        if (object==null)
            return null;
        else if (object instanceof Tag && ((Tag) object).endtag && ((Tag) object).type.compareTo("method")==0)
            return method;
        else
            return null;
    }
    /** Read a XMLConfigField element */
    public AbstractXMLConfig readField(Tag tag)
    {
        XMLConfigField field=new XMLConfigField(tag.name);
        Object value=readElementContent();
        if (value instanceof String)
        {
            try
            {
                int ivalue=Integer.parseInt((String) value);
                field.setValue(ivalue);
            }
            catch (NumberFormatException e1)
            {
                try
                {
                    long lvalue=Long.parseLong((String) value);
                    field.setValue(lvalue);
                }
                catch (NumberFormatException e2)
                {
                    try
                    {
                        double dvalue=Double.parseDouble((String) value);
                        field.setValue(dvalue);
                    }
                    catch (NumberFormatException e3)
                    {
                        if ("true".compareTo(value)==0)
                            field.setValue(true);
                        else if ("false".compareTo(value)==0)
                            field.setValue(false);
                        else
                            field.setValue(stripString((String) value));
                    };
                };
            };
        }
        else if (value instanceof Character)
        {
            field.setValue(((Character) value).charValue());
        }
        else if (value instanceof XMLConfigObject)
        {
            field.setValue((XMLConfigObject) value);
        }
        else
        {
            return null;
        };
        Object endtag=readXMLConfigObject();
        if (endtag instanceof Tag && ((Tag) endtag).endtag && ((Tag) endtag).type.compareTo("field")==0)
            return field;
        else
            return null;
    }
    /** Read e XMLConfigObject element */
    public AbstractXMLConfig readObject(Tag tag)
    {
        XMLConfigObject object=new XMLConfigObject(tag.name);
        Object argument=readXMLConfigObject();
        while (argument instanceof XMLConfigObject || argument instanceof Integer || argument instanceof Long || argument instanceof Double || argument instanceof Boolean || argument instanceof String || argument instanceof Character)
        {
            if (argument instanceof XMLConfigObject)
                object.addArgument((XMLConfigObject) argument);
            else if (argument instanceof Integer)
                object.addArgument(((Integer) argument).intValue());
            else if (argument instanceof Long)
                object.addArgument(((Long) argument).longValue());
            else if (argument instanceof Double)
                object.addArgument(((Double) argument).doubleValue());
            else if (argument instanceof Boolean)
                object.addArgument(((Boolean) argument).booleanValue());
            else if (argument instanceof Character)
                object.addArgument(((Character) argument).charValue());
            else if (argument instanceof String)
                object.addArgument((String) argument);
            argument=readXMLConfigObject();
        };
        if (argument==null)
            return null;
        else if (argument instanceof Tag && ((Tag) argument).endtag && ((Tag) argument).type.compareTo("object")==0)
            return object;
        else
            return null;
    }
    /** Read a XMLConfigArgument element */
    public Object readArgument(Tag tag)
    {
        Object object=readElementContent();
        Object endtag=readXMLConfigObject();
        if (endtag instanceof Tag && ((Tag) endtag).endtag && ((Tag) endtag).type.compareTo("argument")==0)
        {
            if (object instanceof String)
            {
                try
                {
                    return new Integer(Integer.parseInt((String) object));
                }
                catch (NumberFormatException e1)
                {
                    try
                    {
                        return new Long(Long.parseLong((String) object));
                    }
                    catch (NumberFormatException e2)
                    {
                        try
                        {
                            return new Double(Double.parseDouble((String) object));
                        }   
                        catch (NumberFormatException e3)
                        {
                            if ("true".compareTo(object)==0)
                                return new Boolean(true);
                            else if ("false".compareTo(object)==0)
                                return new Boolean(false);
                            else
                                return stripString((String) object);
                        }
                    }
                }
            }
            else if (object instanceof Character)
            {
                return object;
            }
            else if (object instanceof XMLConfigObject)
            {
                return object;
            }
            else
            {
                return null;
            }
        }
        else
        {
            return null;
        }
    }
    /** For a string strip the leading and endind " , in the serialized object a string is delimited by " */
    private String stripString(String string)
    {
        if (string.length()<2)
            return string;
        else 
            return string.substring(1,string.length()-1);
    }
    /** Read element content , read the content of a element (this is either a string, a "string" ,'character' or a XMLConfigElement */
    private Object readElementContent()
    {
        StringBuffer buffer=new StringBuffer();
        int c=-1;
        // Skip the first white spaces
        while ((c=peek())!=-1 && isWhiteSpace(c) && c!='<')
        {
            buffer.append((char) c);
            next(); // Skip white space
        };
        if (c=='<')
        {
            // The content is a object
            return readXMLConfigObject();
        }
        else if (c=='\'')
        {
            // The content is a character
            buffer=new StringBuffer();
            next();
            while ((c=next())!=-1 && c!='\'')
                buffer.append((char) c);
            try 
            {
                return new Character((char) Integer.parseInt(buffer.toString()));
            }
            catch (NumberFormatException e)
            {
                MessageHandler.report("Error parsing character "+e.getMessage(),e);
                return new Character('\0');
            }
        }
        else if (c=='"')
        {
            buffer.append('"');
            next();
            // The content is a string embedded in " "
            while ((c=peek())!=-1 && c!='"')
            {
                if (c=='\\')
                {
                    next();
                    c=peek();
                    buffer.append((char) c);
                    next();
                }
                else
                {
                    buffer.append((char) c);
                    next();
                };
            };
            buffer.append('"');
            next();
            return buffer.toString();
        }
        else
        {
            // The content is a string not embedded in " "
            while ((c=peek())!=-1 && c!='<')
            {
                buffer.append((char) c);
                next();
            };
            return buffer.toString();
        }
    }
    /**  read the the next tag, returns the tag or null if error during read */
    private Tag readTag()
    {
        int c=nextNonWhite();
        if (c=='<')
        {
            c=peekNonWhite();
            if (c=='/')
                return readEndTag();
            else
                return readBeginTag();
        }
        else
        {
            return null;
        }
    }
    /** Read a endtag </type> */
    private Tag readEndTag()
    {
        StringBuffer type=new StringBuffer();
        int c;
        next(); // Skip the /
        c=readWord(type);
        if (c=='>')
        {
            Tag tag=new Tag();
            tag.type=type.toString();
            tag.endtag=true;
            return tag;
        }
        else
        {
            return null;
        }
    }
    /** Read a begin tag <type  :name=value> */
    private Tag readBeginTag()
    {
        StringBuffer type=new StringBuffer();
        StringBuffer name=new StringBuffer();
        StringBuffer value=new StringBuffer();
        int c;
        c=readWord(type);
        if (isWhiteSpace(c))
        {
            c=nextNonWhite();
            if (c==':')
            {
                c=readWord(name);
                if ("name".compareTo(name.toString())==0)
                {
                    if (isWhiteSpace(c))
                        c=nextNonWhite();
                    if (c=='=')
                    {
                        c=readString(value);
                        if (isWhiteSpace(c))
                            c=nextNonWhite();
                        if (c=='>')
                        {
                            Tag tag=new Tag();
                            tag.type=type.toString();
                            tag.name=value.toString();
                            tag.endtag=false;
                            return tag;
                        }
                        else
                        {
                            return null;
                        }
                    }
                    else
                    {
                        return null;
                    }
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }
        else if (c=='>')
        {
            // We have a argument element wih no name attribute
            Tag tag=new Tag();
            tag.type=type.toString();
            tag.name="";
            tag.endtag=false;
            return tag;
        }
        else
        {
            return null;
        }
    }
    /** Read a string of characters "...." */
    private int readString(StringBuffer string)
    {
        int c;
        c=nextNonWhite();
        if (c=='"')
        {
            while ((c=next())!=-1 && c!='"')
            {
                if (c=='\\')
                {
                    c=next();
//                    string.append('\\');
                    if (c!=-1)
                        string.append((char) c);
                }
                else
                {
                    string.append((char) c);
                }
            };
            if (c=='"')
                return next();
            else
                return c;
        }
        else
        {
            return -1;
        }
    }
    /** Read the next non white space character*/
    private int nextNonWhite()
    {
        int c;
        try
        {
            if (peekvalue==-2)
            {
                while ((c=reader.read())!=-1 && isWhiteSpace(c));
                return c;
            }
            else
            {
                c=peekvalue;
                peekvalue=-2;
                return c;
            }
        }
        catch (IOException e)
        {
            return -1;
        }
    }
    /** Read a word in a stringbuffer returning the character after this word */
    private int readWord(StringBuffer word)
    {
        int c=nextNonWhite();
        while (c!=-1 && !(isWhiteSpace(c) || c=='<' || c=='>' || c==':' || c=='"' || c=='='))
        {
            word.append((char) c);
            c=next();
        };
        return c;
    }
    /** Peek for the next non white space character */
    private int peekNonWhite()
    {
        int c;
        try
        {
            if (peekvalue==-2)
            {
                while ((c=reader.read())!=-1 && isWhiteSpace(c));
                peekvalue=c;
                return c;
            }
            else
            {
                return peekvalue;
            }
        }
        catch (IOException e)
        {
            return -1;
        }
    }
    /** Read the next character */
    private int next()
    {
        try
        {
            if (peekvalue==-2)
            {
                return reader.read();
            }
            else
            {
                int c=peekvalue;
                peekvalue=-2;
                return c;
            }
        }
        catch (IOException e)
        {
            return -1;
        }
    }
    /** Peek for the next character */
    private int peek()
    {
        try
        {
            if (peekvalue==-2)
                return peekvalue=reader.read();
            else
                return peekvalue;
        }
        catch (IOException e)
        {
            return -1;
        }
    }
    /** Check if a character is awhite space */
    private boolean isWhiteSpace(int c)
    {
        return c==' ' || c=='\n' || c=='\t' || c=='\r';
    }
    
    /** Private classes to represent a tag */
    private class Tag
    {
        /** Type of the tag */
        private String type;
        /** Name of the tag */
        private String name;
        /** Boolean to indicate this is a end or begin tag */
        private boolean endtag=false;
    }
}
