/*
 * Help.java
 *
 * Created on January 26, 2002, 5:54 PM
 */

package IDE.Help;

import IDE.Messaging.*;
import IDE.Configuration.*;
import java.io.*;
import java.awt.*;
import java.util.Vector;
import java.awt.event.*;
import java.net.URL;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;

/**
 * Popup dialog window for help (via static methods)
 * @author  marc
 *
 */
public class Help
{
    /** Frame to display the native help in */
    private static JFrame helpFrame=null;
    /** Help is displayed in this */
    private static JEditorPane helpPane = new JEditorPane();
    /** Keep track of URLs visited, needed for navigation buttons */
    private static Vector visitedURLs = new Vector();
    /** Current URL number in visitedURLs, needed for navigation */
    private static int currentURLnum = 0;
    /** Temporary URL */
    private static URL url;
    /** Name of the target frame, defined in index.html */
    private static String targetFrame = "main";
    /** Navigation button to go back */
    private static JButton prevButton = new JButton("Back");
    /** Navigation button to go forward */
    private static JButton nextButton = new JButton("Next");;
    /** Navigation button to go to Home page*/
    private static JButton homeButton = new JButton("Home");;
    /** Label for Address field */
    private static JLabel addressLabel = new JLabel("Address:");
    /** Field for typing URL */
    private static JTextField addressField = new JTextField(40);
    
    /*
     * Display the help page for Jabberwocky
     */
    
    public static void displayHelp()
    {
        if (Configuration.helpbrowser.compareTo("native")==0)
        {
            displayHelpNative();
        }
        else
        {
            String browsercommand=Configuration.helpbrowser+" "+getUrlHelp();
            
            Runtime runtime=Runtime.getRuntime();
            try
            {
                Process process=runtime.exec(browsercommand);
                final InputStream in=process.getInputStream();
                final InputStream error=process.getErrorStream();
                // We have to start two threads to read stdout and stderr to avoid that the process hangs
                // waiting on consuming its output
                Thread thread=new Thread(new Runnable()
                {
                    public void run()
                    {
                        try
                        {
                            while (in.read()>0);
                        }
                        catch (Exception e)
                        {
                        };
                    }
                });
                thread.start();
                thread=new Thread(new Runnable()
                {
                    public void run()
                    {
                        try
                        {
                            while (error.read()>0);
                        }
                        catch (Exception e)
                        {
                        };
                    }
                });
                thread.start();
            }
            catch (Exception e)
            {
                MessageHandler.report("Failed to start browser for help",e);
            }
        };
    }
     /*
      * Display help via JEditorPane, code by Y Raja Reddy
      *                                       reddy@cgdatrgdiv.delhi.nic.in
      */
    public static void displayHelpNative()
    {
        displayHelpNative(getUrlHelp());
    }
    public static void displayHelpNative(String urlString)
    {
        if (helpFrame==null)
        {
            displayHelpNativeImp(urlString);
        }
        else
        {
            try
            {
                helpFrame.toFront();
                url = new URL(urlString);
                helpPane.setPage(url);
                visitedURLs.setSize(currentURLnum+1);
                storeURL(url);
            }
            catch(IOException e)
            {
                MessageHandler.report("Cannot go to"  + url.toExternalForm(),e);
            }
        };
        
    }
    private static void displayHelpNativeImp(final String urlString)
    {
        JPanel topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        
        nextButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                // Do nothing if already on the last URL
                if(visitedURLs.size()==currentURLnum+1)
                    return;
                try
                {
                    currentURLnum++;
                    updateButtons();
                    url = (URL)visitedURLs.get(currentURLnum);
                    helpPane.setPage(url);
                    addressField.setText(url.toExternalForm());
                }
                catch(IOException e)
                {
                    MessageHandler.report("Cannot go to"  + url.toExternalForm(),e);
                }
            }
        });
        prevButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                //Do nothing if already on the first URL
                if(0==currentURLnum)
                    return;
                try
                {
                    currentURLnum--;
                    updateButtons();
                    url = (URL)visitedURLs.get(currentURLnum);
                    helpPane.setPage(url);
                    addressField.setText(url.toExternalForm());
                }
                catch(IOException e)
                {
                    MessageHandler.report("Cannot go to"  + url.toExternalForm(),e);
                }
                
            }
        });
        
        homeButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                try
                {
                    url = new URL(urlString);
                    helpPane.setPage(url);
                    storeURL(url);
                }
                catch(IOException e)
                {
                    MessageHandler.report("Cannot go to"  + url.toExternalForm(),e);
                }
                
            }
        });
        
        addressField.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                try
                {
                    url = new URL(addressField.getText());
                    helpPane.setPage(url);
                    visitedURLs.setSize(currentURLnum+1);
                    storeURL(url);
                }
                catch(IOException e)
                {
                    MessageHandler.report("Cannot go to"  + url.toExternalForm(),e);
                }
            }
        });
        
        topPanel.add(prevButton);
        topPanel.add(nextButton);
        topPanel.add(homeButton);
        topPanel.add(addressLabel);
        topPanel.add(addressField);
        
        helpPane.addHyperlinkListener(new LinkActivator());
        helpPane.setEditable(false);
        try
        {
            url = new URL(urlString);
            helpPane.setPage(url);
            helpFrame = new JFrame();
            helpFrame.getContentPane().add(topPanel, BorderLayout.NORTH);
            helpFrame.getContentPane().add(new JScrollPane(helpPane), BorderLayout.CENTER);
            helpFrame.setSize(750, 600);
            helpFrame.setVisible(true);
            
            helpFrame.addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent e)
                {
                    helpFrame=null;
                }
            });
            visitedURLs.clear();
            visitedURLs.add(url);
            currentURLnum=0;
            updateButtons();
            addressField.setText(url.toExternalForm());
        } catch (IOException e)
        {
            MessageHandler.report("Failed to start browser for help",e);
        }
    }
    
    /*
     * Get the URL containing the help file for Jabberwocky
     */
    
    private static String getUrlHelp()
    {
        return "file:"+Configuration.installDirectory+"/doc/index.html";
    }
    private static void storeURL(URL url)
    {
        //Store URL only if it is different from the current one
        if (!url.equals((URL)visitedURLs.get(currentURLnum)))
        {
            visitedURLs.add(url);
            currentURLnum++;
            updateButtons();
            addressField.setText(url.toExternalForm());
        }
    }

    /* 
     * Enable/Disable the next and prev button depending on the fact we have 
     * reached begin or end of the visitedURL's
     */
    private static void updateButtons()
    {
        if (currentURLnum==0)
            prevButton.setEnabled(false);
        else
            prevButton.setEnabled(true);
        if (currentURLnum>=visitedURLs.size()-1)
            nextButton.setEnabled(false);
        else
            nextButton.setEnabled(true);
    }
    
    private static class LinkActivator implements HyperlinkListener
    {
        public void hyperlinkUpdate(HyperlinkEvent event)
        {
            if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
            {
                JEditorPane clickedPane = (JEditorPane) event.getSource();
                HTMLDocument doc;
                if (event instanceof HTMLFrameHyperlinkEvent)
                {
                    HTMLFrameHyperlinkEvent  frameEvent = (HTMLFrameHyperlinkEvent)event;
                    doc = (HTMLDocument)clickedPane.getDocument();
                    doc.processHTMLFrameHyperlinkEvent(frameEvent);
                    addressField.setText(event.getURL().toExternalForm());
                    // If we are not navigating using buttons, most likely we are
                    // taking a new path. So, trim off from the list any URLs beyond the current one.
                    visitedURLs.setSize(currentURLnum+1);
                    storeURL(frameEvent.getURL());
                }
                else
                {
                    try
                    {
                        doc=(HTMLDocument)clickedPane.getDocument();
                        clickedPane.setPage(event.getURL());
                        addressField.setText(event.getURL().toExternalForm());
                        visitedURLs.setSize(currentURLnum+1);
                        storeURL(event.getURL());
                    }
                    catch(IOException e)
                    {
                        MessageHandler.report("Cannot go to"  + event.getURL().toExternalForm(),e);
                    }
                }
            }
        }
    }
    
    //For Testing
    public static void main(String[] args)
    {
        displayHelpNative();
    }
}
