/*
 * HintWindow.java
 *
 * Created on November 5, 2001, 8:01 PM
 */

package IDE.Help;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import IDE.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import IDE.Configuration.*;

/**
 *
 * @author  marc
 */
public class HintWindow extends JSplitPane implements ListSelectionListener
{
    /** Timer object to display hint window */
    java.util.Timer timer=null;
    /** Offset to add to the x coordinate for the origin of the hint window */
    private final static int offsetX=10;
    /** Offset to add to the y coordinate for the origin of the hint window */
    private final static int offsetY=10;
    /** Textarea containing the text to be displayed */
    private static JTextArea text;
    /** Scrollpane to contain the text area */
    private static JScrollPane textpane;
    /** List to contain a list of choices */
    private static JList list;
    /** Scrollpane containing the list */
    private static JScrollPane listpane;
    /** Parser to provide the hint list and the syntax definitions */
    private static Parser parser=null;
    /** Leaf to display hint for */
    private static CodeLeafElement leaf=null;
    /** Creates new HintWindow */
    public HintWindow()
    {
        super(JSplitPane.VERTICAL_SPLIT);

        // Stuff to display a text
        text=new JTextArea();
        text.setEditable(false);
        textpane=new JScrollPane(text);
        // Stuff to display a list of choices
        list=new JList(new DefaultListModel());
        list.addListSelectionListener(this);
        // Replace from the Hint Window with double click of mouse
        list.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent ev)
            {
                if (2 == ev.getClickCount())
                {
                    replaceByHint();
                }
            }
        });
        listpane=new JScrollPane(list);
        add(textpane);
        add(listpane);
        setVisible(true);
        toBack();
    }
    /** replace the leaf used for the hint by the hint */
    public static void replaceByHint()
    {
        if (parser!= null)
            if (leaf!=null)
            {
                parser.replaceLeafByHint(leaf,(String) list.getSelectedValue());
                Main.currentEditor().requestFocus();
            };
    }
    /** Go up in the hint list */
    public static void goUp()
    {
        int i=list.getSelectedIndex();
        if (i>0)
        {
            list.setSelectedIndex(i-1);
            list.ensureIndexIsVisible(i-1);
        };
    }
    /** Go down in the hint list */
    public static void goDown()
    {
        int i=list.getSelectedIndex();
        if (i+1<list.getModel().getSize())
        {
            list.setSelectedIndex(i+1);
            list.ensureIndexIsVisible(i+1);
        };
    }
    /** Show the hint relevant for position position  */
    public void showHint(final int position,final CodeDocument document,final int x,final int y)
    {
        parser=document.getParser();
        // Runnable to switch the syntax hint to the background
        final Runnable toback=new Runnable()
        {
            public void run()
            {
                leaf=null;
                toBack();
            };
        };
        // TimerTask to display the hint
        TimerTask task=new TimerTask()
        {
            public void run()
            {
                HintInfo info=parser.getHintInfo(position);
                if (info==null)
                {
   					SwingUtilities.invokeLater(toback);
   					timer.cancel();
                }
                else
                {
                    DisplayHintWindow display=new DisplayHintWindow(info,x,y);
                    try
                    {
                    	SwingUtilities.invokeAndWait(display);
                    }
                    catch(Exception e)
                    {
                    }
                    timer.cancel();
                };
            }
        };
        // Stop the previous timer if it exist
        if (timer!=null)
            timer.cancel();
        // Start a new timer
        timer=new java.util.Timer();
        timer.schedule(task,50,1000);
    }
    
    // Called when listening to selection changes on the list model */
    public void valueChanged(ListSelectionEvent e)
    {
        if (parser!=null)
        {
            int i=list.getSelectedIndex();
            if (i>=0 && i<list.getModel().getSize())
            {
                String syntax=parser.getSyntaxDescription((String) list.getModel().getElementAt(i));
                if (syntax.length()!=0)
                {
                    text.setText(syntax);
                    text.revalidate();
                };
                sizeHintWindow();
            };
        };
    }
    /** Set the preferred size of the hint window and its components */
    private void sizeHintWindow()
    {
        // Minimum sizes of the components
        int xmax=5000,xmin=100;
        int ymax=500,ymin=30;
        int x, xs,ys,xl,yl;
        FontMetrics fm;
        String str;
        // Calculate the size of the text component
        fm=text.getFontMetrics(text.getFont());
        xs=0;
        ys=fm.getHeight()*(text.getLineCount()-1);
        try
        {
            for (int i=0;i<text.getLineCount();i++)
            {
                str=text.getDocument().getText(text.getLineStartOffset(i),text.getLineEndOffset(i)-text.getLineStartOffset(i));
                xs=Math.max(xs,fm.stringWidth(str));
            };
        }
        catch (BadLocationException ex)
        {
        };
        xs=Math.min(Math.max(xs,xmin),xmax);
        ys=Math.min(Math.max(ys,ymin),ymax);
        xl=xs;
        list.setVisibleRowCount(Math.min(10,list.getModel().getSize()));
        yl=list.getPreferredScrollableViewportSize().height;
        xl=list.getPreferredScrollableViewportSize().width;
        // Ensure that we are no bigger or smaller then min and maximum
        xl=Math.min(Math.max(xl,xmin),xmax);
        yl=Math.min(Math.max(yl,ymin),ymax);
        // Set the sizes
        x=Math.max(xs,xl);
        textpane.setPreferredSize(new Dimension(x+30,ys+30));
        textpane.setMinimumSize(new Dimension(x+30,ys+60));
        listpane.setPreferredSize(new Dimension(x+30,yl+60));
        setDividerSize(2);
        setSize(x+30,yl+ys+66); 
        resetToPreferredSizes();
        
        validateTree();
    }
    /** Bring the hint window to the front */
    public void toFront()
    {
        JLayeredPane lp=Main.frame.getLayeredPane();
        lp.remove(this);
        lp.add(this,JLayeredPane.POPUP_LAYER);
        Main.frame.repaint();
    }
    /** Bring the hint window to the background */
    public void toBack()
    {
        Main.frame.getLayeredPane().remove(this);
        Main.frame.repaint();
    }
    /** Set the x,y position of the hint window */
    private void setPosition(int x,int y)
    {
        Dimension dim;
        int widthFrame,heightFrame;
        int widthWindow,heightWindow;
        int posX;
        int posY;
        
        dim=Main.frame.getSize();
        widthFrame=dim.width;
        heightFrame=dim.height;
        dim=getSize();
        widthWindow=dim.width;
        heightWindow=dim.height;
        if (x+offsetX+widthWindow+30>widthFrame)
            posX=Math.max(0,x-offsetX-widthWindow);
        else
            posX=x+offsetX;
        if (y+offsetY+heightWindow+30>heightFrame)
            posY=Math.max(0,y-offsetY-heightWindow-20);
        else
            posY=y+offsetY;
        
        setLocation(posX,posY);
    }
    
    /** Private class extending runnable to display the hint window */
    private class DisplayHintWindow implements Runnable
    {
        private String[] hints;
        private String syntaxhint;
        private int iHint;
        private int x,y;
        private CodeLeafElement leaf;
        private DisplayHintWindow(HintInfo info,int x,int y)
        {
            this.leaf=info.leaf;
            this.syntaxhint=info.syntaxhint;
            this.iHint=info.iHint;
            this.x=x;
            this.y=y;
			this.hints=info.hints;
        }
        public void run()
        {
            list.setListData(hints);
            if (iHint<0)
                iHint=0;
            if (iHint<hints.length)
                list.setSelectedIndex(iHint);
            text.setText(syntaxhint);
            text.setBackground(Configuration.colorHint);
            // Size the hint window
            sizeHintWindow();
            if (iHint>=0 && iHint<hints.length)
                list.ensureIndexIsVisible(iHint+5);
            // Show the window
            setPosition(x,y);
            toFront();
            // Set the leaf which must be replaced if needed
            HintWindow.leaf=leaf;
        }
    }
}
