/*
 * HyperSpec.java
 *
 * Created on September 29, 2002, 10:04 AM
 */

package IDE.Help;

import IDE.*;
import IDE.Configuration.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import IDE.Messaging.*;
import java.io.*;
import java.util.*;

/**
 * Code to work with the CLHS
 * @author  marc
 */
public class HyperSpec
{
    /** Static hashmap mapping a keyword on a filename */
    private static HashMap keywordToFileName=new HashMap();
    /** Name of the HyperSpec directory */
    private static String hyperSpecDirectory=null;
    /** Get the HyperSpecDirectory */
    public static String getHyperSpecDirectory()
    {
        return hyperSpecDirectory;
    }
    /** Set the HyperSpecDirectory */
    public static void setHyperSpecDirectory(String hyperSpecDirectory)
    {
        HyperSpec.hyperSpecDirectory=hyperSpecDirectory;
        File keywordfile=getKeyWordFile(hyperSpecDirectory);
        if (keywordfile!=null)
        {
            BufferedReader reader=null;
            String keyword;
            String urlfile;
            try
            {
                keywordToFileName.clear();
                reader=new BufferedReader(new FileReader(keywordfile));
                while (true)
                {
                    keyword=reader.readLine();
                    if (keyword==null)
                        break;
                    urlfile=reader.readLine();
                    if (urlfile==null)
                        break;
                    keywordToFileName.put(keyword,urlfile.substring(8));
                };
                reader.close();
            }
            catch (Exception e)
            {
                MessageHandler.warn("Error opening the index file HyperSpec is "+e.getMessage());
            };
        };
    }
    /** Get the file containing the keywordToUrl mapping */
    private static File getKeyWordFile(String directory)
    {
        File file=new File(directory+File.separator+"Data"+File.separator+"Symbol-Table.text");
        if (!file.exists())
            file=new File(directory+File.separator+"Data"+File.separator+"Map_Sym.txt");
        if (!file.exists())
            return null;
        return file;
    }
    /** Get the frontmatter index */
    private static String getFrontMatterIndexUrl()
    {
        String urlFile=hyperSpecDirectory+File.separator+"FrontMatter"+File.separator+"index.html";
        if (new File(urlFile).exists())
            return "File:"+hyperSpecDirectory+"/FrontMatter/index.html";
        urlFile=hyperSpecDirectory+File.separator+"Front"+File.separator+"index.htm";
        if (new File(urlFile).exists())
            return "File:"+hyperSpecDirectory+"/Front/index.htm";
        else
            return null;
    }
    /*
     * Display the HyperSpec documentation for the current keyword or list 
     */
    public static void displayHyperSpecContext()
    {
        if (hyperSpecDirectory==null)
            return;
        CodeEditor editor=Main.currentEditor();
        CodeDocument document;
        int pos,ileaf;
        String url;
        CodeLeafElement leaf;
        CodeElement element;
        LeafVector leafs;
        if (editor!=null)
        {
            document=editor.getDocument();
            if (document!=null && document.getParser() instanceof LispParser)
            {
                // Find the enclosing list and find then the first element of this list
                // and the current non separator element or the element
                pos=editor.cursor.cursorOffset();
                leafs=document.getLeafs();
                ileaf=document.getIndexLeafAt(pos);
                for (;ileaf>=0 && leafs.leafAt(ileaf) instanceof CodeLeafSeparator;ileaf--);
                if (ileaf>=0)
                {
                    leaf=leafs.leafAt(ileaf);
                    for (element=leaf.getParent();element!=null && !(element instanceof LispListElement);element=element.getParent());
                    if (element!=null)
                    {
                        element=((LispListElement) element).getFirst();
                        url=(String) keywordToFileName.get(leaf.getString(document).toUpperCase());
                        for (url=(String) keywordToFileName.get(element.getString(document).toUpperCase());
                             url==null && element!=null;
                             element=element.getParent())
                            if (element instanceof LispListElement)
                                url=(String) keywordToFileName.get(((LispListElement) element).getFirst().getString(document).toUpperCase());
                        if (url!=null)
                            Help.displayHelpNative("file:"+hyperSpecDirectory+"/Body/"+url);
                    };
                };
            };
        };
    }
    /*
     * Display the HyperSpec documentation 
     */
    public static void displayHyperSpec()
    {
        String urlFrontMatter=getFrontMatterIndexUrl();
        if (urlFrontMatter!=null)
            Help.displayHelpNative(urlFrontMatter);
    }
    /** Save the configuration of this file extensions */
    public static void saveConfiguration(Writer writer)
    {
        if (hyperSpecDirectory!=null)
        {
            XMLConfigClass cls=new XMLConfigClass("IDE.Help.HyperSpec");
            XMLConfigMethod method=new XMLConfigMethod("setHyperSpecDirectory");
            method.addArgument(hyperSpecDirectory);
            cls.addMethod(method);
            cls.serialize(writer);
        };
    }
}
