/*
 * IInteraction.java
 *
 * Created on November 7, 2001, 7:25 PM
 */

package IDE.Interaction;

import IDE.CodeEditor.*;
import IDE.Watch.*;
import javax.swing.*;
import java.util.*;

/**
 *  Interface to define the interaction with a process 
 * @author  marc
 */
 
public interface  IInteraction
{
    /** Call the process to let it process a command (this different from sending it to the stdin of the process */
    public abstract String callProcess(String command);
    /** 
     * Get the value of expression in the process, always returns a linked list containing the values returned by the expression 
     * This method must be called outside the UI thread to avoid lockups
     */
    public abstract LinkedList watchExpression(String expression);
    /** Send a string  to the input stream of a process */
    public abstract void sendCommand(String command);
    /** Retireve Error and output from the process */
    public abstract String receiveErrorAndOutput();    
    /** Start the process to interact with  (this must be called in a assync way if you want to avoid bocking) */
    public abstract void startProcess();
    /** Stop the process to interact with (this must be done in a assync way if you want to avoid blocking ) */
    public abstract void stopProcess();
    /** Enable debugging for the given document */
    public abstract boolean enableDebugging(CodeEditor editor);
    /** Disable debugging for the given document, returning true if succeeded */
    public abstract void disableDebugging(CodeEditor editor);
    /** Toggle a breakpoint  */
    public abstract void toggleBreakpoint(CodeEditor editor);
    /** Toggle a conditional breakpoint */
    public abstract void toggleConditionalBreakpoint(CodeEditor editor);
    /** Step into action in the debugger */
    public abstract void stepInto(CodeEditor editor);
    /** Step over action in the debugger */
    public abstract void stepOver(CodeEditor editor);
    /** Stop at action the debugger */
    public abstract void stopAt(CodeEditor editor);
    /** Stop just after evaluating a form */
    public abstract void stopAfter(CodeEditor editor);
    /** Set the return value of  executed code in the debugger */
    public abstract void setNewReturnValue(CodeEditor editor);
    /** Continue */
    public abstract void continueExecution(CodeEditor editor);
    /** Get the value of a leaf at pos (in list the value of a symbol) */
    public abstract String getValueLeaf(CodeEditor editor,int pos);
    /** Get the image icon for a watchnode belonging to a type of data*/
    public abstract ImageIcon getTypeIcon(int type);
    /** Get the text for a certain type of data */
    public abstract String getTypeString(int type);
    /** Used in timetraveling to go to the future */
    public abstract void goToFuture(CodeEditor editor);
    /** Used in timetraveling to go to the past */
    public abstract void goToPast(CodeEditor editor);
    /** Start or reset a profiling session */
    public abstract void startProfiling(CodeEditor editor);
    /** Stop a profiling session */
    public abstract void stopProfiling(CodeEditor editor);
    /** Send signal to the process */
    public abstract void signalProcess(String signal);
    /** Load the editor in the process */
    public abstract void loadEditor(CodeEditor editor);
    /** Compile the content of the editor using the process */
    public abstract void compileEditor(CodeEditor editor);
    /** Get the name to be used in the interaction pane */
    public abstract String getName();
    /** Set the current directory in the process */
    public abstract void setCurrentDirectory(String directory);
    /** Set the interactionpane that is working with this interaction */
    public abstract void setInteractionPane(InteractionPane pane);
    /** Set a watch point in the interaction process (pass on the watch node and the wathc expression*/
    public abstract void setWatchPoint(WatchNode node,String watchExp);
    /** Set a watch point in the interaction process (pass on the watch node and the wathc expression*/
    public abstract void changeWatchPoint(WatchNode node,String watchExp);
    /** Set a watch point in the interaction process (pass on the watch node and the wathc expression*/
    public abstract void removeWatchPoint(WatchNode node);    
}

