/*
 * FilterFiles.java
 *
 * Created on May 30, 2001, 8:47 PM
 */

package IDE.Interaction;

import IDE.Configuration.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

/**
 * Class to hold a mapping of a Interaction to the command to start the interaction process
 * @author  marc
 * @version
 */

/** Contains the list of extensions recognised by the IDE */
public class InteractionCommands
{
    /** Refers to the only instance of FileExtensions , so we can use static methods for inner classes */
    private static InteractionCommands self=new InteractionCommands();
    /** Maps a IInteraction classname to a startup command */
    private static TreeMap interactionClassToCommand=new TreeMap();
    /** Constructor is private so no one outside this file can create a instance */
    private InteractionCommands()
    {
    }
    /** Create a new map from a IInteraction class name to a startup command */
    public static void addCommand(String interactionClass,String command)
    {
        if (command.length()>0)
            interactionClassToCommand.put(interactionClass,command);
    }
    /** Static method to get the command associated with a IInteraction (returns null if no such interaction exist) */
    public static String getInteractionCommand(String classname)
    {
        return (String) interactionClassToCommand.get(classname);
    }
    /** Shows a dialog box to edit the extensions to parser mappings */
    public static void editCommands()
    {
        final JDialog dialog=new JDialog();
        JPanel panel=new JPanel();
        JPanel buttons=new JPanel();
        final DefaultTableModel model=new DefaultTableModel(new String[0][0],new String[]
        {"Interaction Class Name","Startup Command"});
        final JTable table=new JTable(model);
        table.setDefaultEditor(Object.class,new FileExtensionsCellEditor());
        JScrollPane scrollpane=new JScrollPane(table);
        JButton add=new JButton(new AbstractAction("Add")
        {
            public void actionPerformed(ActionEvent ev)
            {
                model.addRow(new String[]
                {"",""});
            }
        });
        JButton remove=new JButton(new AbstractAction("Remove")
        {
            public void actionPerformed(ActionEvent ev)
            {
                int i=table.getSelectedRow();
                TableCellEditor ce=table.getCellEditor();
                if (ce!=null)
                    ce.stopCellEditing();
                if (i>=0)
                    model.removeRow(i);
            }
        });
        JButton ok=new JButton(new AbstractAction("Ok")
        {
            public void actionPerformed(ActionEvent ev)
            {
                int m=model.getRowCount();
                // Stop the cell editing
                TableCellEditor ce=table.getCellEditor();
                if (ce!=null)
                    ce.stopCellEditing();
                // Check if the IInteraction class names are correct
                for (int i=0;i<m;i++)
                {
                    try
                    {
                        Class cls=Class.forName((String) model.getValueAt(i, 0));
                        if (!Class.forName("IDE.Interaction.IInteraction").isAssignableFrom(cls))
                        {
                            if (JOptionPane.showConfirmDialog(null,"Interaction class does not exist, continue edit","Class is not a interaction",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.CANCEL_OPTION)
                            {
                                dialog.setVisible(false);
                                dialog.dispose();
                            }
                            else
                            {
                                return;
                            };
                        }
                    }
                    catch (Exception e)
                    {
                        if (JOptionPane.showConfirmDialog(null,"Interaction class does not exist, continue edit","Class "+model.getValueAt(i,1)+" not found",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.CANCEL_OPTION)
                        {
                            dialog.setVisible(false);
                            dialog.dispose();
                        }
                        else
                        {
                            return;
                        };
                    };
                };
                // Update the extensions
                interactionClassToCommand.clear();
                for (int i=0;i<m;i++)
                    addCommand((String) model.getValueAt(i,0),(String) model.getValueAt(i,1));
                dialog.setVisible(false);
                dialog.dispose();
            }
        });
        JButton cancel=new JButton(new AbstractAction("Cancel")
        {
            public void actionPerformed(ActionEvent ev)
            {
                dialog.setVisible(false);
                dialog.dispose();
            }
        });
        // Set the selection model to single selection
        table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        // Fill up the model
        Iterator iter=interactionClassToCommand.keySet().iterator();
        String interactionclass;
        String command;
        while (iter.hasNext())
        {
            interactionclass=(String) iter.next();
            command=(String) interactionClassToCommand.get(interactionclass);
            model.addRow(new String[]
            {interactionclass,command});
        }
        // Do the layout
        buttons.add(ok);
        buttons.add(cancel);
        buttons.add(add);
        buttons.add(remove);
        
        panel.setLayout(new BorderLayout());
        panel.add(scrollpane,BorderLayout.CENTER);
        panel.add(buttons,BorderLayout.SOUTH);
        
        dialog.setTitle("Edit extensions");
        dialog.setModal(true);
        dialog.setContentPane(panel);
        dialog.setSize(400,500);
        dialog.setLocation(100,200);
        
        // Show the dialog box
        dialog.setVisible(true);
    }
    /** Save the configuration of this file extensions */
    public static void saveConfiguration(Writer writer)
    {
        XMLConfigClass cls=new XMLConfigClass(self.getClass().getName());
        XMLConfigMethod method;
        Iterator iter=interactionClassToCommand.keySet().iterator();
        String interactionclass;
        String command;
        while (iter.hasNext())
        {
            interactionclass=(String) iter.next();
            command=(String) interactionClassToCommand.get(interactionclass);
            method=new XMLConfigMethod("addCommand");
            method.addArgument(interactionclass);
            method.addArgument(command);
            cls.addMethod(method);
        };
        cls.serialize(writer);
    }
    /** 
     * Parse Command String a command string is of the form 
     *   word [word]** 
     *  where word is eather a string containing no spaces or is of the form "...." 
     */
    public static String[] parseCommandString(String command)
    {
        ArrayList result=new ArrayList();
        StringBuffer word=new StringBuffer();
        for (int i=0;i<command.length();i++)
            switch (command.charAt(i))
            {
                case ' ': // End of the previous word
                    if (word.length()>0)
                    {
                        result.add(word.toString());
                        word=new StringBuffer();
                    };
                    break;
                case '"': // Begin of a " delimited string
                    if (word.length()>0)
                    {
                        result.add(word.toString());
                        word=new StringBuffer();
                    };
                    word.append('"');
                    for (i=i+1;i<command.length() && command.charAt(i)!='"';i++)
                        word.append(command.charAt(i));
                    word.append('"');
                    result.add(word.toString());
                    word=new StringBuffer();
                    break;
                default:
                    word.append(command.charAt(i));
                    break;
            };
       if (word.length()>0)
           result.add(word.toString());
       String[] words=new String[result.size()];
       for (int i=0;i<words.length;i++)
           words[i]=(String) result.get(i);
       return words;
    }
    
}
/** Internal class used to do the cell editing */
class FileExtensionsCellEditor extends JTextField implements TableCellEditor
{
    /** Table edited by this cell editor */
    private JTable table;
    /** List of all the listeners to this editor */
    private Vector listeners=new Vector();
    /** Row be edited */
    private int row;
    /** Column be edited */
    private int column;
    /** Adds a listener to the list that's notified when the editor
     * stops, or cancels editing.
     *
     * @param	l		the CellEditorListener
     *
     */
    public void addCellEditorListener(CellEditorListener l)
    {
        listeners.add(l);
    }
    
    /** Tells the editor to cancel editing and not accept any partially
     * edited value.
     *
     */
    public void cancelCellEditing()
    {
        TableModel model=table.getModel();
        for (int i=listeners.size()-1;i>=0;i--)
        {
            ((CellEditorListener) listeners.elementAt(i)).editingCanceled(new ChangeEvent(this));
        };
        if (model!=null && row<model.getRowCount() && column<model.getColumnCount())
        {
            model.setValueAt(getText(),row,column);
            table.setRowSelectionInterval(row,row);
            table.setColumnSelectionInterval(column, column);
        };
        setText("");
    }
    
    /** Returns the value contained in the editor.
     * @return the value contained in the editor
     *
     */
    public Object getCellEditorValue()
    {
        return getText();
    }
    
    /**  Sets an initial <code>value</code> for the editor.  This will cause
     *  the editor to <code>stopEditing</code> and lose any partially
     *  edited value if the editor is editing when this method is called. <p>
     *
     *  Returns the component that should be added to the client's
     *  <code>Component</code> hierarchy.  Once installed in the client's
     *  hierarchy this component will then be able to draw and receive
     *  user input.
     *
     * @param	table		the <code>JTable</code> that is asking the
     * 				editor to edit; can be <code>null</code>
     * @param	value		the value of the cell to be edited; it is
     * 				up to the specific editor to interpret
     * 				and draw the value.  For example, if value is
     * 				the string "true", it could be rendered as a
     * 				string or it could be rendered as a check
     * 				box that is checked.  <code>null</code>
     * 				is a valid value
     * @param	isSelected	true if the cell is to be rendered with
     * 				highlighting
     * @param	row     	the row of the cell being edited
     * @param	column  	the column of the cell being edited
     * @return	the component for editing
     *
     */
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
    {
        this.row=row;
        this.column=column;
        this.table=table;
        if (value instanceof String)
            setText((String) value);
        else
            setText("");
        setEditable(true);
        // Set selected items in model
        TableModel model=table.getModel();
        if (model!=null && row<model.getRowCount() && column<model.getColumnCount())
        {
            table.setRowSelectionInterval(row,row);
            table.setColumnSelectionInterval(column, column);
        };
        return this;
    }
    
    /** Asks the editor if it can start editing using <code>anEvent</code>.
     * <code>anEvent</code> is in the invoking component coordinate system.
     * The editor can not assume the Component returned by
     * <code>getCellEditorComponent</code> is installed.  This method
     * is intended for the use of client to avoid the cost of setting up
     * and installing the editor component if editing is not possible.
     * If editing can be started this method returns true.
     *
     * @param	anEvent		the event the editor should use to consider
     * 				whether to begin editing or not
     * @return	true if editing can be started
     * @see #shouldSelectCell
     *
     */
    public boolean isCellEditable(EventObject anEvent)
    {
        return true;
    }
    
    /** Removes a listener from the list that's notified
     *
     * @param	l		the CellEditorListener
     *
     */
    public void removeCellEditorListener(CellEditorListener l)
    {
        listeners.remove(l);
    }
    
    /** Returns true if the editing cell should be selected, false otherwise.
     * Typically, the return value is true, because is most cases the editing
     * cell should be selected.  However, it is useful to return false to
     * keep the selection from changing for some types of edits.
     * eg. A table that contains a column of check boxes, the user might
     * want to be able to change those checkboxes without altering the
     * selection.  (See Netscape Communicator for just such an example)
     * Of course, it is up to the client of the editor to use the return
     * value, but it doesn't need to if it doesn't want to.
     *
     * @param	anEvent		the event the editor should use to start
     * 				editing
     * @return	true if the editor would like the editing cell to be selected;
     *    otherwise returns false
     * @see #isCellEditable
     *
     */
    public boolean shouldSelectCell(EventObject anEvent)
    {
        return true;
    }
    
    /** Tells the editor to stop editing and accept any partially edited
     * value as the value of the editor.  The editor returns false if
     * editing was not stopped; this is useful for editors that validate
     * and can not accept invalid entries.
     *
     * @return	true if editing was stopped; false otherwise
     *
     */
    public boolean stopCellEditing()
    {
        TableModel model=table.getModel();
        if (model!=null && row<model.getRowCount() && column<model.getColumnCount())
        {
            model.setValueAt(getText(),row,column);
            table.setRowSelectionInterval(row,row);
            table.setColumnSelectionInterval(column, column);
        };
        for (int i=listeners.size()-1;i>=0;i--)
        {
            ((CellEditorListener) listeners.elementAt(i)).editingStopped(new ChangeEvent(this));
        };
        setText("");
        return true;
    }
    
}
