/*
 * Interactions.java
 *
 * JTabbedpane containing the interaction and watch windows
 * Created on March 20, 2002, 7:10 PM
 */

package IDE.Interaction;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import IDE.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import IDE.Configuration.*;
import IDE.Utilities.*;
import Install.*;

/**
 * Pane containing all the interaction and watch panes
 * @author  marc
 */
public class Interactions extends JTabbedPane implements ChangeListener,MouseListener,IIDEAddin
{
    /** Contains the current directory (directory used for the file dialogs and interactions) */
    public static String currentDirectory=System.getProperty("user.dir");
    /** Allows us to work with static methods */
    private static Interactions interactions;
    /** Static boolean value used to avoid a problem with final in the stopping of a interaction  (set stopInteraction for use */
    private static boolean interactionStopped;
    /** Creates a new instance of Interactions */
    public Interactions(int  position)
    {
        super(position);
        interactions=this;
        addChangeListener(this);
        addMouseListener(this);
        setFocusable(false);
    }
    
    /** Make the interactionpane containing the given interaction selected */
    public void selectInteraction(IInteraction interaction)
    {
        InteractionPane interactionpane;
        int m=getComponentCount();
        for (int i=0;i<m;i++)
        {
            if (getComponentAt(i) instanceof InteractionPane)
            {
                interactionpane=(InteractionPane) getComponentAt(i);
                if (interactionpane.getInteraction()==interaction)
                    setSelectedComponent(interactionpane);
            };
        }
    }
    /** Get the selected interaction pane */
    public static InteractionPane getSelectedInteraction()
    {
        Component selected;
        if ((selected=interactions.getSelectedComponent()) instanceof InteractionPane)
            return (InteractionPane) selected;
        else
            return null;
    }
    /** Get the selected watch pane */
    public static WatchPane getSelectedWatch()
    {
        Component selected;
        if ((selected=interactions.getSelectedComponent()) instanceof WatchPane)
            return (WatchPane) selected;
        else
            return null;
    }
    /** Get a array of the InteractionPanes */
    public static InteractionPane[] getInteractionPanes()
    {
        Vector temp=new Vector();
        
        int m=interactions.getComponentCount();
        for (int i=0;i<m;i++)
            if (interactions.getComponentAt(i) instanceof InteractionPane)
                temp.addElement(interactions.getComponentAt(i));
        InteractionPane[] result=new InteractionPane[temp.size()];
        for (int i=0;i<result.length;i++)
            result[i]=(InteractionPane) temp.elementAt(i);
        return result;
    }
    /** Set the current directory for all the interactions */
    public static void setCurrentDirectory(File file)
    {
        InteractionPane[] panes=getInteractionPanes();
        for (int i=0;i<panes.length;i++)
            panes[i].getInteraction().setCurrentDirectory(Util.getCanonicalFileName(file));
        currentDirectory=Util.getCanonicalFileName(file);
    }
    /** Must be implemented for the ChangeListener interface */
    public void stateChanged(ChangeEvent changeEvent)
    {
        setSelectedPaneCurrent();
    }
    /** Make the selected interaction pane the current one */
    private void setSelectedPaneCurrent()
    {
        AbstractInteractionPane pane=(AbstractInteractionPane) getSelectedComponent();
        if (pane!=null)
            Main.setCurrent(pane.getEditor());
        else
            Main.setCurrent(null);
    }
    /** Static method to cleanup all the interactions */
    public static void cleanup()
    {
        InteractionPane[] interactionpanes=getInteractionPanes();
        int m=interactionpanes.length;
        for (int i=0;i<m;i++)
            stopInteraction(interactionpanes[i].getInteraction());
    }
    /** Do a close of the interaction, waiting a maximum of Configuration.timeout on the stop */
    private static void stopInteraction(final IInteraction interaction)
    {
        interactionStopped=false;
        Thread stopInteraction=new Thread(new Runnable()
        {
            public void run()
            {
                interaction.stopProcess();
                interactionStopped=true;
            }
        });
        stopInteraction.start();
        try
        {
            for (int maxsleep=0;maxsleep<Configuration.timeoutStopProcess && !interactionStopped;maxsleep=maxsleep+50)
                Thread.sleep(50);
        }
        catch (InterruptedException e)
        {
        };
    }
    /** Static method to add only a  interaction pane to this tabbedpane (used to restore state)*/
    public static void addInteraction(Parser parser,IInteraction interaction)
    {
        InteractionPane pane=new InteractionPane(parser,interaction);
        interactions.add(pane.getName(),pane);
    }
    /** Static method to add a watch pane associated with the interaction pane at i (used to restore state) */
    public static void addWatch(int index)
    {
        InteractionPane pane=(InteractionPane) interactions.getComponentAt(index);
        interactions.add("Watch "+pane.getName(),new WatchPane(pane));
    }
    /** Enumerate all the editors in a deterministic way */
    public void enumerateEditors(Vector editors)
    {
        int m=getComponentCount();
        for (int i=0;i<m;i++)
            if (getComponentAt(i) instanceof AbstractInteractionPane)
                editors.add(((AbstractInteractionPane) getComponentAt(i)).getEditor());
    }
    /**
     * This method will be called just before exit and allow the addin to save its state, the addin must be registered via a call
     * to Main.registerAddin()
     * Use the XMLConfigClass(Field,Methodor Object) objects to serialize this class
     */
    public void saveState(Writer writer)
    {
        XMLConfigClass cls=new XMLConfigClass(this.getClass().getName());
        XMLConfigMethod method;
        XMLConfigObject parser;
        XMLConfigObject interaction;
        XMLConfigField field;
        
        int m=getTabCount();
        int index;
        Vector savedinteractions=new Vector();
        InteractionPane ipane;
        WatchPane wpane;
        // Add the static current directory field
        field=new XMLConfigField("currentDirectory",OsSystem.correctPathName(currentDirectory));
        cls.addField(field);
        // Second add all the interaction panes
        for (int i=0;i<m;i++)
        {
            if (getComponentAt(i) instanceof InteractionPane)
            {
                ipane=(InteractionPane) getComponentAt(i);
                savedinteractions.add(ipane);
                parser=new XMLConfigObject(ipane.getEditor().getDocument().getParser().getClass().getName());
                parser.addArgument(true);
                interaction=new XMLConfigObject(ipane.getInteraction().getClass().getName());
                method=new XMLConfigMethod("addInteraction");
                method.addArgument(parser);
                method.addArgument(interaction);
                cls.addMethod(method);
            };
        };
        // Now add all the associated watch panes
        for (int i=0;i<m;i++)
        {
            if (getComponentAt(i) instanceof WatchPane)
            {
                wpane=(WatchPane) getComponentAt(i);
                index=savedinteractions.indexOf(wpane.getInteractionPane());
                method=new XMLConfigMethod("addWatch");
                method.addArgument(index);
                cls.addMethod(method);
            };
        };
        cls.serialize(writer);
        // Serialize now the Interaction panes
        AbstractInteractionPane.saveState(writer);
    }
    /* Repaint all the components */
    public void repaintAll()
    {
        CodePane pane=(CodePane) getSelectedComponent();
        if (pane!=null)
        {
            pane.revalidate();
            pane.refreshBackground();
            pane.initAndRepaintVisible();
        }
    }
    /** Check if one of the components is current */
    public boolean isCurrent()
    {
        CodeEditor editor=Main.currentEditor();
        int m=getComponentCount();
        for (int i=0;i<m;i++)
            if (((CodePane) getComponentAt(i)).getEditor()==editor)
                return true;
        return false;
    }
    /** Go to the next interaction pane */
    public void next()
    {
        int index=getSelectedIndex();
        if (index >=0)
        {
            index++;
            if (index>=getComponentCount())
                index=0;
            if (index<getComponentCount())
                setSelectedIndex(index);
        };
    }
    /** Select the workpane containing the editor given as argument */
    public static void selectInteractionPaneContainingEditor(CodeEditor editor)
    {
        int m=interactions.getComponentCount();
        for (int i=0;i<m;i++)
            if (((CodePane) interactions.getComponentAt(i)).getEditor()==editor)
            {
                interactions.removeChangeListener(interactions); // Tempory disable the processing of tab processing
                interactions.setSelectedIndex(i);
                interactions.addChangeListener(interactions);
                break;
            }
    }
    /** Methods which must be implemented for the mouselistener */
    public void mouseClicked(MouseEvent mouseEvent)
    {
    }
    
    public void mouseEntered(MouseEvent mouseEvent)
    {
    }
    
    public void mouseExited(MouseEvent mouseEvent)
    {
    }
    
    public void mousePressed(MouseEvent mouseEvent)
    {
        setSelectedPaneCurrent();
    }
    
    public void mouseReleased(MouseEvent mouseEvent)
    {
    }
    
}
