/*
 * LispInteraction.java
 *
 * Created on November 7, 2001, 7:42 PM
 */

package IDE.Interaction;

import IDE.Messaging.*;
import IDE.*;
import IDE.Configuration.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import IDE.LispConnection.*;
import IDE.WorkArea.*;
import IDE.Watch.*;
import IDE.Utilities.*;
import Install.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import IDE.Project.*;

/**
 * Class to interact with a LISP process
 * @author  marc
 */
public class LispInteraction implements IInteraction
{
    /** Indicates we must stop the output(error) listen thread */
    protected boolean stoplisten=false;
    /** Process if of the process to interact with */
    protected int processid=-1;
    /** Port number used for the connection with lisp */
    private static int portnr=4442;
    /** Static array of  icons */
    private static ImageIcon[] icons;
    /** Static array of type names */
    private static String[] types;
    /** Interaction pane interacting with this interaction */
    protected InteractionPane interactionpane=null;
    static
    {
        icons=new ImageIcon[]
        {
            Configuration.getIcon("watch_unknow.png"), // 0
            Configuration.getIcon("watch_string.png"),    //1
            Configuration.getIcon("watch_bit_vector.png"),
            Configuration.getIcon("watch_vector.png"), // 3;
            Configuration.getIcon("watch_array.png"), // 4;
            Configuration.getIcon("watch_character.png"), //5;
            Configuration.getIcon("watch_complex.png"),// 6;
            Configuration.getIcon("watch_float.png"), //7;
            Configuration.getIcon("watch_function.png"), // 8;
            Configuration.getIcon("watch_hash_table.png"), //9;
            Configuration.getIcon("watch_integer.png"), //10;
            Configuration.getIcon("watch_ration.png"), //11;
            Configuration.getIcon("watch_number.png"), //12;
            Configuration.getIcon("watch_package.png"),//13;
            Configuration.getIcon("watch_pathname.png"), //14;
            Configuration.getIcon("watch_randomstate.png"),//15;
            Configuration.getIcon("watch_readtable.png"),//16;
            Configuration.getIcon("watch_null.png"),//17;
            Configuration.getIcon("watch_list.png"),//18;
            Configuration.getIcon("watch_cons.png"),//19;
            Configuration.getIcon("watch_sequence.png"),//20;
            Configuration.getIcon("watch_stream.png"),//21;
            Configuration.getIcon("watch_symbol.png"),//22;
            Configuration.getIcon("watch_T.png"),//3;
            Configuration.getIcon("watch_structure.png"),//24;
            Configuration.getIcon("watch_class.png"),//25;
            Configuration.getIcon("watch_unknow.png") //26;
        };
        types=new String[]
        {
            new String("unknow"), // 0
            new String("string"),    //1
            new String("bit_vector"),
            new String("vector"), // 3;
            new String("array"), // 4;
            new String("character"), //5;
            new String("complex"),// 6;
            new String("float"), //7;
            new String("function"), // 8;
            new String("hash_table"), //9;
            new String("integer"), //10;
            new String("ration"), //11;
            new String("number"), //12;
            new String("package"),//13;
            new String("pathname"), //14;
            new String("randomstate"),//15;
            new String("readtable"),//16;
            new String("null"),//17;
            new String("list"),//18;
            new String("cons"),//19;
            new String("sequence"),//20;
            new String("stream"),//21;
            new String("symbol"),//22;
            new String("t"),//3;
            new String("structure"),//24;
            new String("class"),//25;
            new String("unknow") //26;
        };
    }
    /** Hash table linking editor to debug info */
    private Hashtable editorToDebuggedEditor;
    /** The command to start the lisp process */
    protected String startcommand;
    /** The first lisp command to send to the lisp system */
    protected String lispcommand;
    /** Lisp process */
    protected Process process;
    /** Input stream of the process */
    protected Writer lispinput;
    /** Output stream of the process */
    protected InputStream lispoutput;
    /** Error stream of the process */
    protected InputStream lisperror;
    /** Connection with the debugging code in LISP */
    protected LispConnection lispconnection=null;
    /** Contains the list of LispInteractions created */
    private static Vector interactions=new Vector();
    /** Constructor of  LispInteraction */
    protected LispInteraction(String startcommand,String lispcommand)
    {
        this.startcommand=startcommand;
        this.lispcommand=lispcommand;
        interactions.add(this); // Register this interaction
        editorToDebuggedEditor=new Hashtable();
    }
    /** Creat a new LispInteraction window  (Making sure that the only one instance can be created) */
    protected static IInteraction createInteractionImp(String startcommand,String lispcommand)
    {
        return new LispInteraction(startcommand,lispcommand);
    }
    /** Ask the lispsystem for the syntax of a function,macro or method */
    public static String getSyntaxDescription(String name)
    {
        InteractionPane interactionpane=Interactions.getSelectedInteraction();
        LispInteraction interaction;
        if (interactionpane==null)
        {
            return "";
        }
        else
        {
            try
            {
                interaction=(LispInteraction) interactionpane.getInteraction();
                Object object=interaction.lispconnection.callLisp("DEBUGGER::get-syntax",new Object[]
                {name},true);
                if (object instanceof String)
                    return (String) object;
                else
                    return "";
            }
            catch (Exception e)
            {
                return "";
            }
        }
    }
    /** Send a string  to the input stream of a process, do not wait on it */
    public void sendCommand(final String command)
    {
        Runnable doSend=new Runnable()
        {
            public void run()
            {
                try
                {
                    MessageHandler.say("Lisp command : "+command);
                    if (command.length()==1 && command.charAt(0)=='\n')
                        lispinput.write(command);
                    else
                        lispinput.write(command+"\n");
                    lispinput.flush();
                }
                catch (IOException e)
                {
                    cleanupAftererror("IOError during write to lispprocess error "+e.getMessage(),e);
                };
            }
        };
        Thread thread=new Thread(doSend);
        thread.start();
    }
    /** Send a string via sockets to the Lisp process and returns the result */
    public String callProcess(String command)
    {
        try
        {
            if (lispconnection!=null)
            {
                Object object=lispconnection.callLisp("DEBUGGER::simple-eval-expression",new Object[]
                {command},true);
                if (object instanceof String)
                    return (String) object+"\n";
                else
                    return "\n";
            }
            else
            {
                return "\n";
            }
        }
        catch (IOException e)
        {
            cleanupAftererror("Error sending watch command to lisp system",e);
            return "\n";
        }
    }
    /** Set a watch point in the LISP process */
    public void setWatchPoint(WatchNode node,String watchExp)
    {
        if (lispconnection!=null)
            asyncCallLisp("DEBUGGER::register-watch-expression",new Object[]{node,watchExp});
    }
    /** Edit a watch point in the LISP process */
    public void changeWatchPoint(WatchNode node,String watchExp)
    {
        if (lispconnection!=null)
            asyncCallLisp("DEBUGGER::change-watch-expression",new Object[]{node,watchExp});
    }
    /** Delete a watch point in the LISP process*/
    public void removeWatchPoint(WatchNode node)
    {
        if (lispconnection!=null)
           asyncCallLisp("DEBUGGER::unregister-watch-expression",new Object[] {node});
    }
    /** Set the current directory in the lisp subsystem */
    public void setCurrentDirectory(String directory)
    {
        if (directory==null || directory.length()==0)
            return; 
        // Ensure that we end the string with a FileSeparator (needed to not confuse the parse-namestring in lisp)
        if (directory.charAt(directory.length()-1)!=File.separatorChar)
            directory=directory+Util.normalize(File.separator);
        if (lispconnection!=null)
            asyncCallLisp("(DEBUGGER::set-current-directory \""+directory+"\")");
    }
    /** Send a watch expression to the lisp process and returns the result, this method must be called assymchronously to avaoid lockups */
    public LinkedList watchExpression(String expression)
    {
        try
        {
            if (lispconnection!=null)
            {
                Object object=lispconnection.callLisp("DEBUGGER::eval-expression",new Object[]
                {expression},true);
                if (object instanceof LinkedList)
                    return (LinkedList) object;
                else
                    return null;
            }
            else
            {
                return null;
            }
        }
        catch (IOException e)
        {
            return null;
        }
    }
    /** First retrieve from error stream if this is blocking then retrieve from output */
    public String receiveErrorAndOutput()
    {
    	
        StringBuffer error=new StringBuffer();
        StringBuffer output=new StringBuffer();
        int n;
        try
        {
            while (true)
            {
                if (stoplisten)
                    return "";
                if (lisperror.available()==0 && lispoutput.available()==0)
                {
                    if (error.length()>0 || output.length()>0)
                        return error.toString()+output.toString();
                    // No output so we should block
                    Thread.sleep(50);
                }
                else if (lisperror.available()>0)
                {
                    error.append((char) lisperror.read());
                }
                else if (lispoutput.available()>0)
                {
                    output.append((char) lispoutput.read());
                };
            }
        }
        catch (InterruptedException e)
        {
            return null;
        }
        catch (IOException e)
        {
            cleanupAftererror("IO Error during read from lispprocess ",e);
            return null;
        }
    }
    /** Stop the process to interact with  */
    public void stopProcess()
    {
        try
        {
            if (lispconnection!=null)
            {
                // Undebug all the editors in debugmode for this interaction
                Iterator iter=editorToDebuggedEditor.values().iterator();
                CodeEditor editor;
                while (iter.hasNext())
                {
                    editor=((DebuggedEditor) iter.next()).editor;
                    disableDebugging(editor);
                    editor.setMode(Mode.getMode("basic"));
                    editor.highlighter.removeHighLight("40breakpoint");
                    editor.highlighter.removeHighLight("50nextstop");
                    editor.highlighter.removeHighLight("70executedcode");
                };
                // Try to stop the process
                asyncCallLisp("(USER::quit)");
                Thread.sleep(1000);
                // Cleanup
                lispconnection.cleanup();
                lispinput.close();
                lisperror.close();
                lispoutput.close();
                stoplisten=true;
                Thread.sleep(1000);
                process.destroy();
                process.waitFor();
                lispconnection=null;
                interactions.remove(this);
            };
        }
        catch (Exception e)
        {
            cleanupAftererror("Error stopping lisp proces because "+e.getMessage(),e);
        }
    }
    /** Function to get the process nr in a synchronised way and increase it by two */
    protected synchronized static int getPortNr()
    {
        portnr=portnr+2;
        int newportnr=LispConnection.getFreeSocketPort(portnr,portnr+100);
        if (newportnr>=0)
        {
            portnr=newportnr;
            return portnr;
        }
        else
        {
            MessageHandler.warn("Error getting new server socket port in the range "+portnr+"-"+(portnr+100));
            return -1;
        }
    }
    /** Start the process to interact with  */
    public void startProcess()
    {
        final int portnr=getPortNr();
        if (portnr<0)
            return;
        final Runnable setWatchResult=new Runnable()
        {
            public void run()
            {
                // Set a watchpoint for the result of a call
                Main.getWatches().addWatchPoint("Result("+getName()+")","DEBUGGER::**values**",LispInteraction.this);
            }
        };
        Thread createlispconnection=new Thread(new Runnable()
        {
            public void run()
            {
                Object result;
                try
                {
                    // Create the lisp connection
                    lispconnection=new LispConnection(portnr,portnr+1);
                    MessageHandler.say("Socket connection to lisp process established");
                    // Pass the interaction object to lisp
                    lispconnection.callLisp("DEBUGGER::set-interaction",new Object[]
                    {LispInteraction.this},true);
                    MessageHandler.say("Creating interaction object finished");
                    // Set the current directory
                    setCurrentDirectory(Interactions.currentDirectory);
                    // Get the process id of the LISP process
                    if (OsSystem.processManagmentIsPossible())
                    {
                        result=lispconnection.callLisp("(DEBUGGER::get-process-id)",true);
                        if (result instanceof Long)
                            processid=((Long) result).intValue();
                    };
                    // Set a watchpoint for the result of a call
                    SwingUtilities.invokeLater(setWatchResult);
                    // Successfull completed
                }
                catch (Exception e)
                {
                    // MessageHandler.report("Failed to establish socket connection to lisp process",e);
                    cleanupAftererror("Failed to establish socket connection to lisp process",e);
                };
            }
        });
        try
        {
            // Start the lisp process
            process=Runtime.getRuntime().exec(InteractionCommands.parseCommandString(startcommand),null,new File(Configuration.installDirectory));
            // Get the input,output and error stream
            lispoutput=process.getInputStream();
            lisperror=process.getErrorStream();
            lispinput=new OutputStreamWriter(process.getOutputStream());
            // Load the code responsible for the debugger (do this in syncrhonous way
            lispinput.write(lispcommand+"\n");
            lispinput.flush();
            // Create the connection with the lisp system
            createlispconnection.start();
            // Start the debugging code
            sendCommand("(DEBUGGER::enter-debug "+String.valueOf(portnr)+" "+String.valueOf(portnr+1)+")");
            // Load the startup code
            if (Configuration.lispStartupFile.length()>0 && new File(Configuration.lispStartupFile).exists())
                sendCommand("(load \""+Configuration.lispStartupFile+"\")");
        }
        catch (IOException e)
        {
            cleanupAftererror("Could not run the command <"+startcommand+">",e);
        }
        catch (Exception e)
        {
            cleanupAftererror("Lisp startup interrupted",e);
        }
    }
    /** Enable debugging of a document, returning true if succeeded, this call can block */
    public boolean enableDebugging(CodeEditor editor)
    {
        if (editor.getCodePane() instanceof AbstractInteractionPane)
            return false;
        else if (lispconnection==null)
            return false;
        try
        {
            File temp1=File.createTempFile("debug",".lisp");
            File temp2=File.createTempFile("instrument",".lisp");
            editor.getDocument().saveTemporyFile(temp1);
            String path1=Util.getCanonicalFileName(temp1);
            String path2=Util.getCanonicalFileName(temp2);
            // Set the **check-error** and **debug-after** flags to influence the generated instrumentation code
            if (Configuration.instrumentCheckError)
                lispconnection.callLisp("(setf DEBUGGER::**check-error** t)",true);
            else
                lispconnection.callLisp("(setf DEBUGGER::**check-error** nil)",true);
            if (Configuration.instrumentDebugAfter)
                lispconnection.callLisp("(setf DEBUGGER::**debug-after** t)",true);
            else
                lispconnection.callLisp("(setf DEBUGGER::**debug-after** nil)",true);
            // Add the instrumentation code
            if (((Boolean) lispconnection.callLisp("DEBUGGER::debug-open-file",new Object[]
            {path1,path2,editor},true)).booleanValue())
            {
                DebuggedEditor debuggededitor=new DebuggedEditor(editor,temp1);
                editorToDebuggedEditor.put(editor,debuggededitor);
                editor.getDocument().setReadOnly(true);
                return true;
            }
            else
            {
                MessageHandler.warn("Failed to enable debugging for this editor");
                return false;
            }
        }
        catch (IOException e)
        {
            MessageHandler.report("Failed to create temp or debug file",e);
            return false;
        }
    }
    /** Disable debugging of a document, this call can block */
    public void disableDebugging(CodeEditor editor)
    {
        try
        {
            editor.getDocument().setReadOnly(false);
            DebuggedEditor debuggededitor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
            if (debuggededitor!=null)
            {
                debuggededitor.stopProfiling();
                String path=Util.getCanonicalFileName(debuggededitor.savedDocument);
                if (lispconnection!=null)
                    lispconnection.callLisp("DEBUGGER::undebug",new Object[]
                    {path,editor},true);
                editorToDebuggedEditor.remove(editor);
            };
        }
        catch (IOException e)
        {
            MessageHandler.report("Failed to disable debugging",e);
        };
    }

    /** Toggle a breakpoint */
    
    public void toggleBreakpoint(final CodeEditor editor)
    {
        final DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        Parser parser=editor.getDocument().getParser();
        final Region region=parser.findHooks(editor.cursor.cursorOffset());
        if (region==null) return;
        region.end++; // Last hook must be included
        final Runnable addHighLight=new Runnable()
        {
            public void run()
            {
                editor.highlighter.addHighLightRegion("40breakpoint",Configuration.colorBreakpoint,region.begin,region.end-region.begin,true);
            }
        };
        final Runnable removeHighLight=new Runnable()
        {
            public void run()
            {
                editor.highlighter.removeHighLightRegion("40breakpoint",region.begin,region.end-region.begin,true);
            }
        };
        Thread setBreakPoint=new Thread(new Runnable()
        {
            public void run()
            {
                if (lispconnection!=null)
                {
                    try
                    {
                        Object result=lispconnection.callLisp("DEBUGGER::toggle-breakpoint",new Object[]
                        {editor,new Integer(region.begin+1),new Integer(region.end+1)},true);
                        if (result instanceof Long)
                        {
                            switch (((Long) result).intValue())
                            {
                                case 1: // Breakpoint has been set
                                    SwingUtilities.invokeLater(addHighLight);
                                    break;
                                case 0: // No valid breakpoint
                                    MessageHandler.warn("No valid breakpoint");
                                    break;
                                case -1:
                                    SwingUtilities.invokeLater(removeHighLight);
                                    break;
                            }
                        }
                        else
                        {
                            MessageHandler.warn("Failed to set/unset breakpoint because of wong type returned");
                        }
                    }
                    catch (Exception e)
                    {
                        cleanupAftererror("Error setting/unsetting breakpoint ,error "+e.getMessage(), e);
                    };
                };
            }
        });
        setBreakPoint.run();
    }
    
    /** Toggle a conditional breakpoint */
    public void toggleConditionalBreakpoint(final CodeEditor editor)
    {
        final DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        Parser parser=editor.getDocument().getParser();
        final Region region=parser.findHooks(editor.cursor.cursorOffset());
        if (region==null) return;
        region.end++; // Last hook must be included
        final Runnable addHighLight=new Runnable()
        {
            public void run()
            {
                editor.highlighter.addHighLightRegion("40breakpoint",Configuration.colorBreakpoint,region.begin,region.end-region.begin,true);
            }
        };
        final Runnable removeHighLight=new Runnable()
        {
            public void run()
            {
                editor.highlighter.removeHighLightRegion("40breakpoint",region.begin,region.end-region.begin,true);
            }
        };
        final Runnable createBreakPoint=new Runnable()
        {
            public void run()
            {
                final String condition=CodeDialogBox.showDialog("Condition breakpoint",LispRepository.lisprepository.createParser(true),"");
                Thread setBreakPoint=new Thread(new Runnable()
                {
                    public void run()
                    {
                        if (lispconnection!=null)
                        {
                            try
                            {
                                Object result=lispconnection.callLisp("DEBUGGER::toggle-breakpoint",new Object[]
                                {editor,new Integer(region.begin+1),new Integer(region.end+1),condition},true);
                                if (result instanceof Long)
                                {
                                    switch (((Long) result).intValue())
                                    {
                                        case 1: // Breakpoint has been set
                                            SwingUtilities.invokeLater(addHighLight);
                                            break;
                                        case 0: // No valid breakpoint
                                            MessageHandler.warn("No valid breakpoint");
                                            break;
                                        case -1:
                                            SwingUtilities.invokeLater(removeHighLight);
                                            break;
                                    }
                                }
                                else
                                {
                                    MessageHandler.warn("Failed to set/unset breakpoint because of wong type returned");
                                }
                            }
                            catch (Exception e)
                            {
                                cleanupAftererror("Error setting/unsetting breakpoint ,error "+e.getMessage(), e);
                            };
                        };
                    }
                });
                if (condition!=null)
                    setBreakPoint.run();
            }
        };
        Thread thread=new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    if (lispconnection!=null)
                    {
                        Object result=lispconnection.callLisp("DEBUGGER::is-breakpoint",new Object[]
                        {editor,new Integer(region.begin+1),new Integer(region.end+1)},true);
                        if (result instanceof Boolean && ((Boolean) result).booleanValue())
                            SwingUtilities.invokeLater(createBreakPoint);
                    }
                }
                catch (IOException e)
                {
                    cleanupAftererror("Error verifying breakpoint",e);
                };
            }
        });
        thread.start();
    }
    
    /** Stop at the next lisp from to be executed */

    public void stepInto(CodeEditor editor)
    {
        DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        editor.highlighter.removeHighLight("50nextstop");
        editor.highlighter.removeHighLight("70executedcode");
        dEditor.currentBreakpoint=null;
        asyncCallLisp("DEBUGGER::step-into-action",new Object[]
        {editor});
    }
    /** Stop at the next lisp form which is in the same list as the currentBreakpoint */

    public void stepOver(CodeEditor editor)
    {
        DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        editor.highlighter.removeHighLight("50nextstop");
        editor.highlighter.removeHighLight("70executedcode");
        if (dEditor.currentBreakpoint!=null)
        {
            // Find the most enclosing list of the currentBreakpoint
            Parser parser=editor.getDocument().getParser();
            Region toplist=null;
            Region region=parser.findHooks(dEditor.currentBreakpoint.end+1);
            while (region!=null)
            {
                toplist=region;
                region=parser.findHooks(region.end+1);
            };
            if (toplist!=null)
                asyncCallLisp("DEBUGGER::step-over-action",new Object[]
                {editor,new Integer(toplist.begin+1),new Integer(toplist.end+1)});
            else
                asyncCallLisp("DEBUGGER::step-over-action",new Object[]
                {editor});
        }
        else
            asyncCallLisp("(DEBUGGER::end-debug-eventloop)");
        dEditor.currentBreakpoint=null;
    }
    
    /** Used in timetraveling to go to the future  */
    public void goToFuture(CodeEditor editor)
    {
        asyncCallLisp("(DEBUGGER::step-forward-in-time)");
    }
    
    /** Used in timetraveling to go to the past  */
    public void goToPast(CodeEditor editor)
    {
        asyncCallLisp("(DEBUGGER::step-back-in-time)");
    }
    
    /** Stop after the the execution of the displayed code (allows you to edit and modify the resulted code */

    public void stopAfter(CodeEditor editor)
    {
        DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        editor.highlighter.removeHighLight("50nextstop");
        editor.highlighter.removeHighLight("70executedcode");
        asyncCallLisp("DEBUGGER::stop-after-action",new Object[]
        {editor});
    }
    /** Set the return value of executed form */
    public void setNewReturnValue(CodeEditor editor)
    {
        DebuggedEditor debuggededitor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (debuggededitor!=null)
        {
            String returnexp=CodeDialogBox.showDialog("Expression to be returned",LispRepository.lisprepository.createParser(true));
            if (returnexp!=null)
            {
                if (lispconnection!=null)
                    asyncCallLisp("DEBUGGER::change-result",new Object[]
                    {returnexp});
            }
        };
    }
    
    /** Stop at the indicated lisp form */

    public void stopAt(CodeEditor editor)
    {
        DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        editor.highlighter.removeHighLight("50nextstop");
        editor.highlighter.removeHighLight("70executedcode");
        Region region=editor.getDocument().getParser().findHooks(editor.cursor.cursorOffset());
        if (region!=null)
        {
            asyncCallLisp("DEBUGGER::stop-at-action",new Object[]
            {editor,new Integer(region.begin+1),new Integer(region.end+2)});
            editor.highlighter.addHighLightRegion("50nextstop",Configuration.colorNextStop,region.begin,region.end-region.begin,true);
        }
        else
        {
            asyncCallLisp("DEBUGGER::stop-at-action",new Object[]
            {editor});
        };
        dEditor.currentBreakpoint=null;
    }
    
    /** Continue the execution */

    public void continueExecution(CodeEditor editor)
    {
        DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (dEditor==null) return;
        editor.highlighter.removeHighLight("50nextstop");
        editor.highlighter.removeHighLight("70executedcode");
        asyncCallLisp("DEBUGGER::continue-execution-action",new Object[]
        {editor});
    }
    /** Called for displaying the result of stepping forward or backward in time */
    public void displayNewTime(final CodeEditor editor,Long begin,Long end)
    {
        final DebuggedEditor debuggededitor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        final int b=begin.intValue();
        final int e=end.intValue();
        Runnable showExecutedCode=new Runnable()
        {
            public void run()
            {
                // Highlight the executed code
                editor.cursor.setCursor(b);
                editor.highlighter.removeHighLight("70executedcode");
                editor.highlighter.removeHighLight("50nextstop");
                editor.highlighter.addHighLightRegion("70executedcode",Configuration.colorExecutingCode,b,e-b,true);
                WorkArea.selectWorkPaneContainingEditor(editor);
            }
        };
        if (debuggededitor!=null)
        {
            SwingUtilities.invokeLater(showExecutedCode);
        };
    }
    /**  Called in case of an error or when we stop after code is executed */
    public void giveControlToInterfaceAfter(final CodeEditor editor,Long begin,Long end)
    {
        final DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        final int b=begin.intValue();
        final int e=end.intValue();
        Runnable showExecutedCode=new Runnable()
        {
            public void run()
            {
                // Highlight the executed code
                CodeEditor current=Main.currentEditor();
                editor.cursor.setCursor(b);
                if (current!=null)
                    Main.setCurrent(current);
                editor.highlighter.removeHighLight("70executedcode");
                editor.highlighter.removeHighLight("50nextstop");
                editor.highlighter.addHighLightRegion("70executedcode",Configuration.colorExecutedCode,b,e-b,true);
                WorkArea.selectWorkPaneContainingEditor(editor);
            }
        };
        {
            if (dEditor!=null)
            {
                dEditor.currentBreakpoint=new Breakpoint(b,e);
                SwingUtilities.invokeLater(showExecutedCode);
            };
        }
    }
    
    /** Called by the lisp process to highlight the part executing and let user command the debugger */
    public boolean giveControlToInterface(final CodeEditor editor,Long begin,Long  end)
    {
        final DebuggedEditor dEditor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        final int b=begin.intValue();
        final int e=end.intValue();
        Runnable showExecutingCode=new Runnable()
        {
            public void run()
            {
                // Highlight the executed code
                CodeEditor current=Main.currentEditor();
                editor.cursor.setCursor(b);
                if (current!=null)
                    Main.setCurrent(current);
                editor.highlighter.removeHighLight("70executedcode");
                editor.highlighter.removeHighLight("50nextstop");
                editor.highlighter.addHighLightRegion("70executedcode",Configuration.colorExecutingCode,b,e-b,true);
                WorkArea.selectWorkPaneContainingEditor(editor);
            }
        };
        if (dEditor==null)
        {
            return true; // Normally this may not occur , just ensure that we go ahead
        }
        else
        {
            dEditor.currentBreakpoint=new Breakpoint(b,e);
            SwingUtilities.invokeLater(showExecutingCode);
            return false; // Ensure that we wait in lisp
        }
    }
    
    /** Compile the content of a editor in the current interaction window */
    public void compileEditor(final CodeEditor editor)
    {
        Thread thread=new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    File file=File.createTempFile("debuggerload",".lisp");
                    editor.getDocument().saveTemporyFile(file);
                    Object result=lispconnection.callLisp("DEBUGGER::checked-compile-file",new Object[]
                    {Util.getCanonicalFileName(file)},false);
                    if (result instanceof Boolean)
                    {
                        if (((Boolean) result).booleanValue())
                            MessageHandler.say("Content compiled");
                        else
                            MessageHandler.warn("Failed to compile content loaded");
                    };
                    file.delete();
                }
                catch (IOException e)
                {
                    MessageHandler.warn("Failed to create tempory file");
                };
            }
        });
        if (lispconnection!=null)
            thread.run();
    }
    /** Load the content of a editor in the current interaction window */
    public void loadEditor(final CodeEditor editor)
    {
        Thread thread=new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    File file=File.createTempFile("debuggerload","lisp");
                    editor.getDocument().saveTemporyFile(file);
                    Object result=lispconnection.callLisp("DEBUGGER::checked-load-file",new Object[]
                    {Util.getCanonicalFileName(file)},false);
                    if (result instanceof Long)
                    {
                        final int pos=((Long) result).intValue();
                        if (pos>0) // Error during load , position in the editor
                        {
                            Runnable position=new Runnable()
                            {
                                public void run()
                                {
                                    editor.cursor.setCursor(pos-1);
                                }
                            };
                            SwingUtilities.invokeLater(position);
                        }
                        else
                        {
                            MessageHandler.say("Content loaded");
                        }
                    };
                    file.delete();
                }
                catch (IOException e)
                {
                    MessageHandler.warn("Failed to create tempory file");
                };
            }
        });
        if (lispconnection!=null)
            thread.run();
    }

    /** Ask the LISP vor the value of the leaf, return null if no value is associated with leaf */
    public String getValueLeaf(CodeEditor editor,int pos)
    {
        CodeLeafElement leaf=editor.getDocument().getLeafAt(pos);
        if (leaf==null)
        {
            return null;
        }
        else if (leaf instanceof CodeLeafSeparator)
        {
            return null;
        }
        else
        {
            try
            {
                if (lispconnection!=null)
                {
                    Object object=lispconnection.callLisp("DEBUGGER::simple-eval-expression",new Object[]
                    {leaf.getString(editor.getDocument())},true);
                    if (object instanceof String)
                    {
                        return (String) object;
                    }
                    else
                    {
                        return null;
                    }
                }
                else
                {
                    return null;
                }
            }
            catch (IOException e)
            {
                cleanupAftererror("Error getting value of leaf",e);
                return null;
            }
        }
    }
    /** Called to tell which interactionpane is working with this interaction */
    public void setInteractionPane(InteractionPane interactionpane)
    {
        this.interactionpane=interactionpane;
    }
    /** Called in case of a error detected in the lisp process */
    protected void cleanupAftererror(String error,Exception e)
    {
        // Warn the user that there is fatal error in the interaction
        MessageHandler.report(error,e);
        // Cleanup all the editors
        Iterator iter=editorToDebuggedEditor.values().iterator();
        CodeEditor editor;
        while (iter.hasNext())
        {
            editor=((DebuggedEditor) iter.next()).editor;
            disableDebugging(editor);
            editor.setMode(Mode.getMode("basic"));
            editor.highlighter.removeHighLight("40breakpoint");
            editor.highlighter.removeHighLight("50nextstop");
            editor.highlighter.removeHighLight("70executedcode");
        };
        
        // Connection cleanup (avoid that we try to stop the process again)
        lispconnection=null;
        // Ask the interaction pane to cleanup
        if (interactionpane!=null)
            interactionpane.cleanup();
    }
    /** Get the icon corresponding with a certain type */
    public ImageIcon getTypeIcon(int type)
    {
        return icons[type];
    }
    /** Get the extension of compiled code */
    public String getCompiledExtension()
    {
        return "";
    }
    /** Get the string corresponding with a certain type */
    public String getTypeString(int type)
    {
        return types[type];
    }
    /* Get the name of this interaction object */
    public String getName()
    {
        return "LISP";
    }
    /** Inner class to represent a breakpoint */
    private class Breakpoint
    {
        private int begin;
        private int end;
        private String condition;
        
        private Breakpoint(int begin,int end)
        {
            this.begin=begin;
            this.end=end;
        }
        private Breakpoint(int begin,int end,String condition)
        {
            this.begin=begin;
            this.end=end;
            this.condition=condition;
        }
    }
    /** Call the lisp process in a assyncrhonous way */
    protected void asyncCallLisp(final String function,final Object[] args)
    {
        Thread thread=new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    if (lispconnection!=null)
                        lispconnection.callLisp(function,args,true);
                }
                catch (Exception e)
                {
                };
            }
        });
        thread.start();
    }
    /** Call the lisp process in a assyncrhonous way */
    protected void asyncCallLisp(final String command)
    {
        Thread thread=new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    if (lispconnection!=null)
                        lispconnection.callLisp(command,true);
                }
                catch (Exception e)
                {
                };
            }
        });
        thread.start();
    }
    
    /** Start or reset a profiling session  */
    public void startProfiling(CodeEditor editor)
    {
        DebuggedEditor debuggededitor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (debuggededitor!=null)
            debuggededitor.startProfiling();
    }
    
    /** Stop a profiling session  */
    public void stopProfiling(CodeEditor editor)
    {
        DebuggedEditor debuggededitor=(DebuggedEditor) editorToDebuggedEditor.get(editor);
        if (debuggededitor!=null)
            debuggededitor.stopProfiling();
    }
    
    /** Signal the process   */
    public void signalProcess(String signal)
    {
        if (OsSystem.processManagmentIsPossible() && processid>0)
        {
            Runtime runtime=Runtime.getRuntime();
            try
            {
                runtime.exec(new String[]
                {"kill",signal,Integer.toString(processid)});
            }
            catch (IOException e)
            {
                MessageHandler.warn("Failed to signal process");
            }
        }
        else
        {
            // The best we can do in the absense of process managment
            asyncCallLisp("(break)");
        };
    }
    /** Generates a file representing the directory where the lisp files are installed in */
    public File lispSubDir(String installdir)
    {
        return new File(installdir);
    }
    /** Inner class containing editors debugged */
    private class DebuggedEditor
    {
        /** Slider for profiling */
        private JSlider slider=null;
        /** Panel to contain slider , slider value and buttons */
        private JPanel sliderPanel=null;
        /** Indicate if we do profiling */
        private boolean profiling=false;
        /** Forms called */
        private HashMap callCount;
        /** Editor */
        private CodeEditor editor;
        /** Indicate step into mode */
        private boolean stepInto=false;
        /**  Contains the current breakpoint reached */
        private Breakpoint currentBreakpoint=null;
        /** File where the editor is saved */
        private File savedDocument;
        /** Create the information containing the editor */
        private DebuggedEditor(CodeEditor editor,File savedDocument)
        {
            this.editor=editor;
            this.savedDocument=savedDocument;
            callCount=new HashMap();
        }
        /** Activate reset profiling */
        private synchronized void startProfiling()
        {
            final Runnable createSliderPanel=new Runnable()
            {
                public void run()
                {
                    if (sliderPanel==null)
                    {
                        // Create the slider panel
                        sliderPanel=new JPanel();
                        sliderPanel.setLayout(new FlowLayout());
                        // Add a ancestor Listener to be notified when the slider becomes visible
                        sliderPanel.addAncestorListener(new AncestorListener()
                        {
                            public void ancestorAdded(AncestorEvent e)
                            {
                                updateCount(slider.getValue());
                            }
                            public void ancestorMoved(AncestorEvent e)
                            {
                            }
                            public void ancestorRemoved(AncestorEvent e)
                            {
                            }
                        });
                        // Create update button
                        JButton update=new JButton(new AbstractAction()
                        {
                            public void actionPerformed(ActionEvent e)
                            {
                                updateCount(slider.getValue());
                            }
                        });
                        update.setText("");
                        update.setBorder(BorderFactory.createEtchedBorder());
                        update.setToolTipText("Update the call count");
                        update.setIcon(Configuration.getIcon("refreshCallCount.png"));
                        // Create up and down buttons
                        JButton up=new JButton(new AbstractAction()
                        {
                            public void actionPerformed(ActionEvent e)
                            {
                                slider.setValue(Math.min(slider.getMaximum(),slider.getValue()+1));
                            }
                        });
                        up.setText("");
                        up.setBorder(BorderFactory.createEtchedBorder());
                        up.setToolTipText("Increase the display threshold");
                        up.setIcon(Configuration.getIcon("increaseThreshold.png"));
                        JButton down=new JButton(new AbstractAction()
                        {
                            public void actionPerformed(ActionEvent e)
                            {
                                slider.setValue(Math.max(1,slider.getValue()-1));
                            }
                        });
                        down.setText("");
                        down.setBorder(BorderFactory.createEtchedBorder());
                        down.setToolTipText("Decrease the display threshold");
                        down.setIcon(Configuration.getIcon("decreaseThreshold.png"));
                        // Create a textfield containing the current silder value
                        final JTextField valueField=new JTextField("1");
                        // Create the slider
                        slider=new JSlider(JSlider.HORIZONTAL);
                        slider.setMinimum(1);
                        slider.setMaximum(2);
                        slider.setValue(1);
                        slider.setMinorTickSpacing(1);
                        slider.setMajorTickSpacing(10);
                        slider.setPaintTicks(true);
                        slider.setSnapToTicks(true);
                        slider.addChangeListener(new ChangeListener()
                        {
                            public void stateChanged(ChangeEvent ev)
                            {
                                highlightCalled(slider.getValue());
                                valueField.setText(Integer.toString(slider.getValue()));
                                valueField.setSize(valueField.getPreferredSize());
                                sliderPanel.revalidate();
                            }
                        });
                        // Populate panel
                        sliderPanel.add(update);
                        sliderPanel.add(down);
                        sliderPanel.add(slider);
                        sliderPanel.add(up);
                        sliderPanel.add(valueField);
                        editor.getMode().addMiscToMode("debug",sliderPanel);
                    };
                }
            };
            Thread enableProfiling=new Thread(new Runnable()
            {
                public void run()
                {
                    try
                    {
                        lispconnection.callLisp("DEBUGGER::start-profiling",new Object[]
                        {editor},true);
                        callCount=new HashMap();
                        profiling=true;
                        SwingUtilities.invokeLater(createSliderPanel);
                    }
                    catch (IOException e)
                    {
                        cleanupAftererror("Error enabling profiling, error "+e.getMessage(), e);
                    };
                }
            });
            enableProfiling.start();
        }
        /** Stopprofiling */
        private synchronized void stopProfiling()
        {
            final Runnable removeSlider=new Runnable()
            {
                public void run()
                {
                    if (sliderPanel!=null)
                        editor.getMode().removeComponentFromMode("debug",sliderPanel);
                    editor.highlighter.removeHighLight("65profiling");
                    profiling=false;
                    sliderPanel=null;
                }
            };
            Thread disableProfiling=new Thread(new Runnable()
            {
                public void run()
                {
                    try
                    {
                        // Ensure that at least the user interface is updated
                        SwingUtilities.invokeAndWait(removeSlider);
                        // Tell the debugger to stop profiling for this editor
                        lispconnection.callLisp("DEBUGGER::stop-profiling",new Object[]
                        {editor},true);
                    }
                    catch (Exception e)
                    {
                    };
                }
            });
            disableProfiling.start();
        }
        /** Highlight the forms called at least count times */
        private void highlightCalled(final int count)
        {
            final Runnable dowork=new Runnable()
            {
                public void run()
                {
                    LeafVector leafs=editor.getDocument().getLeafs();
                    int m=leafs.size();
                    int cnt;
                    CodeLeafElement leaf;
                    
                    editor.highlighter.removeHighLight("65profiling");
                    for (int i=0;i<m;i++)
                    {
                        leaf=leafs.leafAt(i);
                        cnt=getCount(leaf);
                        if (cnt>=count)
                            editor.highlighter.addHighLightRegion("65profiling",Configuration.colorProfiling,leaf.begin(),leaf.end()-leaf.begin(),false);
                    };
                    editor.repaint();
                }
            };
            SwingUtilities.invokeLater(dowork);
        }
        private synchronized void updateCount(final int threshold)
        {
            Thread getCount=new Thread(new Runnable()
            {
                public void run()
                {
                    try
                    {
                        Object result=lispconnection.callLisp("DEBUGGER::get-call-count",new Object[]
                        {editor},true);
                        if (result instanceof LinkedList)
                        {
                            Iterator iter=((LinkedList) result).iterator();
                            LinkedList countinfo;
                            CodeElement element;
                            int b,count;
                            callCount.clear();
                            while (iter.hasNext())
                            {
                                countinfo=(LinkedList) iter.next();
                                b=((Long) countinfo.get(0)).intValue()-1;
                                count=((Long) countinfo.get(2)).intValue();
                                element=editor.getDocument().getElementAt(b);
                                if (element!=null)
                                    callCount.put(element,new Integer(count));
                                slider.setMaximum(Math.max(slider.getMaximum(),count));
                            };
                            highlightCalled(threshold);
                        };
                    }
                    catch (IOException e)
                    {
                        cleanupAftererror("Error getting profile info from lisp, error "+e.getMessage(), e);
                    };
                }
            });
            getCount.start();
        }
        /** Get the count */
        private int getCount(CodeLeafElement leaf)
        {
            Integer cnt;
            CodeElement element=leaf.getParent();
            for (;element!=null && !(element instanceof LispListElement);element=element.getParent());
            cnt=(Integer) callCount.get(element);
            if (cnt!=null)
                return cnt.intValue();
            else
                return -1;
        }
    }
}
