/*
 * WatchPane.java
 *
 * Created on November 27, 2001, 8:28 PM
 */

package IDE.Interaction;

import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import javax.swing.*;

/**
 * Provides a watch pane to send watch commands to the IInteraction object, used to implement
 * interactive watching, the watch is associated with a interaction pane
 * @author  marc
 */
public class WatchPane extends AbstractInteractionPane
{
    /** The interaction pane this Watch pane is associated with */
    private InteractionPane interactionPane;
    /** IInteraction to talk to */
    private IInteraction interaction;
    /** Contains the result of a call to the lisp system */
    private String result;
    /** Creates a new instance of WatchPane */
    protected WatchPane(InteractionPane interactionPane)
    {
        super(interactionPane.getEditor().getDocument().getParser().cloneParser(true));
        this.interactionPane=interactionPane;
        this.interaction=interactionPane.getInteraction();
        interactionPane.addWatchPane(this);
    }
    /** Enable debugging for a piece of code */
    public void enableDebugging(String code)
    {
        interactionPane.enableDebugging(code);
    }
    /** Enable debugging for document in CodeEditor */
    public void enableDebugging(CodeEditor editor)
    {
        interactionPane.enableDebugging(editor); // Let the associated interaction pane do the debugging
    }
    
    /** Process the enter key  */
    public boolean processEnter()
    {
        Parser parser=getEditor().getDocument().getParser();
        // Find the element to send to the interaction system
        CodeDocument document=getEditor().getDocument();
        String command=null;
        return executeCommand(findLastCommand());
    }
    
    /** Disable debugging for document in CodeEditor  */
    public void disableDebugging(CodeEditor editor)
    {
        interactionPane.disableDebugging(editor); // Let the associated interaction pane do the debugging
    }
    
    /** Return the interaction object this pane is interacting with  */
    public IInteraction getInteraction()
    {
        return interaction;
    }
    
    public InteractionPane getInteractionPane()
    {
        return interactionPane;
    }
    /** Get the name of the watch pane */
    public String getName()
    {
        return "Watch "+interactionPane.getName();
    }
    /** Insert a command */
    public void insertCommand(String command)
    {
        getEditor().insertStringAt(command,getEditor().getDocument().length()-1);
        moveCursorToEnd();
    }
    /** Execute the String command  */
    public boolean executeCommand(final String command)
    {
        final Runnable updateContent=new Runnable()
        {
            public void run()
            {
                addCommandToHistory(result);
                getEditor().insertStringAt("\n"+result,getEditor().getDocument().length()-1);
                moveToEnd();
                
            }
        };
        Runnable doWork=new Runnable()
        {
            public void run()
            {
                result=interaction.callProcess(command);
                SwingUtilities.invokeLater(updateContent);
            }
        };
        if (command!=null)
        {
            Thread thread=new Thread(doWork);
            thread.start();
            return true;
        }
        else
        {
            return false;
        }
    }
    /** Do a cleanup */
    public void cleanup()
    {
        // Stop the parsing of the code document
        getEditor().getDocument().getParser().stopParsing();
    }
}
