/*
 * GenerateLispCode.java
 *
 * Created on May 11, 2002, 3:34 PM
 */

package IDE.LispScripting;


import IDE.Messaging.*;
import IDE.Configuration.*;
import IDE.Interaction.*;
import IDE.Utilities.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;

/**
 * Generates the LispCode so that from LISP we can call IDE methods
 * especially Actions and Questions
 * @author  marc
 */
public class GenerateLispCode
{
    /** Creates a new instance of GenerateLispCode, is private to avoid any  */
    private GenerateLispCode()
    {
    }
    /** Generates the LispCode */
    public static void generate()
    {
        MessageHandler.say("Start generating LISP code");
        try
        {
            String sourcedir=InstallParameters.getSourceDir();
            String installdir=InstallParameters.getInstallDir();
            File seedfile=new File(installdir,"seed.lisp");
            File resultfile=new File(installdir,"jabberwocky.lisp");
            File sourcefile=new File(sourcedir,"jabberwocky.lisp");
            if (resultfile.exists())
                resultfile.delete();
            resultfile.createNewFile();
            BufferedReader reader=new BufferedReader(new FileReader(seedfile));
            BufferedWriter writer=new BufferedWriter(new FileWriter(resultfile,false));
            int c;
            /** Copy the contents of sed.lisp in jabberwocky.lisp */
            while ((c=reader.read())!=-1)
                writer.write(c);
            /** Generate the LISP code*/
            writer.write(";;; ***************************************************************\n");
            writer.write(";;; Generated code by Jabberwocky \n");
            writer.write(";;; ***************************************************************\n");
            /** Constants inidicating type of event */
            try
            {
                Class cls=Class.forName("java.awt.event.KeyEvent");
                Field[] fields=cls.getFields();
                for (int i=0;i<fields.length;i++)
                {
                    if (fields[i].getType()==Integer.TYPE &&
                    Modifier.isPublic(fields[i].getModifiers()) &&
                    Modifier.isStatic(fields[i].getModifiers()) &&
                    Modifier.isFinal(fields[i].getModifiers()))
                    {
                        writer.write("(defconstant **"+fields[i].getName().toLowerCase().replace('_','-')+"** "+fields[i].getInt(null)+")\n");
                    };
                }
            }
            catch (Exception e)
            {
                MessageHandler.warn("Error finding fields in KeyEvent class, "+e.getMessage());
            };
            /** Function definitions to execute a action */
            writer.write(";;; ***************************************************************\n");
            writer.write(";;; Functions to execute a action \n");
            writer.write(";;; ***************************************************************\n");
            Iterator iter=Mode.getActionIterator();
            while (iter.hasNext())
            {
                writeActionDefinition((AbstractManagedAction) iter.next(),writer);
            }
            writer.close();
            /** Compiles the code, using the open Lisp Interaction objects */
            InteractionPane[] interactionpanes=Interactions.getInteractionPanes();
            for (int i=0;i<interactionpanes.length;i++)
            {
                if (interactionpanes[i].getInteraction() instanceof LispInteraction)
                {
                    //String compiledfile=installdir+File.separator+"jabberwocky."+((LispInteraction) interactionpanes[i].getInteraction()).getCompiledExtension();
                    String compiledfile=Util.getCanonicalFileName(((LispInteraction) interactionpanes[i].getInteraction()).lispSubDir(installdir))+File.separator+"jabberwocky."+((LispInteraction) interactionpanes[i].getInteraction()).getCompiledExtension();
                    interactionpanes[i].executeCommand("(if (not (find-package \"JABBERWOCKY\")) (make-package \"JABBERWOCKY\"))");
                    interactionpanes[i].executeCommand("(compile-file \""+Util.getCanonicalFileName(resultfile)+"\")");
                    interactionpanes[i].executeCommand("(load \""+compiledfile+"\")");
                }
            };
            /** Copy the generated code (jabberwocky,lisp) to the source directory if it exists */
            if (new File(sourcedir).exists())
            {
                if (sourcefile.exists())
                    sourcefile.delete();
                sourcefile.createNewFile();
                reader=new BufferedReader(new FileReader(resultfile));
                writer=new BufferedWriter(new FileWriter(sourcefile));
                while ((c=reader.read())!=-1)
                    writer.write(c);
                writer.close();
            }
        }
        catch (Exception e)
        {
            MessageHandler.warn("Error generating LISP code, "+e.getMessage());
        }
        MessageHandler.say("Finished generating LISP code");
    }
    /** Write the lisp code to call a action */
    private static void writeActionDefinition(AbstractManagedAction action,Writer writer)
    {
        String functionname=createFunctionName(action);
        try
        {
            writer.write('\n');
            writer.write("(defun IDE-"+functionname+" ()\n");
            writer.write("    (execute-action \""+Mode.getNameAction(action)+"\"))");
            writer.write("\n");
        }
        catch (Exception e)
        {
            MessageHandler.warn("Error generating LISP code for action, "+e.getMessage());
        }
    }
    /** create the function name */
    public static String createFunctionName(AbstractManagedAction action)
    {
        return ((String) action.getValue(Mode.ACTIONNAME)).replace(' ','-').toLowerCase();        
    }
}
