/*
 * LispIDEMethods.java
 *
 * Created on May 11, 2002, 1:40 PM
 */

package IDE.LispScripting;

import java.io.*;
import java.util.regex.*;
import javax.swing.*;
import IDE.*;
import IDE.CodeEditor.*;
import IDE.WorkArea.*;
import IDE.Configuration.*;
import IDE.Messaging.*;
import IDE.Project.Repositories;
import IDE.LispConnection.*;
import IDE.Interaction.*;

/**
 *
 * @author  marc
 */
public class LispIDEMethods
{
    /** This class contains a lot of static java methods solely to be called from the LISP process */
    
    /** Creates a new instance of LispIDEMethods, no instance of this class can be made */
    private LispIDEMethods()
    {
    }
    /** Get the current editor in the IDE */
    public static  CodeEditor getCurrentEditor()
    {
        return Main.currentEditor();
    }
    /** Get the editor buffer editing the content of the given file */
    public static  CodeEditor getEditorByName(String filename)
    {
        return WorkArea.getEditorForLongName(filename);
    }
    /** Set the cursor location for the given editor */
    public static void setCursor(CodeEditor editor,Long position)
    {
        editor.cursor.setCursor(position.intValue());
    }
    /** Get the cursor location for the given editor */
    public static int getCursorOffset(CodeEditor editor)
    {
        return editor.cursor.cursorOffset();
    }
    /** Execute a action */
    public static void executeAction(String actionname)
    {
        AbstractManagedAction action=Mode.getAction(actionname);
        if (action!=null)
            action.actionPerformed(null);
    }
    /** Excute a String Question */
    public static void askStringQuestion(String guidetext,final LispObject function)
    {
        IQuestion question=new StringQuestion(guidetext)
        {
            public void processEnter(String answer)
            {
                try
                {
                    function.getLispConnection().callLisp(function,new Object[]
                    {answer},false);
                }
                catch (Exception e)
                {
                    MessageHandler.warn("Error calling LISP from within string question");
                };
            }
        };
        InstructionField.ask(question);
    }
    /** Excute a number Question */
    public static void askNumberQuestion(String guidetext,final LispObject function)
    {
        IQuestion question=new NumberQuestion(guidetext)
        {
            public void processEnter(int answer)
            {
                try
                {
                    function.getLispConnection().callLisp(function,new Object[]
                    {new Integer(answer)},false);
                }
                catch (Exception e)
                {
                    MessageHandler.warn("Error calling LISP from within number question");
                };
            }
        };
        InstructionField.ask(question);
    }
    /** Get a the text starting at position position with length length for a document in editor buffer editor */
    public static String getText(CodeEditor editor,Long position,Long length)
    {
        return editor.getDocument().getString(position.intValue(),length.intValue());
    }
    /** Add a lisp function (the lisp function may not have any argument)  as a action with the given name, shortdescripition,longdescription */
    public static AbstractIDEAction createAction(String name,String shortdescription,String longdescription,LispObject function)
    {
        return createAction(name,shortdescription,longdescription,null,function);
    }
    public static AbstractIDEAction createAction(String name,String shortdescription,String longdescription,ImageIcon icon,final LispObject function)
    {
        AbstractIDEAction action;
        if (icon==null)
            action=new AbstractIDEAction(name)
            {
                public void doWork()
                {
                    try
                    {
                        function.getLispConnection().callLisp(function,new Object[0],false);
                    }
                    catch (Exception e)
                    {
                        MessageHandler.warn("Error calling LISP action function");
                    };
                }
            };
        else
            action=new AbstractIDEAction(name,icon)
            {
                public void doWork()
                {
                    try
                    {
                        function.getLispConnection().callLisp(function,new Object[0],false);
                    }
                    catch (Exception e)
                    {
                        MessageHandler.warn("Error calling LISP action function");
                    };
                }
            };
          Mode.addAction(name,shortdescription,longdescription,action);
          return action;
    }
    
    /** Create a copy of a mode giving it the name name, returning the mode */
    public static void copyMode(String mode,String name)
    {
        Mode.getMode(mode).createCopy(name);
    }
    /** Bind a action to a menu in a certain mode */
    public static void addActionToMenu(String mode,String action,String menu)
    {
        Mode.getMode(mode).addActionToMenu(menu,action);
    }
    /** Add a separator to a menu */
    public static void addSeparatorToMenu(String mode,String menu)
    {
        Mode.getMode(mode).addSeparatorToMenu(menu);
    }
    /** Add a checkbox to a menu */
    public static void addCheckBoxToMenu(String mode,String action,String menu,Boolean enabled)
    {
        Mode.getMode(mode).addCheckboxToMenu(menu,action,enabled.booleanValue());
    }
    /** Add a separator to a popup menu */
    public static void addSeparatorToPopupMenu(String mode,String menu)
    {
        Mode.getMode(mode).addSeparatorToPopup(menu);
    }
    /** Bind a action to a popup menu in a certain mode */
    public static void addActionToPopupMenu(String mode,String action,String menu)
    {
        if (menu.length()>0)
            Mode.getMode(mode).addActionToPopup(menu,action);
        else
            Mode.getMode(mode).addActionToPopup(action);
    }
    /** Create a key sequence */
    public static KeySequence createKeySequence(Object[] keysequence)
    {
        try
        {
            int m=keysequence.length;
            int[][] intkeystrokes=new int[m][2];
            for (int i=0;i<m;i++)
            {
                Object[] keystroke=(Object[]) keysequence[i];
                intkeystrokes[i][0]=((Long) keystroke[0]).intValue();
                intkeystrokes[i][1]=((Long) keystroke[1]).intValue();
            };
            return new KeySequence(intkeystrokes);
        }
        catch (Exception e)
        {
            MessageHandler.warn("Error creating keysequence "+e.getMessage());
            return null;
        }
    }
    /** Bind a action to a keystroke , for a certain mode */
    public static void bindActionToKeySequence(String mode,String action,KeySequence keysequence)
    {
        Mode.getMode(mode).bindActionToKeySequence(action,keysequence);
    }
    /** Add a button to the toolbar in the given mode */
    public static void addButton(String toolbar,String mode,String action)
    {
        Mode.getMode(mode).addButton(toolbar,action);
    }
    /** set the current editor */
    public static void setCurrentEditor(CodeEditor editor)
    {
        Main.setCurrent(editor);
    }
    /** create a editor */
    public static CodeEditor createEditor()
    {
        CodePane pane=new CodePane(Repositories.createParser(null,true));
        WorkArea.addPane(pane);
        return pane.getEditor();
    }
    public static CodeEditor createEditor(String filename)
    {
        File file=new File(filename);
        CodePane pane=CodePane.createCodePane(Repositories.createParser(file,true),file);
        WorkArea.addPane(pane);
        return pane.getEditor();
    }
    /** Return a icon which can be used in buttons, etc */
    public static ImageIcon createIcon(String filename)
    {
        return new ImageIcon(filename);
    }
    /** Insert text at a certain position of a editor */
    public static void insertText(CodeEditor editor,Long position,String text)
    {
        editor.insertStringAt(text,position.intValue());
    }
    /** Remove some text with length l, from position position for the given editor */
    public static void removeText(CodeEditor editor,Long position,Long length)
    {
        editor.remove(position.intValue(),length.intValue());
    }
    /** Regular expression search for a string in a certain editor, returns a match object */
    public static Matcher regularExpressionSearch(CodeEditor editor,String regexp,Boolean casesensitive)
    {
        CodeDocument document=editor.getDocument();
        String content=document.getString(0,document.length());
        Pattern pattern=casesensitive.booleanValue() ? Pattern.compile(regexp) : Pattern.compile(regexp,Pattern.CASE_INSENSITIVE);
        return pattern.matcher(content);
    }
    /** Calls the find method on a matcher returning the start position or -1 if nothing is found */
    public static int matcherFind(Matcher matcher)
    {
        if (matcher.find())
            return matcher.start();
        else
            return -1;
    }
    public static int matcherFind(Matcher matcher,Long position)
    {
        if (matcher.find(position.intValue()))
            return matcher.start();
        else
            return -1;
    }
    /** Calls the end method on a matcher */
    public static int matcherEnd(Matcher matcher)
    {
        return matcher.end();
    }
    /** Send a command to the standard input of the current Lisp process */
    public static void sendCommand(String command)
    {
        Interactions.getSelectedInteraction().getInteraction().sendCommand(command);
    }
}
