/*
 * Created on May 20, 2003
 */
 
package IDE.LispSyntax;

import IDE.CodeEditor.*;
import IDE.CodeEditor.CodeElement;
import IDE.CodeEditor.CodeElementIterator;
import IDE.CodeEditor.Parser.*;
/**
 * @author marc
 * Class deriving from a LispListElement, used to represent a list containing the remaining elements in a
 * iterator over the elements of a list. Instances of this class are used for the implementation of analyzers
 * over dotted lists. 
 */
public class ListOfLispListElement extends LispListElement
 {
 	/*
 	 * Constructor to build a list of the remaining elements in the iterator (is private and called by 
 	 * the static method getIteratorOnListOfIterator
 	 */
	private ListOfLispListElement(CodeElementIterator iterator)
	{
		CodeElement element;
		while ((element=iterator.next())!=null)
		{
			addElement(element);
		}
	}
	/*
	 * Overide the getElements method of LispListElement to avoid having to figure out again what the real elements are 
	 */
	 public CodeElementVector getElements()
	 {
	 	return elements;
	 }
	 /*
	  * Static method to build a iterator over a CodeElementVector containing as sole element a ListOfLispListElement build 
	  * of the remaining elements of the given iterator
	  * @author marc
	  *
	  */
	  public static CodeElementIterator getIteratorOnListOfIterator(CodeElementIterator iterator)
	  {
	  		CodeElementVector elements=new CodeElementVector();
	  		elements.addCodeElement(new ListOfLispListElement(iterator));
	  		return elements.makeIterator();
	  }
}
