/*
 * IDE.java
 *
 * Created on October 21, 2001, 4:32 PM
 */

package IDE;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import IDE.Messaging.*;
import IDE.Watch.*;
import IDE.Configuration.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import IDE.Interaction.*;
import IDE.Help.*;
import IDE.Search.*;
import IDE.WorkArea.*;
import IDE.Recording.*;
import IDE.Utilities.*;
import IDE.Project.*;
import java.util.*;

/**
 * Main IDE program, every thing in this class must be static with the exception oi the keylistener methods
 * @author  marc
 * @version
 */
public class Main extends WindowAdapter implements IIDEAddin,FocusListener
{
    /**
     * Static list of all the components which has to tell Configuration, their settings on exit for a save of their settings
     * (We can't add this field to Configuration because it considers all its fields as candidate fro a full save of all options
     */
    private static Vector addins=new Vector();
    /**  Frame containing the IDE interface */
    public static JFrame frame;
    /** Splitpane containing the central pane of the IDE*/
    public static JSplitPane central;
    /** Splitpane containing the workarea and the explorers area of the IDE */
    public static JSplitPane deskinteractions;
    /** Label to display the line and position information in */
    private static JLabel cursorinfo;
    /** Label to contain the buttons and other tools */
    private static JPanel toolbar;
    /** Workarea to display editorpanes in */
    private static WorkArea workarea;
    /** Interaction tabbed pane */
    private static Interactions interactions;
    /** List of explores */
    private static JTabbedPane explorers;
    /** Watchwindow  */
    private static Watches watch;
    /** Macro window */
    private static MacroExplorer macroexplorer;
    /** CodePane which has focus */
    private static CodeEditor currentEditor;
    /** Current mode in use */
    private static Mode currentmode;
    /** Configuration settings of the IDE */
    private static Configuration configuration;
    /** Keymap used by IDE */
    public static ActionKeyMap keymap;
    /** Only instance of Main class to do event handling */
    private static Main main;
    /** Constructor for the Main class */
    public Main()
    {
    }
    /**
     * Starting point of the IDE
     * The following arguments are accepted by the IDE
     * -i installdir
     * -s sourcedir
     * These arguments will be stored as static entires in the
     */
    public static void main(String[] args)
    {
        // Check if this is the first instance
        if (!checkFirstInstance())
            System.exit(0);
        boolean argsnok=true;
        // Process the arguments
        if (args.length % 2!=0 || args.length==0)
        {
            JOptionPane.showMessageDialog(null,"Possible arguments are -i installdir -s sourcedir, -i argument is required","Argument error",JOptionPane.ERROR_MESSAGE);
            System.exit(-1);
        };
        for (int i=0;i+1<args.length;i=i+2)
            if (args[i].compareTo("-i")==0)
            {
                InstallParameters.setInstallDir(args[i+1]);
                argsnok=false;
            }
            else if (args[i].compareTo("-s")==0)
                InstallParameters.setSourceDir(args[i+1]);
            else
                argsnok=true;
        if (argsnok)
        {
            JOptionPane.showMessageDialog(null,"Possible arguments are -i installdir -s sourcedir, -i argument is required","Argument error",JOptionPane.ERROR_MESSAGE);
            System.exit(-1);
        };
        // Show the splashscreen
        SplashScreen.showSplash();
        // Load the global configuration 
        Configuration.loadGlobalConfiguration();
        // Define the actions of the interface
        SplashScreen.showMessage("Configure the IDE");
        // Create the questions used by the IDE
        createQuestions();
        // Create the actions used by the IDE
        createActions();
        // Create the a instance of the Main class to do the event handling
        main=new Main();
        // Create the IDE interface
        createIDEWindow();
        // Create the hint
        Hint.initHint();
        // Create the configuration settings
        configuration=new Configuration();
        // Set the root mode
        Mode.getMode("basic").activate();
        // Register the default addins
        Main.registerAddin(workarea);
        Main.registerAddin(interactions);
        Main.registerAddin(main); // Ensure that state main is saved after workarea and interactions
        Main.registerAddin(Recording.getRecording());
        // Replace the default focusmanager
        FocusManager.setCurrentManager(new IDEFocusManager());
        // Dispose the splashscreen
        SplashScreen.disposeSplash();
        // Set the look and feel
        if (Configuration.lookAndFeel.length()>0)
        {
            try
            {
                UIManager.setLookAndFeel(Configuration.lookAndFeel);
                SwingUtilities.updateComponentTreeUI(frame);
                frame.pack();
            }
            catch (ClassNotFoundException e)
            {
                MessageHandler.warn("Look and feel class "+Configuration.lookAndFeel+" not found.");
            }
            catch (Exception e)
            {
                MessageHandler.report(e.getMessage(),e);
            };
        };
        Configuration.loadState();
        // Display IDE
        frame.setVisible(true);
        // Ensure that the current editor has focus
        if (currentEditor!=null)
            currentEditor.requestFocus();
        // Test focus management KeyboardFocusManager.getCurrentKeyboardFocusManager().addVetoableChangeListener(new IDEVetoableChangeListener());
    }
    /** Create the IDE  interface */
    private static void createIDEWindow()
    {
        // Create thew IDE window
        frame=new JFrame(Configuration.IDEName+" "+Configuration.IDEVersion);
        frame.addWindowListener(main);
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addFocusListener(main);
        // Define the different panes that must be displayed
        toolbar=new JPanel();
        toolbar.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
        workarea=WorkArea.createWorkArea();
        watch=new Watches();
        macroexplorer=new MacroExplorer();
        MessageHandler messagehandler=new MessageHandler();
        // Create a pane containing the messages and the instruction field
        JPanel bottempanel=new JPanel();
        GridBagLayout bottempanellayout=new GridBagLayout();
        bottempanel.setLayout(bottempanellayout);
        GridBagConstraints gridbagconstraints=new GridBagConstraints();
        gridbagconstraints.gridx=1;
        gridbagconstraints.gridy=0;
        gridbagconstraints.weightx=0.01;
        cursorinfo=new JLabel("     0-0     ");
        cursorinfo.setForeground(Color.black);
        cursorinfo.setBorder(BorderFactory.createEtchedBorder());
        bottempanellayout.setConstraints(cursorinfo,gridbagconstraints);
        gridbagconstraints.gridx=0;
        gridbagconstraints.gridy=0;
        gridbagconstraints.weightx=0.9;
        gridbagconstraints.fill=GridBagConstraints.BOTH;
        bottempanellayout.setConstraints(messagehandler,gridbagconstraints);
        bottempanel.add(messagehandler);
        bottempanel.add(cursorinfo);
        // Create a tabbed pane containing the interactions
        interactions=new Interactions(JTabbedPane.BOTTOM);
        // Create the pane containing the different explorers
        explorers=new JTabbedPane(JTabbedPane.TOP);
		explorers.addTab("Projects",ProjectExplorer.projectexplorer);
        explorers.addTab("Watches",watch);
        explorers.addTab("Macros",macroexplorer);
        // Create the central pane
        deskinteractions=new JSplitPane(JSplitPane.VERTICAL_SPLIT,workarea,interactions);
        deskinteractions.setDividerSize(3);
        deskinteractions.setDividerLocation(0.8);
        deskinteractions.setResizeWeight(0.8);
        central=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,explorers,deskinteractions);
        central.setDividerSize(3);
        central.setDividerLocation(0.2);
        central.setResizeWeight(0.1);
        // Set the default locations of the sliders
        central.setDividerLocation(-1);
        deskinteractions.setDividerLocation(-1);
        // Creates the content pane of the frame and its components
        JPanel content=new JPanel();
        content.setLayout(new BorderLayout());
        content.add(central,BorderLayout.CENTER);
        content.add(toolbar,BorderLayout.NORTH);
        content.add(bottempanel,BorderLayout.SOUTH);
        frame.setSize(1000,1000);
        frame.getContentPane().add(content);
        //frame.setVisible(true);
    }
    /** Get the watch window */
    public static Watches getWatches()
    {
        return watch;
    }
    /** Get the macro window */
    public static MacroExplorer getMacroExplorer()
    {
        return macroexplorer;
    }
    /** Define the questions */
    private static void createQuestions()
    {
        InstructionField.addQuestion("asklinenumber",new NumberQuestion("Line number ?")
        {
            public void processEnter(int linenr)
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    LineVector lines=editor.getDocument().getLines();
                    if (linenr>0 && linenr+1<lines.size())
                    {
                        editor.cursor.setCursor(lines.lineAt(linenr-1).begin());
                        editor.requestFocus();
                    };
                };
            }
        });
        InstructionField.addQuestion("nextsearch",new IncrementalSearch(true));
        InstructionField.addQuestion("prevsearch",new IncrementalSearch(false));
        InstructionField.addQuestion("searchandreplace",new SearchAndReplace());
    }
    /** Define the actions */
    private static void createActions()
    {
        Mode.addAction("up","Move Up","Move the cursor in the current edit buffer  one line up",new AbstractIDEAction("Move Up")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.cursor.moveUp();
            }
        });
        Mode.addAction("down","Move Down","Move the cursor in the current edit buffer  one line down",new AbstractIDEAction("Move Down")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.cursor.moveDown();
            }
        });
        Mode.addAction("pageup","Move one page up","Move one page up (in the current editor buffer)",new AbstractIDEAction("Move Page Up")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.cursor.pageUp();
            }
        });
        Mode.addAction("pagedown","Move one page down","Move one page down (in the current editor buffer)",new AbstractIDEAction("Move Page Down")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.cursor.pageDown();
            }
        });
        Mode.addAction("left","Move Left","Move the cursor in the current edit buffer to the left",new AbstractIDEAction("Move Left")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.cursor.moveLeft();
            }
        });
        Mode.addAction("right","Move Right","Move the cursor in the current edit buffer to the right",new AbstractIDEAction("Move Right")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.cursor.moveRight();
            }
        });
        Mode.addAction("movetonextleaf","Move the cursor to next word","Move the cursor to the begin of the next element at the same level in its parent list.",
        new AbstractIDEAction("Move to next word")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.moveToNextLeaf();
            };
        });
        Mode.addAction("movetoendlist","Move the cursor to end of list","Move the cursor to the end of the list containing the cursor.",
        new AbstractIDEAction("Move to end list")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.moveToEndList();
            };
        });
        Mode.addAction("movetoprevleaf","Move the cursor to prev word","Move the cursor to the end of the previous element at the same level in its parent list.",
        new AbstractIDEAction("Move to prev word")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.moveToPrevLeaf();
            };
        });
        Mode.addAction("movetobeginlist","Move the cursor to begin list","Move the cursor to begin list",
        new AbstractIDEAction("Move to begin list")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.moveToBeginList();
            };
        });
        Mode.addAction("movetobeginline","Move the cursor to begin line","Move the cursor to the begin lof the current line.",new AbstractIDEAction("Move to begin line")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.moveToBeginLine();
            };
        });
        Mode.addAction("movetoendline","Move the cursor to end of line","Move the cursor to end of the current line",new AbstractIDEAction("Move the cursor to end of line")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.moveToEndLine();
            };
        });
        Mode.addAction("selectcurrentleaf","Select the current leaf","Select the word covering the current cursor position.",
        new AbstractIDEAction("Select the current leaf")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.selectCurrentLeaf();
            };
        });
        Mode.addAction("selectcurrentelement","Select the current element","Select the element covering the current cursor position.",
        new AbstractIDEAction("Select the current element")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.selectCurrentElement();
            };
        });
        Mode.addAction("selecttoplevelelement","Select the toplevel element","Select the toplevel element covering the cursor position.",new AbstractIDEAction("Select the toplevel element")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.selectToplevelElement();
            };
        });
        Mode.addAction("selectline","Select the current line","Select the current line.",new AbstractIDEAction("Select the current line")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.selectLine();
            };
        });
        Mode.addAction("widenselection","Widen the selection","Widen the selection so that it selects the parent element of the current selected element.",
        new AbstractIDEAction("Widen the selection")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.widenSelection();
            };
        });
        Mode.addAction("shortenselection","Shorten the selection","Widen the selection so that it selects the parent element of the current selected element.",new AbstractIDEAction("Shorten the selection")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.shortenSelection();
            };
        });
        Mode.addAction("deletecurrentleaf","Delete the current leaf","Delete the current leaf",new AbstractIDEAction("Delete the current leaf")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.deleteCurrentLeaf();
            };
        });
        Mode.addAction("deletecurrentelement","Delete the current element","Deletes the word covering the cursor position.",
        new AbstractIDEAction("Delete the current element")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.deleteCurrentElement();
            };
        });
        Mode.addAction("deletetoplevelelement","Delete the toplevel element","Delete the toplevel element covering the cursor position.",
        new AbstractIDEAction("Delete the toplevel element")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.deleteToplevelElement();
            };
        });
        Mode.addAction("deleteline","Delete the current line","Delete the line covering the cursor position",new AbstractIDEAction("Delete the current line")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.deleteLine();
            };
        });
        Mode.addAction("gotoline","Go to line","Ask for a line number and jumps to this line in the current editor.",
        new AbstractIDEAction("Go to line")
        {
            public void doWork()
            {
                releaseFocus=false;
                final CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    InstructionField.ask("asklinenumber");
                };
            }
        });
        Mode.addAction("remove","Delete Character","Remove the selected text in the current buffer or if no text is selected delete the character after the cursor.",
        new AbstractIDEAction("Delete Character")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                    editor.removeCharacter(editor.cursor.cursorOffset());
            }
        });
        Mode.addAction("searchforward","Search Forward","Search forwards in the current buffer (Incremental search).",
        new AbstractIDEAction("Search Forward", Configuration.getIcon("searchforward.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    releaseFocus=false;
                    InstructionField.ask("nextsearch");
                };
            }
        });
        Mode.addAction("searchbackward","Search Backward","Search backwards in the current buffer (Incremental search).",
        new AbstractIDEAction("Search Backward", Configuration.getIcon("searchbackward.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    releaseFocus=false;
                    InstructionField.ask("prevsearch");
                };
            }
        });
        Mode.addAction("searchandreplace","Search And Replace","Start a search and replace session.",new AbstractIDEAction("Search And Replace")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    releaseFocus=false;
                    InstructionField.ask("searchandreplace");
                };
            }
        });
        Mode.addAction("findandreplace","Find And Replace","Starts a dialog box for search and replace in the current buffer (or all the files in a choosen project). You can do a regular expression search or a normal case sensitive (insensitive) string search.",
        new AbstractIDEAction("Find And Replace")
        {
            public void doWork()
            {
                RegularExpressionSearch.SearchAndReplace();
            }
        });
        Mode.addAction("backspace","Delete Previous Character","Delete the previous character",new AbstractIDEAction("Backspace")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.removeCharacter(editor.cursor.cursorOffset()-1);
                    editor.cursor.moveLeft();
                };
            }
        });
        Mode.addAction("undo","Undo last edit","Undo last modification in the current buffer.",new AbstractIDEAction("Undo last edit",Configuration.getIcon("undo.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    try
                    {
                        editor.getDocument().getUndoManager().undo();
                    }
                    catch (Exception e)
                    {
                        MessageHandler.warn("No more undo possible ");
                    };
                    editor.revalidate();
                    editor.repaint();
                };
            }
        });
        Mode.addAction("redo","Redo last edit","Redo last edit",new AbstractIDEAction("Redo last edit",Configuration.getIcon("redo.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    try
                    {
                        
                        editor.getDocument().getUndoManager().redo();
                    }
                    catch (Exception e)
                    {
                        MessageHandler.warn("No more redo possible");
                    };
                    editor.revalidate();
                    editor.repaint();
                };
            }
        });
        Mode.addAction("copy","copy","Copies the text selected in the current buffer to the copy buffer.",
        new AbstractIDEAction("copy",Configuration.getIcon("copy.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.copySelectedText();
                };
            }
        });
        Mode.addAction("cut","cut","Copies the text selected in the current buffer to the copy buffer and removes the selected text.",new AbstractIDEAction("cut",Configuration.getIcon("cut.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.cutSelectedText();
                };
            }
        });
        Mode.addAction("delete","delete","Deletes the selected text in the current buffer.",new AbstractIDEAction("delete")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.deleteSelectedText();
                };
            }
        });
        Mode.addAction("paste","paste","Paste the text in the clipboard after the cursor in the current buffer.",new AbstractIDEAction("paste",Configuration.getIcon("paste.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.pasteSelection();
                };
            }
        });
        Mode.addAction("startrecording","Start Recording",
        "Start recording of the activation of all actions (with the exception of course of the replayrecording,startrecording and stoprecording action). If a action activates a question via the instruction field, record also the activity in the instruction field.",
        new StartRecordingAction("Start Recording", Configuration.getIcon("startrecording.png")));
        Mode.addAction("stoprecording","Stop Recording","Stop de macro recording and saves the recorded information in the current macro. Ask then for a name for the current macro and create a macro with that name.",
        new StopRecordingAction("Stop Recording" , Configuration.getIcon("stoprecording.png")));
        Mode.addAction("replayrecording","Replay Recording","Asks for a macro name (if you press enter without giving a name the current macro will be used) and replays the macro.",
        new ReplayRecordingAction("Replay Recording" , Configuration.getIcon("replayrecording.png")));
        Mode.addAction("indentatedocument","Indentate the document","Indentate the document",new AbstractIDEAction("Indentate the document")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.getDocument().indentateAllElements();
                };
            }
        });
        Mode.addAction("new","New File","Opens a new empty editor buffer.",new AbstractIDEAction("New",Configuration.getIcon("new.png"))
        {
            public void doWork()
            {
                CodePane pane=new CodePane(LispRepository.lisprepository.createParser(true));
                WorkArea.addPane(pane);
                setCurrent(pane.getEditor());
            }
        });
        Mode.addAction("open","Open File","Starts a file dialog box to open a file and create a edit buffer for this file.",
        new AbstractIDEAction("Open",Configuration.getIcon("open.png"))
        {
            public void doWork()
            {
                File file;
                JFileChooser chooser=new JFileChooser(Interactions.currentDirectory);
                Parser parser;
                CodePane pane;
                chooser.setFileFilter(Repositories.getChooserFilter());
                chooser.setAcceptAllFileFilterUsed(false);
                if (chooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION)
                {
                    file=chooser.getSelectedFile();
                    file=new File(Util.getCanonicalFileName(file)); // Solves problem with default shell folder
                    parser=Repositories.createParser(file,true);
                    if (parser==null)
                        return;
                    pane=CodePane.createCodePane(parser,file);
                    WorkArea.addPane(pane);
                    setCurrent(pane.getEditor());
                };
            }
        });
        Mode.addAction("save","Save File","Save the contents of the current buffer.",new AbstractIDEAction("Save",Configuration.getIcon("save.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.getCodePane().save();
                }
            }
        });
        Mode.addAction("saveas","Save File As","Starts a dialog box to get a file name and save the current buffer in it.",
        new AbstractIDEAction("Save as",Configuration.getIcon("saveas.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.getCodePane().saveAs();
                }
            }
        });
        Mode.addAction("close","Close File","If the buffer is modified since the last save, ask to save the buffer and then destroys the source buffer.",
        new AbstractIDEAction("Close", Configuration.getIcon("close.png"))
        {
            public void doWork()
            {
                WorkArea.close();
            }
        });
        Mode.addAction("setcurrentdirectory", "Set the current directory","Set the current diretory for all interaction buffers and set the default directory for all file dialogs",
        new AbstractIDEAction("Set Current Directory")
        {
            public void doWork()
            {
                JFileChooser chooser=new JFileChooser(Interactions.currentDirectory);
                chooser.setDialogTitle("Choose the current directory");
                chooser.setDialogType(JFileChooser.OPEN_DIALOG);
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                if (chooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION)
                    Interactions.setCurrentDirectory(chooser.getSelectedFile());
            }
        });
        Mode.addAction("splitvertical","Split Vertical","Split the current buffer in a bottom and upper part buffer, each buffer using the same document.",
        new AbstractIDEAction("Split Vertical", Configuration.getIcon("splitvertical.png"))
        {
            public void doWork()
            {
                workarea.split(JSplitPane.VERTICAL_SPLIT);
            }
        });
        Mode.addAction("splithorizontal","Split Horizontal","Split the current buffer in a left and right buffer, each new buffer is using the same document.",
        new AbstractIDEAction("Split Horizontal", Configuration.getIcon("splithorizontal.png"))
        {
            public void doWork()
            {
                workarea.split(JSplitPane.HORIZONTAL_SPLIT);
            }
        });
        Mode.addAction("replaceeditor","Replace Document","Replace the document associated with a buffer.",
        new AbstractIDEAction("Replace Document")
        {
            public void doWork()
            {
                workarea.replaceEditor();
            }
        });
        Mode.addAction("addproject","Add Project","Adds a project to the projects window.",new AbstractIDEAction("Add Project")
        {
            public void doWork()
            {
                Projects.addProject();
            }
        });
        Mode.addAction("exit","Exit","Closes all the open buffers, stops all lisp processes and exit Jabberwocky.",
        new AbstractIDEAction("Exit")
        {
            public void doWork()
            {
                exitJabberwocky();
            }
        });
        // Only add this action if this is supported
        if (InteractionCommands.getInteractionCommand(LispCmuInteraction.class.getName())!=null)
            Mode.addAction("newinteractioncmucl","New Interaction with CMUCL","Starts a new process to run cmucl and creates a new interaction buffer to interact with this process.",
            new AbstractIDEAction("New Interaction with CMUCL", Configuration.getIcon("lambda.png"))
            {
                public void doWork()
                {
                    IInteraction interaction=new LispCmuInteraction();
                    if (interaction!=null)
                    {
                        AbstractInteractionPane pane=new InteractionPane(LispRepository.lisprepository.createParser(true),interaction);
                        interactions.add(pane.getName(),pane);
                        interactions.setSelectedComponent(pane);
                    };
                }
            });
        else
        {
        };
        // Only add this action if this is supported
        if (InteractionCommands.getInteractionCommand(LispSbclInteraction.class.getName())!=null)
            Mode.addAction("newinteractionsbcl","New Interaction with SBCL","Starts a new process to run sbcl and creates a new interaction buffer to interact with this process.",
            new AbstractIDEAction("New Interaction with SBCL", Configuration.getIcon("lambda.png"))
            {
                public void doWork()
                {
                    IInteraction interaction=new LispSbclInteraction();
                    if (interaction!=null)
                    {
                        AbstractInteractionPane pane=new InteractionPane(LispRepository.lisprepository.createParser(true),interaction);
                        interactions.add(pane.getName(),pane);
                        interactions.setSelectedComponent(pane);
                    };
                }
            });
        else
        {
        };
        // Only add this action if it is supported
        if (InteractionCommands.getInteractionCommand(LispClispInteraction.class.getName())!=null)
            Mode.addAction("newinteractionclisp","New Interaction with CLISP","Starts a new process to run clisp and creates a new interaction buffer to interact with this process.",
            new AbstractIDEAction("New Interaction with CLISP", Configuration.getIcon("lambda.png"))
            {
                public void doWork()
                {
                    IInteraction interaction=new LispClispInteraction();
                    if (interaction!=null)
                    {
                        AbstractInteractionPane pane=new InteractionPane(LispRepository.lisprepository.createParser(true),interaction);
                        interactions.add(pane.getName(),pane);
                        interactions.setSelectedComponent(pane);
                    };
                }
            });
        else
        {
        };
        // Only add this action if it is supported
        if (InteractionCommands.getInteractionCommand(LispAclInteraction.class.getName())!=null)
            Mode.addAction("newinteractionacl","New Interaction with ACL","Starts a new process to run ACL and creates a new interaction buffer to interact with this process.",
            new AbstractIDEAction("New Interaction with ACL", Configuration.getIcon("lambda.png"))
            {
                public void doWork()
                {
                    IInteraction interaction=new LispAclInteraction();
                    if (interaction!=null)
                    {
                        AbstractInteractionPane pane=new InteractionPane(LispRepository.lisprepository.createParser(true),interaction);
                        interactions.add(pane.getName(),pane);
                        interactions.setSelectedComponent(pane);
                    };
                }
            });
        else
        {
        };
		// Only add this action if it is supported
		if (InteractionCommands.getInteractionCommand(LispAclWindowsInteraction.class.getName())!=null)
			Mode.addAction("newinteractionaclwindows","New Interaction with ACL (Windows)","Starts a new process to run ACL and creates a new interaction buffer to interact with this process.",
			new AbstractIDEAction("New Interaction with ACL (Windows)", Configuration.getIcon("lambda.png"))
			{
				public void doWork()
				{
					IInteraction interaction=new LispAclWindowsInteraction();
					if (interaction!=null)
					{
						AbstractInteractionPane pane=new InteractionPane(LispRepository.lisprepository.createParser(true),interaction);
						interactions.add(pane.getName(),pane);
						interactions.setSelectedComponent(pane);
					};
				}
			});
		else
		{
		};
        Mode.addAction("newwatch","New Watch Window","Creates a watch buffer associated with the current interaction buffer. This watch buffer can be used to type in lisp forms and see there results while debugging a program.",
        new AbstractIDEAction("New Watch Window")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane !=null)
                {
                    AbstractInteractionPane watchpane=((InteractionPane) interactionpane).createWatchPane();
                    interactions.add("Watch "+interactionpane.getName(),watchpane);
                    interactions.setSelectedComponent(watchpane);
                }
            }
        });
        Mode.addAction("loadeditor","Load Editor Content","Load the content of the current editor buffer in the process in the current interaction buffer.",
        new AbstractIDEAction("Load Editor Content")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                CodeEditor editor=currentEditor();
                if (interactionpane !=null && editor!=null && !editor.isInteractionEditor())
                {
                    interactionpane.getInteraction().loadEditor(editor);
                }
            }
        });
        Mode.addAction("compileeditor","Compile Editor Content","Compile the content of the current editor buffer, using the process in the current interaction buffer.",
        new AbstractIDEAction("Compile Editor Content", Configuration.getIcon("compile.png"))
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                CodeEditor editor=currentEditor();
                if (interactionpane !=null && editor!=null && !editor.isInteractionEditor())
                {
                    interactionpane.getInteraction().compileEditor(editor);
                }
            }
        });
        Mode.addAction("deleteinteraction","Delete Interaction","Put all buffers in debugmode which uses this interaction in edit mode. Remove all watchpoints using this interaction. Stops the lispprocess and close the buffer associated with this lisp process.",
        new AbstractIDEAction("Delete Interaction", Configuration.getIcon("deleteinteraction.png"))
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                {
                    interactionpane.cleanup();
                    interactions.remove(interactionpane);
                };
            }
        });
        Mode.addAction("deletewatch","Delete Watch","Remove the watch pane",
        new AbstractIDEAction("Delete Interaction")
        {
            public void doWork()
            {
                WatchPane watchpane=Interactions.getSelectedWatch();
                if (watchpane!=null)
                {
                    watchpane.cleanup();
                    interactions.remove(watchpane);
                };
            }
        });
        Mode.addAction("sigint","Send Interrupt","Send Interrupt signal to process in the current interaction buffer",new AbstractIDEAction("Send Interrupt")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                {
                    interactionpane.signalProcess(AbstractInteractionPane.SIGINT);
                    setCurrent(interactionpane.getEditor());
                };
            }
        });
        Mode.addAction("sigstop","Send Stop","Send Stop signal to process in the current interaction buffer",new AbstractIDEAction("Send Stop")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                {
                    interactionpane.signalProcess(AbstractInteractionPane.SIGSTOP);
                    setCurrent(interactionpane.getEditor());
                };
            }
        });
        Mode.addAction("sigcont","Send Continue","Send Continue signal to process in the current interaction buffer",new AbstractIDEAction("Send Continue")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                {
                    interactionpane.signalProcess(AbstractInteractionPane.SIGCONT);
                    setCurrent(interactionpane.getEditor());
                };
            }
        });
        Mode.addAction("sigquit","Send Quit","Send Quit signal to process in the current interaction buffer",new AbstractIDEAction("Send Quit")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                {
                    interactionpane.signalProcess(AbstractInteractionPane.SIGQUIT);
                    setCurrent(interactionpane.getEditor());
                };
            }
        });
        Mode.addAction("sigkill","Send Kill","Send Kill signal to process in the current interaction buffer",new AbstractIDEAction("Send Kill")
        {
            public void doWork()
            {
                InteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                {
                    interactionpane.signalProcess(AbstractInteractionPane.SIGINT);
                    setCurrent(interactionpane.getEditor());
                };
            }
        });
        Mode.addAction("togglehint","Show/Hide Hint","Show/Hide the hint window.",new AbstractIDEAction("Show/Hide Hint", Configuration.getIcon("hint.png"))
        {
            public void doWork()
            {
                Hint.toggleHint();
            }
        });
        Mode.addAction("help","Show Help","Show Help",new AbstractIDEAction("Show Help", Configuration.getIcon("help.png"))
        {
            public void doWork()
            {
                Help.displayHelp();
            }
        });
        Mode.addAction("replacebyhint","Replace by the Hint","Replace the element containing the cursor by the selected hint value.",
        new AbstractIDEAction("Replace by the hint")
        {
            public void doWork()
            {
                Hint.replaceByHint();
            }
        });
        Mode.addAction("hintdown","Go down in hint list","Move one line down in the hint list.",new AbstractIDEAction("Go down in hint list")
        {
            public void doWork()
            {
                Hint.goDown();
            }
        });
        Mode.addAction("hintup","Go up in hint list","Move one line up in the hint list.",new AbstractIDEAction("Go up in hint")
        {
            public void doWork()
            {
                Hint.goUp();
            }
        });
        Mode.addAction("hyperspeccontext","Hyperspec Context Help","Show the hyperspec help for the current context",new AbstractIDEAction("Hyperspec Context Help")
        {
            public void doWork()
            {
                HyperSpec.displayHyperSpecContext();
            }
        });
        Mode.addAction("hyperspec","Hyperspec Help","Show the hyperspec documentation",new AbstractIDEAction("Hyperspec Context Help")
        {
            public void doWork()
            {
                HyperSpec.displayHyperSpec();
            }
        });
        Mode.addAction("selectcommand","Select command of command history","Select command of command hsitory",new AbstractIDEAction("Command History")
        {
            public void doWork()
            {
                AbstractInteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                    interactionpane.chooseCommand();
            }
        });
        Mode.addAction("execute","Execute","Execute the selected command in the command history.",new AbstractIDEAction("Execute", Configuration.getIcon("execute.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                AbstractInteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (editor!=null && interactionpane!=null)
                {
                    String str=editor.highlighter.getHighLightedCode("30select");
                    if (str!=null)
                    {
                        interactionpane.executeCommand(str);
                    }
                    else
                    {
                        Region region=editor.getDocument().getParser().findHooks(editor.cursor.cursorOffset());
                        if (region!=null)
                            interactionpane.executeCommand(editor.getDocument().getString(region.begin,region.end-region.begin+1));
                    }
                };
            }
        });
        Mode.addAction("return","Return","Send a Return to the process",new AbstractIDEAction("Return")
        {
            public void doWork()
            {
                AbstractInteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (interactionpane!=null)
                    interactionpane.sendReturn();
            }
        });
        Mode.addAction("indentate","Indentate Line","Add spaces to the begin of the line so that the code on this line is correctly indentated.",
        new AbstractIDEAction("Indentate Line")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.getDocument().indentateLine();
                };
            }
        });
        Mode.addAction("indentateelement","Indentate Element","Indentate the toplevel element containing the cursor.",
        new AbstractIDEAction("Indentate Element")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null)
                {
                    editor.getDocument().indentateElement(editor.cursor.cursorOffset());
                };
            }
        });
        Mode.addAction("debug","Debug Document","Starts a debugging session for the current source buffer using the current interaction window.",
        new AbstractIDEAction("Debug Document",Configuration.getIcon("debug.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                AbstractInteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (editor!=null && interactionpane!=null)
                {
                    interactionpane.enableDebugging(editor);
                };
            }
        });
        Mode.addAction("debugform","Debug Selection/Form","Debug the selected code (or the current lispform) in a separate window.",
        new AbstractIDEAction("Debug Selection/Form",Configuration.getIcon("debug.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                AbstractInteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (editor!=null && interactionpane!=null)
                {
                    String str=editor.highlighter.getHighLightedCode("30select");
                    if (str!=null)
                    {
                        interactionpane.enableDebugging(str);
                    }
                    else
                    {
                        Region region=editor.getDocument().getParser().findHooks(editor.cursor.cursorOffset());
                        if (region!=null)
                            interactionpane.enableDebugging(editor.getDocument().getString(region.begin,region.end-region.begin+1));
                    }
                };
            }
        });
        Mode.addAction("undebug","Finish Debugging Document","Load the original code of a source, disabling the debugging of this source.",
        new AbstractIDEAction("Finish Debugging Document",Configuration.getIcon("undebug.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                AbstractInteractionPane interactionpane=Interactions.getSelectedInteraction();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null && interactionpane!=null)
                {
                    interactionpane.disableDebugging(editor);
                };
            }
        });
        Mode.addAction("togglebreakpoint","Toggle Breakpoint","Enable/ disable a breakpoint on the current form.",
        new AbstractIDEAction("Toggle Breakpoint",Configuration.getIcon("togglebreakpoint.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.toggleBreakpoint(editor);
                    };
                };
            }
        });
        Mode.addAction("togglecondbreakpoint","Toggle Conditional Breakpoint","Enable/Disable a conditional breakpoint on the current form.",
        new AbstractIDEAction("Toggle Conditional Breakpoint",Configuration.getIcon("toggleconditionalbreakpoint.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.toggleConditionalBreakpoint(editor);
                    };
                };
            }
        });
        Mode.addAction("stepinto","Step Into","Give control to the program, stopping before execution of the next form.",
        new AbstractIDEAction("Step Into",Configuration.getIcon("stepinto.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.stepInto(editor);
                    };
                };
            }
        });
        Mode.addAction("stepover","Step Over","Give control to the program, stopping at the next form having the same parent as the current form.",
        new AbstractIDEAction("Step Over",Configuration.getIcon("stepover.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.stepOver(editor);
                    };
                };
            }
        });
        Mode.addAction("stopat","Stop At","Stop At",new AbstractIDEAction("Give control to the program and stop at the next breakpoint or the highlighted form (what comes first).",
        Configuration.getIcon("stopat.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.stopAt(editor);
                    };
                };
            }
        });
        Mode.addAction("stopafter","Stop After","Execute the current lisp form and returns to the debugger before executing the next form.",
        new AbstractIDEAction("Stop After",Configuration.getIcon("stopafter.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.stopAfter(editor);
                    };
                };
            }
        });
        Mode.addAction("setnewreturnvalue","Set New Return Value","Change the value of the just executed lispform (only when current editor is in debug mode and when you have done a stepafter).",
        new AbstractIDEAction("Set New Return Value",Configuration.getIcon("setnewreturnvalue.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.setNewReturnValue(editor);
                    };
                };
            }
        });
        Mode.addAction("continue","Continue","Returns control from the debugger to the debugged program.",
        new AbstractIDEAction("Continue",Configuration.getIcon("continue.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.continueExecution(editor);
                    }
                };
            }
        });
        Mode.addAction("future","Step Forward In Time","Go one frame forward in time. The lisp form which was executed after the current highlighted form is highlighted and the saved lexical environment of that moment is restored. If the maximum depth of the time buffer is reached a wrap around is taken place.",
        new AbstractIDEAction("Step Forward In Time",Configuration.getIcon("future.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.goToFuture(editor);
                    };
                };
            }
        });
        Mode.addAction("past","Step Backward In Time","Go one frame back in time. The lisp form which was executed before the current highlighted form is highlighted and the saved lexical environment of that moment is restored. If the maximum depth of the time buffer is reached a wrap around is taken place.",
        new AbstractIDEAction("Step Backward In Time",Configuration.getIcon("past.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.goToPast(editor);
                    };
                };
            }
        });
        Mode.addAction("startprofiling","Start (reset) profiling","Only valid for a editor in debug mode. Starts profiling or reset the profile counters. The debugger will count the number of times a lisp form is called during execution. Displays also a slider to highlight the forms with call count higer or equal to the sliders value.",
        new AbstractIDEAction("Start (reset) profiling",Configuration.getIcon("startprofiling.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.startProfiling(editor);
                    };
                };
            }
        });
        Mode.addAction("stopprofiling","Stop Profiling","Stop the profiling of a lisp program. The debugger will not track any more the amout of time that a lisp form is called. Will remove also the slider to display the profile information.",
        new AbstractIDEAction("Stop Profiling",Configuration.getIcon("stopprofiling.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        interactions.selectInteraction(interaction);
                        interaction.stopProfiling(editor);
                    };
                };
            }
        });
        Mode.addAction("addwatch","Add Watch","Adds a watch point to the current buffer if this buffer is in debug mode.",
        new AbstractIDEAction("Add Watch",Configuration.getIcon("watch.png"))
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        watch.addWatchPoint(editor,interaction);
                    };
                };
            };
        });
        Mode.addAction("watchleaf","Watch variable","Watch the current word (is considered a variable).",
        new AbstractIDEAction("Watch variable")
        {
            public void doWork()
            {
                CodeEditor editor=currentEditor();
                if (editor!=null) // If this is a editor for a interaction pane ensure that we have associated editor
                    editor=editor.getEditorForInteraction();
                if (editor!=null)
                {
                    IInteraction interaction=editor.getInteraction();
                    if (interaction!=null)
                    {
                        watch.addWatchedLeaf(editor,interaction);
                    };
                };
            }
        });
        Mode.addAction("editwatch","Edit Watch","Starts a editor buffer allowing you to change the expression associated with the watchpoint selected in the watch explorer.",
        new AbstractIDEAction("Edit Watch")
        {
            public void doWork()
            {
                watch.editWatchPoint();
            }
        });
        Mode.addAction("removewatch","Remode Watch","Remove the watchpoint selected in the watch explorer.",new AbstractIDEAction("Remove Watch")
        {
            public void doWork()
            {
                watch.removeWatchPoint();
            }
        });
        Mode.addAction("keybinding","Change Keybinding","Displays a dialog box allowing you to change the keysequences acting as a shortcut for actions.",
        new AbstractIDEAction("keybinding")
        {
            public void doWork()
            {
                //keymap.changeKeyMap();
                ActionKeyMap.changeKeyMappings();
            }
        });
        Mode.addAction("switch","Switch","Switch between the editor pane and the interaction pan.",
        new AbstractIDEAction("Switch")
        {
            public void doWork()
            {
                if (workarea.isCurrent())
                {
                    // Make interactions current
                    CodePane pane=(CodePane) interactions.getSelectedComponent();
                    if (pane!=null)
                        setCurrent(pane.getEditor());
                }
                else if (interactions.isCurrent())
                {
                    // Make workarea current
                    WorkPane pane=(WorkPane) workarea.getSelectedComponent();
                    if (pane!=null)
                        setCurrent(pane.getFirstEditor());
                };
            }
        });
        Mode.addAction("next","Next","Switch to the next editor buffer in the interaction or editor pane. If the last buffer is reached switch back to the first buffer.",
        new AbstractIDEAction("next")
        {
            public void doWork()
            {
                if (workarea.isCurrent())
                    workarea.next();
                else if (interactions.isCurrent())
                    interactions.next();
            }
        });
    }
    /** Set the current codeeditor */
    public static void setCurrent(CodeEditor editor)
    {
        //System.out.println("Set current "+editor);
        if (editor==null)
        {
            currentEditor=null;
            Mode.getMode("basic").activate();
        }
        else if (currentEditor!=editor)
        {
            currentEditor=editor;;
            Hint.setEditor(currentEditor);
            editor.getMode().activate();
            WorkArea.selectWorkPaneContainingEditor(editor);
            Interactions.selectInteractionPaneContainingEditor(editor);
            currentEditor.requestFocus();
            currentEditor.cursor.setCursor(currentEditor.cursor.cursorOffset());
            currentEditor.getDocument().setCodeEditor(currentEditor);
            ValueWindow.removeValueLeaf(); // Remove every trace of a possible value displayed in a debugged editor
        };
    }
    /** Set the mode of the IDE */
    public static void setMode(Mode mode,JMenuBar menubar,JPanel toolbar,ActionKeyMap keymap)
    {
        if (currentmode!=mode)
        {
            // Set the Menus
            frame.setJMenuBar(menubar);
            // Set the keymap
            Main.keymap=keymap;
            // Set the buttons displayed in the button bar
            Main.toolbar.removeAll();
            Main.toolbar.add(toolbar);
            Main.toolbar.revalidate();
            Main.toolbar.repaint();
            frame.validate();
            // Set the current mode
            currentmode=mode;
            MessageHandler.displayMode(currentmode.getName());
        };
    }
    /** Update the position of the cursor indication */
    public static void updateCursorInfo(int line,int column)
    {
        cursorinfo.setText(line+"-"+column);
        cursorinfo.revalidate();
    }
    /** Set the mouse pointer  for the main window */
    public static void setMousePointer(Cursor cursor)
    {
        frame.getContentPane().setCursor(cursor);
    }
    public static void setMousePointer(int cursortype)
    {
        frame.getContentPane().setCursor(new Cursor(cursortype));
    }
    /** Find the current document , based on the CodePane current in use */
    public static CodeEditor currentEditor()
    {
        return currentEditor;
    }
    /** Get the current mode that is active */
    public static Mode currentMode()
    {
        return currentmode;
    }
    /** Get the workarea of this editor */
    public static WorkArea getWorkArea()
    {
        return workarea;
    }
    /** Return the interactions */
    public static JTabbedPane getInteractions()
    {
        return interactions;
    }
    /** Repaint the Main window and all its contents */
    public static void repaintEverything()
    {
        workarea.repaintAll();
        Main.interactions.repaintAll();
        ProjectExplorer.projectexplorer.repaintAll();
        watch.repaintAll();
        macroexplorer.repaintAll();
    }
    /** Called when IDE window is closed */
    public void windowClosing(WindowEvent e)
    {
        // Change the cursor in a wait cursor
        exitJabberwocky();
    }
    /** Exit the program */
    private static void exitJabberwocky()
    {
        Thread stop=new Thread()
        {
            public void run()
            {
                Runnable doStop=new Runnable()
                {
                    public void run()
                    {
                        cleanupTempFiles();
                        Interactions.cleanup();
                        saveStateAddins();
                        workarea.closeAll();
                        SplashScreen.disposeSplash();
                        frame.setVisible(false);
                        frame.dispose();
                        System.out.flush();
                    }
                };
                try
                {
                    Thread.sleep(200); // Hopefully we wait long enough to let the screen be drawn
                    SwingUtilities.invokeAndWait(doStop);
                }
                catch(Exception e)
                {
                };
                System.exit(0);
            }
        };
        if (workarea.saveAll())
        {
            frame.getRootPane().setCursor(new Cursor(Cursor.WAIT_CURSOR));
            SplashScreen.showSplash();
            SplashScreen.showMessage("Exiting Jabberwocky");
            stop.start();
        }
        else
        {
            frame.getRootPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        };
    }
    /** Cleanup the temp files */
    private static void cleanupTempFiles()
    {
        // Close all the created tempfiles
        try
        {
            File tempdir=new File(System.getProperty("java.io.tmpdir"));
            File[] tempfiles=tempdir.listFiles();
            for (int i=0;i<tempfiles.length;i++)
                if (tempfiles[i].getName().startsWith("debug") || tempfiles[i].getName().startsWith("instrument") || tempfiles[i].getName().startsWith("lock"))
                    tempfiles[i].delete();
        }
        catch (Exception ex)
        {
            MessageHandler.report("Error cleaning up temp files",ex);
        };
    }
    /** Checks if a instance of Jabberwocky is already started */
    private static boolean checkFirstInstance()
    {
        File tempdir=new File(System.getProperty("java.io.tmpdir"));
        File[] lockfiles=tempdir.listFiles(new FileFilter()
        {
            public boolean accept(File file)
            {
                if (file.getName().startsWith("LockJabberwocky"))
                    return true;
                else
                    return false;
            }
        });
        
        if (lockfiles.length>0 && JOptionPane.showConfirmDialog(null,"Jabberwocky is already started or was not terminated correctly the previous time.\nProceed with startup?","Startup Jabberwocky",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.CANCEL_OPTION)
            return false;
        // Create a lock file for this instance
        try
        {
            if (lockfiles.length>0 && JOptionPane.showConfirmDialog(null,"Remove the lock files (Choose OK if Jabberwocky was not terminated correctly the previous time) ?","Cleanup Lock files",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION)
            {
                for (int i=0;i<lockfiles.length;i++)
                    lockfiles[i].delete();
            };
            File lockfile=File.createTempFile("LockJabberwocky","lck",tempdir);
            lockfile.deleteOnExit();
        }
        catch (Exception e)
        {
        };
        return true;
    }
    /** Add a persistant object to the list of persistent objects */
    public static void registerAddin(IIDEAddin addin)
    {
        addins.add(addin);
    }
    /** Get the list of addins */
    public static void saveStateAddins()
    {
        try
        {
            File file=new File(System.getProperty("user.home")+File.separator+".Jabberwocky"+File.separator+".LispIDEState");
            Writer writer=new BufferedWriter(new FileWriter(file));
            for (int i=0;i<addins.size();i++)
                ((IIDEAddin) addins.elementAt(i)).saveState(writer);
            writer.close();
        }
        catch (IOException e)
        {
            MessageHandler.report("Failed to save state of components",e);
        };
    }
    /**
     * Enumerate all the editorpanes in a determenistic order, used to be able to determine the current editor
     * during save and load of the state
     */
    private static Vector enumerateEditors()
    {
        Vector result=new Vector();
        workarea.enumerateEditors(result);
        interactions.enumerateEditors(result);
        return result;
    }
    /** Make the editor indicated as current in the state the current editor */
    public static void restoreCurrent(int max,int current)
    {
        Vector editors=enumerateEditors();
        if (max==editors.size()) // Ensure that we have the same number of editors as when the state was saved
            if (current<max && editors.elementAt(current) instanceof CodeEditor)
                setCurrent((CodeEditor) editors.elementAt(current));
    }
    /**
     * This method will be called just before exit and allow the addin to save its state, the addin must be registered via a call
     * to Main.registerAddin()
     * Use the XMLConfigClass(Field,Methodor Object) objects to serialize this class
     */
    public void saveState(Writer writer)
    {
        XMLConfigClass cls=new XMLConfigClass(this.getClass().getName());
        // Save the size of the IDE
        XMLConfigMethod method=new XMLConfigMethod("restoreState");
        Dimension dim=frame.getSize();
        method.addArgument((int) dim.getWidth());
        method.addArgument((int) dim.getHeight());
        method.addArgument(deskinteractions.getDividerLocation());
        method.addArgument(central.getDividerLocation());
        cls.addMethod(method);
        // Save who is the current editor
        if (currentEditor!=null)
        {
            Vector editors=enumerateEditors();
            int m=editors.size();
            int current;
            for (current=0;current<m && editors.elementAt(current)!=currentEditor;current++);
            if (current<m)
            {
                method=new XMLConfigMethod("restoreCurrent");
                method.addArgument(m);
                method.addArgument(current);
                cls.addMethod(method);
            };
        };
        cls.serialize(writer);
    }
    /** Set the size of the frame and the divider locations */
    public static void restoreState(int width,int height,int divDeskinteractions,int divCentral)
    {
        frame.setSize(width,height);
        deskinteractions.setDividerLocation(divDeskinteractions);
        central.setDividerLocation(divCentral);
    }
    
    /** Invoked when a component gains the keyboard focus.
     *
     */
    public void focusGained(FocusEvent e)
    {
        if (currentEditor!=null)
            currentEditor.requestFocus();
    }
    
    /** Invoked when a component loses the keyboard focus.
     *
     */
    public void focusLost(FocusEvent e)
    {
    }
    
}
