/*
 * InstructionBox.java
 *
 * Created on September 16, 2001, 2:45 PM
 */

package IDE.Messaging;

import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import IDE.*;
import IDE.Recording.*;
/**
 * The text input field below the edit window
 * To ask something via the InstructionField, use InstructionField.ask(question)
 * @author  marc
 * @version
 */
public class InstructionField extends JTextField implements FocusListener,KeyListener,DocumentListener
{
    /** Contain the name to question mapping for questions used by the instructionField */
    public static HashMap nameToQuestion=new HashMap();
    /** Contains the question to name mapping for serialization purposes */
    public static HashMap questionToName=new HashMap();
    /** Some general constants indicate the type of call made */
    /** indicates a completeAnswer is send to the question */
    public static final int completeAnswer=0;
    /** indicates a newAnswer is send to the question */
    public static final int newAnswer=1;
    /** indicates a keyPressed has been sended to the question */
    public static final int keyPressed=2;
    /** indicates the initialText has been sended to the question */
    public static final int initialText=3;
    /** Contains the question which must be answered */
    private static IQuestion question=null;
    /** Contains a reference to the only created InstructionField */
    private static InstructionField self;
    /** Creates new InstructionBox */
    public InstructionField()
    {
        setBorder(BorderFactory.createEtchedBorder());
        addFocusListener(this);
        addKeyListener(this);
        getDocument().addDocumentListener(this);
        self=this;
    }
    /** Add a question to the instruction field */
    public static void addQuestion(String name,IQuestion question)
    {
        nameToQuestion.put(name,question);
        questionToName.put(question,name);
    }
    /** Get the name of a question */
    public static String getQuestionName(IQuestion question)
    {
        return (String) questionToName.get(question);
    }
    /** Get the question by name */
    public static IQuestion getQuestion(String name)
    {
        return (IQuestion) nameToQuestion.get(name);
    }
    /* *
     * Ask the user to type in extra information,
     */
    public static void ask(String name)
    {
        IQuestion question=(IQuestion) nameToQuestion.get(name);
        if (question!=null && self!=null && !Recording.isReplaying())
        {
            if (question.restoreState()) // Restore the state of the question
            {
                InstructionField.question=question;
                self.setText(question.initialText());
                self.selectAll();
                Recording.recordQuestionEvent(question,0,0,self.getText(),initialText);
                self.requestFocus();
                MessageHandler.say(question.guideText());
            };
        }
    }
    /** May only be called from LISP, so no macro recording is needed */
    public static void ask(IQuestion question)
    {
        if (self!=null)
        {
            if (question.restoreState())
            {
                InstructionField.question=question;
                self.setText(question.initialText());
                self.selectAll();
                self.requestFocus();
                MessageHandler.say(question.guideText());
            }
        };
    }
    /** Will be called by the question to set the text in the input field */
    public static void updateText(String text)
    {
        self.setText(text);
    }
    /**
     * Check if the instruction field has the focus
     */
    public static boolean inFocus()
    {
        return self.hasFocus();
    }
    /** Clear the message area and the inputfield when leaviung the focus */
    public void focusLost(FocusEvent e)
    {
        if (!Recording.isReplaying())
        {
            if (question==null || question.abort())
            {
                MessageHandler.say("");
                question=null;
                setText("");
            }
            else
            {
                requestFocus();
            }
        }
    }
    
    public void focusGained(FocusEvent e)
    {
    }
    
    public void keyTyped(KeyEvent e)
    {
        if (!Recording.isReplaying())
            if (question==null)
                e.consume();
    }
    
    public void keyReleased(KeyEvent e)
    {
        if (question==null || Recording.isReplaying())
            e.consume();
    }
    
    public void keyPressed(KeyEvent e)
    {
        if (question==null || Recording.isReplaying())
        {
            e.consume();
        }
        else if (e.getKeyChar()==KeyEvent.VK_ESCAPE)
        {
            if (question==null || question.abort())
            {
                MessageHandler.say("");
                question=null;
                setText("");
                Main.currentEditor().grabFocus();
            };
        }
        else if (e.getKeyChar()==KeyEvent.VK_ENTER)
        {
            Recording.recordQuestionEvent(question,e.getKeyCode(),e.getModifiers(),getText(),completeAnswer);
            question.completeAnswer(getText());
            MessageHandler.say(question.guideText());
            e.consume();
        }
        else
        {
            Recording.recordQuestionEvent(question,e.getKeyCode(),e.getModifiers(),getText(),keyPressed);
            question.keyPressed(e.getKeyCode(),e.getModifiers());
            MessageHandler.say(question.guideText());
        }
    }
    public void removeUpdate(DocumentEvent e)
    {
        if (question!=null && !Recording.isReplaying())
        {
            Recording.recordQuestionEvent(question,0,0,getText(),newAnswer);
            question.newAnswer(getText());
        };
    }
    
    public void insertUpdate(DocumentEvent e)
    {
        if (question!=null && !Recording.isReplaying())
        {
            Recording.recordQuestionEvent(question,0,0,getText(),newAnswer);
            question.newAnswer(getText());
        }
    }
    
    public void changedUpdate(DocumentEvent e)
    {
        if (question!=null && !Recording.isReplaying())
        {
            Recording.recordQuestionEvent(question,0,0,getText(),newAnswer);
            question.newAnswer(getText());
        }
    }
}
