/*
 * Created on Aug 17, 2003
 */
 
package IDE.Project;

import IDE.CodeEditor.*;
import IDE.Utilities.*;
import java.util.*;

/**
 * @author marc
 * Partial implementation of a Repository
 */
public abstract class AbstractRepository implements IRepository 
{
	/** Counter used to ensure that getName() returns a unique value for each Definition created */
	private static int counter=0;
	/** Timer to run the analyze of the elements in a seperate thread on */
	public final  static java.util.Timer timer=new java.util.Timer();
	/* (non-Javadoc)
	 * @see IDE.Project.IRepository#analyzeElements(IDE.CodeEditor.CodeElementVector, IDE.CodeEditor.CodeDocument, boolean)
	 */
	public void analyzeElements(final CodeElementVector elements,final CodeDocument doc) 
	{
		timer.schedule(new TimerTask()
		{
			public void run()
			{
				analyzeElementsImp(elements,doc);
			}
		},10);
	}
	/**
	 * Loops over all the elements and calls updateDefinition on it, this causes a callback on the repository by the element
	 * so that the repository displays the new information
	 * @param elements
	 * @param doc
	 */
	private synchronized void analyzeElementsImp(CodeElementVector elements,CodeDocument doc)
	{
		Iterator iter;
		String path=Util.getCanonicalFileName(doc.getFile());
		// Start a refresh of a file 
		FileReference.beginAnalyze(path);
		// Tell myself that the analying has started to set the initial state
		beginAnalyze();
		// Let each element update the repository
		int m=elements.size();
		for (int i=0;i<m;i++)
			elements.codeElementAt(i).updateDefinition(this,doc);
		// Tell myself that the analyzing is finished
		endAnalyze();
		// End the referesh of the file
		FileReference.endAnalyze();
	}
}
