/*
 * Created on Jul 16, 2003
 */
 
package IDE.Project;

import java.awt.event.MouseEvent;
import java.util.*;

import IDE.Configuration.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * @author marc
 * Explorer node used to display in a ProjectExplorer tree
 */
public class ExplorerNode implements IExplorerNode 
{
	/** List of the child nodes of this node */
	private ArrayList children=new ArrayList();
	/** Parent of this node */
	private IExplorerNode parent;
	/** INode associated with this ExplorerNode */
	private INode node;
	/** Termpory instance of IExplorerNode to search for a node */
	private static ExplorerNode search=new ExplorerNode(null);
	/**
	 * Initialize a newly create ExplorerNode
	 */
	public ExplorerNode(INode node)
	{
		this.node=node;
	}
	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#getChildAt(int)
	 */
	public TreeNode getChildAt(int childIndex) 
	{
		return (TreeNode) children.get(childIndex);
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#getChildCount()
	 */
	public int getChildCount() 
	{
		return children.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#getParent()
	 */
	public TreeNode getParent() 
	{
		return parent;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#getIndex(javax.swing.tree.TreeNode)
	 */
	public int getIndex(TreeNode node) 
	{
		int i=Collections.binarySearch(children,node);
		if (i<0)
			return -1;
		else
			return i;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#getAllowsChildren()
	 */
	public boolean getAllowsChildren() 
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#isLeaf()
	 */
	public boolean isLeaf() 
	{
		return children.size()==0;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeNode#children()
	 */
	public Enumeration children() 
	{
		return new Enumeration()
		{
			private Iterator iter=children.iterator();
			public boolean hasMoreElements() 
			{
				return iter.hasNext();
			}
			public Object nextElement() 
			{
				return iter.next();
			}
			
		};
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#DestroyNode()
	 */
	public void DestroyNode() 
	{
		if (parent!=null)
		{
			parent.removeChild(this);
			parent=null;
		}
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#addChild()
	 */
	public void addChild(IExplorerNode child) 
	{
		/** Find the position where the child should be inserted in */
		int i=Collections.binarySearch(children,child);
		if (i<0)
		{
			children.add(-i-1,child);
			ProjectExplorer.model.nodesWereInserted(this,new int[]{-i-1});
		}
		else
		{
			children.set(i,child);
			ProjectExplorer.model.nodeChanged(child);
		};
		child.setParent(this);
	}
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) 
	{
		return getName().compareTo(((IExplorerNode) o).getName());
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#setParent(IDE.Project.IExplorerNode)
	 */
	public void setParent(IExplorerNode parent) 
	{
		this.parent=parent;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#getName()
	 */
	public String getName() 
	{
		return node.getUniqueName();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#getDisplayName()
	 */
	public String getDisplayName() 
	{
		return node.getDisplayName();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#removeChild(IDE.Project.IExplorerNode)
	 */
	public void removeChild(IExplorerNode child) 
	{
		int i=Collections.binarySearch(children,child);
		if (i>=0)
		{
			children.remove(i);
			ProjectExplorer.model.nodesWereRemoved(this,new int[]{i},new Object[]{child});
			if (children.size()==0)
				DestroyNode(); // Remove myself if I have no children anymore
		}
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#contains(java.lang.String)
	 */
	public boolean contains(String name) 
	{
		return Collections.binarySearch(children,name)>0;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#getIndexForName(java.lang.String)
	 */
	public int getIndexForNode(INode node) 
	{
		search.node=node;
		int i=Collections.binarySearch(children,search);
		if (i<0)
			return -1;
		else
			return i;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#getIcon()
	 */
	public ImageIcon getIcon() 
	{
		return Configuration.getIcon(node.getIconFile());
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#getToolTip()
	 */
	public String getToolTip() 
	{
		return node.getTooltip();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#getPopupMenu(java.awt.event.MouseEvent)
	 */
	public JPopupMenu getPopupMenu(MouseEvent e) 
	{
		return node.getPopupMenu();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#resetINode(IDE.Project.INode)
	 */
	public void resetINode(INode node) 
	{
		this.node=node;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.IExplorerNode#doubleClick()
	 */
	public void doubleClick() 
	{
		node.doubleClick();
	}
}
