/*
 * Created on Aug 24, 2003
 */
 
package IDE.Project;

import java.lang.reflect.*;
import java.util.*;

import javax.swing.SwingUtilities;
/**
 * @author marc
 * Class containing static methods to maintain what is done because of the analyzing of a certain source file and undo this operations
 * if the source file becomes invalid
 */
public class FileReference 
{
	/** Mapping of a path to a collection of IDefinitions */
	private static SortedMap pathToDefinitions=new TreeMap();
	/** Mapping of a path to a TreeSet of a array of INode's (a tree path in the project explorer) */
	private static SortedMap pathToTreepaths=new TreeMap();
	/** Collection of IDefinitions that belong to the file that is analyzed */
	private static ArrayList registeredDefinitions;
	/** Collection of INode[] that belong to the file that is analyzed */
	private static SortedSet registeredTreepaths;
	/** Collection of TreePaths that must be removed */
	private static SortedSet removedTreepaths;
	/** Path of the file beeing analyzed */
	private static String path;
	/** Close a file (call close on all the definitions associated with this file) */
	public static void closeFile(String file)
	{
		ArrayList definitions=(ArrayList) pathToDefinitions.get(file);
		if (definitions!=null)
		{
			int m=definitions.size();
			for (int i=0;i<m;i++)
				((IDefinition) definitions.get(i)).close();
		}
	}
	/**
	 * Called to release all the file references to a certain file from the filereferences 
	 * @param path Path of the file removed
	 */
	public static void removeFileReference(final String path)
	{
		// Undo definitions
		int m;
		ArrayList definitions=(ArrayList) pathToDefinitions.remove(path);
		if (definitions!=null)
		{
			m=definitions.size();
			for (int i=m-1;i>-1;i--)
				((IDefinition) definitions.get(i)).undoDefinition();
		};
		// Runnable to remove tree paths
		Runnable dowork=new Runnable()
		{
			public void run()
			{
				SortedSet treepaths=(SortedSet) pathToTreepaths.remove(path);
				Iterator iter=treepaths.iterator();
				while (iter.hasNext())
					ProjectExplorer.removeNode((INode[]) iter.next());
			}
		};
		// remove treepaths 
		try
		{
			if (SwingUtilities.isEventDispatchThread())
				dowork.run();
			else
				SwingUtilities.invokeAndWait(dowork);
		}
		catch (Exception e)
		{
		};
	}
	/**
	 * Called at the start of the analyze of a source file, undo the effect of the previous analyze and initialize the 
	 * system (set registeredDefinitions and registeredTreepaths) for a new analyze
	 * @param path Pathname of the source file
	 */
	public static void beginAnalyze(String path)
	{
		int m;
		FileReference.path=path;
		// Get the definitions registered for this path 
		registeredDefinitions=(ArrayList) pathToDefinitions.get(path);
		if (registeredDefinitions==null)
		{
			registeredDefinitions=new ArrayList();
			pathToDefinitions.put(path,registeredDefinitions);
		};
		// Undo all the previous definitions in  reverse order of registration
		m=registeredDefinitions.size();
		for (int i=m-1;i>-1;i--)
			((IDefinition) registeredDefinitions.get(i)).undoDefinition();
		// Clear the list of registered definitions for this file
		registeredDefinitions.clear();
		// Get the treepaths registerd for this path that must be removed
		removedTreepaths=(SortedSet) pathToTreepaths.get(path);
		// Create a new TreeSet to hold the newly added tree paths
		registeredTreepaths=new TreeSet(INodeArrayComparator.comparator);
		pathToTreepaths.put(path,registeredTreepaths);
		// Clear the list of all the registerd treepaths for this file
		registeredTreepaths.clear();
	}
	/**
	 * Gets called at the end of the analyzing of a source file
	 *
	 */
	public static  void endAnalyze()
	{
		// On the user interface thread create all the paths added and remove the previous nodes (if they are not created again)
		try
		{
			SwingUtilities.invokeAndWait(new Runnable()
			{
				public void run()
				{
					// Remove the tree nodes which are not added again
					INode[] treepath;
					Iterator iter;
					if (removedTreepaths!=null)
					{
						iter=removedTreepaths.iterator();
						while (iter.hasNext())
						{
							treepath=(INode[]) iter.next();
							if (!registeredTreepaths.contains(treepath))
								ProjectExplorer.removeNode(treepath);
						};
					};
					// Next add or update the added treepaths
					iter=registeredTreepaths.iterator();
					while (iter.hasNext())
					{
						treepath=(INode[]) iter.next();
						if (removedTreepaths!=null && removedTreepaths.contains(treepath))
							// Only reset the the associated ProjectExplorer node
							ProjectExplorer.resetNode(treepath);
						else
							// Add the new path 
							ProjectExplorer.addNode(treepath); 
					};
				}
			});
		}
		catch (InterruptedException e)
		{	 
		}
		catch (InvocationTargetException e)
		{
		}
	}
	/**
	 * Register a definition
	 * @param definition The definition to register
	 */
	public static void registerDefinition(IDefinition definition)
	{
		registeredDefinitions.add(definition);
	}
	/**
	 * Register a treepath (array of INode's 
	 * @param treepath The treepath to register
	 */
	public static void registerTreePath(INode[] treepath)
	{
		registeredTreepaths.add(treepath);
	}
	/**
	 * Retrieve the path of the file that is currently analyzed
	 * @return The path of the file that is analyzed
	 */
	public static String getPath()
	{
		return path;
	}
	/**
	 * Returns a array of the definitions (in order of creation) associared with the path
	 * @param path The path of the filename containing the definitions
	 * @param type The type of the definitions to return
	 * @return The array of requested definitions
	 */
	public static  IDefinition[] getDefinitionsOfType(String path,Class type)
	{
		ArrayList result=new ArrayList();
		ArrayList definitions=(ArrayList) pathToDefinitions.get(path);
		if (definitions==null)
			return new IDefinition[0];
		int m=definitions.size();
		for (int i=0;i<m;i++)
			if (definitions.get(i).getClass()==type)
				result.add(definitions.get(i));
		IDefinition[] found=new IDefinition[result.size()];
		for (int i=0;i<found.length;i++)
			found[i]=(IDefinition) result.get(i);
		return found;
	}
}
