/*
 * Created on Aug 17, 2003
 */
 
package IDE.Project;

import java.util.Comparator;

import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;

/**
 * @author marc
 * A repository keeps track of all the definitions maintained by a parser
 */
public interface IRepository extends INode
{
	/** Called by a parser when he has updated the element structure of a document., this method is responsible
	 * to look at all the elements and ask them to callback and tell the repository what they have found
	 * @param elements CodeElement vector containing the elements of a document
	 * @param doc The document containing the elements
	 */
	public abstract void analyzeElements(CodeElementVector elements,CodeDocument doc);
	/**
	 * Check if this repository is able to work with this kind of file (the LispRepository ) is able to work with files with 
	 * extensions lisp or lsp
	 * @param File File to check
	 */
	public abstract boolean supportFile(String filename);
	/** 
	 * Returns a array of the extensions this repository recognizes
	 * @return array of the extensins supported by this repository
	 */
	public abstract String[] getExtensions();
	/**
	 * Returns the default extension used for example in save and save-as operations
	 * @return The default extension
	 */
	public abstract String getDefaultExtension();
	/**
	 * Create the parser associated with this repository (LispRepository creates a instance of LispParser
	 * @param assync indicates if the parse must parse a document asyncrhounous or synchronous
	 * @return Parser instance of the parser understood by this repository
	*/
	public abstract Parser createParser(boolean assync);
	/**
	 * Tells the repository that the analyze of a document has been started, the repository can use
	 * this to initialize state information like the current package, ....
	 */
	public void beginAnalyze();
	/**
	 *  Tells the repository that the analyze of a document has been finished
	 */
	public void endAnalyze();
	/** 
	 * Returns a comparator to compare hint strings (specific for the repository involved)
	 * @return A Comparator that is able to compare the strings in the list of hints (for example in the LispRepository you have to take
	 * care of the package name)
	 */
	public Comparator getHintComparator();
}
