/*
 * Created on Aug 31, 2003
 */
 
package IDE.Project;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.event.*;
import IDE.*;
import IDE.CodeEditor.*;

/**
 * @author marc
 *
 */
public abstract class LispSymbolDefinitionNode extends AbstractDefinition implements INode
{
	/** The symbol associated with this node */
	private LispSymbol symbol;
	/** The type of the symbol (we have a concrete definition) */
	private String type;
	public LispSymbolDefinitionNode(LispSymbol symbol,Position begin,int length)
	{
		this(symbol,null,begin,length);
	}
	public LispSymbolDefinitionNode(LispSymbol symbol,String type,Position begin,int length)
	{
		super(begin,length);
		this.symbol=symbol;
		if (type!=null)
			this.type=type.toLowerCase();
	}
	/**
	 * Returns the symbol associated with this definition
	 * @return The smbol associated with this definition
	 */
	public LispSymbol getSymbol()
	{
		return symbol;
	}
	
	/**
	 * If the document containing this definition is already open, jump to the location of the definition
	 * otherwise open the file containing the definition and jump to the correct location
	 */
	private void openDefinition()
	{
		int pos=getOffset();
		CodeDocument doc=Repositories.openFile(getPath());
		CodeEditor editor=doc.getCodeEditor();
		if (editor!=null)
		{
			Main.setCurrent(editor);
			editor.cursor.setCursor(pos);
		};
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#getUniqueName()
	 */
	public String getUniqueName() 
	{
		return symbol.getName()+getPath()+getOffset();
	}
	/**
	 * Get the name name of the symbol this definition stands for
	 * @return A string of the name of the symbol this is a definition of
	 */
	public String getSymbolName()
	{
		return symbol.getName();
	}
	/**
	 * Get the type of this lisp definition
	 * @return The type of this lisp definition
	 */
	public String getType()
	{
		return type;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#getDisplayName()
	 */
	public String getDisplayName() 
	{
		return symbol.getName();
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getIconFile()
	 */
	public String getIconFile() 
	{
		if (type==null)
			return "big_defun.png";
		else
			return "small_"+type+".png";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getPopupMenu()
	 */
	public JPopupMenu getPopupMenu() 
	{
		JPopupMenu popup=new JPopupMenu();
		popup.add(new AbstractAction("Open")
		{
				public void actionPerformed(ActionEvent ev)
				{
					openDefinition();
				}
		});
		return popup;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getTooltip()
	 */
	public String getTooltip() 
	{
		if (type==null)
			return "( "+symbol.getHomePackage()+"::"+symbol.getName()+")  at ["+getPath()+"::"+getOffset()+"]";
		else
			return "("+type+" "+symbol.getHomePackage()+"::"+symbol.getName()+")  at ["+getPath()+"::"+getOffset()+"]";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#doubleClick()
	 */
	public void doubleClick() 
	{
		openDefinition();
	}	
}
