/*
 * Created on Jul 16, 2003
 */
 
package IDE.Project;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.BorderLayout;
import java.awt.event.*;
import IDE.*;
import IDE.CodeEditor.*;

/**
 * @author marc
 * Represents the user interface for all the repositories
 */
public class ProjectExplorer extends JPanel implements MouseListener
 {
 	/** Top node in the tree model */
 	public final static IExplorerNode rootnode;
 	/** Tree model used for the JTree in this window */
	static DefaultTreeModel model;
	/** Scrollpanel containing the tree */
	private static JScrollPane scrollpane;
	/** Tree displaying the tree model */
	private static JTree tree;
	/** Only instance of this class */
	public final  static ProjectExplorer projectexplorer;

	/** Static initialization */
	static
	{
		rootnode=new ExplorerNode(RootNode.rootnode);
		model=new DefaultTreeModel(rootnode,true);
		tree=new JTree(model);
		tree.setLargeModel(true);
		tree.setRowHeight(0);
		tree.setCellRenderer(new ProjectNodeRenderer());
		// Ensure that we handle focus events correctly
		tree.addFocusListener(new FocusListener()
		{
			public void focusGained(FocusEvent ev)
			{
				CodeEditor editor=Main.currentEditor();
				if (editor!=null)
					CodeCursor.cancelCursor();
			}
			public void focusLost(FocusEvent ev)
			{
			}
		});
		scrollpane=new JScrollPane(tree);
		projectexplorer=new ProjectExplorer();
		tree.addMouseListener(projectexplorer);
		// Enable tooltips
		ToolTipManager.sharedInstance().registerComponent(tree);
	} 
	// Used to create the only ProjectExplorer possible
	private ProjectExplorer()
	{
		setLayout(new BorderLayout());
	 	add(scrollpane,BorderLayout.CENTER);
	}
	/** Repaint the tree if the fonts are changed */
	public void repaintAll()
	{
		model.nodeStructureChanged(rootnode);
		repaint();
	}
	/**
	 * Gets the root containing all the repositories
	 * @return Get the root node
	 */
	public static IExplorerNode getRoot()
	{
		return rootnode;
	}
	/**
	 * Sets for the explorer nodes the new INode that sits behind them
	 * @param path
	 */
	public static void resetNode(INode[] path)
	{
		IExplorerNode explorernode=rootnode;
		IExplorerNode temp;
		int index;
		for (int i=0;i<path.length;i++)
		{
			if ((index=explorernode.getIndexForNode(path[i]))<0)
			{
				temp=new ExplorerNode(path[i]);
				explorernode.addChild(temp);
				explorernode=temp;
			}
			else
			{
				explorernode=(IExplorerNode) explorernode.getChildAt(index);
				explorernode.resetINode(path[i]);
			};
		}
		
	}
	/**
	 * Add a node to the Project explorer
	 * @param path
	 */
	public static void addNode(INode[] path)
	{
		IExplorerNode explorernode=rootnode;
		IExplorerNode temp;
		int index;
		for (int i=0;i<path.length;i++)
		{
			if ((index=explorernode.getIndexForNode(path[i]))<0)
			{
				temp=new ExplorerNode(path[i]);
				explorernode.addChild(temp);
				explorernode=temp;
			}
			else
			{
				explorernode=(IExplorerNode) explorernode.getChildAt(index);
			};
		}
	}
	/**
	 * Remove a node from the Project explorer
	 * @param path
	 */
	public static void removeNode(INode[] path)
	{
		IExplorerNode node=rootnode;
		int index;
		for (int i=0;i<path.length;i++)
		{
			index=node.getIndexForNode(path[i]);
			if (index<0)
				return; // Unexisting node do nothing
			node=(IExplorerNode) node.getChildAt(index);
			if (i+1==path.length)
				node.DestroyNode(); // Found the node and delete it
		};
	}
	/**
	 * Check if the MouseEvent indicates that we must show a popup menu
	 * @param e The mouse event causing the call
	 */
	private static void checkPopup(MouseEvent e)
	{
		if (e.isPopupTrigger())
		{
			tree.clearSelection();
			tree.addSelectionPath(tree.getClosestPathForLocation(e.getX(),e.getY()));
			IExplorerNode node=(IExplorerNode) tree.getLastSelectedPathComponent();
			if (node!=null)
			{
				JPopupMenu popup=node.getPopupMenu(e);
				if (popup!=null)
					popup.show(tree,e.getX(),e.getY());
			}
		}
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) 
	{
		if (e.getClickCount()==2)
		{
			tree.clearSelection();
			tree.addSelectionPath(tree.getClosestPathForLocation(e.getX(),e.getY()));
			IExplorerNode node=(IExplorerNode) tree.getLastSelectedPathComponent();
			if (node!=null)
			{
				node.doubleClick();
			};
		}
		else
		{	
			checkPopup(e);
		}
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) 
	{
		checkPopup(e);
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e)
	 {
		checkPopup(e);
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e)
	 {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e)
	{
	}
}
