/*
 * Created on Aug 19, 2003
 */
 
package IDE.Project;

import IDE.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;
import IDE.Messaging.*;
import IDE.Utilities.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * @author marc
 * A project represents all the files, and subdirectories beneath a certain directory
 */
public class Projects implements IIDEAddin
{
	/** Top node of all the projects */
	private static final ProjectsNode projectsNode=new ProjectsNode();
	/** Default project directory */
	private static final ProjectNode defaultProjectNode=new ProjectNode();
	/** Map of pathName to ProjectNodes */
	private static final SortedMap pathToProjectNode=new TreeMap();
	/** Map of pathName to FileNodes */
	private static final SortedMap pathToFileOrDirectoryNode=new TreeMap();
	/** Contains the directory or file copied */
	private static String copybuffer="";
	/** Used to indicate that the pending copy is actually a move */
	private static boolean copyIsMove=false;
	static
	{
		// Add the topnode for all the projects
		ProjectExplorer.addNode(new INode[] {projectsNode});
		// Register the only instance of this classt at the Main class */
		Main.registerAddin(new Projects());	
	}
	
	private Projects()
	{
		
	}
	/**
	 * Show a Filechoser dialog asking for a directory and use that to add a project
	 */
	public static void addProject()
	{
		JFileChooser chooser=new JFileChooser();
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		if (chooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION)
		{
			addProject(chooser.getSelectedFile());
		}
	}
	/**
	 *  Add a new project for the directory with path path 
	 * Called via the state managment
	 * @param path The path of the directory
	 */
	public static void addProject(String path)
	{
		File projectdir=new File(path);
		if (projectdir.exists())
			addProject(projectdir);
	}
	/**
	 * Add a new project associated with this directory to the list of projects
	 * @param directory The directory associated with this project
	 */
	public static void addProject(File directory)
	{
		String path=Util.getCanonicalFileName(directory);
		ArrayList files=new ArrayList();
		if (pathToProjectNode.get(path)!=null)
			return; // Project exist already
		pathToProjectNode.put(path,new ProjectNode(directory));
		scanDirectory(path,directory,new TreeSet(),files);
		// Create the directories and file nodes if they don't exist yet
		addProjectFileDirectoryNodes(path,files);
	}
	/** Scan a directory and starts a analyzing of all the files in this directory by the repositories that recognize these files
	 * 
	 * @param directory Directory to scann
	 */
	private static void scanDirectory(String projectpath,File directory,SortedSet visited,ArrayList filesFound)
	{
		String path=Util.getCanonicalFileName(directory);
		if (visited.contains(path))
			return; // Avoid that we recursive calls ourself again
		visited.add(path);
		File[] files=directory.listFiles();
		for (int i=0;i<files.length;i++)
		{
			if (files[i].isDirectory())
			{
				scanDirectory(projectpath,files[i],visited,filesFound);
				filesFound.add(files[i]);
			}
			else
			{
				if (Repositories.supportsFile(files[i]))
				{
					Repositories.scanFile(files[i]);
					filesFound.add(files[i]);
				};
			}
		};
	}
	/**
	 * Add the directory node to the explorer (this will not analyze the files in this directory)
	 * @param directory The directory which node must be added
	 * @param ProjectNode The project node that must be added
	 */
	private static void addProjectFileDirectoryNodes(final String projectpath,final ArrayList files)
	{
		// On the timer thread of the AbstractRepository add the different nodes for a directory
		AbstractRepository.timer.schedule(new TimerTask()
		{
			public void run()
			{
				int m=files.size();
				FileReference.beginAnalyze(projectpath);
				INode projectnode=(INode) pathToProjectNode.get(projectpath);
				FileReference.registerTreePath(new INode[]{projectsNode,projectnode});
				for (int k=0;k<m;k++)
				{
					File file=(File) files.get(k);
					File[] paths=Util.calculateExclusivePathDifference(new File(projectpath),file);
					final INode[] nodes=new INode[paths.length+3];
					nodes[0]=projectsNode;
					nodes[1]=projectnode;
					nodes[paths.length+2]=getOrCreateFileOrDirectoryNode(file);
					for (int i=0;i<paths.length;i++)
						nodes[i+2]=getOrCreateFileOrDirectoryNode(paths[i]);
					FileReference.registerTreePath(nodes);
				}
				FileReference.endAnalyze();
			}
		},100);
	}
	/** 
	 * Returns a array of strings representing the path of all the filese supported by a project in the directory associated with this project
	 * @return The array of paths of all files (subdirectories)  in this projects that are by the repositories
	 */
	public static String[] getPathOfAllFiles()
	{
		ArrayList found=new ArrayList();
		SortedSet visited=new TreeSet();
		Iterator iter=pathToProjectNode.keySet().iterator();
		while (iter.hasNext())
			getPathOfAllFilesIn(new File((String) iter.next()),visited,found);
		int m=found.size();
		String[] result=new String[m];
		for (int i=0;i<m;i++)
			result[i]=(String) found.get(i);
		return result;
	}
	/**
	 * Returns a array of strings representing all the files supported by the repositories in this directory
	 * @param directory The path name of the directory to examine
	 * @return The array of all paths found
	 */
	public static String[] getPathOfAllFilesIn(String directory)
	{
		ArrayList found=new ArrayList();
		getPathOfAllFilesIn(new File(directory),new TreeSet(),found);
		int m=found.size();
		String[] result=new String[m];
		for (int i=0;i<m;i++)
			result[i]=(String) found.get(i);
		return result;		
	}
	/**
	 * Find all the supported files (by the repositories) in this directory
	 * @param directory Directory to scan
	 * @param visited ArrayList of directories already scanned
	 * @param found ArrayList containing the pathnames of the files found
	 */
	public  static void getPathOfAllFilesIn(File directory,SortedSet visited,ArrayList found)
	{
		String path=Util.getCanonicalFileName(directory);
		if (visited.contains(path))
			return; // Avoid visiting same directory twice
		visited.add(path);
		File[] children=directory.listFiles();
		for (int i=0;i<children.length;i++)
			if (children[i].isDirectory())
			{
				getPathOfAllFilesIn(children[i],visited,found);	
			}
			else
			{
				if (Repositories.supportsFile(children[i]))
					found.add(Util.getCanonicalFileName(children[i]));
			};
	}
	/**
	 * Returns a array of string representing the paths of all the defined projects
	*/
	public static String[] getProjects()
	{
		String[] paths=new String[pathToProjectNode.size()];
		Iterator iter=pathToProjectNode.keySet().iterator();
		for (int i=0;i<paths.length;i++)
			paths[i]=(String) iter.next();
		return paths;
	}
	/**
	 * Checks if the file with path path is contained in one of the project directories (npt the default project)
	 * @param path The path name of the file
	 * @return true if the file sits in a open project false otherwise
	 */
	public static boolean isInProject(String path)
	{
		Iterator iter=pathToProjectNode.keySet().iterator();
		while (iter.hasNext())
			if (path.startsWith((String) iter.next()))
				return true;
		return false;
	}
	
	/**
	 * Removes a project from the repository on the user interface thread, this function must be called on the user interface thread
	 * @param directory
	 */
	public static void removeProject(final File directory)
	{
		final String path=Util.getCanonicalFileName(directory);
		// Delete the project
		pathToProjectNode.remove(path);
		// Build a list of the files which references must be removed and which references must be reestablished
		final SortedSet toUpdate=new TreeSet();
		final SortedSet toRemove=new TreeSet();
		// toUpdate contains initially all the supported paths in the other projects directories
		String[] paths=getPathOfAllFiles(); // We have already removed this project so this is ok
		for (int i=0;i<paths.length;i++)
			toUpdate.add(paths[i]);
		// To Remove contains all the files in this directory minus the open files and the files referenced in the other directories
		paths=getPathOfAllFilesIn(path);
		for (int i=0;i<paths.length;i++)
			if (Repositories.isOpen(paths[i]))
				toUpdate.add(paths[i]);
			else if (!(toUpdate.contains(paths[i])))
				toRemove.add(paths[i]);
		// To avoid conflicts run this on the Timer thread of the AbstractRepository class
		AbstractRepository.timer.schedule(new TimerTask()
		{
			public void run()
			{
				// Remove all path references to this project
				FileReference.removeFileReference(path);
				// Remove all references to the files in toRemove
				Iterator iter=toRemove.iterator();
				while (iter.hasNext())
					FileReference.removeFileReference((String) iter.next());
				// Rescan all the other files 
				iter=toUpdate.iterator();
				while (iter.hasNext())
					Repositories.scanFile(new File((String) iter.next()));
			}
		},100);
	}
	/**
	 * Returns a array arraylist of INodes representing the treepath for each project for the file associated with the 
	 * definition node 
	 * @param definition The definition node where we have to create the filepaths for
	 * @return The array of arraylists of INodes representing the treepath for each projects of the file associated with this definition
	 */
	public synchronized static ArrayList[] calculateTreePathsForDefinitionNode(IDefinition definition)
	{
		ArrayList treepaths=new ArrayList();
		ArrayList treepath;
		Iterator iter=pathToProjectNode.values().iterator();
		ProjectNode projectnode;
		while (iter.hasNext())
		{
			projectnode=(ProjectNode) iter.next();
			treepath=calculateTreePathsForDefinitionNodeInProject(definition,projectnode);
			if (treepath.size()!=0)
				treepaths.add(treepath);
		};
		if (treepaths.size()==0)
		{
			treepath=new ArrayList();
			treepath.add(projectsNode);
			treepath.add(defaultProjectNode);
			treepath.add(getOrCreateFileOrDirectoryNode(new File(definition.getPath())));
			treepaths.add(treepath);
		};
		ArrayList[] result=new ArrayList[treepaths.size()];
		for (int i=0;i<result.length;i++)
			result[i]=(ArrayList) treepaths.get(i);
		return result;
	}
	/**
	 * Returns a arraylist of treenodes representing the treepath for the project and the file associated with this definition node
	 * @param definition The definition to calculate the treepath for
	 * @param projectnode The projectnode containing this treepath
	 * @return The treepath for this definition in this project
	 */
	private static synchronized ArrayList calculateTreePathsForDefinitionNodeInProject(IDefinition definition,ProjectNode projectnode)
	{
		if (definition.getPath().startsWith(projectnode.getUniqueName()))
		{
			File definitionFile=new File(definition.getPath());
			File projectdirFile=new File(projectnode.getUniqueName());
			File[] paths=Util.calculateExclusivePathDifference(projectdirFile,definitionFile);
			ArrayList result=new ArrayList();
			result.add(projectsNode);
			result.add(projectnode);
			for (int i=0;i<paths.length;i++)
				result.add(getOrCreateFileOrDirectoryNode(paths[i]));
			result.add(getOrCreateFileOrDirectoryNode(definitionFile));
			return result;
		}
		else 
		{
			return new ArrayList();
		}
	}
	/**
	 * Check if a file or directory node exist already then returns it, otherwise returns a newly created node and register it
	 * @param path Full pathname of directory or file node has to be created for
	 * @return The node found or created
	 */
	private static synchronized INode getOrCreateFileOrDirectoryNode(File file)
	{
		String path=Util.getCanonicalFileName(file);
		INode node=(INode) pathToFileOrDirectoryNode.get(path);
		if (node==null)
		{
			if (file.isDirectory())
			{
				node=new DirectoryNode(file);
				pathToFileOrDirectoryNode.put(path,node);
				return node;
			}
			else
			{
				node=new FileNode(file);
				pathToFileOrDirectoryNode.put(path,node);
				return node;
			}
		}
		else
		{
			return node;
		}
	}
	/* (non-Javadoc)
	 * @see IDE.IIDEAddin#saveState(java.io.Writer)
	 */
	public void saveState(Writer writer) 
	{
		XMLConfigClass cls=new XMLConfigClass(Projects.class.getName());
		XMLConfigMethod method;
		Iterator iter=pathToProjectNode.keySet().iterator();
		String path;
		while (iter.hasNext())
		{
			path=(String) iter.next();
			if (path.length()>0)
			{
				method=new XMLConfigMethod("addProject");
				method.addArgument(path);
				cls.addMethod(method);
			};
		};
		cls.serialize(writer);
	}
	/**
	 * Rescans all the files in the project associated with the directory with filename path
	 * @param path The directory path of the project
	 */
	protected static void rescanProject(String path)
	{
		ArrayList files=new ArrayList();
		scanDirectory(path,new File(path),new TreeSet(),files);
		addProjectFileDirectoryNodes(path,files);
	}
	/**
	 * Rescan all the projects containing path path
	 * @param path
	 */
	private  static void rescanAllProjectsContaining(String path)
	{
		Iterator iter=pathToProjectNode.keySet().iterator();
		String projectpath;
		while (iter.hasNext())
		{
			projectpath=(String) iter.next();
			if (path.startsWith(projectpath))
				rescanProject(projectpath);
		}
	}
	/**
	 * Rescan all the projects containing the paths in tthe array paths
	 * @param paths array of paths
	 */	
	private  static void rescanAllProjectsContaining(String[] paths)
	{
		Iterator iter=pathToProjectNode.keySet().iterator();
		String projectpath;
		while (iter.hasNext())
		{
			projectpath=(String) iter.next();
			for (int i=0;i<paths.length;i++)
				if (paths[i].startsWith(projectpath))
				{
					rescanProject(projectpath);
					break;
				};
		}
	}
	
	/**
	 * Ask for a new directory to create and creates this directory int the given dirpath directory
	 */
	protected static void newDirectory(String dirpath)
	{
		String newdir=(String) JOptionPane.showInputDialog("Folder name ?");
		if (newdir!=null)
		{
			File file=new File(dirpath,newdir);
			file.mkdir();
			// Rescan all the projects containing this directory
			rescanAllProjectsContaining(dirpath);
		};
	}
	/**
	 * Creates a new file
	 * @param filepath The path of the new file
	 * @param dirpath The directory to creat the file in 
	 * @param extension The extension of the file
	 */
	protected static void newFile(String dirpath,String extension)
	{
		String path=(String) JOptionPane.showInputDialog("File name  (without extension and directory)")+"."+extension;
		File file=new File(dirpath,path);
		try
		{
			file.createNewFile();
			// Rescan all the projects containing this directory
			rescanAllProjectsContaining(Util.getCanonicalFileName(file));
		}
		catch (Exception e)
		{
			MessageHandler.warn("Failed to create file "+file.getAbsolutePath()+" because of "+e.getMessage());
		};
	}
	/**
	 * Deletes the file with path filepath
	 * @param filepath The pat of the file to delete
	 */
	protected static void deleteFile(String filepath)
	{
		// Ask if we really want to delete this file
		if (JOptionPane.showConfirmDialog(null,"Delete file "+filepath+" ?","Delete file",JOptionPane.OK_OPTION)==JOptionPane.YES_OPTION)
			return;
		deleteFileImp(filepath);
	}
	private static void deleteFileImp(String filepath)
	{
		File file =new File(filepath);
		CodeDocument doc;
		try
		{
			if (Repositories.isOpen(filepath))
			{
				doc=Repositories.getOpenDocument(filepath);
				CodeEditor.closeAll(doc);
			};
			file.delete();
			// Rescan all the projects containing this file
			rescanAllProjectsContaining(filepath);
		}	
		catch (SecurityException e)
		{
			MessageHandler.warn("Failed to delete file "+filepath+" because of "+e.getMessage());
		};
	}  
	/**
	 * Delete the directory with file path filepath
	 * @param filepath
	 */
	protected static void deleteDirectory(String directory)
	{
		// Ask if we really want to delete this file
		if (JOptionPane.showConfirmDialog(null,"Delete the directory and its entire content, are you sure ?","Delete Directory",JOptionPane.OK_OPTION)==JOptionPane.YES_OPTION)
			return;
		deleteDirectoryAndContent(directory);
	}
	private static void deleteDirectoryAndContent(String directory)
	{
		// Check if we try to delete a project directory
		if (pathToProjectNode.containsKey(directory))
		{
			JOptionPane.showMessageDialog(null,"You may not delete a projectdirectory, close the project associated with "+directory+" first.");
			return;	
		};
		SortedSet visited=new TreeSet();
		ArrayList removed=new ArrayList();
		File dir=new File(directory);
		deleteDirectoryImp(new File(directory),removed,visited);
		try
		{
			dir.delete(); // Delete the directory
			removed.add(dir);
		}
		catch (SecurityException e)
		{
			MessageHandler.warn("Failed to delete the directory "+directory+" because of "+e.getMessage());
		};
		// Scann all the directories containing the removed files
		String[] paths=new String[removed.size()];
		for (int i=0;i<paths.length;i++)
			paths[i]=Util.getCanonicalFileName((File) removed.get(i));
		Projects.rescanAllProjectsContaining(paths);
	}
	private static void deleteDirectoryImp(File directory,ArrayList removed,SortedSet visited)
	{
		String path=Util.getCanonicalFileName(directory);
		if (visited.contains(path))
			return;
		visited.add(path);
		// Check if we try to delete a project directory
		if (pathToProjectNode.containsKey(path))
		{
			JOptionPane.showMessageDialog(null,"You may not delete a projectdirectory, close the project associated with "+path+" first.");
			return;	
		}
		CodeDocument doc;
		try
		{
			// First empty the contents of this directory
			File[] files=directory.listFiles();
			for (int i=0;i<files.length;i++)
			{
				path=Util.getCanonicalFileName(files[i]);
				if (files[i].isDirectory())
				{
					deleteDirectoryImp(files[i],removed,visited);
					files[i].delete();
					removed.add(files[i]);
				}
				else
				{
					if (Repositories.isOpen(path))
					{
						doc=Repositories.getOpenDocument(path);
						CodeEditor.closeAll(doc);
					};
					files[i].delete();
					removed.add(files[i]);
				}
			};
			path=Util.getCanonicalFileName(directory);
		}
		catch (SecurityException e)
		{
			MessageHandler.warn("Failed to delete directory or file "+path+" because of "+e.getMessage());
		};		
	}
	/**
	 * OPen the file with filename path
	 * @param path The file name of the file to open
	 */
	protected static void openFile(String path)
	{
		CodeDocument doc=Repositories.openFile(path);
		CodeEditor editor=doc.getCodeEditor();
		if (editor!=null)
		{
			Main.setCurrent(editor);
			editor.cursor.setCursor(0);
		};
	}
	protected static void renameFileOrDirectory(String oldname)
	{
		// Check if we try to rename a project directory
		if (pathToProjectNode.containsKey(oldname))
		{
			JOptionPane.showMessageDialog(null,"You may not rename a projectdirectory, close the project associated with "+oldname+" first.");
			return;	
		}
		File oldfile=new File(oldname);
		File directory=oldfile.getParentFile();
		String newname=JOptionPane.showInputDialog("New filename ?");
		if (newname!=null)
		{
			File newfile=oldfile==null ? new File(newname): new File(directory,newname);
			if (oldfile.isFile() && Repositories.isOpen(oldname))
			{
				CodeDocument doc=Repositories.getOpenDocument(oldname);
				doc.setFile(newfile);
				Repositories.renameDocument(doc,new File(oldname));
			};
			oldfile.renameTo(newfile);
			Projects.rescanAllProjectsContaining(oldname);
		};
	}
	/**
	 * Add all the emnu items to create a file for the different recognised extensions to the popup menu
	 * @param popup
	 */
	protected static void addNewFileMenuToPopup(final String path,JPopupMenu popup)
	{
		String[] extensions=Repositories.getExtensions();
		final StringBuffer buffer=new StringBuffer();
		for (int i=0;i<extensions.length;i++)
		{
			buffer.delete(0,buffer.length());
			buffer.append(extensions[i]);
			popup.add(new AbstractAction("Create *."+extensions[i]+" file")
			{
				public void actionPerformed(ActionEvent e)
				{
					Projects.newFile(path,buffer.toString());
				}
			});
		}
	}
	/**
	 * Check if paste is possible
	 * @return True if past is possible false otherwise
	 */
	protected static boolean pastePossible()
	{
		return copybuffer.length()>0;
	}
	/**
	 * Called when we want to start a copy/paste operation
	 * @param path The path of the file(directory) copied
	 */
	protected static void copyFileDirectory(String path)
	{
		copybuffer=path;
		copyIsMove=false;
	}
	/**
	 * Called when we want to start a cut/paste operation
	 * @param path The path of the file(directory) copied
	 */
	protected static void moveFileDirectory(String path)
	{
		copybuffer=path;
		copyIsMove=true;
	}	
	/**
	 * Called by a paste operation on a file 
	 * @param directory
	 */
	protected static void pasteFileDirectory(String directory)
	{
		if (copybuffer.length()==0)
			return;
		File copyOrCut=new File(copybuffer);
		ArrayList created=new ArrayList();
		if (copyOrCut.isDirectory())
		{
			copyDirectory(copyOrCut,new File(directory),new TreeSet(),created);
			if (copyIsMove)
				deleteDirectoryAndContent(copybuffer);
		}
		else
		{
			copyFile(copyOrCut,new File(directory),created);
			if (copyIsMove)
				deleteFileImp(copybuffer);
		};
		// Update all the 
		String[] files=new String[created.size()];
		for (int i=0;i<files.length;i++)
			files[i]=(String) Util.getCanonicalFileName((File) created.get(i));
		Projects.rescanAllProjectsContaining(files);
		// Reset
		copybuffer="";
		copyIsMove=false;
	}
	/**
	 * Copy a directory into another directory
	 * @param directory The directory to copy
	 * @param destination The destination directory to create
	 */
	private static void copyDirectory(File directory,File destination,SortedSet visited,ArrayList created)
	{
		String pathdirectory=Util.getCanonicalFileName(directory);
		String pathdestination=Util.getCanonicalFileName(destination);
		if (visited.contains(directory))
			return;
		visited.add(pathdirectory);
		if (pathdestination.startsWith(pathdirectory))
		{
			JOptionPane.showMessageDialog(null,"You may not copy a directory to its own subdirectory");
			return;
		};
		File newdirectory=new File(destination,directory.getName());
		int i=0;
		while (newdirectory.exists())
		{
			newdirectory=new File(destination,i+"."+directory.getName());
			i++;
		};
		try
		{
			newdirectory.mkdir();
			created.add(newdirectory);
			File[] files=directory.listFiles();
			for (int k=0;k<files.length;k++)
			{
				if (files[k].isDirectory())
					copyDirectory(files[k],newdirectory,visited,created);
				else
					copyFile(files[k],newdirectory,created);
			};
		}
		catch (Exception e)
		{
			MessageHandler.warn("Failed to create directory "+Util.getCanonicalFileName(newdirectory)+" because "+e.getMessage());
		}
	}
	/**
	 * Copy a file into another directory
	 * @param directory  The file to copy
	 * @param destination The destination directory
	 */
	private static void copyFile(File file,File destination,ArrayList created)
	{
		try
		{
			// Copy the content of the file
			File newfile=new File(destination,file.getName());
			int i=0;
			while (newfile.exists())
			{
				newfile=new File(destination,i+"."+file.getName());
				i++;
			};
			BufferedInputStream reader=new BufferedInputStream(new FileInputStream(file));
			BufferedOutputStream writer=new BufferedOutputStream(new FileOutputStream(newfile));
			byte[] buffer=new byte[(int) file.length()];
			reader.read(buffer);
			writer.write(buffer);
			writer.close();
			created.add(newfile);
		}
		catch (Exception e)
		{
			MessageHandler.warn("Failed to copy file "+Util.getCanonicalFileName(file)+" to "+Util.getCanonicalFileName(destination)+" because "+e.getMessage());
		}
	}
}
class ProjectNode implements INode
{
	/** Full pathname of the project directory */
	private String path;
	/** Name of the project directory */
	private String name;
	public ProjectNode()
	{
		path=null;
		name="Default Project";
	}
	public ProjectNode(File directory)
	{
		path=Util.getCanonicalFileName(directory);
		name=directory.getName();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#getUniqueName()
	 */
	public String getUniqueName() 
	{
		if (path==null)
			return "";
		else
			return path;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getDisplayName()
	 */
	public String getDisplayName() 
	{
		return name;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getIconFile()
	 */
	public String getIconFile() 
	{
		return "small_projects.png";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getPopupMenu()
	 */
	public JPopupMenu getPopupMenu() 
	{
		JPopupMenu popup=new JPopupMenu();
		popup.add(Mode.getAction("findandreplace"));
		popup.add(new AbstractAction("Rescan")
		{
			public void actionPerformed(ActionEvent ev)
			{
				Projects.rescanProject(path);
			}
		});
		popup.addSeparator();
		popup.add(new AbstractAction("Close Project")
		{
			public void actionPerformed(ActionEvent ev)
			{
				Projects.removeProject(new File(path));
			}
		});
		popup.addSeparator();
		popup.add(new AbstractAction("New Folder")
		{
			public void actionPerformed(ActionEvent ev)
			{
				Projects.newDirectory(path);
			}
		});
		Projects.addNewFileMenuToPopup(path,popup);
		popup.addSeparator();
		if (Projects.pastePossible())
			popup.add(new AbstractAction("Paste")
			{
				public void actionPerformed(ActionEvent e)
				{
					Projects.pasteFileDirectory(path);
				}
			});
		return popup;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getTooltip()
	 */
	public String getTooltip() 
	{
		if (path==null)
			return "Default directory, containing all the open files not sitting in a open project";
		else
			return path;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#doubleClick()
	 */
	public void doubleClick() {
		// TODO Auto-generated method stub
		
	}
	
}
class DirectoryNode implements INode
{
	/** Full pathname of the directory */
	String path;
	/** Relative name of the directory */
	String name;
	public DirectoryNode(File directory)
	{
		path=Util.getCanonicalFileName(directory);
		name=directory.getName();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#getUniqueName()
	 */
	public String getUniqueName() 
	{
		return path;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getDisplayName()
	 */
	public String getDisplayName() 
	{
		return name;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getIconFile()
	 */
	public String getIconFile() 
	{
		return "directory.png";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getPopupMenu()
	 */
	public JPopupMenu getPopupMenu() 
	{
		JPopupMenu popup=new JPopupMenu();
		popup.add(new AbstractAction("New Folder")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.newDirectory(path);
			}
		});
		popup.add(new AbstractAction("Rename")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.renameFileOrDirectory(path);
			}
		});
		popup.add(new AbstractAction("Delete")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.deleteDirectory(path);
			}
		});	
		popup.addSeparator();
		Projects.addNewFileMenuToPopup(path,popup);
		popup.addSeparator();
		popup.add(new AbstractAction("Copy")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.copyFileDirectory(path);
			}
		});
		popup.add(new AbstractAction("Cut")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.moveFileDirectory(path);
			}
		});
		if (Projects.pastePossible())
			popup.add(new AbstractAction("Paste")
			{
				public void actionPerformed(ActionEvent e)
				{
					Projects.pasteFileDirectory(path);
				}
			});
		return popup;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getTooltip()
	 */
	public String getTooltip() 
	{
		return path;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#doubleClick()
	 */
	public void doubleClick() {
		// TODO Auto-generated method stub
		
	}
	
}
class FileNode implements INode
{
	/** Full path name of the file */
	private String path;
	/** Relative name of the file */
	private String name;
	public FileNode(File file)
	{
		path=Util.getCanonicalFileName(file);
		name=file.getName();
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#getUniqueName()
	 */
	public String getUniqueName() 
	{
		return path;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#getDisplayName()
	 */
	public String getDisplayName() 
	{
		return name;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getIconFile()
	 */
	public String getIconFile() 
	{
		return "file.png";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getPopupMenu()
	 */
	public JPopupMenu getPopupMenu() 
	{
		JPopupMenu popup=new JPopupMenu();
		popup.add(new AbstractAction("Open")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.openFile(path);
			}
		});
		popup.add(new AbstractAction("Rename")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.renameFileOrDirectory(path);
			}
		});	
		popup.add(new AbstractAction("Delete")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.deleteFile(path);
			}
		});
		popup.addSeparator();
		popup.add(new AbstractAction("Copy")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.copyFileDirectory(path);
			}
		});
		popup.add(new AbstractAction("Cut")
		{
			public void actionPerformed(ActionEvent e)
			{
				Projects.moveFileDirectory(path);
			}
		});
		return popup;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getTooltip()
	 */
	public String getTooltip() 
	{
		return path;
	}
	/* (non-Javadoc)
	 * @see IDE.Project.INode#doubleClick()
	 */
	public void doubleClick() 
	{
		Projects.openFile(path);
	}
}
/**
 * Represents the top node containing all the projects known in Jabberwocky
 * @author marc
 *
 */
class ProjectsNode implements INode
{

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getUniqueName()
	 */
	public String getUniqueName() 
	{
		return "Projects";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getDisplayName()
	 */
	public String getDisplayName() 
	{
		return "Projects";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getIconFile()
	 */
	public String getIconFile() 
	{
		return "big_projects.png";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getPopupMenu()
	 */
	public JPopupMenu getPopupMenu() 
	{
		JPopupMenu popup=new JPopupMenu();
		popup.add(new AbstractAction("Open Project")
		{
			public void actionPerformed(ActionEvent ev)
			{
				Projects.addProject();
			}
		});
		popup.add(Mode.getAction("findandreplace"));
		return popup;
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#getTooltip()
	 */
	public String getTooltip() 
	{
		return "List of all the projects";
	}

	/* (non-Javadoc)
	 * @see IDE.Project.INode#doubleClick()
	 */
	public void doubleClick() 
	{
	}	
}
