/*
 * Records.java
 *
 * Created on May 4, 2002, 7:28 PM
 */

package IDE.Recording;

import java.util.*;
import IDE.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;
import IDE.Messaging.*;

/**
 * This class contain all the records of the actions during begin/end recording
 * @author  marc
 */
public class Macro
{
    /** Vector containing the records */
    private LinkedList records;
    /** Current Record or Question, avoid that the question has to pass on itself */
    private Object current=null;
    /** Name of the macro */
    private String name=null;
    /** Creates a new instance of Records */
    public Macro()
    {
        records=new LinkedList();
    }
    /** Set the name for the macro */
    public void setName(String name)
    {
        this.name=name;
    }
    /** Get the name of the macro */
    public String getName()
    {
        return name;
    }
    /** Record a action and event in the records */
    protected void recordActionEvent(AbstractIDEAction action)
    {
        records.add(new RecordActionEvent(action));
    }
    /** Recortd the insertion of a deadkey event */
    protected void recordInsertDeadKeyEvent(char unicodechar)
    {
        records.add(new RecordInsertDeadKeyEvent(unicodechar));
    }
    
    /** Record a question asked */
    protected void recordQuestionEvent(IQuestion question,int keycode,int modifiers,String text,int type)
    {
        records.add(new RecordQuestionEvent(question,keycode,modifiers,text,type));
    }
    /** Record a KeyPressed KeyEvent */
    protected void recordKeyEvent(int id,long when,int modifiers,int keycode,char keychar,int keylocation,int type)
    {
        records.add(new RecordKeyEvent(id,when,modifiers,keycode,keychar,keylocation,type));
    }
    /** Return a iterator over the records */
    public Iterator getRecordIterator()
    {
        return records.iterator();
    } 
    /** Replay the recording */
    protected void replay()
    {
        /* Now we are replaying the objects */
        Object record;
        HashSet staterestored=new HashSet();
        Iterator iter=records.listIterator();
        CodeEditor current=Main.currentEditor();
        if (current==null)
            return;
        while (iter.hasNext())
        {
            record=iter.next();
            if (record instanceof RecordActionEvent)
            {
                // Replay the record
                RecordActionEvent actionevent=(RecordActionEvent) record;
                // Restore the state  in case this action is used for the first time (simulates the constructor)
                if (!staterestored.contains(actionevent.getAction()))
                {
                    actionevent.getAction().restoreState();
                    staterestored.add(actionevent.getAction());
                }
                actionevent.getAction().doWork();
            }
            else if (record instanceof RecordQuestionEvent)
            {
                RecordQuestionEvent questionevent=(RecordQuestionEvent) record;
                IQuestion question=questionevent.getQuestion();
                // Restore the state in this action for the first time (stimulates the constructor)
                if (!staterestored.contains(question))
                {
                    question.restoreState();
                    staterestored.add(question);
                };
                switch (questionevent.getType())
                {
                    case InstructionField.completeAnswer:
                        question.completeAnswer(questionevent.getText());
                        break;
                    case InstructionField.initialText:
                        question.setInitialText(questionevent.getText());
                        question.restoreState();
                        InstructionField.updateText(question.initialText());
                        break;
                    case InstructionField.keyPressed:
                        question.keyPressed(questionevent.getKeyCode(),questionevent.getModifiers());
                        break;
                    case InstructionField.newAnswer:
                        question.newAnswer(questionevent.getText());
                        break;
                }
            }
            else if (record instanceof RecordKeyEvent)
            {
                if (current!=null)
                {
                    int type=((RecordKeyEvent) record).getType();
                    if (type==RecordKeyEvent.KeyPressed)
                        current.processKeyPressed(((RecordKeyEvent) record).createKeyEvent(current),true);
                    else if (type==RecordKeyEvent.KeyTyped)
                        current.processKeyTyped(((RecordKeyEvent) record).createKeyEvent(current),true);
                };
            }
            else if (record instanceof RecordInsertDeadKeyEvent)
            {
                if (current!=null)
                {
                    current.insertCharacter(((RecordInsertDeadKeyEvent) record).getChar());
                }
            }
        };
    }
}
