/*
 * RecordingExplorer.java
 *
 * Created on May 9, 2002, 11:54 AM
 */

package IDE.Recording;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import IDE.Configuration.*;


/**
 * Class to repesent a tree of macros and lisp extension code
 * @author  marc
 */
public class MacroExplorer extends JPanel implements MouseListener
{
    /** Tree containing the macros and lisp extension code */
    private JTree tree;
    /** TreeModel used */
    private DefaultTreeModel treemodel;
    /** Root node */
    private DefaultMutableTreeNode root;
    /** Scrollpane */
    private JScrollPane scrollpane;
    /** Root node menu */
    private JPopupMenu rootnodemenu;
    /** Popupmenu for a macro node */
    private JPopupMenu macronodemenu;
    /** Creates a new instance of RecordingExplorer */
    public MacroExplorer()
    {
        root=new DefaultMutableTreeNode("All Macros And Lisp Scripts");
        treemodel=new DefaultTreeModel(root);
        tree=new JTree(treemodel);
        tree.setCellRenderer(new MacroRenderer());
        scrollpane=new JScrollPane(tree);
        setLayout(new BorderLayout());
        add(scrollpane,BorderLayout.CENTER);
        tree.addMouseListener(this);
        // Create the different popup menus
        createPopupMenus();
    }
    /** Repaint all the noded (if for example the font has changed */
    public void repaintAll()
    {
        treemodel.nodeStructureChanged(root);
        repaint();
    }
    /** Create the different popupmenus */
    private void createPopupMenus()
    {
        rootnodemenu=new JPopupMenu("Macros/Scripts");
        rootnodemenu.add(Mode.getAction("startrecording")).setIcon(null);
        rootnodemenu.add(Mode.getAction("stoprecording")).setIcon(null);
        rootnodemenu.add(Mode.getAction("replayrecording")).setIcon(null);
        
        macronodemenu=new JPopupMenu("Macro");
        macronodemenu.add(new AbstractAction("Replay")
        {
            public void actionPerformed(ActionEvent ev)
            {
                replayMacroNode();
            }
        });
        macronodemenu.add(new AbstractAction("Delete")
        {
            public void actionPerformed(ActionEvent ev)
            {
                removeMacroNode();
            }
        });
    }
    /** Replay the selected macronode */
    private void replayMacroNode()
    {
        Object node=tree.getLastSelectedPathComponent();
        if (node instanceof MacroNode)
            Recording.replay(((MacroNode) node).getName());
    }
    /** Remove the selected macronode */
    private void removeMacroNode()
    {
        Object node=tree.getLastSelectedPathComponent();
        if (node instanceof MacroNode)
        {
            Recording.removeMacro(((MacroNode) node).getName());
            treemodel.removeNodeFromParent((MacroNode) node);
        };
    }
    /** Find the index to add macro to */
    private int findMacroIndex(String name)
    {
        int m=root.getChildCount();
        int i;
        for (i=0;i<m && name.compareTo(((AbstractMacroNode) root.getChildAt(i)).getName())<0;i++);
        return i;
    }
    /** Add a macro to this tree */
    public void addMacroNode(String name)
    {
        Macro macro=Recording.getMacro(name);
        if (macro!=null)
        {
            // First add the macro node
            MacroNode macronode=new MacroNode(name);
            int index=findMacroIndex(name);
            if (index<root.getChildCount() && (root.getChildAt(index) instanceof AbstractMacroNode)
            && ((AbstractMacroNode) root.getChildAt(index)).getName().compareTo(name)==0)
                treemodel.removeNodeFromParent((AbstractMacroNode) root.getChildAt(index));
            treemodel.insertNodeInto(macronode,root,findMacroIndex(name));
            // Next add the recorded information
            Iterator iter=macro.getRecordIterator();
            int i=0;
            while (iter.hasNext())
            {
                addRecordNode(macronode,iter.next(),i);
                i++;
            };
            tree.expandRow(0);
        };
    }
    /** Add a record node to the macro node */
    private void addRecordNode(MacroNode macronode,Object record,int i)
    {
        AbstractMacroNode node;
        if (record instanceof RecordActionEvent)
        {
            node=new MacroActionNode((RecordActionEvent) record);
            treemodel.insertNodeInto(node,macronode,i);
        }
        else if (record instanceof RecordQuestionEvent)
        {
            node=new MacroQuestionNode((RecordQuestionEvent) record);
            treemodel.insertNodeInto(node,macronode,i);
        }
        else if (record instanceof RecordKeyEvent)
        {
            node=new MacroKeyNode((RecordKeyEvent) record);
            treemodel.insertNodeInto(node,macronode,i);
        }
        else if (record instanceof RecordInsertDeadKeyEvent)
        {
            node=new MacroInsertDeadKey(((RecordInsertDeadKeyEvent) record).getChar());
            treemodel.insertNodeInto(node,macronode,i);
        }
    }
    /** Check if we have a popup event for the tree and display then the popup menu */
    private void checkpopup(MouseEvent ev)
    {
        if (ev.isPopupTrigger())
        {
            tree.clearSelection();
            tree.addSelectionPath(tree.getClosestPathForLocation(ev.getX(),ev.getY()));
            Object node=tree.getLastSelectedPathComponent();
            if (node == root)
                rootnodemenu.show(tree,ev.getX(),ev.getY());
            else if (node instanceof MacroNode)
                macronodemenu.show(tree,ev.getX(),ev.getY());
        };
    }
    /** Methods which must be implemented because this is a MouseListener */
    public void mouseClicked(MouseEvent ev)
    {
    }
    
    public void mouseEntered(MouseEvent ev)
    {
        checkpopup(ev);
    }
    
    public void mouseExited(MouseEvent ev)
    {
        checkpopup(ev);
    }
    
    public void mousePressed(MouseEvent ev)
    {
        checkpopup(ev);
    }
    
    public void mouseReleased(MouseEvent ev)
    {
        checkpopup(ev);
    }
    
}
