/*
 * IncrementalSearch.java
 *
 * Created on September 16, 2001, 4:15 PM
 */

package IDE.Search;

import IDE.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;
import java.awt.event.*;
import java.util.*;
import IDE.Messaging.*;
import IDE.Recording.*;
/**
 *  Do a incremental search in a given document
 * @author  marc
 * @version
 */
public class IncrementalSearch implements IQuestion
{
    /** Boolean value to indicate a forward or backward search */
    private boolean forward;
    /**  Indicates if we have reached the end (begin) of the source */
    private boolean endorbegin=false;
    /**  Start position of the last found match */
    private int start;
    /** Search string */
    private String search="";
    /** Document to search */
    private CodeEditor editor;
    /** Hash table containing found string */
    private HashMap founds;
    /** Array of characters to search */
    char[] chars;
    /** Highlighter used to highlight found entries */
    private CodeHighLighter highlighter;
    /** Creates new IncrementalSearch */
    public IncrementalSearch(boolean forward)
    {
        this.forward=forward;
    }
    /** Called to initialize the state */
    public boolean restoreState()
    {
        CodeEditor editor=Main.currentEditor();
        if (editor!=null)
        {
            this.editor=editor;
            founds=new HashMap();
            endorbegin=false;
            if (Recording.isReplaying())
                search="";
            // set the start of the search
            start=editor.cursor.cursorOffset();
            // Get the array of characters of the document in the editor
            chars=editor.getDocument().getChars();
            // Get the highlighter of the editor
            highlighter=editor.highlighter;
            highlighter.removeHighLight("60search");
            if (forward)
                searchForward();
            else
                searchBackward();
            return true;
        }
        else
            return false;
    }
    /** Set the initial Text */
    public void setInitialText(String text)
    {
        search=text;
    }
    /** Check if we have a match from this position */
    private boolean matches(int offset)
    {
        int l=search.length();
        int i;
        for (i=0;i<l && i+offset<chars.length && chars[i+offset]==search.charAt(i);i++);
        if (i==l)
            return true;
        else
            return false;
    }
    
    /** Search forwards for the string */
    private void searchForward()
    {
        int i;
        // Remove any highlighting of found matches
        highlighter.removeHighLight("60search");
        // Search for a match
        for (i=start;i>-1 && i<chars.length && !matches(i);i++);
        if (i<chars.length && i>-1)
        {
            //We found a occurance of the search string
            endorbegin=false;
            if (founds.get(search)==null)
                founds.put(search,new Integer(i));
            start=i;
            editor.cursor.setCursor(i);
            highlighter.addHighLightRegion("60search",Configuration.colorSearch,i,search.length(),true);
        }
        else
        {
            endorbegin=true;
            start=0;
        };
    }
    /** Search backwards for the string */
    private void searchBackward()
    {        
        int i;
        // Remove any highlighting of found matches
        highlighter.removeHighLight("60search");
        // Search for a match
        for (i=start;i>-1 && i<chars.length && !matches(i);i--);
        if (i<chars.length && i>-1)
        {
            //We found a occurance of the search string
            endorbegin=false;
            if (founds.get(search)==null)
                founds.put(search,new Integer(i));
            start=i;
            editor.cursor.setCursor(i);
            highlighter.addHighLightRegion("60search",Configuration.colorSearch,i,search.length(),true);
        }
        else
        {
            endorbegin=true;
            start=chars.length-1;
        };
    }
    /** Will be called by the InstructonField if the user has confirmed its answer  */
    public void completeAnswer(String answer)
    {
    }
    /** Will be called by the instructionfield if the user has changed the inputfield */
    public void newAnswer(String answer)
    {
        if (search.startsWith(answer))
        {
            search=answer;
            Integer newstart=(Integer)  founds.get(search);
            if (newstart!=null)
            {
                start=newstart.intValue();
                if (forward)
                    searchForward();
                else
                    searchBackward();
            }
        }
        else
        {
            search=answer;
            if (forward)
                searchForward();
            else
                searchBackward();
        };
    }
    /** Will be celled by the InstructionField if the user has confirmed its answer */
    public void keyPressed(int keycode,int modifiers)
    {
        if (modifiers==KeyEvent.CTRL_MASK)
        {
            if (keycode==KeyEvent.VK_DELETE)
            {
                search="";
                InstructionField.updateText("");
            }
            else if (keycode==KeyEvent.VK_N)
            {
                start=start+1;
                searchForward();
            }
            else if (keycode==KeyEvent.VK_P)
            {
                start=start-1;
                searchBackward();
            };
        };
    }
    /** Return the quide text for the questioner  */
    public String guideText()
    {
        if (endorbegin)
            return "Search (wrap around)  ctrl-(delete=clear,n=next,p=previous)";
        else
            return "Search expression ctrl-(delete=clear,n=next,p=previous)";
    }
    /** Return the initial field */
    public String initialText()
    {
        return search;
    }
    /** Called to inidicate that we abort the input, returns true if aborting is allowed  */
    public boolean abort()
    {
        highlighter.removeHighLight("60search");
        return true;
    }
}
