/*
 * RegularExpressionSearch.java
 *
 * Created on March 9, 2002, 2:59 PM
 */

package IDE.Search;

/**
 * Dialog box for search and replace action (used in static way)
 * @author  marc
 */

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;
import IDE.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;
import IDE.Configuration.*;
import IDE.WorkArea.*;
import IDE.Messaging.*;
import IDE.Project.Projects;
import IDE.Project.Repositories;
import IDE.Utilities.*;

public class RegularExpressionSearch
{
    // Dialog box displayed
    private static JDialog dialog;
    // Text field containing search expression
    private static JTextField searchField;
    // Text field containing replace expression
    private static JTextField replaceField;
    // Radio button for case sensitivity
    private static JRadioButton caseSensitiveButton;
    // Radio button for case insensitivity
    private static JRadioButton caseInsensitiveButton;
    // Radio button for regexpression use
    private static JRadioButton regexpressionButton;
    // Checkbox for multifile search
    private static JCheckBox multifileCheckBox;
    // Combo box to show the current projects
    private static JComboBox projectsBox;
    // Button corresponding with replace action
    private static JButton replaceButton;
    // Button to replace all occurances 
    private static JButton replaceAllButton;
    // Button corresponding with close action
    private static JButton closeButton;
    // Button corresponding with search action
    private static JButton searchButton;
    // List containing the found items
    private static JTable resultList;
    // Contains the sources containing a match
    private static Vector sources=new Vector(0);
    // Contain the start point of the matches
    private static Vector starts=new Vector(0);
    // Containt the end points of the matches
    private static Vector ends=new Vector(0);
    // Contain the lines containing the matches
    private static Vector lines=new Vector(0);
    // Contain the list of all the projects
	private static String[] projectPaths;    
    public static void SearchAndReplace()
    {
        // Content of the dialog box
        JPanel content=new JPanel();
        content.setLayout(new BoxLayout(content,BoxLayout.Y_AXIS));
        // Build the search dialog box
        dialog=new JDialog();
        dialog.setTitle("Search");
        dialog.setModal(false);
        dialog.setSize(600,300);
        dialog.setContentPane(content);
        dialog.addWindowListener(new WindowAdapter()
        {
            public void windowClosed(WindowEvent ev)
            {
                processClose();
            }
        });
        // Arrange search and replace fields
        GridBagLayout gbl=new GridBagLayout();
        GridBagConstraints gbc=new GridBagConstraints();
        gbc.fill=GridBagConstraints.BOTH;
        JPanel searchAndReplace=new JPanel(gbl);
        content.add(searchAndReplace);
        
        gbc.gridx=0;
        gbc.gridy=0;
        gbc.weightx=0.2;
        JLabel searchLabel=new JLabel("Search For:");
        gbl.setConstraints(searchLabel,gbc);
        searchAndReplace.add(searchLabel);
        
        gbc.gridx=1;
        gbc.gridy=0;
        gbc.weightx=0.8;
        searchField=new JTextField();
        gbl.setConstraints(searchField,gbc);
        searchAndReplace.add(searchField);
        
        gbc.gridx=0;
        gbc.gridy=1;
        gbc.weightx=0.2;
        JLabel replaceLabel=new JLabel("Replace By:");
        gbl.setConstraints(replaceLabel,gbc);
        searchAndReplace.add(replaceLabel);
        
        gbc.gridx=1;
        gbc.gridy=1;
        gbc.weightx=0.8;
        replaceField=new JTextField();
        gbl.setConstraints(replaceField,gbc);
        searchAndReplace.add(replaceField);
        
        // Arrange the  radiobuttons
        JPanel radiobuttons=new JPanel(new FlowLayout(FlowLayout.LEFT));
        content.add(radiobuttons);
        
        ButtonGroup buttongroup=new ButtonGroup();
        
        regexpressionButton=new JRadioButton("Regular expression search");
        caseSensitiveButton=new JRadioButton("Case sensitive search");
        caseInsensitiveButton=new JRadioButton("Case insensitive search");
        caseInsensitiveButton.setSelected(true);
        
        buttongroup.add(regexpressionButton);
        buttongroup.add(caseSensitiveButton);
        buttongroup.add(caseInsensitiveButton);
        
        radiobuttons.add(regexpressionButton);
        radiobuttons.add(caseSensitiveButton);
        radiobuttons.add(caseInsensitiveButton);
        
        // Arrange the checkbox for search of all project files
        JPanel checkbuttons=new JPanel(new FlowLayout(FlowLayout.LEFT));
        multifileCheckBox=new JCheckBox(new AbstractAction("Search in project(s)")
        {
        	public void actionPerformed(ActionEvent ev)
        	{
        		if (multifileCheckBox.isSelected())
        			projectsBox.setEnabled(true);
        		else
        			projectsBox.setEnabled(false);
        	}
        });
		multifileCheckBox.setSelected(true);
        projectsBox=new JComboBox();
        projectsBox.addItem("(All)");
        projectPaths=Projects.getProjects();
        for (int i=0;i<projectPaths.length;i++)
            projectsBox.addItem(projectPaths[i]);
        checkbuttons.add(multifileCheckBox);
        checkbuttons.add(projectsBox);
        content.add(checkbuttons);
        
        
        // Arrange the list of found entries
        resultList=new JTable();
        resultList.setDefaultRenderer(Boolean.class,new BooleanRenderer());
        resultList.setDefaultEditor(Boolean.class,new BooleanEditor(resultList));
        resultList.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        resultList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        resultList.addMouseListener(new ResultMouseListener());
        JScrollPane scrollpane=new JScrollPane(resultList);
        scrollpane.setSize(600,200);
        content.add(scrollpane);
        
        // Arrange the action buttons
        JPanel buttons=new JPanel(new FlowLayout(FlowLayout.LEFT));
        content.add(buttons);
        searchButton=new JButton(new AbstractAction("Search")
        {
            public void actionPerformed(ActionEvent ev)
            {
                processSearch();
            }
        });
        replaceButton=new JButton(new AbstractAction("Replace")
        {
            public void actionPerformed(ActionEvent ev)
            {
                processReplace();
            }
        });
        replaceAllButton=new JButton(new AbstractAction("Replace All")
        {
            public void actionPerformed(ActionEvent ev)
            {
                processReplaceAll();
            }
        });
        closeButton=new JButton(new AbstractAction("Close")
        {
            public void actionPerformed(ActionEvent ev)
            {
                processClose();
            }
        });
        buttons.add(searchButton);
        buttons.add(replaceButton);
        buttons.add(replaceAllButton);
        buttons.add(closeButton);
        
        // Show  the dialog box
        dialog.setVisible(true);
    }
    
    /** 
     * Replace all the matches by the replace string
     */
    private static void processReplaceAll()
    {
        // Set the replace flag on for every entry in the table
        int m=resultList.getRowCount();
        for (int i=0;i<m;i++)
            resultList.setValueAt(new Boolean(true),i,0);
        processReplace();
    }
    /**
     * Process the press of the replace button to optimize we make use of the fact that the table is sorted
     * on source, so that we can use a break mechanisme to avoid opening and closing files for every replace
     * and calculating the deltas if for the begin of the replacements
     */
    private static void processReplace()
    {
        String oldpath=null; // Force a break the first time
        String path;
        File file=null;
        Object source=null;
        int delta=0;
        int begin;
        int end;
        TableModel tm=resultList.getModel();
        int m=tm.getRowCount();
        for (int i=0;i<m;i++)
        {
            if (((Boolean) tm.getValueAt(i,0)).booleanValue())
            {
                path=(String) tm.getValueAt(i,1);
                begin=((Integer) tm.getValueAt(i,3)).intValue();
                end=((Integer) tm.getValueAt(i,4)).intValue();
                if (oldpath==null || path.compareTo(oldpath)!=0)
                {
                    // Set the delta on 0, because we have not yet replaced anything
                    delta=0;
                    // We have a break on the source so we must save the old string if needed
                    if (source instanceof StringBuffer)
                        saveBuffer((StringBuffer) source,file);
                    // Because of a break we have to reset the oldpath
                    oldpath=path;
                    file=new File(path);
                    // Set the new source
                    source=WorkArea.getEditorForLongName(path);
                    if (source==null)
                    {
                        try
                        {
                            BufferedReader reader=new BufferedReader(new FileReader(file));
                            char[] chars=new char[(int) file.length()];
                            reader.read(chars);
                            //source=new StringBuffer(new String(chars));
                            source=new StringBuffer(CodeDocument.removeTabs(new String(chars)));
                        }
                        catch (IOException e)
                        {
                            MessageHandler.warn(e.getMessage());
                        };
                    };
                };
                // Do the replacements
                if (source instanceof StringBuffer)
                    replaceBuffer((StringBuffer) source,begin+delta,end+delta);
                else if (source instanceof CodeEditor)
                    replaceDocument((CodeEditor) source,begin+delta,end+delta);
                // Calculate the delta to be added for the begin and end of the next replacements because of the replacement
                delta=delta-end+begin+replaceField.getText().length();
            };
        };
        if (source instanceof StringBuffer)
            saveBuffer((StringBuffer) source,file);
        // Refresh the search list
        processSearch();
    }
    /** Replace in a file all occurrances of the matches by replace string in a given document */
    private static void replaceDocument(CodeEditor editor,int begin,int end)
    {
        CodeDocument document=editor.getDocument();
        document.remove(begin,end-begin);
        document.insertStringAt(replaceField.getText(),begin);
    }
    /** Replace in a stringbuffer all occurances of the matches by the replace string in a given document */
    private static void replaceBuffer(StringBuffer buffer,int begin,int end)
    {
        buffer.replace(begin,end,replaceField.getText());
    }
    /** Save the buffer to the file */
    private static void saveBuffer(StringBuffer buffer,final File file)
    {
        try
        {
            BufferedWriter writer=new BufferedWriter(new FileWriter(file));
            writer.write(buffer.toString());
            writer.close();
            // In async thread repopulate the file node
            Thread reparse=new Thread()
            {
                public void run()
                {
                    /// TODO Remove this when tested Explorer.refreshFileNode(file);
                    Repositories.scanFile(file);
                }
            };
            reparse.start();
        }
        catch (IOException e)
        {
            MessageHandler.warn(e.getMessage());
        };
    }
    /**
     * Process the press of the search button
     */
    private static void processSearch()
    {
        sources.clear();
        starts.clear();
        ends.clear();
        lines.clear();
        
        if (regexpressionButton.isSelected())
        {
            if (multifileCheckBox.isSelected())
            {
                regExpSearchFiles();
            }
            else
            {
                CodeEditor editor=Main.currentEditor();
                if (editor!=null)
                {
                    regExpSearchDocument(editor.getDocument());
                };
            };
        }
        else if (multifileCheckBox.isSelected())
        {
            searchFiles();
        }
        else
        {
            CodeEditor editor=Main.currentEditor();
            if (editor!=null)
            {
                searchDocument(editor.getDocument());
            };
        };
        resultList.setModel(new ResultTableModel(sources,starts,ends,lines));
        // Set the sizes of the columns
        TableColumnModel cm=resultList.getColumnModel();
        cm.getColumn(0).setMaxWidth(20);
        cm.getColumn(2).setMinWidth(200);
        cm.getColumn(3).setPreferredWidth(50);
        resultList.doLayout();
    }
    /**
     * Process the press of the close button
     */
    private static void processClose()
    {
        dialog.setVisible(false);
        sources.clear();
        starts.clear();
        ends.clear();
        lines.clear();
    }
    /** Search the files in the selected project for a match using case sensitive (insensitive) search  */
    private static void searchFiles()
    {
    	String[] paths;
    	if (projectsBox.getSelectedIndex()==0)
         	paths=Projects.getPathOfAllFiles();
         else 
         	paths=Projects.getPathOfAllFilesIn((String) projectsBox.getSelectedItem()); 
         for (int i=0;i<paths.length;i++)
         	searchFile(new File(paths[i]));
    }
    /** Do a search in a file */
    private static void searchFile(File file)
    {
        try
        {
            Reader reader=new BufferedReader(new FileReader(file));
            char[] content=new char[(int) file.length()];
            reader.read(content);
            String str=CodeDocument.removeTabs(new String(content));
            SimpleMatcher matcher=new SimpleMatcher(str,searchField.getText(),caseInsensitiveButton.isSelected());
            String path=Util.getCanonicalFileName(file);
            while (matcher.find())
            {
                sources.add(path);
                starts.add(new Integer(matcher.start()));
                ends.add(new Integer(matcher.end()));
                lines.add(getLine(str,matcher.start()));
            };
        }
        catch (Exception e)
        {
        };
    }
    /** Search the files in the the selected project for a match using regular expression  */
    private static void regExpSearchFiles()
    {
		String[] paths;
		if (projectsBox.getSelectedIndex()==0)
			paths=Projects.getPathOfAllFiles();
		else 
			paths=Projects.getPathOfAllFilesIn((String) projectsBox.getSelectedItem()); 
		for (int i=0;i<paths.length;i++)
		   searchFile(new File(paths[i]));
    }
     /*
      * Do a regular expression search for the given document
      */
    private static void regExpSearchFile(File file)
    {
        try
        {
            Reader reader=new BufferedReader(new FileReader(file));
            char[] content=new char[(int) file.length()];
            reader.read(content);
            String str=CodeDocument.removeTabs(new String(content));
            Pattern pattern=Pattern.compile(searchField.getText());
            Matcher matcher=pattern.matcher(str);
            String path=Util.getCanonicalFileName(file);
            while (matcher.find())
            {
                sources.add(path);
                starts.add(new Integer(matcher.start()));
                ends.add(new Integer(matcher.end()));
                lines.add(getLine(str,matcher.start()));
            };
        }
        catch (Exception e)
        {
        };
    }
    /** Get the string of a line covering a position in char array */
    private static String getLine(String str,int pos)
    {
        int begin;
        int end;
        int i;
        for (i=pos;i>=0 && i<str.length() && str.charAt(i)!='\n';i--);
        begin=i+1;
        for (i=pos;i<str.length() && str.charAt(i)!='\n';i++);
        end=i;
        return str.substring(begin,end);
    }
    /**
     * Do a case sensitive/case insensitive  expression search for the given document
     */
    private static void searchDocument(CodeDocument document)
    {
        CodeEditor editor=document.getCodeEditor();
        String content=document.getString(0,document.length());
        try
        {
            SimpleMatcher matcher=new SimpleMatcher(content,searchField.getText(),caseInsensitiveButton.isSelected());
            String found=(editor==null) ? Util.getCanonicalFileName(document.getFile()) : editor.getLongName();
            CodeLine line;
            while (matcher.find())
            {
                sources.add(found);
                starts.add(new Integer(matcher.start()));
                ends.add(new Integer(matcher.end()));
                line=document.getLineAt(matcher.start());
                if (line!=null)
                    lines.add(document.getString(line.begin(),line.end()-line.begin()));
            };
        }
        catch (PatternSyntaxException e)
        {
            MessageHandler.warn(e.getMessage());
        };
    }
    /**
     * Do a regular expression search for the given document
     */
    private static void regExpSearchDocument(CodeDocument document)
    {
        CodeEditor editor=document.getCodeEditor();
        String content=document.getString(0,document.length());
        try
        {
            Pattern pattern=Pattern.compile(searchField.getText());
            Matcher matcher=pattern.matcher(content);
            String found=(editor==null) ? Util.getCanonicalFileName(document.getFile()) : editor.getLongName();
            CodeLine line;
            while (matcher.find())
            {
                sources.add(found);
                starts.add(new Integer(matcher.start()));
                ends.add(new Integer(matcher.end()));
                line=document.getLineAt(matcher.start());
                if (line!=null)
                    lines.add(document.getString(line.begin(),line.end()-line.begin()));
            };
        }
        catch (PatternSyntaxException e)
        {
            MessageHandler.warn(e.getMessage());
        };
    }
    /** Select the matched entries in the document */
    protected static void selectMatch(CodeEditor editor,JTable table,int row)
    {
        // Make the editor the current editor
        Main.setCurrent(editor);
        TableModel model=table.getModel();
        int begin=((Integer) model.getValueAt(row,3)).intValue();
        int end=((Integer) model.getValueAt(row,4)).intValue();
        editor.cursor.setCursor(begin);
        editor.highlighter.removeHighLight("60search");
        editor.highlighter.addHighLightRegion("60search",Configuration.colorSearch,begin,end-begin,true);
    }
    /** Open a code editor associated with the file name and display it in the work area, selecting the matched area */
    protected static void selectMatch(File file,JTable table,int row)
    {
        CodePane codepane;
        TableModel model=table.getModel();
        int begin=((Integer) model.getValueAt(row,3)).intValue();
        int end=((Integer) model.getValueAt(row,4)).intValue();
        Parser parser=Repositories.createParser(file,false);
        if (parser==null)
            return;
        else
            codepane=CodePane.createCodePane(parser,file);
        WorkArea.addPane(codepane);
        CodeEditor editor=codepane.getEditor();
        Main.setCurrent(editor);
        editor.cursor.setCursor(begin);
        editor.highlighter.removeHighLight("60search");
        editor.highlighter.addHighLightRegion("60search",Configuration.colorSearch,begin,end-begin,true);
    }
}
/** TableModel used in the JTable */
class ResultTableModel extends AbstractTableModel
{
    /** Contains the list of the sources searched */
    private String[] sources;
    /** Contains the list of the start of the matches */
    private int[] starts;
    /** Contains the list of the end of the matches */
    private int[] ends;
    /** Contains the list of  the line contents of the matched */
    private String[] lines;
    /** Contains the list of the booleans indicating that the value must be replaced */
    private boolean[] mustBeReplaced;
    
    /** Creates a new ResultTableModel and fills it up it is assumed that all vectors have equal length */
    ResultTableModel(Vector sources,Vector starts,Vector ends,Vector lines)
    {
        int m=sources.size();
        this.sources=new String[m];
        for (int i=0;i<m;i++)
            this.sources[i]=(String) sources.elementAt(i);
        
        m=starts.size();
        this.starts=new int[m];
        for (int i=0;i<m;i++)
            this.starts[i]=((Integer) starts.elementAt(i)).intValue();
        
        m=ends.size();
        this.ends=new int[m];
        for (int i=0;i<m;i++)
            this.ends[i]=((Integer) ends.elementAt(i)).intValue();
        
        m=lines.size();
        this.lines=new String[m];
        for (int i=0;i<m;i++)
            this.lines[i]=(String) lines.elementAt(i);
        
        mustBeReplaced=new boolean[m];
    }
    /** Get the number of the columns in this tablemodel */
    public int getColumnCount()
    {
        return 4;
    }
    /** Get the name of the column in this table model */
    public String getColumnName(int col)
    {
        switch (col)
        {
            case 0:
                return "Replace";
            case 1:
                return "Source";
            case 2:
                return "Line";
            case 3:
                return "Begin";
            default:
                return "";
        }
    }
    /** Get the column class (needed for the renderers */
    public Class getColumnClass(int col)
    {
        if (col==0)
            return Boolean.class;
        else
            return String.class;
    }
    /** Get the number of rows */
    public int getRowCount()
    {
        return sources.length;
    }
    /** Indicates that the first column is edible */
    public boolean isCellEditable(int row,int col)
    {
        return (col==0);
    }
    /* Get the value at row column */
    public Object getValueAt(int row, int column)
    {
        switch (column)
        {
            case 0:
                return new Boolean(mustBeReplaced[row]);
            case 1:
                return  sources[row];
            case 2:
                return lines[row];
            case 3:
                return new Integer(starts[row]);
            case 4:
                return new Integer(ends[row]);
            default:
                return null;
        }
    }
    /** Set the value at row column */
    public void setValueAt(Object value,int row,int col)
    {
        if (col==0)
        {
            mustBeReplaced[row]=((Boolean) value).booleanValue();
        };
    }
}
/** Class to render the cells of the result table */
class BooleanRenderer extends JCheckBox implements TableCellRenderer
{
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col)
    {
        if (isSelected)
            setBackground(table.getSelectionBackground());
        else
            setBackground(table.getBackground());
        setSelected(((Boolean) value).booleanValue());
        return this;
    }
}
/** Class to edit the cell values of the result table */
class BooleanEditor extends JCheckBox implements TableCellEditor
{
    // Tablemodel to change if the settings of the editor changes
    JTable table;
    // Set of listeners for editing events
    private Vector listeners=new Vector();
    // Original value
    boolean original=false;
    // Constructor of the editor
    public BooleanEditor(JTable table)
    {
        this.table=table;
        // Add a listener to the checkbox so that it updates the state of the listmodel if pressed
        addActionListener(new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                BooleanEditor.this.table.setValueAt(new Boolean(isSelected()),BooleanEditor.this.table.getSelectedRow(),0);
            };
        });
    }
    public void addCellEditorListener(CellEditorListener listener)
    {
        listeners.addElement(listener);
    }
    
    public void cancelCellEditing()
    {
        setSelected(original);
        int m=listeners.size();
        ChangeEvent ev=new ChangeEvent(this);
        for (int i=0;i<m;i++)
            ((CellEditorListener) listeners.elementAt(i)).editingCanceled(ev);
    }
    
    public Object getCellEditorValue()
    {
        return new Boolean(isSelected());
    }
    
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int col)
    {
        setSelected(((Boolean) value).booleanValue());
        original=((Boolean) value).booleanValue();
        table.setRowSelectionInterval(row,row);
        table.setColumnSelectionInterval(col,col);
        return this;
    }
    
    public boolean isCellEditable(EventObject eventObject)
    {
        return true;
    }
    
    public void removeCellEditorListener(CellEditorListener listener)
    {
        listeners.remove(listener);
    }
    
    public boolean shouldSelectCell(EventObject eventObject)
    {
        return true;
    }
    
    public boolean stopCellEditing()
    {
        int m=listeners.size();
        ChangeEvent ev=new ChangeEvent(this);
        for (int i=0;i<m;i++)
            ((CellEditorListener) listeners.elementAt(i)).editingStopped(ev);
        return true;
    }
}
/** LIsterner to selection model changes */
class ResultMouseListener implements MouseListener
{
    public void mouseClicked(MouseEvent ev)
    {
        JTable table=(JTable) ev.getSource();
        CodeEditor editor;
        int row=table.getSelectedRow();
        String path=(String) table.getValueAt(row,1);
        if (ev.getClickCount()==1)
        {
            // Bring the document if it exist to the foreground and highlight the selected values
            editor=WorkArea.getEditorForLongName(path);
            if (editor!=null)
                RegularExpressionSearch.selectMatch(editor,table,row);
        }
        else if (ev.getClickCount()==2)
        {
            // Bring the document if it exist to the foreground and highlight the selected values
            editor=WorkArea.getEditorForLongName(path);
            if (editor!=null)
                RegularExpressionSearch.selectMatch(editor,table,row);
            else
                // Open a editor to highlight the found match
                RegularExpressionSearch.selectMatch(new File(path),table,row);
        }
    }
    
    public void mouseEntered(MouseEvent ev)
    {
    }
    
    public void mouseExited(MouseEvent ev)
    {
    }
    
    public void mousePressed(MouseEvent ev)
    {
    }
    
    public void mouseReleased(MouseEvent ev)
    {
    }
}
/** Private class, used as a matcher in a case sensitive (insensitive) way (determined at creation time) */
class SimpleMatcher
{
    // Position to start the next match
    private int pos=0;
    // String to scan
    private String source;
    // Match expression for matching
    private String match;
    // Cases sensitivity
    private boolean caseinsensitive;
    // begin of match
    int begin=0;
    // end of the match
    int end=0;
    // Constructor
    protected SimpleMatcher(String source,String match,boolean caseinsensitive)
    {
        this.caseinsensitive=caseinsensitive;
        this.source=source;
        this.match=(caseinsensitive) ? match : match.toUpperCase();
    };
    /** Tries to find a match, changing the state for the next match */
    protected boolean find()
    {
        int m=source.length();
        int ml=match.length();
        for (begin=pos;begin<m && !source.regionMatches(caseinsensitive,begin,match,0,ml);begin++);
        if (begin<m)
        {
            end=begin+ml;
            pos=begin+1;
            return true;
        }
        else
        {
            return false;
        }
    }
    /** Return the begin of a match */
    protected int start()
    {
        return begin;
    }
    /** Return the end of a match */
    protected int end()
    {
        return end;
    }
}
