/*
 * SearchAndReplace.java
 *
 * Created on January 1, 2002, 12:13 PM
 */

package IDE.Search;

import IDE.Configuration.*;
import IDE.Messaging.*;
import IDE.*;
import IDE.CodeEditor.*;
import java.awt.event.*;
import java.util.*;

/**
 *
 * @author  marc
 */
public class SearchAndReplace implements IQuestion
{
    /** Boolean indicating we are ddoing a replace or not */
    private boolean replace=false;
    /** Boolean inidicating a forward / backword search */
    private boolean forward;
    /** Indicates if we have reached the end (begin) of the source */
    boolean endorbegin=false;
    /** Search string */
    private String search="";
    /** start of string found */
    private int start;
    /** Editor to do the search operation */
    private CodeEditor editor;
    /** Hash table containing found string */
    private HashMap founds;
    /** Array of characters to search */
    private char[] chars;
    /** Highlighter used to highlight found entries */
    private CodeHighLighter highlighter;
    /** Creates a new instance of SearchAndReplace */
    public SearchAndReplace()
    {
    }
    /** Called to initialize the Question during replaying  */
    public boolean restoreState()
    {
        CodeEditor editor=Main.currentEditor();
        if (editor!=null)
        {
            this.editor=editor;
            highlighter=editor.highlighter;
            founds=new HashMap();
            forward=true;
            start=editor.cursor.cursorOffset();
            chars=editor.getDocument().getChars();
            highlighter.removeHighLight("60search");
            search="";
            replace=false;
            endorbegin=false;
            searchForward();
            return true;
        }
        else
            return false;
    }
        /** Set the initial Text */
    public void setInitialText(String text)
    {
        search=text;
    }
    /** Will be called by the InstructonField if the user has confirmed its answer by pressing return */
    public void completeAnswer(String answer)
    {
        if (!replace && matches(start))
        {
            replace=true;
        }
        else if (replace)
        {
            if (matches(start))
            {
                editor.replace(start,search.length(),answer);
                chars=editor.getDocument().getChars();
            };
            if (forward)
                searchForward();
            else
                searchBackward();
        };
    }
    /** Return the quide text for the questioner  */
    public String guideText()
    {
        if (replace)
            return "Replace <"+search+"> by ";
        else if (endorbegin)
            return "Search and replace (wrap around) ctrl-(delete=clear,n=next,p=previous), enter=replace";
        else
            return "Search and replace ctrl-(delete=clear,n=next,p=previous), enter-replace";
    }
    /** Return the initial value for the input field  */
    public String initialText()
    {
        return "";
    }
    /** Will be called whenever a key is pressed on the input field  */
    public void keyPressed(int keycode,int modifiers)
    {
        if (modifiers==KeyEvent.CTRL_MASK)
        {
            if (keycode==KeyEvent.VK_DELETE)
            {
                search="";
                InstructionField.updateText("");
            }
            else if (keycode==KeyEvent.VK_N)
            {
                start=start+1;
                searchForward();
            }
            else if (keycode==KeyEvent.VK_P)
            {
                start=start-1;
                searchBackward();
            };
        };
    }
    
    /** Will be called by the InstructionField everytime the user types in something in the instruction field , returns the new string */
    public void newAnswer(String answer)
    {
        if (!replace)
        {
            if (search.startsWith(answer))
            {
                search=answer;
                Integer newstart=(Integer)  founds.get(search);
                if (newstart!=null)
                {
                    start=newstart.intValue();
                    searchForward();
                }
            }
            else
            {
                search=answer;
                searchForward();
            };
        };
    }
    /** Check if we have a match from this position */
    private boolean matches(int offset)
    {
        int l=search.length();
        int i;
        for (i=0;i<l && i+offset<chars.length && chars[i+offset]==search.charAt(i);i++);
        if (i==l)
            return true;
        else
            return false;
    }
    
    /** Search forwards for the string */
    private void searchForward()
    {
        int i;
        // Remove any highlighting of found matches
        highlighter.removeHighLight("60search");
        // Search for a match
        for (i=start;i>-1 && i<chars.length && !matches(i);i++);
        if (i<chars.length && i>-1)
        {
            //We found a occurance of the search string
            endorbegin=false;
            if (founds.get(search)==null)
                founds.put(search,new Integer(i));
            start=i;
            editor.cursor.setCursor(i,false);
            highlighter.addHighLightRegion("60search",Configuration.colorSearch,i,search.length(),true);
        }
        else
        {
            endorbegin=true;
            start=0;
        };
    }
    /** Search backwards for the string */
    private void searchBackward()
    {
        int i;
        // Remove any highlighting of found matches
        highlighter.removeHighLight("60search");
        // Search for a match
        for (i=start;i>-1 && i<chars.length && !matches(i);i--);
        if (i<chars.length && i>-1)
        {
            //We found a occurance of the search string
            endorbegin=false;
            if (founds.get(search)==null)
                founds.put(search,new Integer(i));
            start=i;
            editor.cursor.setCursor(i,false);
            highlighter.addHighLightRegion("60search",Configuration.colorSearch,i,search.length(),true);
        }
        else
        {
            endorbegin=true;
            start=chars.length-1;
        };
    }
    
    /** Called to inidicate that we abort the input, returns true if aborting is allowed  */
    public boolean abort()
    {
        return true;
    }
    
}
