/*
 * Utilities.java
 *
 * Created on January 13, 2003, 7:55 PM
 */

package IDE.Utilities;

import IDE.Messaging.*;
import java.io.*;
import java.util.*;
/**
 * Contains static methods which could not be classified in another way
 * @author  marc
 */
public class Util
{
	/** Used to generate a unique string */
	public static int count;
	/** Used to speed up the find of a the canonical file name */
	public static HashMap fileToPath=new HashMap();
	/** 
	 * Creates a set of paths representing all the directories that lie between the top and the end (top and end not included)
	 * @param top The directory containing end
	 * @param end The directory or file sitting in end
	 * @return Returns the intermediate paths
	 */
	public static File[] calculateExclusivePathDifference(File top,File end)
	{
		ArrayList paths=new ArrayList();
		File parent;
		for (parent=end.getParentFile();parent!=null && parent.compareTo(top)!=0;parent=parent.getParentFile())
			paths.add(parent);
		if (parent==null)
			return new File[0];
		File[] result=new File[paths.size()];
		for (int i=0,j=result.length-1;i<result.length;i++,j--)
			result[j]=(File) paths.get(i);
		return result;
	}
    /** Get the Canonical file name of a file, if this fails report a error and returns the absolute filename */
    public static String getCanonicalFileName(File file)
    {
    	String path=(String) fileToPath.get(file);
    	if (path==null)
    	{
        	try
        	{
            	path=file.getCanonicalPath();
            	fileToPath.put(file,path);
            	return path;
        	}
        	catch (Exception e)
        	{
            	MessageHandler.report("IO Error when trying to determine the canonical file name of a file, returning the absolute path name",e);
            	return file.getAbsolutePath();
        	}
    	}
    	else
    	{
    		return path;
    	}
    }
    /** Compare two files based on the Canonical File name */
    public static boolean FileEqual(File f1,File f2)
    {
    	if (f1==null || f2==null)
    		return false;
        return getCanonicalFileName(f1).compareTo(getCanonicalFileName(f2))==0;
    }
    /** Test of a directory contains a file, directly or inderectly */
    public static boolean dirContains(File directory,File file)
    {
    	if (directory==null || file==null)
    		return false;
    	return getCanonicalFileName(file).startsWith(getCanonicalFileName(directory));
    }
    /**
     * Given a directory and a file produces a new file that sits in this directory with same filename
     * Example : /usr/tmp    /second/test/file.lisp ===> /usr/tmp/file.lisp
     */
    public static File renameFileInDir(File file,File dir)
    {
        return new File(dir,file.getName());
    }
    /** Search (recursively) for a file (dir) based on its base name, returns null if nothing is found */
    public static File findFile(String name)
    {
        File roots[]=File.listRoots();
        File found;
        if (roots==null)
            return null;
        for (int i=0;i<roots.length;i++)
            if ((found=findFile(roots[i],name))!=null)
                return found;
            else
                return null;
        return null;
    }
    /** Search (recursively) for a file(dir)  based on its name in directory */
    public static File findFile(File dir,String name)
    {
        File found;
        if (!dir.isDirectory())
            return null;
        if (dir.getName().compareTo(name)==0)
            return dir;
        File files[]=dir.listFiles();
        for (int i=0;i<files.length;i++)
            if (files[i].getName().compareTo(name)==0)
                return files[i];
            else if (files[i].isDirectory())
                if ((found=findFile(files[i],name))!=null)
                    return found;
        return null;
    }
    /** Normalize a string (repleace \ by \\ and " by \" */
    public static String normalize(String input)
    {
        StringBuffer buffer=new StringBuffer();
        char c;
        for (int i=0;i<input.length();i++)
        {
            c=input.charAt(i);
            switch (c)
            {
                case '\\':
                    buffer.append('\\');
                    buffer.append(c);
                    break;
                case '"':
                    buffer.append('\\');
                    buffer.append(c);
                    break;
                default:
                    buffer.append(c);
            };
        };
        return buffer.toString();
    }

	/** Generates a unique name (is not part of a absolute pathname) and is not equal to previous generated name */
	public static String makeUniqueName()
	{
		count++;
		return "_UNIQUE"+count;
	}
	/**
	 * Returns a array of all the paths of the files or subdirectories in a given directory
	 * @param directory , the directory to enumerate
	 * @return A array of the paths of all the files and directories in the given directory
	 */
	public String[] getPathsFilesOrDirectoriesIn(File directory)
	{
		SortedSet visited=new TreeSet();
		ArrayList found=new ArrayList();
		getFilesOrDirectoriesInImp(found,visited,directory);
		String[] paths=new String[found.size()];
		for (int i=0;i<paths.length;i++)
			paths[i]=getCanonicalFileName((File) found.get(i));
		return paths;
	}
	/**
	 * Returns a array of all the the files or subdirectories in a given directory
	 * @param directory , the directory to enumerate
	 * @return A array of all the files and directories in the given directory
	 */
	public File[] getFilesOrDirectoriesIn(File directory)
	{
		SortedSet visited=new TreeSet();
		ArrayList found=new ArrayList();
		getFilesOrDirectoriesInImp(found,visited,directory);
		File[] paths=new File[found.size()];
		for (int i=0;i<paths.length;i++)
			paths[i]=(File) found.get(i);
		return paths;
	}
	/**
	 * Fill up found with a list of the strings of all the discovered directories
	 * @param found
	 * @param visited
	 * @param directory
	 */
	private void getFilesOrDirectoriesInImp(ArrayList found,SortedSet visited,File directory)
	{
		String path=getCanonicalFileName(directory);
		if (visited.contains(path))
			return; // Avoid to loop 
		visited.add(path);
		if (directory.isDirectory())
		{
			File[] files=directory.listFiles();
			for (int i=0;i<files.length;i++)
			{
				if (files[i].isDirectory())
				{
					found.add(files[i]);
					getFilesOrDirectoriesInImp(found,visited,files[i]);		
				}
				else
				{
					found.add(files[i]);
				}
			}
		}
	}
 /// TODO Cache the retrieving of Canonical filenames to avoid to much access to the file system
}
