/*
 * WatchNode.java
 *
 * Created on December 1, 2001, 3:14 PM
 */

package IDE.Watch;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;
import IDE.Interaction.*;
import IDE.CodeEditor.*;
import IDE.CodeEditor.Parser.*;

/**
 *  Represents a watch node
 * @author  marc
 */
public class WatchNode extends AbstractWatchNode
{
    /** Interaction object to get the value of a expression */
    private IInteraction interaction;
    /** Watch expression */
    private String watch;
    /**  String containing the result of watching watch */
    private String text;
    /** Icon of this node */
    private ImageIcon icon;
    /** Treemodel of this node */
    private DefaultTreeModel treemodel;
    /** Parser used to edit watch expression */
    private Parser parser;
    /** Creates a new instance of WatchNode */
    public WatchNode(String title,String watch,IInteraction interaction,Parser parser,DefaultTreeModel treemodel)
    {
        super(watch);
        this.interaction=interaction;
        this.title=title;
        this.watch=watch;
        this.parser=parser;
        this.treemodel=treemodel;
        // Insert yourself in the tree
        DefaultMutableTreeNode root=(DefaultMutableTreeNode) treemodel.getRoot();
        treemodel.insertNodeInto(this, root, root.getChildCount());
        // Set the watch point
        interaction.setWatchPoint(this, watch);
    }
    /** Cleanup the watch node */
    public void cleanup()
    {
        IInteraction saved=interaction;
        // Ensure that we will not by accident refresh this watchpoint
        interaction=null;
        // Tell the process to not watch anymore
        saved.removeWatchPoint(this);
    }
    /** Use a CodeDialog box to alow for editing of the watch expression */
    public void editWatch()
    {
        if (parser!=null)
        {
            String str=CodeDialogBox.showDialog("Watch Expression",parser.cloneParser(false),watch);
            if (str!=null)
            {
                watch=str;
                title=str;
                // Tell the process to change its stored watch point
                interaction.changeWatchPoint(this, watch);
            };
        };
    }
    /** Get the interaction object */

    public IInteraction getInteraction()
    {
        return interaction;
    }
    
    /**  Get the type of the value in the watch point */
    public ImageIcon  getIcon()
    {
        //return interaction.getTypeIcon(0);
        return icon;
    }
    /** Populate the WatchNode */
    protected void populateWatchNode(int type,Object object,AbstractWatchNode node)
    {
        // Remove all the child nodes
        while (node.getChildCount()>0)
            treemodel.removeNodeFromParent((MutableTreeNode) treemodel.getChild(node,0));
        // Populate the watch node
        if (object instanceof LinkedList)
        {
            node.setTextAndIcon(interaction.getTypeString(type),interaction.getTypeIcon(type));
            populateWatchNodeList((LinkedList) object,node);
        }
        else
        {
            node.setTextAndIcon("["+interaction.getTypeString(type)+"]="+object.toString(),interaction.getTypeIcon(type));
            treemodel.nodeChanged(node);
        }
    }
    /** Create child nodes to represent a list of values */
    protected void populateWatchNodeList(LinkedList list,AbstractWatchNode node)
    {
        Iterator iterator=list.iterator();
        LinkedList typeValue;
        WatchValueNode child;
        // Remove all the child nodes
        while (node.getChildCount()>0)
            treemodel.removeNodeFromParent((MutableTreeNode) treemodel.getChild(node,0));
        // Create the child nodes
        int i=0;
        while (iterator.hasNext())
        {
            typeValue=(LinkedList) iterator.next();
            child=new WatchValueNode("Test");
            if (typeValue.get(0) instanceof Long)
            {
                populateWatchNode(((Long) typeValue.get(0)).intValue(),typeValue.get(1),child);
            }
            else if (typeValue.get(0) instanceof String)
            {
                // Case of slots in classes and structs
                child.title=(String) typeValue.get(0);
                typeValue=(LinkedList) typeValue.get(1);
                populateWatchNode(((Long) typeValue.get(0)).intValue(),typeValue.get(1), child);
            };
            treemodel.insertNodeInto(child,node,i);
            i++;
        }
    }
    /** Set the text and the icon of this node */
    public void setTextAndIcon(String text,ImageIcon icon)
    {
        // We don't have to set the icon because this node is intellegent enough to get the icon
        this.text=text;
        this.icon=icon;
        treemodel.nodeChanged(this);
    }
    /** Get the display string */
    public String getDisplayString()
    {
        return title+"==>"+text;
    }
    /** Is not a leaf element */
    public boolean isLeaf()
    {
        return false;
    }
    /** Refresh the watch node */
    public void refresh(LinkedList list)
    {
        // If the interaction is null this watch point is removed, there is a chance that the
        // process does not now this yet
        if (interaction==null)
            return;
        if (list==null)
        {
            text="Unknown";
            icon=interaction.getTypeIcon(0);
            treemodel.nodeChanged(this);
        }
        else if (list.size()>1)
        {
            text="Multiple Values";
            icon=interaction.getTypeIcon(0);
            populateWatchNodeList(list,WatchNode.this); // ((t1 v1) ... (tn vn))
        }
        else
        {
            list=(LinkedList) list.get(0);
            populateWatchNode(((Long) list.get(0)).intValue(),list.get(1),WatchNode.this);
        }
    }
}
