/*
 * ListOfChoices.java
 *
 * Created on January 2, 2002, 12:20 PM
 */

package IDE.WorkArea;

import IDE.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Represent a list of choices to be displayed in the WorkAre
 * The following key controls are possible
 *  up,down,pageup,pagedown to scroll to the list of choices
 * enter  return selected choice
 * esc return null do nothing
 *
 * @author  marc
 */
public class ListOfChoices
{
    /** Result to be returned */
    private static Object result=null;
    /** Creates a new instance of ListOfChoices  make it private so no one can create this instance */
    private ListOfChoices()
    {
    }
    /** Show the list dialog and return the choice */
    public static Object showList(String title,Vector choices)
    {
        return showList(title,choices,null);
    }
    public static Object showList(String title,Vector choices,Point point)
    {
        // Create a modal dialog box containing a ok and cancel button 
        final JDialog dialog=new JDialog(Main.frame);
        dialog.setTitle(title);
        dialog.setModal(true);
        JPanel pane=new JPanel(new BorderLayout());
        JPanel buttons=new JPanel();
        final JList list=new JList(choices);
        JScrollPane scrollpane=new JScrollPane(list);
        // Create actions and buttons for the ok and cancel actions
        AbstractAction okAction=new AbstractAction("Ok")
        {
            public void actionPerformed(ActionEvent ev)
            {
                result=list.getSelectedValue();
                dialog.setVisible(false);
                dialog.dispose();
            }
        };
        AbstractAction cancelAction=new AbstractAction("Cancel")
        {
            public void actionPerformed(ActionEvent ev)
            {
                result=null;
                dialog.setVisible(false);
                dialog.dispose();
            }
        };
        JButton ok=new JButton(okAction);
        JButton cancel=new JButton(cancelAction);
        buttons.add(ok);
        buttons.add(cancel);
        // Create the list and ensure that it reacts on the enter,exit and  up and down keys
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        if (choices.size()>0)
            list.setSelectedIndex(0);
        list.addKeyListener(new KeyListener()
        {
            public void keyTyped(KeyEvent ev)
            {
            }
            public void keyPressed(KeyEvent ev)
            {
                if (ev.getKeyCode()==KeyEvent.VK_ENTER)
                {
                    result=list.getSelectedValue();
                    dialog.setVisible(false);
                    dialog.dispose();
                    ev.consume();
                }
                else if (ev.getKeyCode()==KeyEvent.VK_ESCAPE)
                {
                    result=null;
                    dialog.setVisible(false);
                    dialog.dispose();
                    ev.consume();
                };
            }
            public void keyReleased(KeyEvent ev)
            {
            }
        });
        // Add components to the dialog box
        pane.add(scrollpane,BorderLayout.CENTER);
        pane.add(buttons,BorderLayout.SOUTH);
        // Ensure that the first component getting focus is the list 
        pane.setFocusTraversalPolicy(new LinearFocusTraversalPolicy(new Component[] {list,ok,cancel}));
        dialog.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent ev)
            {
                list.requestFocus();
            }
            public void focusLost(FocusEvent ev)
            {
            }
        });
        // Finish the creating of the dialog box
        dialog.getContentPane().add(pane);
        dialog.setSize(400,500);
        // Calculating the position of the dialogbox
        Rectangle bounds=GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().getBounds();
        Dimension dim=dialog.getSize();
        int x,y;
        if (point==null)
        {
            x=Math.max(0,(bounds.width-dim.width)/2);
            y=Math.max(0,(bounds.height-dim.height)/2);
        }
        else
        {
            if (point.x+dim.width>bounds.width)
                x=Math.max(0,bounds.width-dim.width);
            else
                x=point.x;
            if (point.y+dim.height>bounds.height)
                y=Math.max(0,bounds.height-dim.height);
            else
                y=point.y;
        };
        dialog.setLocation(x,y);
        dialog.show();
        return result;
    }
}
