/*
 * WorkArea.java
 *
 * Created on December 29, 2001, 10:44 AM
 */

package IDE.WorkArea;

import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import IDE.*;
import IDE.Interaction.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;
import IDE.Utilities.*;
import java.io.*;
import java.awt.event.*;

/**
 * Container class to contain the different editors (mix of JSplitPane and JTabbedPane) (only one instance of this class my exist)
 * @author  marc
 */
public class WorkArea extends JTabbedPane implements ChangeListener,IIDEAddin,MouseListener
{
    /** List of editors displayed in this WorkArea */
    private static Vector editors=new Vector();
    /** Static field to represent the only instance of this class */
    private static  WorkArea workarea;
    /** Contains the currentEditor in this workpane (null if there is no current editor) */
    private static CodeEditor currentEditor;
    /** Creates a new instance of WorkArea */
    private WorkArea()
    {
        addChangeListener(this);
        addMouseListener(this);
        workarea=this;
        setFocusable(false);
    }
    /** Create workarea */
    public static WorkArea createWorkArea()
    {
        if (workarea!=null)
            return workarea;
        else
            return new WorkArea();
    }
    /** Select the workpane containing the editor given as argument */
    public static void selectWorkPaneContainingEditor(CodeEditor editor)
    {
        int m=workarea.getComponentCount();
        for (int i=0;i<m;i++)
            if (((WorkPane) workarea.getComponentAt(i)).containsEditor(editor))
            {
                workarea.removeChangeListener(workarea); // Tempory disable the processing of tab processing
                workarea.setSelectedIndex(i);
                workarea.addChangeListener(workarea);
                // Indicate that this is the current editor in the workpane
                currentEditor=editor;
                break;
            }
    }
    /** Remove the code panes editing the file file */
    public static void removeCodePanes(File file)
    {
        CodeEditor editor;
        int i=0;
        while (i<editors.size())
        {
            editor=(CodeEditor) editors.elementAt(i);
            if (Util.getCanonicalFileName(editor.getDocument().getFile()).compareTo(Util.getCanonicalFileName(file))==0)
            {
                if (editor==currentEditor)
                    currentEditor=null;
                editors.remove(i);
                for (int j=0;j<workarea.getComponentCount();j++)
                {
                    Main.setCurrent(editor);
                    ((WorkPane) workarea.getComponent(j)).close();
                };
            }
            else
            {
                i++;
            };
        };
        WorkArea.refreshTitles();
    }
    /** Add a CodePane to the work area  */
    public static void addPane(CodePane pane)
    {
        CodeEditor editor=pane.getEditor();
        WorkPane workpane=new WorkPane(pane);
        editors.add(editor);
        workarea.addTab(editor.getName(),workpane);
        workarea.setSelectedComponent(workpane);
        WorkArea.refreshTitles();
    }
    /** Add a editor to the list of editors in this work area (used during restore of state), avoids duplicated entries */
    protected static void addEditor(CodeEditor editor)
    {
        if (editors.indexOf(editor)==-1)
            editors.addElement(editor);
    }
    /** Split the current selected pane vertical */
    public void split(int orientation)
    {
        WorkPane workpane=(WorkPane) getSelectedComponent();
        if (workpane!=null)
        {
            workpane.split(orientation,editors);
            refreshTitles();
        };
    }
    /** Called when the state of the codepane is changed */
    public void stateChanged(ChangeEvent changeEvent)
    {
        setSelectedWorkpaneCurrent();
    }
    /** Set the focus on the editor in the current workpane */
    private void setSelectedWorkpaneCurrent()
    {
        WorkPane workpane=(WorkPane) getSelectedComponent();
        if (workpane!=null)
            Main.setCurrent(workpane.getFirstEditor());
        else
            Main.setCurrent(null);
    }
    /** Refresh the titles of all the workpanes */
    public static void refreshTitles()
    {
        int m=workarea.getTabCount();
        WorkPane workpane;
        for (int i=0;i<m;i++)
        {
            workpane=(WorkPane) workarea.getComponentAt(i);
            workarea.setTitleAt(i,workpane.getShortTitle());
            workarea.setToolTipTextAt(i,workpane.getLongTitle());
        };
    }
    /** Replace the editor displayed in a workpane by a copy of another */
    public void replaceEditor()
    {
        WorkPane workpane=(WorkPane) getSelectedComponent();
        if (workpane!=null)
        {
            workpane.replaceEditor(editors);
            refreshTitles();
        };
    }
    /** Close the current workpane */
    public static void close()
    {
        WorkPane workpane=(WorkPane)  workarea.getSelectedComponent();
        if (workpane!=null)
        {
            workpane.close();
            WorkArea.refreshTitles();
        };
    }
    /** Close the whole work area */
    public void closeAll()
    {
        CodeEditor editor;
        /** Close all the editors  */
        currentEditor=null;
        for (int i=0;i<editors.size();i++)
        {
            editor=(CodeEditor) editors.elementAt(i);
            editor.closeEditor();
            removeWorkPaneContaining(editor);
        };
    }
    /** Try to save all open CodeEditors, return false if this fails */
    public boolean saveAll()
    {
        Vector processedDocuments=new Vector();
        CodeEditor editor;
        boolean notsaved=false;
        for (int i=0;i<editors.size();i++)
        {
            editor=(CodeEditor) editors.elementAt(i);
            if (!(processedDocuments.contains(editor.getDocument())))
                if (!(editor.ensureSaved()))
                    notsaved=true;
            processedDocuments.add(editor.getDocument());
        };
        if (notsaved)
            if (JOptionPane.showConfirmDialog(null,"Not all changes are saved, proceed with exit ?","Exit Warning",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                return true;
            else
                return false;
        else
            return true;
    }
    /** Remove editor */
    public static void removeEditor(CodeEditor editor)
    {
        if (currentEditor==editor)
            currentEditor=null;
        editors.remove(editor);
    }
    /** Remove the workpane containing the editor editor */
    public static void removeWorkPaneContaining(CodeEditor editor)
    {
        WorkPane pane;
        editors.remove(editor);
        for (int i=0;i<workarea.getComponentCount();i++)
        {
            pane=(WorkPane) workarea.getComponentAt(i);
            if (pane.containsEditor(editor))
            {
                if (editor==currentEditor)
                    currentEditor=null;
                pane.removePane(editor);
            };
        };
    }
    /** Remove tab */
    public static void removePane(JComponent component)
    {
        workarea.remove(component);
    }
    /**
     * This method will be called just before exit and allow the addin to save its state, the addin must be registered via a call
     * to registerAddin()
     * Use the XMLConfigClass(Field,Method or Object) objects to serialize this class
     */
    public void saveState(Writer writer)
    {
        // Ensure that all the panes specific for debugging are closed  so there state is not saved
        int m=editors.size();
        for (int i=0;i<m;i++)
            if (((CodeEditor) editors.elementAt(i)).getCodePane() instanceof DebugCodePane)
                removeWorkPaneContaining((CodeEditor) editors.elementAt(i));
        // Save the remaining state
        XMLConfigClass cls=new XMLConfigClass(this.getClass().getName());
        XMLConfigMethod method;
        for (int i=0;i<getTabCount();i++)
        {
            method=new XMLConfigMethod("addWorkPane");
            method.addArgument(((WorkPane) getComponentAt(i)).createStateWorkPaneObject());
            cls.addMethod(method);
        };
        cls.serialize(writer);
    }
    /** Add a workpane to the set of workpanes */
    public static void addWorkPane(WorkPane workpane)
    {
        workarea.addTab(workpane.getShortTitle(),workpane);
        workpane.updateEditors(editors);
    }
    /** Get a array of documents in the workarea */
    public static CodeDocument[] getDocuments()
    {
        int m=editors.size();
        CodeDocument[] result=new CodeDocument[m];
        for (int i=0;i<m;i++)
            result[i]=((CodeEditor) editors.elementAt(i)).getDocument();
        return result;
    }
    /** Check if the editor with the given long name is in the work area and return the associated editor of null if not found **/
    public static CodeEditor getEditorForLongName(String longname)
    {
        CodeDocument[] documents=getDocuments();
        CodeEditor editor;
        for (int i=0;i<documents.length;i++)
        {
            editor=documents[i].getCodeEditor();
            if (editor!=null && editor.getLongName().compareTo(longname)==0)
                return editor;
        };
        return null;
    }
    /** Check if the editor containing a document associated with the file is in the work area and return the associated editor or null if not found */
    public static CodeEditor getEditorForFile(File file)
    {
        CodeDocument[] documents=getDocuments();
        for (int i=0;i<documents.length;i++)
            if (documents[i]!=null && documents[i].getFile()!=null && Util.getCanonicalFileName(documents[i].getFile()).compareTo(Util.getCanonicalFileName(file))==0)
                return documents[i].getCodeEditor();
        return null;
    }
    /** Check if there is another CodeDocument then the document of the editor, that is associated with this file */
    public static boolean fileOpenByAnotherEditor(File file,CodeEditor editor)
    {
        int m=editors.size();
        for (int i=0;i<m;i++)
            if (((CodeEditor) editors.elementAt(i)).getDocument()!=editor.getDocument()
            && Util.FileEqual(file, ((CodeEditor) editors.elementAt(i)).getDocument().getFile()))
                return true;
        return false;
    }
    /* Repaint all the components */
    public void repaintAll()
    {
        WorkPane workpane=(WorkPane) getSelectedComponent();
        if (workpane!=null)
            workpane.repaintAll();
    }
    /** Checks if the work area contains the current editor */
    public boolean isCurrent()
    {
        CodeEditor editor=Main.currentEditor();
        return editors.contains(editor);
    }
    /** Get the current editor in the workarea */
    public static CodeEditor getCurrentEditor()
    {
        return currentEditor;
    }
    /** Enumerate all the editors in a deterministic way */
    public void enumerateEditors(Vector editors)
    {
        int m=getComponentCount();
        for (int i=0;i<m;i++)
            if (getComponentAt(i) instanceof WorkPane)
                ((WorkPane) getComponentAt(i)).enumerateEditors(editors);
    }
    /** Go to the editor after the current editor */
    public void next()
    {
        int index=getSelectedIndex();
        CodeEditor editor=Main.currentEditor();
        CodeEditor[] editors;
        if (index>=0 && editor!=null)
        {
            // Find the current editor
            editors=((WorkPane) getComponentAt(index)).getEditors();
            int inext;
            for (inext=0;inext<editors.length && editors[inext]!=editor;inext++);
            inext++;
            if (inext<editors.length)
            {
                Main.setCurrent(editors[inext]);
            }
            else
            {
                index++;
                if (index>=getComponentCount())
                    index=0;
                if (index<getComponentCount())
                    Main.setCurrent(((WorkPane) getComponentAt(index)).getFirstEditor());
            };
        };
    }
    
    public void mouseClicked(java.awt.event.MouseEvent mouseEvent)
    {
    }
    
    public void mouseEntered(java.awt.event.MouseEvent mouseEvent)
    {
    }
    
    public void mouseExited(java.awt.event.MouseEvent mouseEvent)
    {
    }
    
    public void mousePressed(java.awt.event.MouseEvent mouseEvent)
    {
        setSelectedWorkpaneCurrent();
    }
    
    public void mouseReleased(java.awt.event.MouseEvent mouseEvent)
    {
    }
    
}
