/*
 * WorkPane.java
 *
 * Created on December 29, 2001, 11:05 AM
 */

package IDE.WorkArea;

import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.awt.*;
import IDE.CodeEditor.*;
import IDE.Configuration.*;
import java.util.*;
import IDE.*;
import IDE.CodeEditor.Parser.*;
import IDE.Project.*;
import IDE.Utilities.*;
import Install.*;

/**
 *  Panel that has a way of splitting itself horizontal or vertical
 * @author  marc
 */
public class WorkPane extends JSplitPane implements FocusListener
{
    /** First part of this splitpane , is either a CodePane or a JSplitPane */
    JComponent first;
    /** Second part of this splitpane, is either null,a CodePane or a JSplitPane */
    JComponent second=null;
    /** Creates a new instance of WorkPane */
    public WorkPane(CodePane pane)
    {
        super(JSplitPane.VERTICAL_SPLIT);
        first=pane;
        add(pane);
        setDividerSize(3);
        setDividerLocation(0.5);
        setResizeWeight(0.5);
        addFocusListener(this);
    }
    /** Constructor for reconstructing the state */
    public WorkPane(JComponent first,JComponent second,int orientation,int dividerlocation)
    {
        super(orientation);
        this.first=first;
        this.second=second;
        setTopComponent(first);
        setBottomComponent(second);
        setDividerSize(3);
        setDividerLocation(dividerlocation);
        WorkArea.refreshTitles();
        addFocusListener(this);
    }
    /** Constructor for reconstructing the state */
    public WorkPane(JComponent first,int orientation,int dividerlocation)
    {
        super(orientation);
        this.first=first;
        this.second=null;
        setTopComponent(first);
        setDividerSize(3);
        setDividerLocation(dividerlocation);
        WorkArea.refreshTitles();
        addFocusListener(this);
    }
    
    /** Constructor for reconstructing the state */
    public WorkPane(Object first,int orientation,int dividerlocation)
    {
        super(orientation);
        this.first=getMember(first);
        this.second=null;
        setTopComponent(this.first);
        setDividerSize(3);
        setDividerLocation(dividerlocation);
        WorkArea.refreshTitles();
        addFocusListener(this);
    }
    
    public WorkPane(Object first,Object second,int orientation,int dividerlocation)
    {
        super(orientation);
        this.first=getMember(first);
        this.second=getMember(second);
        setTopComponent(this.first);
        setBottomComponent(this.second);
        setDividerSize(3);
        setDividerLocation(dividerlocation);
        WorkArea.refreshTitles();
        addFocusListener(this);
    }
    /** Return either the workpane or a codepane depending on the type of the argument */
    private JComponent getMember(Object object)
    {
        if (object instanceof WorkPane)
        {
            return (JComponent) object;
        }
        else if (object instanceof CodePaneState)
        {
            CodePane codepane;
            String filename=((CodePaneState) object).getFilename();
            Parser parser=((CodePaneState) object).getParser();
            if (filename.length()==0)
            {
                codepane=new CodePane(parser);
            }
            else
            {
                codepane=CodePane.createCodePane(parser,new File(filename));
                WorkArea.addEditor(codepane.getEditor());
            };
            codepane.getEditor().cursor.setCursor(((CodePaneState) object).getCursorPosition());
            return codepane;
        }
        else
        {
            return new CodePane(Repositories.createParser(null,true));
        }
    }
    /** Check if a workpane contains the editor given as argument */
    public boolean containsEditor(CodeEditor editor)
    {
        if (first instanceof CodePane && ((CodePane) first).getEditor()==editor)
            return true;
        else if (second instanceof CodePane && ((CodePane) second).getEditor()==editor)
            return true;
        else if (first instanceof WorkPane && ((WorkPane) first).containsEditor(editor))
            return true;
        else if (second instanceof WorkPane && ((WorkPane) second).containsEditor(editor))
            return true;
        else
            return false;
    }
    /** Get the focussed workpane (this is the workpane containing the editor which has the focus) */
    private WorkPane getFocussedWorkPane()
    {
        WorkPane workpane;
        if (first instanceof CodePane && ((CodePane) first).getEditor().isCurrent())
            return this;
        else if (second instanceof CodePane && ((CodePane) second).getEditor().isCurrent())
            return this;
        else if (first instanceof WorkPane && (workpane=((WorkPane) first).getFocussedWorkPane())!=null)
            return workpane;
        else if (second instanceof WorkPane && (workpane=((WorkPane) second).getFocussedWorkPane())!=null)
            return workpane;
        else
            return null;
    }
    /** Repaint and revalidate all the codepanes in this workpane */
    public void repaintAll()
    {
        if (first instanceof CodePane)
        {
            ((CodePane) first).revalidate();
            ((CodePane) first).refreshBackground();
            ((CodePane) first).initAndRepaintVisible();
        }
        else
        {
            ((WorkPane) first).repaintAll();
        };
        if (second instanceof CodePane)
        {
            ((CodePane) second).revalidate();
            ((CodePane) second).refreshBackground();
            ((CodePane) second).initAndRepaintVisible();
        }
        else if (second !=null)
        {
            ((WorkPane) second).repaintAll();
        };
    }
    /** Get the list of the editors in this workpane */
    public CodeEditor[] getEditors()
    {
        Vector editors=new Vector();
        getEditorsImp(editors);
        int m=editors.size();
        CodeEditor[] result=new CodeEditor[m];
        for (int i=0;i<m;i++)
            result[i]=(CodeEditor) editors.elementAt(i);
        return result;
    }
    private void getEditorsImp(Vector editors)
    {
        if (first instanceof CodePane)
            editors.add(((CodePane) first).getEditor());
        else
            ((WorkPane) first).getEditorsImp(editors);
        if (second instanceof CodePane)
            editors.add(((CodePane) second).getEditor());
        else if (second!=null)
            ((WorkPane) second).getEditorsImp(editors);
    }
    /** Replace the selected editor by another editor */
    public void replaceEditor(Vector editors)
    {
        if (editors.size()>0)
        {
            WorkPane workpane=getFocussedWorkPane();
            if (workpane!=null)
                workpane.replaceEditorDialog(editors);
        };
    }
    /** Dot the actual work of replacing the editor */
    private void replaceEditorImp(CodeEditor editor,Vector editors)
    {
        CodePane codepane;
        if (first instanceof CodePane && ((CodePane) first).getEditor().isCurrent())
        {
            codepane=editor.getCodePane().cloneCodePane();
            editors.add(codepane.getEditor());
            first=codepane;
            setTopComponent(codepane);
        }
        else if (second instanceof CodePane && ((CodePane) second).getEditor().isCurrent())
        {
            codepane=editor.getCodePane().cloneCodePane();
            editors.add(codepane.getEditor());
            second=codepane;
            setBottomComponent(codepane);
        }
    }
    /** Show a list of editors to choose from for displacement */
    private void replaceEditorDialog(final Vector editors)
    {
        final JDialog dialog=new JDialog();
        dialog.setTitle("Choose a document");
        dialog.setModal(true);
        JPanel pane=new JPanel(new BorderLayout());
        JPanel buttons=new JPanel();
        final JList list=new JList(editors);
        JScrollPane scrollpane=new JScrollPane(list);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setSelectedIndex(0);
        JButton ok=new JButton(new AbstractAction("Ok")
        {
            public void actionPerformed(ActionEvent ev)
            {
                replaceEditorImp((CodeEditor) list.getSelectedValue(),editors);
                dialog.setVisible(false);
                dialog.dispose();
            }
        });
        JButton cancel=new JButton(new AbstractAction("Cancel")
        {
            public void actionPerformed(ActionEvent ev)
            {
                dialog.setVisible(false);
                dialog.dispose();
            }
        });
        buttons.add(ok);
        buttons.add(cancel);
        pane.add(scrollpane,BorderLayout.CENTER);
        pane.add(buttons,BorderLayout.SOUTH);
        // Ensure that the first component getting focus is the list then next the ok button and then cancel en then list again
        pane.setFocusTraversalPolicy(new LinearFocusTraversalPolicy(new Component[] {list,ok,cancel}));
        dialog.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent ev)
            {
                list.grabFocus();
            }
            public void focusLost(FocusEvent ev)
            {
            }
        });
        dialog.getContentPane().add(pane);
        dialog.setSize(400,500);
        dialog.setVisible(true);
    }
    /** Split this pane */
    public void split(int orientation,Vector editors)
    {
        WorkPane workpane=getFocussedWorkPane();
        if (workpane!=null)
            workpane.splitImp(orientation,editors);
    }
    /** Split implementation */
    private void splitImp(int orientation,Vector editors)
    {
        // We may assume that this WorkPane contains the editor which has focus
        int index=0;
        CodePane codepane,newcodepane;
        WorkPane workpane;
        if (first instanceof CodePane && ((CodePane) first).getEditor().isCurrent())
        {
            codepane=(CodePane) first;
            if (second==null)
            {
                // This plane is not yet splitted
                setOrientation(orientation);
                second=codepane.cloneCodePane();                
                add((CodePane) second);
                setDividerLocation(0.5);
                setDividerSize(3);
                setOneTouchExpandable(true);
                resetToPreferredSizes();
            }
            else
            {
                // Change first in a Workpane
                workpane=new WorkPane(codepane);
                workpane.second=codepane.cloneCodePane();
                workpane.setOrientation(orientation);
                workpane.add((CodePane) workpane.second);
                workpane.setDividerLocation(0.5);
                workpane.setOneTouchExpandable(true);
                first=workpane;
                setTopComponent(workpane);
                setDividerLocation(0.5);
                setDividerSize(3);
                setOneTouchExpandable(true);
                resetToPreferredSizes();
                workpane.resetToPreferredSizes();
            }
        }
        else if (second instanceof CodePane && ((CodePane) second).getEditor().isCurrent())
        {
            codepane=(CodePane) second;
            // Change second in a workpane
            workpane=new WorkPane(codepane);
            workpane.second=codepane.cloneCodePane();
            workpane.setOrientation(orientation);
            workpane.add((CodePane) workpane.second);
            workpane.setDividerLocation(0.5);
            workpane.setDividerSize(3);
            workpane.setOneTouchExpandable(true);
            second=workpane;
            setBottomComponent(workpane);
            setDividerLocation(0.5);
            setDividerSize(3);
            setOneTouchExpandable(true);
            resetToPreferredSizes();
            workpane.resetToPreferredSizes();
        };
        updateEditors(editors);
    }
    /** Get the first editor in this workpane */
    public CodeEditor getFirstEditor()
    {
        if (first instanceof WorkPane)
            return ((WorkPane) first).getFirstEditor();
        else if (first instanceof CodePane)
            return ((CodePane) first).getEditor();
        else
            return null;
    }
    /** Get the short title to be used for this workpane */
    public String getShortTitle()
    {
        String title="";
        if (first instanceof CodePane)
            title=((CodePane) first).getEditor().getShortName();
        else if (first instanceof WorkPane)
            title=((WorkPane) first).getShortTitle();
        if (second instanceof CodePane)
            title=title+"-"+((CodePane) second).getEditor().getShortName();
        else if (second instanceof WorkPane)
            title=title+"-"+((WorkPane) second).getShortTitle();
        return title;
    }
    /** Get the long title to be used for this workpane */
    public String getLongTitle()
    {
        String title="";
        if (first instanceof CodePane)
            title=((CodePane) first).getEditor().getLongName();
        else if (first instanceof WorkPane)
            title=((WorkPane) first).getLongTitle();
        if (second instanceof CodePane)
            title=title+"-"+((CodePane) second).getEditor().getLongName();
        else if (second instanceof CodePane)
            title=title+"-"+((WorkPane) second).getLongTitle();
        return title;
    }
    /** Close the current CodeEditor */
    
    public void close()
    {
        if (removeCodePane())
            WorkArea.removePane(this);
    }
    /** Remove the code pane from the workpane , returning true if nothing is in workpane false if other entries are left in workpane */
    private boolean removeCodePane()
    {
        CodeEditor editor;
        if (first instanceof CodePane && ((CodePane) first).getEditor().isCurrent())
        {
            // Close the editor and remove every reference to the editor from the workarea
            editor=((CodePane) first).getEditor();
            // Check if the editor content is saved
            if (!editor.ensureSaved())
                if (JOptionPane.showConfirmDialog(null,"Changes are not saved, continue close ?","Error closing",JOptionPane.YES_NO_OPTION)!=JOptionPane.OK_OPTION)
                    return false;
            editor.closeEditor();
            WorkArea.removeEditor(editor);
            // Remove the first
            remove(first);
            if (second==null)
            {
                first=null;
                return true;
            }
            else
            {
                // Move the second to the first
                first=second;
                second=null;
                return false;
            }
        }
        else if (second instanceof CodePane && ((CodePane) second).getEditor().isCurrent())
        {
            // Close the editor and remove every reference to the editor from the workarea
            editor=((CodePane) second).getEditor();
            // Ensure that the editor content is saved
            if (!editor.ensureSaved())
                if (JOptionPane.showConfirmDialog(null,"Changes are not saved, continue close ?","Error closing",JOptionPane.YES_NO_OPTION)!=JOptionPane.OK_OPTION)
                    return false;
            editor.closeEditor();
            WorkArea.removeEditor(editor);
            // remove the second instance
            remove(second);
            second=null;
            return false;
        }
        else
        {
            // First nor second is a code pane containing the current editor
            if (first instanceof WorkPane)
            {
                if (((WorkPane) first).removeCodePane())
                {
                    if (second==null)
                    {
                        // The last entry in this workpane is removed
                        remove(first);
                        first=null;
                        return true;
                    }
                    else
                    {
                        // Move the second to the first
                        remove(first);
                        first=second;
                        second=null;
                        return false;
                    }
                }
                else
                {
                    // We don't have to remove the first entry
                    return false;
                }
            };
            if (second instanceof WorkPane)
            {
                if (((WorkPane) second).removeCodePane())
                {
                    // Remove the second entry
                    remove(second);
                    second=null;
                    return true;
                };
            };
            return false;
        }
    }
    /** Enumerate the list of all codeeditors in deterministic way */
    public void enumerateEditors(Vector editors)
    {
        if (first instanceof CodePane)
            editors.add(((CodePane) first).getEditor());
        else if (first instanceof WorkPane)
            ((WorkPane) first).enumerateEditors(editors);
        if (second instanceof CodePane)
            editors.add(((CodePane) second).getEditor());
        else if (second instanceof WorkPane)
            ((WorkPane) second).enumerateEditors(editors);
    }
    /** Remove the workpane containing the editor editor (the editor is already closed and workarea has already removed a reference to the editor) */
    public void removePane(CodeEditor editor)
    {
        CodePane codepane;
        if (first instanceof CodePane && ((CodePane) first).getEditor()==editor)
        {
            codepane=(CodePane) first;
            if (second==null)
            {
                // We close the last codepane in this window
                WorkArea.removePane(this);
            }
            else
            {
                // We close only the first codepane in this window
                remove(first);
                first=second;
                second=null;
            }
        }
        else if (second instanceof CodePane && ((CodePane) second).getEditor()==editor)
        {
            codepane=(CodePane) second;
            // We close the last codepane in this window
            remove(second);
            second=null;
        };
    }
    /** Create a XMLConfigObject for saving the state of this workpane */
    public XMLConfigObject createStateWorkPaneObject()
    {
        XMLConfigObject object=new XMLConfigObject(this.getClass().getName());
        if (first instanceof CodePane)
            object.addArgument(createStateCodePaneObject((CodePane) first));
        else if (first instanceof WorkPane)
            object.addArgument(((WorkPane) first).createStateWorkPaneObject());
        if (second instanceof CodePane)
            object.addArgument(createStateCodePaneObject((CodePane) second));
        else if (second instanceof WorkPane)
            object.addArgument(((WorkPane) second).createStateWorkPaneObject());
        object.addArgument(getOrientation());
        object.addArgument(getDividerLocation());
        return object;
    }
    
    /** Create a XMLConfig object saving the state of a CodePane */
    private XMLConfigObject createStateCodePaneObject(CodePane codepane)
    {
        XMLConfigObject object=new XMLConfigObject(new CodePaneState(null,null).getClass().getName());
        XMLConfigObject parser=new XMLConfigObject(codepane.getEditor().getDocument().getParser().getClass().getName());
        parser.addArgument(true);
        object.addArgument(parser);
        File file=codepane.getEditor().getDocument().getFile();
        object.addArgument((file==null) ? "" : OsSystem.correctPathName(Util.getCanonicalFileName(file)));
        object.addArgument(codepane.getEditor().cursor.cursorOffset());
        return object;
    }

    /** Update the vector of editors in its arguments */
    public void updateEditors(Vector editors)
    {
        if (first instanceof CodePane)
            uniqueAdd(((CodePane) first).getEditor(),editors);
        else if (first instanceof WorkPane)
            ((WorkPane) first).updateEditors(editors);
        if (second instanceof CodePane)
            uniqueAdd(((CodePane) second).getEditor(),editors);
        else if (second instanceof WorkPane)
            ((WorkPane) second).updateEditors(editors);
    }
    /**  Add a elelement to a vector ensuring that there are no duplicates */
    private static void uniqueAdd(Object object,Vector objects)
    {
        if (!objects.contains(object))
            objects.add(object);
    }
    /** Ensure that the current editor has the focus and not just a container containing the editor */
    public void focusGained(java.awt.event.FocusEvent focusEvent)
    {
        CodeEditor editor=Main.currentEditor();
        if (editor!=null)
            editor.requestFocus();
    }
    public void focusLost(java.awt.event.FocusEvent focusEvent)
    {
        if (!(focusEvent.getOppositeComponent() instanceof CodeEditor))
        {
            CodeEditor editor=Main.currentEditor();
            if (editor!=null)
                CodeCursor.cancelCursor();
        };
    }
    
}
